PRO CHANGE_ALM_RESOLUTION, alm_in, alm_out, beam_in=beam_in, beam_out=beam_out, globalfactor=globalfactor, lmax=lmax, double=double, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CHANGE_ALM_RESOLUTION: procedure to change the resolution of alm coefficients'
     PSM_INFO_MESSAGE, 'SYNTAX: CHANGE_ALM_RESOLUTION, alm_in, alm_out, beam_in=, beam_out=, globalfactor=, lmax=, /double'
     PSM_INFO_MESSAGE, 'set /double to force double precision for output alm'
     PSM_INFO_MESSAGE, 'beam_in and beam_out are structures for the input and output beam'
     PSM_INFO_MESSAGE, 'globalfactor is an optional global factor to apply to the alm'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF


  s = SIZE(alm_in,/type)

  IF s EQ 7 THEN BEGIN
     ;; Assume the input alm is a file

     IF NOT DEFINED(beam_in) THEN BEGIN
        beam_in = FITSFILE_BEAM(alm_in)
     ENDIF ELSE BEGIN
        filebeam_in = FITSFILE_BEAM(alm_in)
        IF filebeam_in.type EQ 'UNKNOWN' THEN BEGIN
           ;; use provided beam_in
        ENDIF ELSE BEGIN
           IF beam_in.type EQ 'UNKNOWN' THEN BEGIN
              beam_in = filebeam_in
           ENDIF ELSE BEGIN
              ok = STRUCT_EQUAL(filebeam_in,beam_in)
              IF NOT ok THEN BEGIN
                 PSM_DEBUG_MESSAGE, 'Implement appropriate comparison of assumed input beam and beam stored in file', routine='CHANGE_ALM_RESOLUTION'
                 STOP
              ENDIF
           ENDELSE
        ENDELSE
     ENDELSE

     IF NOT DEFINED(beam_in) THEN BEGIN
        IF DEFINED(beam_out) THEN BEGIN
           MESSAGE, 'Input beam is not defined';, /info
           PROHELP, 'change_alm_resolution'
           RETURN
        ENDIF
     ENDIF ELSE BEGIN
        IF beam_in.type EQ 'UNKNOWN' THEN BEGIN
           MESSAGE, 'Input beam not defined';, /info
           PROHELP, 'change_alm_resolution'
           RETURN
        ENDIF
     ENDELSE

     ;; Find polarisation and lmax
     polar = ISPOLARISED(alm_in,/alm)
     IF NOT DEFINED(lmax) THEN lmax = FITSFILE_LMAX(alm_in)

     ;; Compute beam ratio

     IF DEFINED(beam_out) THEN BEGIN
        IF polar THEN ratio = PSM_BL_RATIO(beam_out,beam_in,lmax=lmax,double=double, dim=3) $
        ELSE ratio = PSM_BL_RATIO(beam_out,beam_in,lmax=lmax,double=double)
        IF DEFINED(globalfactor) THEN ratio = ratio*globalfactor
     ENDIF ELSE BEGIN
        IF polar THEN BEGIN
           IF KEYWORD_SET(double) THEN ratio = DBLARR(lmax+1,3) ELSE ratio = FLTARR(lmax+1,3)
        ENDIF ELSE BEGIN
           IF KEYWORD_SET(double) THEN ratio = DBLARR(lmax+1) ELSE ratio = FLTARR(lmax+1)
        ENDELSE
        IF DEFINED(globalfactor) THEN ratio[*] = globalfactor ELSE ratio[*] = 1
     ENDELSE

     ;; Read the alm
     READ_ALM, alm_in, alm, index, dtshp='array'
     ;INDEX2L_ONLY, TEMPORARY(index), l
     INDEX2L_ONLY, index, l

     ;; Apply the beam ratio
     IF polar THEN BEGIN
        alm[*,0,0] = alm[*,0,0] * ratio[l,0]
        alm[*,0,1] = alm[*,0,1] * ratio[l,1]
        alm[*,0,2] = alm[*,0,2] * ratio[l,2]
        alm[*,1,0] = alm[*,1,0] * ratio[l,0]
        alm[*,1,1] = alm[*,1,1] * ratio[l,1]
        alm[*,1,2] = alm[*,1,2] * ratio[l,2]
     ENDIF ELSE BEGIN
        alm[*,0] = alm[*,0] * ratio[l]
        alm[*,1] = alm[*,1] * ratio[l]
     ENDELSE

     IF DEFINED(alm_out) EQ 0 THEN alm_out = TEMPORARY(alm) $
     ELSE BEGIN
        s = SIZE(alm_out,/type)

        IF s EQ 7 THEN BEGIN
           ;; Assume the output alm is a file
           WRITE_ALM, alm, alm_out

           ;; Fix the header, quick and dirty
           ;;---------------------------------
           PSM_DEBUG_MESSAGE, 'Implement header in the writing of output alm', routine='CHANGE_ALM_RESOLUTION', infolimit=2, stoplimit=3
           
           newbeamhdr = PSM_BEAM_HDR(beam_out)
           IF polar THEN nxt=3 ELSE nxt=1

           FOR e=1, nxt DO BEGIN

              ;; update beam header
              ;;--------------------
              hdr_in = HEADFITS(alm_in,exten=e)
              UPDATE_PSMHDRBLOCK, hdr_in, newbeamhdr
              
              ;; update base header
              ;;--------------------
              EXTRACT_PSMHDRBLOCK, hdr_in, bashdrblock, blockname='base', whereblock=whereblock
              IF whereblock[0] EQ -1 THEN BEGIN
                 bashdrblock = PSM_BASE_HDR(dataform='ALM', /setval)
                 hdr_in = MERGE_FITSHDRS(hdr_in, bashdrblock)
              ENDIF ELSE BEGIN
                 PSM_ID_KEYGEN, file_key
                 SXADDPAR, hdr_in, 'PSM_FLID', file_key
              ENDELSE

              ;; alm header
              ;;------------
              EXTRACT_PSMHDRBLOCK, hdr_in, almhdrblock, blockname='alm', whereblock=whereblock
              IF whereblock[0] EQ -1 THEN BEGIN
                 almhdrblock = PSM_ALM_HDR(lmax=lmax)
                 hdr_in = MERGE_FITSHDRS(hdr_in, almhdrblock)
              ENDIF ELSE BEGIN
                 ;; do nothing
              ENDELSE

              MODFITS, alm_out, 0, hdr_in, exten=e
           ENDFOR

        ENDIF ELSE BEGIN
           alm_out = TEMPORARY(alm)
           RETURN
        ENDELSE


     ENDELSE

  ENDIF 

  IF( (s LE 5) OR (s EQ 8)) THEN BEGIN
     
     ;; Write the alm
     file = PSM_TMPFILENAME(extension='.fits')
     WRITE_ALM, alm_in, file, beam=beam_in
     
     ;; Change the resolution
     CHANGE_ALM_RESOLUTION, file, alm_out, beam_out=beam_out, globalfactor=globalfactor, lmax=lmax, double=double

     ;; Erase the file
     FILE_DELETE, file, /quiet
     
  ENDIF
  

END
