; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ALL_INSTRUMENTS
;
; PURPOSE:
;	Functions that makes a single instrument structure for
;	describing all instruments loaded in a PSM run
;
; CATEGORY:
;	Instrument model
;
; CALLING SEQUENCE:
;	Results = ALL_INSTRUMENTS()
;
; INPUTS:
;	None
;
; OUTPUTS:
;	Result is an output structure modeling the uninon of all
;	instruments in the PSM run.
;
; COMMON BLOCKS:
;	OBS:	observation parameters for a PSM run
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	18/01/2009:	Jacques Delabrouille, First version.
; 	06/07/2009:	JD, removed explicit instrument list from
; 			call, replaced by common block.
;-
FUNCTION ALL_INSTRUMENTS, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'ALL_INSTRUMENTS: function which returns a structure for all PSM instruments together'
     PSM_INFO_MESSAGE, 'SYNTAX: result = ALL_INSTRUMENTS()'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  COMMON obs, obser
  ni = N_ELEMENTS(obser.inst)
  
  CASE obser.inst[0] OF
     'PSM_IDEAL': inst = LOAD_INSTRUMENT('PSM_IDEAL')
     'WMAP': BEGIN
        channel_list = obser.wmap.frequencies
        version = obser.wmap.version
        inst = LOAD_INSTRUMENT('WMAP',channel_list=channel_list,version=version)
     END
     'LFI_RIMO': BEGIN
        channel_list = obser.lfi_rimo.frequencies
        version = obser.lfi_rimo.version
        inst = LOAD_INSTRUMENT('LFI_RIMO',channel_list=channel_list,version=version)
     END
     'HFI_RIMO': BEGIN
        channel_list = obser.hfi_rimo.frequencies
        version = obser.hfi_rimo.version
        inst = LOAD_INSTRUMENT('HFI_RIMO',channel_list=channel_list,version=version)
     END
     ELSE: BEGIN
        ok = EXECUTE('channel_list = obser.'+obser.inst[0]+'.frequencies')
        inst = LOAD_INSTRUMENT(obser.inst[0],channel_list=channel_list)
     END
  ENDCASE

  IF ni GE 2 THEN BEGIN

     FOR i=1, ni-1 DO BEGIN
        ;newinst = LOAD_INSTRUMENT(obser.inst[i])
        CASE obser.inst[i] OF
           'PSM_IDEAL': inst = LOAD_INSTRUMENT(obser.inst[i])
           'WMAP': BEGIN
              channel_list = obser.wmap.frequencies
              version = obser.wmap.version
              newinst = LOAD_INSTRUMENT('WMAP',channel_list=channel_list,version=version)
           END
           'LFI_RIMO': BEGIN
              channel_list = obser.lfi_rimo.frequencies
              version = obser.lfi_rimo.version
              newinst = LOAD_INSTRUMENT('LFI_RIMO',channel_list=channel_list,version=version)
           END
           'HFI_RIMO': BEGIN
              channel_list = obser.hfi_rimo.frequencies
              version = obser.hfi_rimo.version
              newinst = LOAD_INSTRUMENT('HFI_RIMO',channel_list=channel_list,version=version)
           END
           ELSE: BEGIN
              ok = EXECUTE('channel_list = obser.'+obser.inst[i]+'.frequencies')
              newinst = LOAD_INSTRUMENT(obser.inst[i],channel_list=channel_list)
           END
        ENDCASE
        n = inst.nchannel
        creation_string = "instr = {name:inst.name+'+'+newinst.name, nchannel:n+newinst.nchannel"
        FOR j=1,n DO creation_string = creation_string+",channel_"+NUMBER2STRING(j)+":inst.channel_"+NUMBER2STRING(j)
        FOR j=1,newinst.nchannel DO creation_string = creation_string+",channel_"+NUMBER2STRING(j+n)+":newinst.channel_"+NUMBER2STRING(j) 
        creation_string = creation_string+'}'
        ok = EXECUTE(creation_string)
        inst = instr
     ENDFOR

  ENDIF

  RETURN, inst
END
