; -----------------------------------------------------------------------------
;
; Copyright (C) 2008-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	LOAD_INSTRUMENT
;
; PURPOSE:
;	Code for creating a structure used for a simplified
;	description of instrument properties.
;
; CATEGORY:
;	Instrument model
;
; CALLING SEQUENCE:
;	Result = LOAD_INSTRUMENT(Instrument_name, CHANNEL_LIST=, VERSION=, FREQS=)
;
; INPUTS:
;	Instrument_name:	Name of the instrument to be loaded. 
;				The current options are:
;				PSM_IDEAL:	versions = N/A
;				HFI_IDEAL:	versions = N/A
;				LFI_IDEAL:	versions = N/A
;				HFI_BLUEBOOK:	versions = N/A
;				LFI_BLUEBOOK:	versions = N/A
;				HFI_RIMO:	versions = {'DX7-v1','DX8-v1','DX9-v1','FFP4-v2','FFP6-v1'}
;				LFI_RIMO:	versions = {'DX6-v1','DX7-v1','DX8-v1','DX9-v1','FFP4-v1','FFP4-v2','FFP6-v1'}
;				WMAP:		versions = {'1yr','3yr','5yr','7yr'}
;				IRAS_IDEAL:	versions = N/A
;				IRAS_RIMO:	versions = N/A
;
; KEYWORD PARAMETERS:
;	CHANNEL_LIST:		list of channels to be included in the instrument structure.
;
; OUTPUTS:
;	Result is the output structure modeling the instrument
;
; SIDE EFFECTS:
;	An instrument model contains pointers for part of the
;	information, make sure that heap variables are cleaned to
;	avoid dangling references.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	02/12/2008:	Jacques Delabrouille, First version.
; 	17/09/2012:	JD, implemented LFI_RIMO version FFP6-v1,
; 			including a patch for LFI square band approximation.
; 	22/09/2012:	JD, implemented HFI_RIMO version FFP6-v1.
;-

;;==========================================================================================================
;;   SUBROUTINE
;;==========================================================================================================
FUNCTION ONE_CHANNEL, band=band, beam=beam, pix=pix, obs_units=obs_units, compute_conversion=compute_conversion, inst=inst, version=version

  conversion = { ysz2units:0d0, kcmb2units:0d0, krj2units:0d0, megajysr2units:0d0}
  IF NOT KEYWORD_SET(inst) THEN inst=''
  IF NOT KEYWORD_SET(version) THEN version=''

  onechannel = { inst:inst, $             ;
                 version:version, $       ;
                 name:'', $               ;
                 band: band, $            ;
                 beam: beam, $            ; 
                 pix: pix, $              ;
                 stokes: '', $            ; 
                 obs_units:'', $          ;
                 conversion:conversion $  ;
               }
  IF KEYWORD_SET(compute_conversion) NE 0 THEN BEGIN
     IF DEFINED(obs_units) THEN BEGIN
        onechannel.obs_units = obs_units
        onechannel.conversion.ysz2units = CONVERSION_FACTOR('y_sz',obs_units,band)
        onechannel.conversion.krj2units = CONVERSION_FACTOR('K_RJ',obs_units,band)
        onechannel.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',obs_units,band)
        onechannel.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',obs_units,band)
     ENDIF ELSE BEGIN
        MESSAGE, 'Cannot compute conversion factor (obs_units missing)'
     ENDELSE
  ENDIF   
  RETURN, onechannel

END

;;==========================================================================================================
;;   SUBROUTINE
;;==========================================================================================================
FUNCTION N_CHANNEL_INSTRUMENT, nchannel, version=version, bandshape=bandshape, beamtype=beamtype, pixtype=pixtype, instname=instname, polar=polar
 
  IF NOT DEFINED(version) THEN version=''
  IF NOT DEFINED(bandshape) THEN bandshape='DIRAC'
  IF NOT DEFINED(beamtype) THEN beamtype='GAUSSIAN'
  IF NOT DEFINED(pixtype) THEN pixtype='HEALPIX'
  IF NOT KEYWORD_SET(polar) THEN dim=1 ELSE dim=3

  IF nchannel NE 0 THEN BEGIN
     band = GET_BAND_STRUCT(bandshape)
     beam = GET_BEAM_STRUCT(beamtype,dim=dim)
     pix = GET_PIX_STRUCT(pixtype)
     
     onechannel = ONE_CHANNEL(band=band, beam=beam, pix=pix, inst=instname, version=version)
  ENDIF

  IF NOT KEYWORD_SET(instname) THEN name='' ELSE name=instname
  creation_string = "inst = {name:name,version:version,nchannel:nchannel"
  IF nchannel NE 0 THEN FOR i=0, nchannel-1 DO creation_string = creation_string + ',channel_'+NUMBER2STRING(i+1)+':onechannel'
  creation_string = creation_string + '}'

  ok = EXECUTE(creation_string)
  RETURN, inst

END

;;==========================================================================================================
;;   SUBROUTINE
;;==========================================================================================================
PRO ADD_A_CHANNEL, inst, channel, version=version
  
  nchannel = inst.nchannel+1
  name = inst.name
  creation_string = 'newinst = {name:inst.name,version:version,nchannel:nchannel'
  IF nchannel GE 2 THEN FOR i=0, nchannel-2 DO creation_string = creation_string + ',channel_'+NUMBER2STRING(i+1)+':inst.channel_'+NUMBER2STRING(i+1)
  creation_string = creation_string + ',channel_'+NUMBER2STRING(nchannel)+':channel}'
  ok = EXECUTE(creation_string)
  inst = newinst

END

;;==========================================================================================================
;;   SUBROUTINE
;;==========================================================================================================
FUNCTION HFI_TABLEBAND_INSTRUMENT, detnum

  COMMON obs, obser

  nfreq = N_ELEMENTS(detnum)

  chanbeams = [10.,7.1,5.,5.,5.,5.]
  chanunits = ['mK_CMB','mK_CMB','mK_CMB','MJy/sr','MJy/sr','MJy/sr']
  freq = [100,143,217,353,545,857]
  fwhm = chanbeams(WHERE(freq EQ firstchar(detnum[0],3))) & fwhm = fwhm[0]
  obs_units = chanunits(WHERE(freq EQ firstchar(detnum[0],3))) & obs_units = obs_units[0]
  IF DEFINED(obser) THEN BEGIN
     IF obser.global.obs_pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
     IF obser.global.obs_res EQ 'instr' THEN BEGIN
        dobeam = 1
     ;;; beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=10.,sizeunits='arcmin') $
     ENDIF ELSE BEGIN
        beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin')
        dobeam = 0
     ENDELSE
  ENDIF ELSE BEGIN
     pixtype = 'HEALPIX'
     dobeam = 1
     ;;; beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=10.,sizeunits='arcmin')
  ENDELSE
  inst = N_CHANNEL_INSTRUMENT(0) 
  ;; Pixelisation
  ;;--------------
  pix = GET_PIX_STRUCT(pixtype)
  IF pixtype EQ 'HEALPIX' THEN BEGIN
     pix.nside = 2048L
     pix.npix = 2048L^2*12L
     pix.ordering = 'RING'
     pix.object = 'FULLSKY'
     pix.firstpix = 0L
     pix.lastpix = pix.npix-1L
     pix.indxschm = 'IMPLICIT'
     pix.coordsys = 'G'
  ENDIF
  ;; Where to find bandpass data
  ;;-----------------------------
  ;; dirbands = GETPSMDIR('psmdata')+'instrument/planck/bands/bandpass_spencer_v2.02/SpecTransFiles_v202/'
  ;; Add all channels
  ;;------------------
  FOR i=0, nfreq-1 DO BEGIN

     ;; datafile = 'HFI__bc*_'+detnum[i]+'_Apod5_v202.dat'
     ;; ok = TEST_PSM_DATA(dirbands+datafile, /get)

     ;; IF ok THEN res = FILE_SEARCH(dirbands,'HFI__bc*_'+detnum[i]+'_Apod5_v202.dat') ELSE MESSAGE, 'Bandpass data file not available'

     ;; READCOL, res[0], nu, value, format='D,D'
     band = GET_BAND_STRUCT('INSTR',instr='HFI_TABLEBANDS',version='spencer_202',channel=detnum[i])
     FILL_BAND_DATA, band

     IF dobeam THEN BEGIN
        beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=fwhm, sizeunits='arcmin')
        ;print, 'det = ',detnum[i], ' fwhm = ',fwhm
     ENDIF
     channel = ONE_CHANNEL(band=band, beam=beam, pix=pix, obs_units=obs_units, /compute_conversion)
     channel.name = 'detector_'+detnum[i]
     IF ( (LASTCHAR(detnum[i]) EQ 'a') OR (LASTCHAR(detnum[i]) EQ 'b')) THEN channel.stokes = 'TQU' ELSE channel.stokes = 'T'
     ADD_A_CHANNEL, inst, channel
  ENDFOR
  RETURN, inst
END

;;==========================================================================================================
;;   SUBROUTINE
;;==========================================================================================================
FUNCTION HFI_RIMO_INSTRUMENT, detnum, version=version, allbands=allbands

  COMMON obs, obser
  
  IF NOT DEFINED(version) THEN instr = LATEST_HFI_RIMO(version=version)
  
  rimofile = GETPSMDIR('psmdata')+'instrument/planck/RIMO/HFI/HFI_RIMO_'+version+'.fits'
  ok = TEST_PSM_DATA(rimofile,/get)

  ;; BEING DEVELOPED TO INCLUDE ALL FREQUENCY BANDS IF NEEDED
  ;;-----------------------------------------------------------
  ;; IF NOT DEFINED(detnum) THEN detnum=['']
  ;; IF KEYWORD_SET(allbands) NE 0 THEN detnum = [detnum,'F100','F143','F217','F353','F545','F857']

  ;; uniqlist = PSM_UNIQ(detnum)
  ;; detnum = detnum[uniqlist]
  ;; wh = WHERE(STRLEN(detnum) NE 0)
  ;; detnum = detnum[wh]

  nfreq = N_ELEMENTS(detnum)

  chanunits = ['K_CMB','K_CMB','K_CMB','K_CMB','K_CMB','K_CMB']
  freq = [100,143,217,353,545,857]

  ;; units
  obs_units = STRARR(N_ELEMENTS(detnum))
  IF DEFINED(obser) THEN BEGIN
     IF obser.hfi_rimo.nfreq NE 0 THEN chanunits = obser.hfi_rimo.channel_units
  ENDIF
  FOR i=0, N_ELEMENTS(detnum)-1 DO BEGIN
     whichfreq = FIRSTCHAR(detnum[i],3)
     wh = WHERE(NUMBER2STRING(freq) EQ STRING(whichfreq), nwh)
     IF nwh NE 1 THEN BEGIN
        wh = WHERE('F'+NUMBER2STRING(freq) EQ detnum[i], nwh)
        IF nwh NE 1 THEN MESSAGE, 'Invalid HFI RIMO detector: '+detnum[i]
        obs_units[i] = chanunits[wh[0]]
     ENDIF ELSE obs_units[i] = chanunits[wh[0]]
  ENDFOR

  ;; pixellisation
  IF DEFINED(obser) THEN BEGIN
     IF obser.hfi_rimo.nfreq NE 0 THEN BEGIN
        IF obser.hfi_rimo.pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
     ENDIF ELSE BEGIN
        pixtype='HEALPIX'
     ENDELSE
  ENDIF ELSE BEGIN
     pixtype='HEALPIX'
  ENDELSE
  CASE pixtype OF
     'HEALPIX': BEGIN
        pix = GET_PIX_STRUCT(pixtype)
        IF pixtype EQ 'HEALPIX' THEN BEGIN
           pix.nside = 2048L
           pix.npix = 2048L^2*12L
           pix.ordering = 'RING'
           pix.object = 'FULLSKY'
           pix.firstpix = 0L
           pix.lastpix = pix.npix-1L
           pix.indxschm = 'IMPLICIT'
           pix.coordsys = 'G'
        ENDIF
     END
     'SKYPIX': BEGIN
        pix = SKY_PIX()
     END
  ENDCASE

  ;; beams
  IF DEFINED(obser) THEN BEGIN
     IF obser.global.obs_res EQ 'instr' THEN BEGIN
        dobeam = 1
     ENDIF ELSE BEGIN
        beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin')
        dobeam = 0
     ENDELSE
  ENDIF ELSE BEGIN
     dobeam = 1
  ENDELSE

  inst = N_CHANNEL_INSTRUMENT(0,version=version) 
  FOR i=0, nfreq-1 DO BEGIN

     ;; define and get bands
     CASE version OF
        ;; '20110412': BEGIN
        ;;    band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='20110412',channel=detnum[i])
        ;; END
        ;; '20101213': BEGIN
        ;;    band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='20101213',channel=detnum[i])
        ;; END
        ;; '20110720': BEGIN
        ;;    band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='20110720',channel=detnum[i])
        ;; END
        ;; 'FFP4_bandpass-only': BEGIN
        ;;    band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='FFP4_bandpass-only',channel=detnum[i])
        ;; END
        ;; '20120124': BEGIN
        ;;    band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='20120124',channel=detnum[i])
        ;; END
        'DX7-v1': BEGIN
           band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='DX7-v1',channel=detnum[i])
        END
        'DX8-v1': BEGIN
           band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='DX8-v1',channel=detnum[i])
        END
        'DX9-v1': BEGIN
           band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='DX9-v1',channel=detnum[i])
        END
        ;; 'FFP4-v1': BEGIN
        ;;    band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='FFP4-v1',channel=detnum[i])
        ;; END
        'FFP4-v2': BEGIN
           band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='FFP4-v2',channel=detnum[i])
        END
        'FFP6-v1': BEGIN
           band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='FFP6-v1',channel=detnum[i])
        END
        ;; 'FFP4_60s_weights_fcbp': BEGIN
        ;;    band = GET_BAND_STRUCT('INSTR',instr='HFI_RIMO',version='FFP4_60s_weights_fcbp',channel=detnum[i])
        ;; END
        ELSE: MESSAGE, 'HFI Instrument version not available'
     ENDCASE
     FILL_BAND_DATA, band

     ;; define and get beams
     CASE version OF
        'FFP4-v1': dobeam = 0
        'FFP4-v2': dobeam = 0
        ;; 'FFP4_bandpass_only': dobeam = 0
        ;; 'FFP4_60s_weights_fcbp': dobeam=0
        'DX7-v1':
        'DX8-v1':
        'DX9-v1':
        'FFP6-v1': 
        ELSE: MESSAGE, 'HFI Instrument version not available'
     ENDCASE
     IF dobeam THEN BEGIN
        rimofile = GETPSMDIR('psmdata')+'instrument/planck/RIMO/HFI/HFI_RIMO_'+band.version+'.fits'
        IF FIRSTCHAR(detnum[i]) EQ 'F' THEN BEGIN
           par = MRDFITS(rimofile,'FREQUENCY MAP PARAMETERS',/silent) 
           wh = WHERE(STRTRIM(par.frequency,2) EQ LASTCHAR(detnum[i],3), nwh)
        ENDIF ELSE BEGIN
           par = MRDFITS(rimofile,'CHANNEL PARAMETERS',/silent) 
           wh = WHERE(STRTRIM(par.detector,2) EQ PSM_STR_REPLACE(detnum[i],'_','-'), nwh)
        ENDELSE
        IF nwh EQ 1 THEN BEGIN
           beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=(par.fwhm)[wh[0]], sizeunits='arcmin') 
        ENDIF ELSE BEGIN
           CASE FIRSTCHAR(detnum[i]) OF
              'F':BEGIN
                 
                 GOTO, effective
                 ;; average rimo beams
                 freq_channel = LASTCHAR(detnum[i],3)
                 detlist = ALL_HFI_DETECTORS(freq=[freq_channel])
                 detlist = PSM_STR_REPLACE(detlist, '_', '-')
                 fwhms = FLTARR(N_ELEMENTS(detlist))
                 FOR k=0, N_ELEMENTS(detlist)-1 DO BEGIN
                    whk = WHERE(par.detector EQ detlist[k])
                    fwhms[k] = (par.fwhm)[whk]
                 ENDFOR
                 beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=AVG(fwhms), sizeunits='arcmin')
                 
                 effective:
                 ;; effective beams 
                 freq_channel = LASTCHAR(detnum[i],3)
                 mapinfo = MRDFITS(rimofile,2)
                 
                 ;; from early paper
                 ;; effbeams = [9.88,7.18,4.87,4.65,4.72,4.39]
                 ;; freqsnames = ['100','143','217','353','545','857']
                 
                 ;; from RIMO
                 effbeams = mapinfo.fwhm
                 freqsnames = mapinfo.frequency
                 
                 wh = WHERE(freq_channel EQ freqsnames, nwh)
                 beamsize = effbeams(wh)
                 beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=beamsize[0], sizeunits='arcmin')
                 
              END
           ENDCASE
        ENDELSE
     ENDIF ELSE BEGIN
        beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=SKY_RESOLUTION(), sizeunits='arcmin')
     ENDELSE

     ;; define channels
     channel = ONE_CHANNEL(band=band, beam=beam, pix=pix, obs_units=obs_units[i], /compute_conversion, version=version)
     channel.inst = 'HFI_RIMO'
     channel.name = 'detector_'+detnum[i]
     IF ( (LASTCHAR(detnum[i]) EQ 'a') OR (LASTCHAR(detnum[i]) EQ 'b')) THEN channel.stokes = 'TQU' ELSE channel.stokes = 'T'
     ADD_A_CHANNEL, inst, channel, version=version
  ENDFOR
  RETURN, inst
END

;;==========================================================================================================
;;   SUBROUTINE
;;==========================================================================================================
FUNCTION LFI_RIMO_INSTRUMENT, detnum, version=version

  IF NOT DEFINED(version) THEN instr = LATEST_LFI_RIMO(version=version)
  COMMON obs, obser

  nfreq = N_ELEMENTS(detnum)

  chanunits = ['K_CMB','K_CMB','K_CMB']
  freq = ['030','044','070']

  ;; units
  obs_units = STRARR(N_ELEMENTS(detnum))
  IF DEFINED(obser) THEN BEGIN
     IF obser.lfi_rimo.nfreq NE 0 THEN chanunits = obser.lfi_rimo.channel_units
  ENDIF

  FOR i=0, N_ELEMENTS(detnum)-1 DO BEGIN
     whichfreq = FIRSTCHAR(detnum[i],3)
     wh = WHERE(freq EQ whichfreq, nwh)
     IF nwh NE 1 THEN BEGIN
        wh = WHERE('F'+NUMBER2STRING(freq) EQ detnum[i], nwh)
        IF nwh NE 1 THEN MESSAGE, 'Invalid LFI RIMO detector: '+detnum[i]
        obs_units[i] = chanunits[wh[0]]
     ENDIF ELSE obs_units[i] = chanunits[wh]
  ENDFOR

  ;; pixellisation
  IF DEFINED(obser) THEN BEGIN
     IF obser.lfi_rimo.nfreq NE 0 THEN BEGIN
        IF obser.lfi_rimo.pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
     ENDIF ELSE BEGIN
        pixtype='HEALPIX'
     ENDELSE
  ENDIF ELSE BEGIN
     pixtype='HEALPIX'
  ENDELSE
  CASE pixtype OF
     'HEALPIX': BEGIN
        pix = GET_PIX_STRUCT(pixtype)
        IF pixtype EQ 'HEALPIX' THEN BEGIN
           pix.nside = 1024L
           pix.npix = 1024L^2*12L
           pix.ordering = 'RING'
           pix.object = 'FULLSKY'
           pix.firstpix = 0L
           pix.lastpix = pix.npix-1L
           pix.indxschm = 'IMPLICIT'
           pix.coordsys = 'G'
        ENDIF
     END
     'SKYPIX': BEGIN
        pix = SKY_PIX()
     END
  ENDCASE

  ;; beams
  IF DEFINED(obser) THEN BEGIN
     IF obser.global.obs_res EQ 'instr' THEN BEGIN
        dobeam = 1
     ENDIF ELSE BEGIN
        beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin')
        dobeam = 0
     ENDELSE
  ENDIF ELSE BEGIN
     dobeam = 1
  ENDELSE

  inst = N_CHANNEL_INSTRUMENT(0) 
  FOR i=0, nfreq-1 DO BEGIN

     ;; define and get bands

     CASE version OF
        ;; 'FFP4_bandpass-only': BEGIN
        ;;    file = GETPSMDIR('psmdata')+'instrument/planck/RIMO/LFI/LFI_RIMO_FFP4_bandpass-only.fits'
        ;;    ok = TEST_PSM_DATA(file,/get)
        ;;    h = HEADFITS(file,exten='BANDPASS_'+detnum[i])
        ;;    nu_min = SXPAR(h,'B_LOW') * 1e9
        ;;    nu_max = SXPAR(h,'B_HIGH') * 1e9
        ;;    band = GET_BAND_STRUCT('TOPHAT',nu_min=nu_min,nu_max=nu_max)
        ;; END
        'FFP4-v1': BEGIN
           file = GETPSMDIR('psmdata')+'instrument/planck/RIMO/LFI/LFI_RIMO_FFP4-v1.fits'
           ok = TEST_PSM_DATA(file,/get)
           h = HEADFITS(file,exten='BANDPASS_'+detnum[i])
           nu_min = SXPAR(h,'B_LOW') * 1e9
           nu_max = SXPAR(h,'B_HIGH') * 1e9
           band = GET_BAND_STRUCT('TOPHAT',nu_min=nu_min,nu_max=nu_max)
        END
        ELSE: BEGIN
           file = GETPSMDIR('psmdata')+'instrument/planck/RIMO/LFI/LFI_RIMO_'+version+'.fits'
           ok = TEST_PSM_DATA(file,/get)
           band = GET_BAND_STRUCT('INSTR',instr='LFI_RIMO',version=version,channel=detnum[i])
           FILL_BAND_DATA, band
        ENDELSE
     ENDCASE

     ;; define and get beams
     CASE version OF
        ;; 'FFP4_bandpass-only': dobeam = 0
        'FFP4-v1': dobeam = 0
        'FFP4-v2': dobeam = 0
        'FFP6-v1':
        ;; 'FFP4_60s_weights_fcbp': dobeam = 0
        'DX6-v1': 
        'DX7-v1': 
        'DX8-v1': 
        'DX9-v1': 
        ELSE: MESSAGE, 'LFI Instrument version not available'
     ENDCASE
     IF dobeam THEN BEGIN
        rimofile = GETPSMDIR('psmdata')+'instrument/planck/RIMO/LFI/LFI_RIMO_'+version+'.fits'
        IF FIRSTCHAR(detnum[i]) EQ 'F' THEN BEGIN
           par = MRDFITS(rimofile,'FREQUENCY_MAP_PARAMETERS',/silent) 
           wh = WHERE(STRTRIM(par.frequency,2) EQ LASTCHAR(detnum[i],3), nwh)
        ENDIF ELSE BEGIN
           par = MRDFITS(rimofile,'CHANNEL_PARAMETERS',/silent) 
           wh = WHERE(STRTRIM(par.detector,2) EQ 'LFI'+LASTCHAR(detnum[i],3), nwh)
        ENDELSE
        IF nwh EQ 1 THEN BEGIN
           beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=(par.fwhm)[wh[0]], sizeunits='arcmin') 
        ENDIF ELSE BEGIN
           CASE FIRSTCHAR(detnum[i]) OF
              'F':BEGIN
                 
                 ;; effective beams 
                 freq_channel = LASTCHAR(detnum[i],3)
                 mapinfo = MRDFITS(rimofile, 2)
                 
                 ;; from early paper
                 ;; effbeams = [32.65,27.92,13.01]
                 ;; freqsnames = ['030','044','070']
                 
                 ;; from RIMO
                 effbeams = mapinfo.fwhm
                 freqsnames = mapinfo.frequency
                 
                 wh = WHERE(freq_channel EQ freqsnames, nwh)
                 beamsize = effbeams(wh)
                 beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=beamsize[0], sizeunits='arcmin')
                 
              END
           ENDCASE
        ENDELSE
     ENDIF ELSE BEGIN
        beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=SKY_RESOLUTION(), sizeunits='arcmin')
     ENDELSE

     ;; define channels
     channel = ONE_CHANNEL(band=band, beam=beam, pix=pix, obs_units=obs_units[i], /compute_conversion, version=version)
     channel.inst = 'LFI_RIMO'
     channel.name = 'detector_'+detnum[i]
     channel.stokes = 'TQU'
     ADD_A_CHANNEL, inst, channel, version=version
  ENDFOR
  
  RETURN, inst
END

;;==========================================================================================================
;;   SUBROUTINE
;;==========================================================================================================
FUNCTION WMAP_INSTRUMENT, detnum, version=version

  IF NOT DEFINED(version) THEN version='7yr'
  COMMON obs, obser

  nfreq = N_ELEMENTS(detnum)

  chanunits = ['mK_CMB','mK_CMB','mK_CMB','mK_CMB','mK_CMB']
  freq = ['K','Ka','Q','V','W']

  ;; units
  obs_units = STRARR(N_ELEMENTS(detnum))
  nuc = FLTARR(N_ELEMENTS(detnum))
  IF DEFINED(obser) THEN BEGIN
     IF obser.wmap.nfreq NE 0 THEN chanunits = obser.wmap.channel_units
  ENDIF

  FOR i=0, N_ELEMENTS(detnum)-1 DO BEGIN
     wh = WHERE(detnum EQ 'K', nwh) 
     IF (nwh NE 0) THEN BEGIN
        obs_units[wh] = chanunits[0]
        nuc[wh] = 23.
     ENDIF
     wh = WHERE(detnum EQ 'Ka', nwh) 
     IF (nwh NE 0) THEN BEGIN
        obs_units[wh] = chanunits[1]
        nuc[wh] = 33.
     ENDIF
     wh = WHERE(FIRSTCHAR(detnum) EQ 'Q', nwh) 
     IF (nwh NE 0) THEN BEGIN
        obs_units[wh] = chanunits[2]
        nuc[wh] = 41.
     END
     wh = WHERE(FIRSTCHAR(detnum) EQ 'V', nwh) 
     IF (nwh NE 0) THEN BEGIN
        obs_units[wh] = chanunits[3]
        nuc[wh] = 61.
     END
     wh = WHERE(FIRSTCHAR(detnum) EQ 'W', nwh) 
     IF (nwh NE 0) THEN BEGIN
        obs_units[wh] = chanunits[4]
        nuc[wh] = 94.
     END
  ENDFOR

  ;; pixellisation
  IF DEFINED(obser) THEN BEGIN
     IF obser.wmap.nfreq NE 0 THEN BEGIN
        IF obser.wmap.pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
     ENDIF ELSE BEGIN
        pixtype='HEALPIX'
     ENDELSE
  ENDIF ELSE BEGIN
     pixtype='HEALPIX'
  ENDELSE

  CASE pixtype OF
     'HEALPIX': BEGIN
        pix = GET_PIX_STRUCT(pixtype)
        IF pixtype EQ 'HEALPIX' THEN BEGIN
           pix.nside = 512L
           pix.npix = 512L^2*12L
           pix.ordering = 'NESTED'
           pix.object = 'FULLSKY'
           pix.firstpix = 0L
           pix.lastpix = pix.npix-1L
           pix.indxschm = 'IMPLICIT'
           pix.coordsys = 'G'
        ENDIF
     END
     'SKYPIX': BEGIN
        pix = SKY_PIX()
     END
  ENDCASE

  ;; beams
  IF DEFINED(obser) THEN BEGIN
     IF obser.global.obs_res EQ 'instr' THEN BEGIN
        dobeam = 1
     ENDIF ELSE BEGIN
        beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin')
        dobeam = 0
     ENDELSE
  ENDIF ELSE BEGIN
     dobeam = 1
  ENDELSE

  inst = N_CHANNEL_INSTRUMENT(0, version=version) 
  FOR i=0, nfreq-1 DO BEGIN

     ;; define bands
     band = GET_BAND_STRUCT('DIRAC',nu_c=nuc[i]*1e9)

     ;; define beams
     IF dobeam THEN BEGIN
        beam = GET_BEAM_STRUCT('INSTR',instr='WMAP',version=version,channel=detnum[i])
     ENDIF ELSE BEGIN
        beam = GET_BEAM_STRUCT('GAUSSIAN', beamsize=SKY_RESOLUTION(), sizeunits='arcmin')
     ENDELSE

     ;; define channels
     channel = ONE_CHANNEL(band=band, beam=beam, pix=pix, obs_units=obs_units[i], /compute_conversion, version=version)
     channel.inst = 'WMAP'
     channel.name = detnum[i]
     channel.stokes = 'TQU'
     ADD_A_CHANNEL, inst, channel, version=version
  ENDFOR
  RETURN, inst
END

;;==========================================================================================================
;;   MAIN PROGRAM
;;==========================================================================================================
FUNCTION LOAD_INSTRUMENT, instrument_name, channel_list=channel_list, version=version, freqs=freqs, help=help

  ON_ERROR, 2

  IF (KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 1) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'LOAD_INSTRUMENT: function which returns a structure describing an instrument'
     PSM_INFO_MESSAGE, 'SYNTAX: result = LOAD_INSTRUMENT(instrument_name, channel_list=, version=, freqs=)'
        PSM_INFO_MESSAGE, "Choose instrument_name from the following list:"
        PSM_INFO_MESSAGE, "    PSM_IDEAL"
        PSM_INFO_MESSAGE, "    HFI_IDEAL"
        PSM_INFO_MESSAGE, "    LFI_IDEAL"
        PSM_INFO_MESSAGE, "    HFI_BLUEBOOK"
        PSM_INFO_MESSAGE, "    LFI_BLUEBOOK"
        PSM_INFO_MESSAGE, "    HFI_RIMO"
        PSM_INFO_MESSAGE, "    LFI_RIMO"
        ;PSM_INFO_MESSAGE, "    HFIxxx_TABLEBANDS, where xxx can be 100, 143, 217, 353, 545, 857"
        ;PSM_INFO_MESSAGE, "    HFI_TABLEBANDS"
        ;PSM_INFO_MESSAGE, "    WMAP5yr_IDEALBANDS"
        ;PSM_INFO_MESSAGE, "    WMAP7yr_IDEALBANDS"
        ;PSM_INFO_MESSAGE, "    WMAP7yr_IDEALBANDS_DA"
        PSM_INFO_MESSAGE, "    WMAP"
        PSM_INFO_MESSAGE, "    IRAS_IDEAL"
        PSM_INFO_MESSAGE, "    IRAS_RIMO"
     PSM_INFO_MESSAGE, 'channel_list is the list of channels to be included' 
     PSM_INFO_MESSAGE, '(default: all individual detectors, or all frequency bands if /freqs is set)'
     PSM_INFO_MESSAGE, "version sets the version for HFI_RIMO, LFI_RIMO, WMAP"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  COMMON psm, param
  COMMON obs, obser
  COMMON sky, model

  CASE instrument_name OF 
     
     ;;----------------------------------------------------------------------------------------------------
     ;; PSM instrument
     ;;----------------------------------------------------------------------------------------------------

     ;; Simple, idealized, PSM instrument
     ;;-----------------------------------
     'PSM_IDEAL': BEGIN
        defok = DEFINED(obser)
        IF defok THEN BEGIN
           nfreq = obser.psm_ideal.nfreq
           IF nfreq NE 0 THEN BEGIN       
              inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='GAUSSIAN',pixtype='SKYPIX')
              inst.name = instrument_name
              FOR i=0, nfreq-1 DO BEGIN
                 ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
                 chan.inst = instrument_name
                 chan.name = 'band' + NUMBER2STRING(i+1)
                 chan.band.nu_c = obser.psm_ideal.frequencies[i]*1E9
                 chan.band.shape = "DIRAC"
                 IF obser.global.obs_res EQ 'sky' THEN chan.beam.size = SKY_RESOLUTION() ELSE chan.beam.size = obser.psm_ideal.resolution[i]
                 chan.beam.sizeunits = "arcmin"
                 chan.stokes = obser.psm_ideal.stokes[i]
                 chan.obs_units = obser.psm_ideal.channel_units[i]
                 chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
                 chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
                 chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
                 chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
                 ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
              ENDFOR
              
           ENDIF 
        ENDIF ELSE BEGIN
           inst = {name: instrument_name, $
                   nchannels: 0}
        ENDELSE
     END
     
     ;;----------------------------------------------------------------------------------------------------
     ;; Planck HFI instrument (various versions)
     ;;----------------------------------------------------------------------------------------------------

     ;; Ideal HFI instrument, monofrequency bands, HFI only
     ;;----------------------------------------------------
     'HFI_IDEAL': BEGIN
        nfreq = 6
        IF DEFINED(obser) THEN BEGIN
           IF obser.hfi_ideal.pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
           IF obser.global.obs_res EQ 'instr' THEN beam = [10.,7.1,5.,5.,5.,5.] $
           ELSE beam = REPLICATE(SKY_RESOLUTION(),nfreq)
           IF obser.hfi_ideal.nfreq EQ 6 THEN units = obser.hfi_ideal.units $
              ELSE units = ['K_CMB','K_CMB','K_CMB','K_CMB','K_CMB','K_CMB']
        ENDIF ELSE BEGIN
           units = ['K_CMB','K_CMB','K_CMB','K_CMB','K_CMB','K_CMB']           
           pixtype= 'HEALPIX'
           IF DEFINED(model) THEN beam = REPLICATE(SKY_RESOLUTION(),nfreq) ELSE BEGIN
              pixtype='HEALPIX'
              beam = [10.,7.1,5.,5.,5.,5.]
           ENDELSE
        ENDELSE
        inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='GAUSSIAN',pixtype=pixtype)
        inst.name = instrument_name
        bandname = ['100GHz','143GHz','217GHz','353GHz','545GHz','857GHz']
        nuc = ['100.','143.','217.','353.','545.','857.']*1E9
        stokes = ['TQU','TQU','TQU','TQU','T','T']
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.inst = instrument_name
           chan.name = bandname[i]
           chan.band.nu_c = nuc[i]
           chan.beam.size = beam[i]
           chan.beam.sizeunits = 'arcmin'
           chan.stokes = stokes[i]
           chan.obs_units = units[i]
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 2048L
              chan.pix.npix = 2048L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR
     END

     ;; Bluebook HFI instrument, tophat bands, HFI only
     ;;-------------------------------------------------
     'HFI_BLUEBOOK': BEGIN
        nfreq = 6
        IF DEFINED(obser) THEN BEGIN
           IF obser.hfi_bluebook.pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
           IF obser.global.obs_res EQ 'instr' THEN beam = [10.,7.1,5.,5.,5.,5.] $
              ELSE beam = REPLICATE(SKY_RESOLUTION(),nfreq)
           IF obser.hfi_bluebook.nfreq EQ 6 THEN units = obser.hfi_bluebook.units $
              ELSE units = ['K_CMB','K_CMB','K_CMB','K_CMB','K_CMB','K_CMB']
        ENDIF ELSE BEGIN
           units = ['K_CMB','K_CMB','K_CMB','K_CMB','K_CMB','K_CMB']
           pixtype= 'HEALPIX'
           IF DEFINED(model) THEN beam = REPLICATE(SKY_RESOLUTION(),nfreq) ELSE BEGIN
              pixtype='HEALPIX'
              beam = [10.,7.1,5.,5.,5.,5.]
           ENDELSE
        ENDELSE
        inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='TOPHAT',beamtype='GAUSSIAN',pixtype=pixtype)
        inst.name = instrument_name
        bandname = ['100GHz','143GHz','217GHz','353GHz','545GHz','857GHz']
        nuc = ['100.','143.','217.','353.','545.','857.']*1E9
        stokes = ['TQU','TQU','TQU','TQU','T','T']
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.inst = instrument_name
           chan.name = bandname[i]
           chan.band.nu_c = nuc[i]
           chan.band.nu_min = nuc[i] * 0.835
           chan.band.nu_max = nuc[i] * 1.165
           chan.band.integ_dnu = chan.band.nu_max - chan.band.nu_min
           chan.beam.size = beam[i]
           chan.beam.sizeunits = 'arcmin'
           chan.stokes = stokes[i]
           chan.obs_units = units[i]
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 2048L
              chan.pix.npix = 2048L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR
     END

     ;; HFI 100GHz, ideal beams
     ;;-------------------------
     ;; 'HFI100_TABLEBANDS': BEGIN
     ;;    nfreq = 8
     ;;    detnum = ['100_1a','100_1b','100_2a','100_2b','100_3a','100_3b','100_4a','100_4b']
     ;;    inst = HFI_TABLEBAND_INSTRUMENT(detnum)
     ;;    inst.name = instrument_name
     ;;    FOR i=1, nfreq DO ok = EXECUTE('inst.channel_'+NUMBER2STRING(i)+'.inst = instrument_name')
     ;; END

     ;; HFI 143GHz, ideal beams
     ;;-------------------------
     ;; 'HFI143_TABLEBANDS': BEGIN
     ;;    nfreq = 12
     ;;    detnum = ['143_1a','143_1b','143_2a','143_2b','143_3a','143_3b','143_4a','143_4b','143_5','143_6','143_7','143_8']
     ;;    inst = HFI_TABLEBAND_INSTRUMENT(detnum)
     ;;    inst.name = instrument_name
     ;;    FOR i=1, nfreq DO ok = EXECUTE('inst.channel_'+NUMBER2STRING(i)+'.inst = instrument_name')
     ;; END

     ;; HFI 217GHz, ideal beams
     ;;-------------------------
     ;; 'HFI217_TABLEBANDS': BEGIN
     ;;    nfreq = 12
     ;;    detnum = ['217_1','217_2','217_3','217_4','217_5a','217_5b','217_6a','217_6b','217_7a','217_7b','217_8a','217_8b']
     ;;    inst = HFI_TABLEBAND_INSTRUMENT(detnum)
     ;;    inst.name = instrument_name
     ;;    FOR i=1, nfreq DO ok = EXECUTE('inst.channel_'+NUMBER2STRING(i)+'.inst = instrument_name')
     ;; END

     ;; HFI 353GHz, ideal beams
     ;;-------------------------
     ;; 'HFI353_TABLEBANDS': BEGIN
     ;;    nfreq = 12
     ;;    detnum = ['353_1','353_2','353_3a','353_3b','353_4a','353_4b','353_5a','353_5b','353_6a','353_6b','353_7','353_8']
     ;;    inst = HFI_TABLEBAND_INSTRUMENT(detnum)
     ;;    inst.name = instrument_name
     ;;    FOR i=1, nfreq DO ok = EXECUTE('inst.channel_'+NUMBER2STRING(i)+'.inst = instrument_name')
     ;; END

     ;; HFI 545GHz, ideal beams
     ;;-------------------------
     ;; 'HFI545_TABLEBANDS': BEGIN
     ;;    nfreq = 4
     ;;    detnum = ['545_1','545_2','545_3','545_4']
     ;;    inst = HFI_TABLEBAND_INSTRUMENT(detnum)
     ;;    inst.name = instrument_name
     ;;    FOR i=1, nfreq DO ok = EXECUTE('inst.channel_'+NUMBER2STRING(i)+'.inst = instrument_name')
     ;; END

     ;; HFI 857GHz, ideal beams
     ;;-------------------------
     ;; 'HFI857_TABLEBANDS': BEGIN
     ;;    nfreq = 4
     ;;    detnum = ['857_1','857_2','857_3','857_4']
     ;;    inst = HFI_TABLEBAND_INSTRUMENT(detnum)
     ;;    inst.name = instrument_name
     ;;    FOR i=1, nfreq DO ok = EXECUTE('inst.channel_'+NUMBER2STRING(i)+'.inst = instrument_name')
     ;; END

     ;; 'HFI_TABLEBANDS': BEGIN
     ;;    list = ['HFI100_TABLEBANDS','HFI143_TABLEBANDS','HFI217_TABLEBANDS','HFI353_TABLEBANDS','HFI545_TABLEBANDS','HFI857_TABLEBANDS']
     ;;    inst = LOAD_INSTRUMENT(list)
     ;;    inst.name = 'HFI_TABLEBANDS'
     ;; END

     'HFI_RIMO': BEGIN
        IF NOT KEYWORD_SET(channel_list) THEN BEGIN
           IF NOT KEYWORD_SET(freqs) THEN channel_list = ALL_HFI_DETECTORS() ELSE channel_list = ['F100','F143','F217','F353','F545','F857']
        ENDIF
        inst = HFI_RIMO_INSTRUMENT(channel_list, version=version)
        inst.name = 'HFI_RIMO'
     END

     ;;----------------------------------------------------------------------------------------------------
     ;; Planck LFI instrument (various versions)
     ;;----------------------------------------------------------------------------------------------------
     
     ;; Ideal LFI instrument, monofrequency bands, LFI only
     ;;-----------------------------------------------------
     'LFI_IDEAL': BEGIN
        nfreq = 3
        IF DEFINED(obser) THEN BEGIN
           IF obser.lfi_ideal.pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
           IF obser.global.obs_res EQ 'instr' THEN beam = [33.,24.,14.] $
           ELSE beam = REPLICATE(SKY_RESOLUTION(),nfreq)
           IF obser.lfi_ideal.nfreq EQ 3 THEN units = obser.lfi_ideal.units $
              ELSE units = ['K_CMB','K_CMB','K_CMB']
        ENDIF ELSE BEGIN
           units = ['K_CMB','K_CMB','K_CMB']
           pixtype= 'HEALPIX'
           IF DEFINED(model) THEN beam = REPLICATE(SKY_RESOLUTION(),nfreq) ELSE BEGIN
              pixtype='HEALPIX'
              beam = [33.,24.,14.]
           ENDELSE
        ENDELSE
        inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='GAUSSIAN',pixtype=pixtype)
        inst.name = instrument_name
        bandname = ['30GHz','44GHz','70GHz']
        nuc = ['30.','44.','70.']*1E9        
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.inst = instrument_name
           chan.name = bandname[i]
           chan.band.nu_c = nuc[i]
           chan.beam.size = beam[i]
           chan.beam.sizeunits = 'arcmin'
           chan.stokes = 'TQU'
           chan.obs_units = units[i]
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 1024L
              chan.pix.npix = 1024L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR
     END

     ;; Bluebook LFI instrument, tophat bands, LFI only
     ;;-------------------------------------------------
     'LFI_BLUEBOOK': BEGIN
        nfreq = 3
        IF DEFINED(obser) THEN BEGIN
           IF obser.lfi_bluebook.pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
           IF obser.global.obs_res EQ 'instr' THEN beam = [33.,24.,14.] $
           ELSE beam = REPLICATE(SKY_RESOLUTION(),nfreq)
           IF obser.lfi_bluebook.nfreq EQ 3 THEN units = obser.lfi_bluebook.units $
              ELSE units = ['K_CMB','K_CMB','K_CMB']
        ENDIF ELSE BEGIN
           units = ['K_CMB','K_CMB','K_CMB']
           pixtype= 'HEALPIX'
           IF DEFINED(model) THEN beam = REPLICATE(SKY_RESOLUTION(),nfreq) ELSE BEGIN
              pixtype='HEALPIX'
              beam = [33.,24.,14.]
           ENDELSE
        ENDELSE
        inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='TOPHAT',beamtype='GAUSSIAN',pixtype=pixtype)
        inst.name = instrument_name
        bandname = ['30GHz','44GHz','70GHz']
        nuc = [30.,44.,70.]*1E9
        numin = nuc * 0.9
        numax = nuc * 1.1
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.inst = instrument_name
           chan.name = bandname[i]
           chan.band.nu_c = nuc[i]
           chan.band.nu_min = numin[i]
           chan.band.nu_max = numax[i]
           chan.band.integ_dnu = chan.band.nu_max - chan.band.nu_min
           chan.beam.size = beam[i]
           chan.beam.sizeunits = 'arcmin'
           chan.stokes = 'TQU'
           chan.obs_units = units[i]
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 1024L
              chan.pix.npix = 1024L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR
     END
     
     'LFI_RIMO': BEGIN
        IF NOT KEYWORD_SET(channel_list) THEN BEGIN
           IF NOT KEYWORD_SET(freqs) THEN channel_list = ALL_LFI_DETECTORS() ELSE channel_list = ['F030','F044','F070']
        ENDIF
        inst = LFI_RIMO_INSTRUMENT(channel_list, version=version)
        inst.name = 'LFI_RIMO'
     END


     ;;----------------------------------------------------------------------------------------------------
     ;; WMAP instrument (various versions)
     ;;----------------------------------------------------------------------------------------------------
     
     'WMAP5yr_IDEALBANDS': BEGIN
        nfreq = 5
        IF DEFINED(obser) THEN BEGIN
           IF obser.global.obs_pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
           IF obser.global.obs_res EQ 'instr' THEN BEGIN
              wmapbeam=1 
           ENDIF ELSE BEGIN
              wmapbeam=0
              beam = REPLICATE(SKY_RESOLUTION(),nfreq)
           ENDELSE
        ENDIF ELSE BEGIN
           pixtype= 'HEALPIX'
           wmapbeam=1
        ENDELSE
        IF wmapbeam THEN BEGIN
           inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='INSTR',pixtype=pixtype,instname='WMAP5yr_IDEALBANDS',/polar)        
        ENDIF ELSE BEGIN
           inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='GAUSSIAN',pixtype=pixtype,instname='WMAP5yr_IDEALBANDS',/polar)
        ENDELSE
        bandname = ['K','Ka','Q','V','W']
        nu_c = [23.,33.,41.,61.,94.]*1E9
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.name = bandname[i]
           chan.band.nu_c = nu_c[i]
           chan.stokes = 'TQU'
           chan.obs_units = 'mK_CMB'
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF wmapbeam THEN BEGIN
              chan.beam.instr = 'WMAP5yr'
              chan.beam.channel = bandname[i]
           ENDIF ELSE BEGIN
              chan.beam.size = SKY_RESOLUTION()
              chan.beam.sizeunits = 'arcmin'
           ENDELSE
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 512L
              chan.pix.npix = 512L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR

     END

     'WMAP7yr_IDEALBANDS': BEGIN
        nfreq = 5
        IF DEFINED(obser) THEN BEGIN
           IF obser.global.obs_pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
           IF obser.global.obs_res EQ 'instr' THEN BEGIN
              wmapbeam=1 
           ENDIF ELSE BEGIN
              wmapbeam=0
              beam = REPLICATE(SKY_RESOLUTION(),nfreq)
           ENDELSE
        ENDIF ELSE BEGIN
           pixtype= 'HEALPIX'
           wmapbeam=1
        ENDELSE
        IF wmapbeam THEN BEGIN
           inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='INSTR',pixtype=pixtype,instname='WMAP7yr_IDEALBANDS',/polar)        
        ENDIF ELSE BEGIN
           inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='GAUSSIAN',pixtype=pixtype,instname='WMAP7yr_IDEALBANDS',/polar)
        ENDELSE
        bandname = ['K','Ka','Q','V','W']
        nu_c = [23.,33.,41.,61.,94.]*1E9
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.name = bandname[i]
           chan.band.nu_c = nu_c[i]
           chan.stokes = 'TQU'
           chan.obs_units = 'mK_CMB'
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF wmapbeam THEN BEGIN
              chan.beam.instr = 'WMAP7yr'
              chan.beam.channel = bandname[i]
           ENDIF ELSE BEGIN
              chan.beam.size = SKY_RESOLUTION()
              chan.beam.sizeunits = 'arcmin'
           ENDELSE
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 512L
              chan.pix.npix = 512L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR

     END

     'WMAP7yr_IDEALBANDS_DA': BEGIN
        nfreq = 10
        IF DEFINED(obser) THEN BEGIN
           IF obser.global.obs_pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
           IF obser.global.obs_res EQ 'instr' THEN BEGIN
              wmapbeam=1 
           ENDIF ELSE BEGIN
              wmapbeam=0
              beam = REPLICATE(SKY_RESOLUTION(),nfreq)
           ENDELSE
        ENDIF ELSE BEGIN
           pixtype= 'HEALPIX'
           wmapbeam=1
        ENDELSE
        inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='INSTR',pixtype=pixtype,instname='WMAP7yr_IDEALBANDS_DA',/polar)        
        bandname = ['K','Ka','Q1','Q2','V1','V2','W','W2','W3','W4']
        nu_c = [23.,33.,41.,41.,61.,61.,94.,94.,94.,94.]*1E9
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.name = bandname[i]
           chan.band.nu_c = nu_c[i]
           chan.stokes = 'TQU'
           chan.obs_units = 'mK_CMB'
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF wmapbeam THEN BEGIN
              chan.beam.instr = 'WMAP7yr'
              chan.beam.channel = bandname[i]
           ENDIF ELSE BEGIN
              chan.beam.size = SKY_RESOLUTION()
              chan.beam.sizeunits = 'arcmin'
           ENDELSE
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 512L
              chan.pix.npix = 512L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR

     END

     'WMAP': BEGIN
        IF NOT KEYWORD_SET(channel_list) THEN BEGIN
           IF NOT KEYWORD_SET(freqs) THEN channel_list = ['K','Ka','Q1','Q2','V1','V2','W1','W2','W3','W4'] $
           ELSE channel_list = ['K','Ka','Q','V','W']
        ENDIF
        
        inst = WMAP_INSTRUMENT(channel_list, version=version)
        inst.name = 'WMAP'
     END

     ;;----------------------------------------------------------------------------------------------------
     ;; IRAS instrument (various versions)
     ;;----------------------------------------------------------------------------------------------------
     
     'IRAS_IDEAL': BEGIN
        nfreq = 2
        IF DEFINED(obser) THEN BEGIN
           IF obser.global.obs_res EQ 'instr' THEN beam = [5.,4.7] $ ;arcmin [not diffraction limited]
           ELSE beam = REPLICATE(SKY_RESOLUTION(),nfreq)
        ENDIF ELSE BEGIN
           pixtype= 'HEALPIX'
           beam = [5.,4.7]
        ENDELSE
        inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='DIRAC',beamtype='GAUSSIAN',pixtype=pixtype)
        inst.name = instrument_name
        bandname = ['100micron','60micron']
        DEFSYSV, '!phyconst', exists=exists
        IF NOT exists THEN DEFINE_PHYSICAL_CONSTANTS
        nu_c = !phyconst.c / [100.,60.] * 1e6
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.inst = instrument_name
           chan.name = bandname[i]
           chan.band.nu_c = nu_c[i]
           chan.beam.size = beam[i]
           chan.beam.sizeunits = 'arcmin'
           chan.stokes = 'T'
           chan.obs_units = 'MJy/sr'
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 2048L
              chan.pix.npix = 2048L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR
     END

     'IRAS_RIMO': BEGIN
        nfreq = 2
        IF DEFINED(obser) THEN BEGIN
           IF obser.global.obs_res EQ 'instr' THEN beam = [5.,4.7] $ ;arcmin [not diffraction limited]
           ELSE beam = REPLICATE(SKY_RESOLUTION(),nfreq)
        ENDIF ELSE BEGIN
           pixtype= 'HEALPIX'
           beam = [5.,4.7]
        ENDELSE
        inst = N_CHANNEL_INSTRUMENT(nfreq,bandshape='INSTR',beamtype='GAUSSIAN',pixtype=pixtype)
        inst.name = instrument_name
        bandname = ['100micron','60micron']
        DEFSYSV, '!phyconst', exists=exists
        IF NOT exists THEN DEFINE_PHYSICAL_CONSTANTS
        nu_c = !phyconst.c / [100.,60.] * 1e6
        FOR i=0, nfreq-1 DO BEGIN
           ok = EXECUTE('chan = inst.channel_'+NUMBER2STRING(i+1))
           chan.inst = instrument_name
           chan.name = bandname[i]
           chan.band.instr = instrument_name
           chan.band.version = ''
           chan.band.channel = bandname[i]
           chan.beam.size = beam[i]
           chan.beam.sizeunits = 'arcmin'
           chan.stokes = 'T'
           chan.obs_units = 'MJy/sr'
           chan.conversion.ysz2units = CONVERSION_FACTOR('y_sz',chan.obs_units,chan.band)
           chan.conversion.kcmb2units = CONVERSION_FACTOR('K_CMB',chan.obs_units,chan.band)
           chan.conversion.krj2units = CONVERSION_FACTOR('K_RJ',chan.obs_units,chan.band)
           chan.conversion.megajysr2units = CONVERSION_FACTOR('MJy/sr',chan.obs_units,chan.band)
           IF pixtype EQ 'HEALPIX' THEN BEGIN
              chan.pix.nside = 2048L
              chan.pix.npix = 2048L^2*12L
              chan.pix.ordering = 'RING'
              chan.pix.object = 'FULLSKY'
              chan.pix.firstpix = 0L
              chan.pix.lastpix = chan.pix.npix-1L
              chan.pix.indxschm = 'IMPLICIT'
              chan.pix.coordsys = 'G'
           ENDIF ELSE BEGIN
              STOP
           ENDELSE
           ok = EXECUTE('inst.channel_'+NUMBER2STRING(i+1)+' = chan')
        ENDFOR
     END

     ;; IRAS instrument, actual tabulated bands
     ;;-----------------------------------------
     ;; 'IRAS_TABLEBANDS': BEGIN
     ;;    nfreq = 4
     ;;    fix_size = 0B
     ;;    IF DEFINED(obser) THEN BEGIN
     ;;       IF obser.global.obs_pix EQ 'instr' THEN pixtype='HEALPIX' ELSE pixtype='SKYPIX'
     ;;       IF obser.global.obs_res EQ 'instr' THEN BEGIN
     ;;          beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=0.,sizeunits='arcmin') 
     ;;          beamsize = [25.,25.,60.,100.] ;arcsec
     ;;          fix_size = 1B
     ;;       ENDIF ELSE BEGIN
     ;;          beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin')
     ;;       ENDELSE
     ;;    ENDIF ELSE BEGIN
     ;;       pixtype= 'HEALPIX'
     ;;       beamsize = [25.,25.,60.,100.] ;arcsec
     ;;       beam = GET_BEAM_STRUCT('GAUSSIAN',beamsize=0.,sizeunits='arcmin')
     ;;       fix_size = 1B
     ;;    ENDELSE
     ;;    inst = N_CHANNEL_INSTRUMENT(0) & inst.name = instrument_name
     ;;    ;; Pixelisation
     ;;    ;;--------------
     ;;    pix = GET_PIX_STRUCT(pixtype)
     ;;    IF pixtype EQ 'HEALPIX' THEN BEGIN
     ;;       pix.nside = 2048L
     ;;       pix.npix = 2048L^2*12L
     ;;       pix.ordering = 'RING'
     ;;       pix.object = 'FULLSKY'
     ;;       pix.firstpix = 0L
     ;;       pix.lastpix = pix.npix-1L
     ;;       pix.indxschm = 'IMPLICIT'
     ;;       pix.coordsys = 'G'
     ;;    ENDIF
     ;;    ;; Where to find bandpass data
     ;;    ;;-----------------------------
     ;;    dirbands = GETPSMDIR('psmsoft')+'datafiles/IRAS/bands/'
     ;;    ;; Add all channels
     ;;    ;;------------------
     ;;    detnum = ['12microns','25microns','60microns','100microns']
     ;;    FOR i=0, nfreq-1 DO BEGIN
     ;;       res = FILE_SEARCH(dirbands,'bandpass_'+detnum[i]+'.txt')
     ;;       READCOL, res[0], lambda, value, format='D,D', comment='#'
     ;;       DEFSYSV, '!phyconst', exists=exists
     ;;       IF NOT exists THEN DEFINE_PHYSICAL_CONSTANTS
     ;;       nu = !phyconst.c/lambda *1e6
     ;;       nu = nu(SORT(nu))
     ;;       value = value(SORT(nu))
     ;;       band = GET_BAND_STRUCT('TABLE',nutab=nu, value=value)
     ;;       IF fix_size THEN beam.size = beamsize[i]/60.
     ;;       channel = ONE_CHANNEL(band=band, beam=beam, pix=pix, obs_units='MJy/sr', /compute_conversion)
     ;;       channel.inst = instrument_name
     ;;       channel.name = 'detector_'+detnum[i]
     ;;       channel.stokes = 'T'
     ;;       ADD_A_CHANNEL, inst, channel
     ;;    ENDFOR
     ;; END

     ELSE: BEGIN
        PSM_INFO_MESSAGE, "Sorry, '"+ instrument_name + "' is not a valid instrument"
        PSMHELP, 'LOAD_INSTRUMENT'
     END

  ENDCASE

RETURN, inst

END
