; Copyright 2008-2011 CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Generate homogeneous white noise maps for LFI, HFI, or PLANCK (=LFI+HFI), with level according to the Planck 'blue book' sensitivities
;
; @param instrument  {in}{required}{type=string} name of instrument (can be 'LFI', or 'HFI' or 'PLANCK')
; @param duration_months {in}{required}{type=float} duration of the mission in months
; @keyword polar {in}{optional}{type=boolean} if set take into account the polarisation also.
; @keyword nside {in}{optional}{type=long} specify the nside of
; the output map. If not set maps are at nside=1024 for the LFI, and nside=2048 for the HFI.
; @keyword ordering {in}{optional}{type=string} healpix ordering written in the ouput (default = 'RING')
; @keyword output_directory {in}{optional}{type=string} directory in which files are written (see also output_filenames)
; @keyword output_filenames {in}{optional}{type=array of string} names of the files to be written default, e.g. 'HFI_bluebook_noise-353-GHz.fits'
; @uses fix_separator, getseed, write_fits_map, write_tqu
;
; @author Jacques Delabrouille, 
; @history december 2008 : first version
; @copyright 2008 CNRS
;-
FUNCTION BLUEBOOK_NOISE, freqchan, units, duration_months=duration_months, $
                    polar=polar, nside=nside, ordering=ordering

  IF NOT KEYWORD_SET(ordering) THEN ordering = 'RING'
  IF NOT KEYWORD_SET(duration_months) THEN duration_months=14.

  freqchans = [30,44,70,100,143,217,353,545,857]
  wh = WHERE(freqchans EQ freqchan, nwh)
  IF nwh EQ 1 THEN ichan=wh[0] ELSE BEGIN
     MESSAGE, 'freqchan must be in [30,44,70,100,143,217,353,545,857]'
  ENDELSE
  resol_arcmin = [33,24,14,10,7.1,5,5,5,5]
  
  deltaT_over_T_14mth_per_resol = [2., 2.7, 4.7, 2.5, 2.2, 4.8, 14.7, 147, 6700]    ; from bluebook, in uK/KCMB
  deltaT_over_T_per_resol = deltaT_over_T_14mth_per_resol * SQRT(14./duration_months) ; for number of months different from 14
  
  deltaP_over_P_14mth_per_resol = [2.8, 3.9, 6.7, 4., 4.2, 9.8, 29.8]               ; from bluebook, in uK/KCMB
  deltaP_over_P_per_resol = deltaP_over_P_14mth_per_resol * SQRT(14./duration_months) ; for number of months different from 14
  
  usenside = LONARR(9)
  IF NOT KEYWORD_SET(nside) THEN usenside=LONG([1024,1024,1024,2048,2048,2048,2048,2048,2048]) ELSE usenside[*]=LONG(nside)
  npix = 12L*usenside*usenside
  pixsize_arcmin = SQRT(4D0*!DPI / npix) * (180D0/!DPI) * 60D0
  sigmanoise_TT = deltaT_over_T_per_resol * (resol_arcmin/pixsize_arcmin)            ; per pixel
  sigmanoise_PP = deltaP_over_P_per_resol * (resol_arcmin/pixsize_arcmin[0:6])       ; per pixel
  
  IF NOT KEYWORD_SET(polar) THEN BEGIN
     seed = GETSEED()
     map = RANDOMN(seed,npix[ichan])
     map = TEMPORARY(map) * FLOAT(sigmanoise_TT[ichan])
  ENDIF ELSE BEGIN
     seed = GETSEED()
     map = RANDOMN(seed,npix[ichan],3)
     map[*,0] = map[*,0] * FLOAT(sigmanoise_TT[ichan])
     map[*,1] = map[*,1] * FLOAT(sigmanoise_PP[ichan])
     map[*,2] = map[*,2] * FLOAT(sigmanoise_PP[ichan])
  ENDELSE

  ;; the map is in uK_CMB
  
  RETURN, map * CONVERSION_FACTOR('uK/KCMB',units,freqchans[ichan]*1e9)

END
