; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012 
;   Jacques Delabrouille
;   Marc Betoule
;   Guillaume Castex
;   Maude Le Jeune
;   Jean-Baptiste Melin
;   Marc-Antoine Miville-Deschenes
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_MAIN
;
; PURPOSE:
;	This procedure is the main program to run a sky prediction or
;	simulation with the PSM at frequencies in the range 1 - 3000 GHz.
;
; CATEGORY:
;	PSM pipeline.
;
; CALLING SEQUENCE:
;	PSM_MAIN, Config_file
;
; INPUTS:
;	None.
;
; OPTIONAL INPUTS:
;	Config_file:  The input PSM configuration file.
;	
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;	None.
;
; OPTIONAL OUTPUTS:
;	None
;
; COMMON BLOCKS:
;	PSM: param - A structure containing parameters of the PSM run (created) 
;	SKY: model - A structure containing parameters of the sky model (created)
;
; SIDE EFFECTS:
;	Creates or fills in (and possibly overwrites) an output
;	directory hierarchy containing several files. Downloads data
;	files from the web. See the PSM documentation.
;
; RESTRICTIONS:
;	None
;
; PROCEDURE:
;
; EXAMPLE:
;	
;
; MODIFICATION HISTORY:
;	30 January 2007     Marc Betoule, first version.
;	05 February 2007    MB, Pipe modification: split prediction and simulation.
;	06 February 2007    MB, Model structure creation (parameters reading becomes conditional).
;	22 June 2007        MB, Put the coaddition at the end of the pipe.
;	12 July 2007        Maude le Jeune, Seed is set here (in simulation mode).
;	14 July 2007        Jacques Delabrouille, Fix debug mode for galactic, SZ and PS pipes.
;	18 July 2007        JD, Keyword check_param added
;	19 July 2007        JD, Added param as an optional output
;	09 October 2007     MB, Capability of writing noise map
;	18 January 2009     JD, Observation parameters added
;	19 January 2009     JD, New handling of seeds
;	20 January 2009     JD, Complete revision for PSM 1.7
;	11 March 2009       JD, Added keyword to check memory, updated
;	                    creation of output directory and subdirectories 
;	24 April 2010       JD, Some general cleaning, rearrangement and debugging, getting ready for PSM v1.7
;	03 July 2009        JD, Check memory automatically
;	11 May 2010         JD, Some general reorganization
;	25 October 2011     JD, Added the possibility to use an
;	                    existing psm_seeds structure to initialize the random
;	                    generators (for MC purposes) 
;	03 January 2012	    JD, Added standard IDL documentation header
;-
PRO PSM_MAIN, config_file, psm_seeds=psm_seeds, output_dir=output_dir, test=test, check_param=check_param, carefulness=carefulness, verbosity=verbosity, debug=debug, help=help

  ;; Handling of errors
  ;;--------------------
  error_status = 0
  IF NOT KEYWORD_SET(debug) THEN BEGIN
     CATCH, error_status  
     IF error_status NE 0 THEN BEGIN 
        PSM_INFO_TITLE, 'An error was encountered while running the PSM -- run aborted.'
        PRINT, 'Error Index = ' + NUMBER2STRING(error_status) 
        PRINT, 'Error Message: ', !ERROR_STATE.MSG
        GOTO, closing
        ;;CATCH, /CANCEL 
     ENDIF
  ENDIF

  ;; Define and open log file for PSM run
  ;;--------------------------------------
  logfile = PSM_TMPFILENAME(extension='.txt')
  ;; GET_LUN, unit
  DEFSYSV, '!psm_logfile', logfile

  ;; Common parameters
  ;;-------------------
  COMMON psm, param
  COMMON sky, model

  PSM_INFO_TITLE, 'STARTING THE PSM RUN'
  PSM_STARTUP, carefulness=carefulness, verbosity=verbosity, debug=debug
  MEMO, /current
  run_start_date = SYSTIME() & run_start_date = PSM_STR_REPLACE(run_start_date,' ','-')

  ;cpu, tpool_nthreads=1

;;-----------------------------------------------------------------------------------------------------------------
;; Read input parameters
;;-----------------------------------------------------------------------------------------------------------------

  IF N_PARAMS() LT 1 THEN BEGIN
     IF NOT KEYWORD_SET(test) THEN BEGIN
        PSM_INFO_MESSAGE, 'parameter file not specified, using default (psm/config/config.psm)'
        config_file = FIX_SEPARATOR(!PSMROOT)+'Soft/psm/config/config.psm'
     ENDIF ELSE BEGIN
        PSM_INFO_MESSAGE, 'parameter file not specified, using test mode (psm/config/test_config.psm)'
        config_file = FIX_SEPARATOR(!PSMROOT)+'Soft/psm/config/test.psm'
     ENDELSE
  ENDIF

  ;; Read the file and add start date
  ;;----------------------------------
  READ_PARAM_FILE, config_file, table
  nl = N_ELEMENTS(table[0,*])
  param_table = STRARR(2,nl+1)
  param_table[*,0:nl-1] = table
  param_table[0,nl] = 'RUN_START_DATE '
  param_table[1,nl] = ' '+run_start_date

  IF verbosity GE 1 THEN PSM_INFO_MESSAGE, "CONFIGURATION FILE SUCCESSFULLY READ"

  ;; Output_file 
  ;;-------------
  IF NOT KEYWORD_SET(output_dir) THEN use_output_dir = FIX_SEPARATOR(!PSMROOT)+"PSM_OUTPUT" ELSE use_output_dir=output_dir
  out = {output_directory:use_output_dir}
  ;FILL_STRUCT, param_table, out, carefulness=carefulness

  ;; Assign values to psm parameters
  ;;---------------------------------
  param = READ_PSM_PARAM(param_table, carefulness=carefulness, verbosity=verbosity, debug=debug)

  IF KEYWORD_SET(output_dir) NE 0 THEN BEGIN
     IF param.run.output_directory NE out.output_directory THEN BEGIN
        param.run.output_directory = out.output_directory 
        param.run.psm_biblio_file = out.output_directory+'psminfo/psm_bibliography.txt'
        param.run.psm_cite_file = out.output_directory+'psminfo/psm_citations.txt'
        PSM_INFO_MESSAGE, "Output directory overwritten by user, set to '"+out.output_directory+"'"
     ENDIF
  ENDIF

  ;; IF KEYWORD_SET(main_seed) NE 0 THEN BEGIN
  ;;    param.global.seed = main_seed
  ;;    PSM_INFO_MESSAGE, "Main seed overwritten by user, set to "+NUMBER2STRING(main_seed)
  ;; ENDIF

  IF KEYWORD_SET(psm_seeds) NE 0 THEN BEGIN
     param.global.seed = !psm_seeds.current.value
     PSM_INFO_MESSAGE, "Main seed overwritten by user, set to "+NUMBER2STRING(!psm_seeds.current.value)
  ENDIF
 
  use_output_dir = param.run.output_directory
  runp = param.run
  IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_SUBTITLE, "PSM PARAMETERS SUCCESFULLY DEFINED FOR PSM RUN"
  IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_MESSAGE, "PSM version = "+PSM_VERSION()
  IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_MESSAGE, "PSM run ID key = '"+PSM_ID()+"'"
  IF PSM_VERBOSITY() GE 1 THEN PRINT, ''
  IF KEYWORD_SET(check_param) NE 0 THEN BEGIN
     PRTSTRUCT, param, name='PARAM'
     GOTO, seedinit
  ENDIF ELSE BEGIN
     IF PSM_VERBOSITY() GE 1 THEN PRTSTRUCT, param, name='PARAM'
  ENDELSE

;;-----------------------------------------------------------------------------------------------------------------
;; Prepare the output directory, erasing old subdirectories and files if requested
;;-----------------------------------------------------------------------------------------------------------------

  outdir = GETPSMDIR('output')
  PREPARE_OUTPUT_DIR, outdir, config_file=config_file

;;-----------------------------------------------------------------------------------------------------------------
;; Seeds -- a tricky issue
;;-----------------------------------------------------------------------------------------------------------------
    ;; seeds are distributed across all aspects of PSM simulations
    ;; from the same, unique, input seed.
    ;; This permits complete reproducibility of a simulation, which would
    ;; not be possible if seeds were obtained from the computer internal clock, for instance.
    ;; However, as the PSM offers the possibility to 
    ;; - update the sky (for instance, to replace one single component, changing the model
    ;;   for some components, but possibly keeping some of them unchanged);
    ;; - observe the sky with different instruments;
    ;; - ultimately, do Monte Carlo simulations with various realizations of everything; 
    ;; a way to make sure that the same seed is not used to generate different components, 
    ;; or instrumental noise, is needed.

  seedinit:
  PSM_INFO_TITLE, 'INITIALISATION OF INPUT PSM SEEDS'
  IF NOT KEYWORD_SET(psm_seeds) THEN BEGIN
     PSM_SEEDINIT, main_seed=param.global.seed
     IF KEYWORD_SET(check_param) THEN BEGIN
        PSM_SEEDSTATUS, verbosity = 2
        GOTO, skysim
     ENDIF ELSE BEGIN
        IF (PSM_VERBOSITY() GE 1) THEN PSM_SEEDSTATUS, verbosity = PSM_VERBOSITY()
     ENDELSE
  ENDIF ELSE BEGIN
     PSM_INFO_MESSAGE, "Using psm_seeds as set by keyword"
     DEFSYSV, '!psm_seeds', psm_seeds
     PSM_SEEDSTATUS, verbosity = 2
  ENDELSE

;;-----------------------------------------------------------------------------------------------------------------
;; GENERATION OF THE MODEL SKY
;;-----------------------------------------------------------------------------------------------------------------
  skysim:
  PSM_SKYSIM, param_table, debug=debug, check_param=check_param
  HEAP_GC
  PSM_DEBUG_MESSAGE, 'Need to track memory leaks in sky simulation', routine='PSM_MAIN', infolimit=2, stoplimit=3

;;-----------------------------------------------------------------------------------------------------------------
;; OBSERVATION OF THE OUTPUT MODEL WITH SPECIFIED INSTRUMENTS
;;-----------------------------------------------------------------------------------------------------------------

  PSM_SKYOBS, param_table, check_param=check_param

;;-----------------------------------------------------------------------------------------------------------------
;; ENDING THE PSM RUN
;;-----------------------------------------------------------------------------------------------------------------

  IF NOT KEYWORD_SET(check_param) THEN BEGIN
     psm_seeds = !psm_seeds
     SAVE, psm_seeds, file=GETPSMDIR('private')+'psm_seeds.sav'
  ENDIF

CLOSING:

;;-----------------------------------------------------------------------------------------------------------------
;; GENERAL CLEANUP
;;-----------------------------------------------------------------------------------------------------------------

  IF error_status EQ 0 THEN BEGIN 

     PSM_INFO_TITLE, "PSM production successfully completed"

     ;; Move log file to appropriate output directory
     ;;-----------------------------------------------
     DEFSYSV, '!psm_logfile', exists=exists
     IF exists THEN BEGIN
        IF NOT KEYWORD_SET(check_param) THEN BEGIN
           RESTORE, FIX_SEPARATOR(GETPSMDIR('info'))+'psm_history.sav'
           run_number = N_ELEMENTS(psm_history)
           SPAWN, 'cp '+ !psm_logfile + ' ' + FIX_SEPARATOR(GETPSMDIR('info'))+'logfile-'+NUMBER2STRING(run_number)+'_'+PSM_ID()+'.txt'
        ENDIF
        FILE_DELETE, !psm_logfile
     ENDIF
     
  ENDIF ELSE BEGIN

     ;; Erase temporary log file 
     ;;--------------------------
     DEFSYSV, '!psm_logfile', exists=exists
     IF exists THEN BEGIN
        FILE_DELETE, !psm_logfile
     ENDIF

  ENDELSE

  ;; Check for temporary files
  ;;---------------------------
  tmpfiles_thisrun = PSM_TMPFILES()
  wh = WHERE(tmpfiles_thisrun NE '', nwh)
  IF nwh NE 0 THEN BEGIN
     IF PSM_CAREFULNESS() LE 1 THEN BEGIN
        ERASE_TMPFILES 
     ENDIF ELSE BEGIN
        PSM_DEBUG_MESSAGE, 'Some temporary PSM files still remain from this run!'
        IF PSM_VERBOSITY() GE 1 THEN PRINT, TRANSPOSE(tmpfiles_thisrun)
        PSM_DEBUG_MESSAGE, 'Type .c to erase them and end the PSM run'
        STOP
        ERASE_TMPFILES
     ENDELSE
  ENDIF
  
  PTR_FREE, !psm_tmpfilenames
  HEAP_GC


END
