; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Reads the specific parameters used for the generation of the CMB in the PSM
;
; @categories pipe
;   
; @param table {in}{required}{type=strarr(*,2)} vector of couple (keyword, value)
; @param cmb_seed {in}{required}{type=long} cmb seed value
; @param cmbp {out}{required}{type=struct} cmb model specific parameters
; @keyword carefulness {in}{optional}{type=integer} level of carefulness in handling missing or repeated keywords in the table. 
;
; @uses fill_struct
; 
; @history <p>19/01/2009: First version, Jacques Delabrouille </p>
; @history <p>22/01/2009: Some clean-up: obsolete models removed, Jacques Delabrouille </p>
; @history <p>05/02/2009: Default cosmological parameters adjusted to match WMAP best fit power 
;                         spectrum when CAMB is used, Jacques Delabrouille </p>
; @history <p>28/04/2009: Removed unnecessary intermediate 'model' structure 
;                         + added output file names in structure, Jacques Delabrouille </p>
; @history <p>22/05/2009: Keyword for carefulness added, improved handling of missing or repeated keywords, Jacques Delabrouille</p>
; @history <p>18/09/2009: Now is a function + restore keyword added, Jacques Delabrouille</p>
; @copyright <p>Jacques Delabrouille, CNRS, 2009</p>
;-
FUNCTION READ_CMB_PARAM, table, carefulness=carefulness, restore=restore
  
  ;; File where parameters will be written
  ;;---------------------------------------
  file = GETPSMDIR('private',/create)+'cmb_parameters.sav'

  IF NOT KEYWORD_SET(restore) THEN BEGIN

     which_cmb = {cmb_model: 'no_cmb'}  
     FILL_STRUCT, table, which_cmb, tags=['CMB_MODEL'], carefulness=carefulness
     
     ;; Seed used here for drawing random numbers when needed
     ;;-------------------------------------------------------
     SET_PSM_SEEDCHAIN, 'cmb-setup'
     
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;; CMB pipeline parameters
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     CASE which_cmb.cmb_model OF
;;-----------------------------------------------------------------------------------------------------------------------
        ;; Gaussian model
        ;;----------------
        'gaussian': BEGIN
           cmbp = { $
                  model:'gaussian', $                 ; {'gaussian'}
                  cmb_constrained:'no', $             ; {'yes', 'no'} used only for cmb_model = 'gaussian'
                  cmb_cl_source:'CAMB', $             ; {'standard-LCDM', 'CAMB', 'CLASS'}
                  cmb_lensing:'none', $               ; {'cl', 'ilens', 'none'}
                  save_cmb_map:'yes', $
                  save_cmb_alm:'yes' $
                  }
           tags = 'CMB_'+['CONSTRAINED','CL_SOURCE','LENSING']
           FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
        END
;;-----------------------------------------------------------------------------------------------------------------------
        ;; Prediction 
        ;;------------
        'prediction': BEGIN
           cmbp = { $
                  model:'prediction', $               ; {'prediction'}
                  ;; cmb_lens_cls:'yes', $
                  cmb_cl_source:'standard-LCDM', $    ; {'standard-LCDM', 'CAMB', 'CLASS'}
                  cmb_lensing:'cl', $                 ; {'cl', 'ilens', 'none'}
                  save_cmb_map:'yes', $
                  save_cmb_alm:'yes' $
                  }
           ;tags = 'CMB_'+['LENS_CLS']
           ;tags = [tags, 'SAVE_CMB_MAP', 'SAVE_CMB_ALM']
           ;FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
        END
;;-----------------------------------------------------------------------------------------------------------------------
        ;; Soumen Basak's lensed model
        ;;-----------------------------
        ;; 'lensed_sb': BEGIN
        ;;    cmbp = { $
        ;;           model:'lensed_sb', $ ; {'lensed_sb'}
        ;;           draw_lensed_map_number:'no', $
        ;;           lensed_map_number_min:1, $
        ;;           lensed_map_number_max:1, $
        ;;           lensed_map_number:1, $
        ;;           save_cmb_map:'yes', $
        ;;           save_cmb_alm:'yes' $
        ;;           }
        ;;    tags = ['DRAW_LENSED_MAP_NUMBER']
        ;;    ;tags = [tags, 'SAVE_CMB_MAP', 'SAVE_CMB_ALM']
        ;;    FILL_STRUCT, table, cmbp, carefulness=carefulness
        ;;    CASE STRLOWCASE(cmbp.draw_lensed_map_number) OF 
        ;;       'yes': BEGIN
        ;;          tags = ['LENSED_MAP_NUMBER_MIN','LENSED_MAP_NUMBER_MAX']
        ;;          FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
        ;;          cmbp.lensed_map_number = FIX(cmbp.lensed_map_number_min + (cmbp.lensed_map_number_max+1-cmbp.lensed_map_number_min)*RANDOMU(GETSEED())) < cmbp.lensed_map_number_max
        ;;       END
        ;;       'no': BEGIN
        ;;          tags = ['LENSED_MAP_NUMBER']
        ;;          FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
        ;;          cmbp.lensed_map_number_min = cmbp.lensed_map_number
        ;;          cmbp.lensed_map_number_max = cmbp.lensed_map_number
        ;;       END
        ;;       ELSE: MESSAGE, 'SELECTION FOR DRAW_LENSED_MAP_NUMBER NOT UNDERSTOOD'
        ;;    ENDCASE
        ;; END
;;-----------------------------------------------------------------------------------------------------------------------
        ;; Non Gaussian
        ;;--------------
        'nongaussian_fnl': BEGIN
           cmbp = { $
                  model:'nongaussian_fnl', $                 ; {'nongaussian_fnl'}
                  ng_simuset:'elsner3500', $                 ;
                  hi_ell_extend_gaussian:'no', $             ;
                  readjust_ng_spectrum:'no', $               ;
                  cmb_cl_source:'standard-LCDM', $           ; {'standard-LCDM', 'CAMB', 'CLASS'}
                  cmb_lensing:'none', $                      ; {'cl', 'ilens', 'none'}
                  draw_f_nl:'no', $
                  f_nl_min:-30., $
                  f_nl_max:30., $
                  f_nl:10., $
                  draw_ng_map_number:'no', $
                  ng_map_number_min:0, $
                  ng_map_number_max:0, $
                  ng_map_number:0, $
                  save_cmb_map:'yes', $
                  save_cmb_alm:'yes' $
                  }
           tags = ['NG_SIMUSET','HI_ELL_EXTEND_GAUSSIAN','READJUST_NG_SPECTRUM','CMB_CL_SOURCE','CMB_LENSING','DRAW_F_NL','DRAW_NG_MAP_NUMBER']
           FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
           CASE cmbp.ng_simuset OF 
              'elsner1024': BEGIN
                 cmbp.ng_map_number_min = 1
                 cmbp.ng_map_number_max = 1000
                 cmbp.ng_map_number = 1
               END
              'elsner3500': BEGIN
                 cmbp.ng_map_number_min = 1
                 cmbp.ng_map_number_max = 100
                 cmbp.ng_map_number = 1
               END
           ENDCASE
           CASE STRLOWCASE(cmbp.draw_f_nl) OF 
              'yes': BEGIN
                 tags = ['F_NL_MIN','F_NL_MAX']
                 FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
                 cmbp.f_nl = cmbp.f_nl_min + (cmbp.f_nl_max-cmbp.f_nl_min)*RANDOMU(GETSEED())
              END
              'no': BEGIN
                 tags = ['F_NL']
                 FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
                 cmbp.f_nl_min = cmbp.f_nl
                 cmbp.f_nl_max = cmbp.f_nl
              END
              ELSE: MESSAGE, 'SELECTION FOR DRAW_F_NL NOT UNDERSTOOD'
           ENDCASE
           CASE STRLOWCASE(cmbp.draw_ng_map_number) OF 
              'yes': BEGIN
                 tags = ['NG_MAP_NUMBER_MIN','NG_MAP_NUMBER_MAX']
                 FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
                 cmbp.ng_map_number = FIX(cmbp.ng_map_number_min + (cmbp.ng_map_number_max+1-cmbp.ng_map_number_min)*RANDOMU(GETSEED())) < cmbp.ng_map_number_max
              END
              'no': BEGIN
                 tags = ['NG_MAP_NUMBER']
                 FILL_STRUCT, table, cmbp, tags = tags, carefulness=carefulness
                 cmbp.ng_map_number_min = cmbp.ng_map_number
                 cmbp.ng_map_number_max = cmbp.ng_map_number
              END
              ELSE: MESSAGE, 'SELECTION FOR DRAW_NG_MAP_NUMBER NOT UNDERSTOOD'
           ENDCASE
        END
;;-----------------------------------------------------------------------------------------------------------------------
        ;; Template map
        ;;--------------
        'template': BEGIN
                                ;find whether cmb_template_alm and/or cmb_template_map are defined and exist
           dummy = {cmb_template_alm:'',cmb_template_map:''}
           FILL_STRUCT, table, dummy, carefulness=carefulness
           alm_exists = FILE_TEST(dummy.cmb_template_alm)
           map_exists = FILE_TEST(dummy.cmb_template_map)
           IF alm_exists THEN BEGIN
              cmb_template_alm = dummy.cmb_template_alm
              cmb_template_map = ''
           ENDIF ELSE BEGIN
              IF map_exists THEN BEGIN
                 cmb_template_map = dummy.cmb_template_map
                 cmb_template_alm = ''
              ENDIF ELSE BEGIN
                 cmb_template_alm = fix_separator(!PSMROOT)+'data/Ancillary/Simulations/CMB/Templates/CMB_lensed_lmax3250_alm_seed10.fits'
                 cmb_template_map = ''
              ENDELSE
           ENDELSE
           cmbp = { $
                  model:'template', $ ; {'template'}
                  cmb_template_alm:cmb_template_alm, $
                  cmb_template_map:cmb_template_map, $
                  save_cmb_map:'yes', $
                  save_cmb_alm:'yes' $
                  }
           ;tags = ['SAVE_CMB_MAP','SAVE_CMB_ALM']
           ;FILL_STRUCT, table, cmbp, tags=tags, carefulness=carefulness
        END 
;;-----------------------------------------------------------------------------------------------------------------------
        ;; Prediction
        ;;------------
        'prediction': BEGIN
           cmbp = { $
                  model:'prediction', $           ; {'prediction'}
                  cmb_obs_template:'ADAMIS_NILC5', $  ; {'ADAMIS_NILC5'}
                  save_cmb_map:'yes', $
                  save_cmb_alm:'yes' $
                  }
           ;FILL_STRUCT, table, cmbp, carefulness=carefulness
        END
;;-----------------------------------------------------------------------------------------------------------------------
        ;; Lenspix
        ;;---------
        ;; 'lenspix': BEGIN
        ;;    cmbp = { $
        ;;           model:'lenspix', $           ; {'prediction'}
        ;;           mpi_nproc:1, $
        ;;           lenspix_cls_file:fix_separator(!PSMROOT)+'data/Ancillary/CMB/Lenspix/WMAP3_scalCls.dat',$
        ;;           lenspix_lens_method:1,$
        ;;           lenspix_interp_method:1,$   
        ;;           lenspix_interp_factor:-1, $
        ;;           save_cmb_alm:'yes' $
        ;;           }
        ;;    FILL_STRUCT, table, cmbp, carefulness=carefulness
        ;; END
        'no_cmb': cmbp = {model:'no_cmb'}
        ELSE: MESSAGE, 'CMB MODEL NOT ALLOWED'
     ENDCASE
     
     ;; Write the parameters
     ;;----------------------
     SAVE, filename=file, cmbp

  ENDIF ELSE BEGIN
     RESTORE, filename=file
  ENDELSE

  RETURN, cmbp
     
  END
  
  
  
