;FUNCTION ANG_DIST, z_array, cosmo=cosmo
;+
; NAME:
;    ANG_DIST
; ONE LINE HELP:
;    give the angular distance of a source (Mpc)
; PURPOSE:
;    give the angular distance of a source (Mpc)
; CALLING SEQUENCE:
;    ang_dist_array = ANG_DIST(z_array, cosmo=cosmo)
; INPUT:
;    z_array : an array of redshift values
;
; OUTPUT:
;    ang_dist_array : the array of angular distances corresponding to z_array
;    
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;    MATTIG, HUBBLE_AT_Z
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin April 2001
;    Keyword /DOUBLE added in QROMB and new example in header, JBM May 2001
;    Correction of a small bug in the order of the result, JBM May 21 2001
;    Does not return a 1 element array anymore
;      if z_array is not an array at input, JBM+JD, may 29 2001
;    Loops in LONG, JBM and JD, may 29 2001
;    Distances in Mpc, JBM, May 30 2001
;    HUBBLE_AT_Z introduced, JBM, Sept 24 2001
;    Keyword cosmo introduced, JBM, Sept 27 2001
;    Bug (H0 forgotten) fixed for spherical and hyperbolic cosmo, JBM,
;    Nov 22 2002
;    Modified for quintessence and renamed ang_dist_q, JBM, July 2004
;    Last loop in LONG, JBM, April 12 2006
;    Fully rewritten from COM_DIST, JBM, June 30 2009
;    Merged ang_dist and ang_dist_q, Matthieu Roman, April 2012
;-

FUNCTION ANG_DIST, z_array, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : ad = ANG_DIST(z_array)'
        print,'Keyword : cosmo=cosmo'
     GOTO, closing
ENDIF

c_light = 2.9979246d+08 ; in m/s

; BE CAREFUL WITH QUINTESSENCE HERE !
;; omega_c = cosmo.omega_m+cosmo.omega_de-1.
omega_c = cosmo.omega_k

; case of a flat universe
IF (omega_c EQ 0.) THEN BEGIN
  res = COM_DIST(z_array, cosmo=cosmo)/(1.+z_array)
ENDIF ELSE BEGIN
  ; case of a spherical universe
  IF (omega_c GT 0.) THEN BEGIN
    res = c_light/1000./SQRT(omega_c)/(1.+z_array)/(100.*cosmo.h)* $
          SIN(100.*cosmo.h*SQRT(omega_c)/c_light*1000.*COM_DIST(z_array, cosmo=cosmo))
  ENDIF ELSE BEGIN
    ; case of an hyperbolic universe
    res = c_light/1000./SQRT(ABS(omega_c))/(1.+z_array)/(100.*cosmo.h)* $
          SINH(100.*cosmo.h*SQRT(ABS(omega_c))/c_light*1000.*COM_DIST(z_array, cosmo=cosmo))
  ENDELSE
ENDELSE

RETURN, res

closing:
END
