;+
; NAME:
;     LOOKBACK_TIME
; ONE LINE HELP:
;     Compute the lookback time (=INT(1/(H*(1+z))*dz)) if the redshift z and the cosmology are given (quintessence models).
; PURPOSE:
;     Compute the lookback time (=INT(1/(H*(1+z))*dz)) if the redshift z and the cosmology are given (quintessence models).
; CALLING SEQUENCE:
;     t=LOOKBACK_TIME(z_array, cosmo=cosmo)
; INPUT:
;     z_array : the redshift array
; OUTPUT:
;     t : the lookback times (years)
; KEYWORDS:
;     cosmo : the structure containing the cosmological parameters (cf. DEFINE_COSMO)
; EXAMPLE :
;     
; RESTRICTIONS:
;     Not fully tested yet. Report any bugs to Jacques Delabrouille
;     delabrouille@apc.univ-paris7.fr
;
; PROCEDURES CALLED:
;     HUBBLE_AT_Z
; REVISION HISTORY
;     Written, JBM, 2002
;     Modified for quintessence and renamed lookback_time_q,
;     Jean-Baptiste Melin, March 13 2006
;     Merged lookback_time and lookback_time_q, Matthieu Roman, April
;     3, 2012
;-

;------------------------------------------------------------------------------
FUNCTION SUB_LOOKBACK_TIME, u
;-----------------------------
  COMMON SUB_LOOKBACK_TIME_COSMO, sub_lookback_time_cosmo 
  hz = HUBBLE_AT_Z(u, cosmo=sub_lookback_time_cosmo)
  RETURN, 1./(hz*(1.+u))
END
;------------------------------------------------------------------------------

FUNCTION LOOKBACK_TIME, z_array, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : t=LOOKBACK_TIME(z_array)'
        print,'Keyword : cosmo=cosmo'
     GOTO, closing
ENDIF

COMMON SUB_LOOKBACK_TIME_COSMO, sub_lookback_time_cosmo
sub_lookback_time_cosmo = cosmo

save_z = z_array
z_array = z_array[UNIQ(z_array, SORT(z_array))]
n = N_ELEMENTS(z_array)
new_z = [z_array[0],z_array]
z_sup = MAX([new_z[n],10000.])
integrale = QROMB('sub_lookback_time_q', new_z[n], z_sup, /DOUBLE)
result = DBLARR(n)

FOR i=0L, n-1 DO BEGIN
  result[n-1-i] = integrale
  integrale = integrale + QROMB('sub_lookback_time_q', new_z[n-1-i], new_z[n-i], /DOUBLE)
ENDFOR

Mpc2m = 3.0856775807d+22 ; in m
result = result/1d3*Mpc2m
yr2s = 365.*24.*60.*60.
result = result/yr2s
res = DBLARR(N_ELEMENTS(save_z))
FOR i=0, n-1 DO BEGIN
  wh = WHERE(z_array[i] EQ save_z)
  res[wh]=result[i]
ENDFOR
z_array = save_z
IF NOT ISARRAY(z_array) AND N_ELEMENTS(res) EQ 1 THEN res = res(0)
RETURN, res
closing:
END
