;;-------------------------------------------------------------------------------------------
;; INITIALISATION OF COSMO OBJECT
;;-------------------------------------------------------------------------------------------
FUNCTION COSMO::INIT, t_cmb=t_cmb, h=h, omega_m=omega_m, omega_rad=omega_rad, omega_b=omega_b, omega_k=omega_k, omega_nu=omega_nu, omega_de=omega_de, $
                       sigma_8=sigma_8, n_scal=n_scal, n_s_running=n_s_running, n_tens=n_tens, r_tens_scal=r_tens_scal, tau_reion=tau_reion, fraction_he=fraction_he, $
                       n_massless_nu=n_massless_nu, w_de=w_de, scalar_amplitude=scalar_amplitude

  ;; DEFAULT VALUES
  ;;----------------
  self.T_CMB=2.725                         ; CMB temperature
  self.h=0.704                             ; Hubble parameter
  self.Omega_m=0.272                       ; Omega matter
  self.Omega_rad=0.                        ; Omega radiation
  self.Omega_de=0.728                      ; Omega dark energy (for w not necessarily equal to -1)
  self.Omega_k=0.                          ; Omega curvature
  self.Omega_b=0.0456                      ; Omega baryon
  self.Omega_nu=0.                         ; Omega neutrino
  self.Sigma_8=0.809                       ; sigma_8 (for LSS power normalisation, use in SZ)
  self.n_s=0.963                           ; scalar spectral index
  self.n_s_running=0.                      ; running of the scalar spectral index
  self.n_t=0.                              ; tensor spectral index
  self.r=0.00                              ; tensor to scalar ratio
  self.tau_reion=0.087                     ; reionisation optical depth
;          self.z_reion:11.                ; reionisation redshift
  self.he_fraction=0.24                    ; helium fraction by mass
  self.n_massless_nu=3.04                  ; number of effective massless neutrinos species
  self.n_massive_nu=0.                     ; number of massive neutrino species
  self.w_de=-1.                            ; dark energy equation of state
  self.scalar_amplitude=2.441e-9           ; scalar amplitude
  
  ;; CMB temperature
  ;;-----------------
  IF DEFINED(t_cmb) NE 0 THEN self.t_cmb = t_cmb

  ;; Hubble parameter h
  ;;--------------------
  IF DEFINED(h) NE 0 THEN self.h = h

  ;; Density parameters
  ;;--------------------
  omeganames = ['omega_m','omega_rad','omega_de','omega_k']
  nomega = N_ELEMENTS(omeganames)
  omegavalue = FLTARR(nomega)
  defomega = INTARR(nomega)
  FOR i=0, nomega-1 DO BEGIN
     ok = EXECUTE('IF DEFINED('+omeganames[i]+') THEN defomega('+NUMBER2STRING(i)+') = 1')
     IF NOT ok THEN MESSAGE, 'Error while executing IDL command'
  ENDFOR

  ;; Assign default values
  ;;-----------------------
  ndefomega = FIX(TOTAL(defomega))
  WHILE ndefomega LE nomega-2 DO BEGIN
     IF DEFINED(omega_rad) EQ 0 THEN BEGIN
        omega_rad = 0.
        whomega = WHERE(omeganames EQ 'omega_rad')
        defomega[whomega] = 1
        ndefomega = ndefomega+1
     ENDIF ELSE BEGIN
        IF DEFINED(omega_k) EQ 0 THEN BEGIN
           omega_k = 0.
           whomega = WHERE(omeganames EQ 'omega_k')
           defomega[whomega] = 1
           ndefomega = ndefomega+1
        ENDIF ELSE BEGIN
           IF DEFINED(omega_m) EQ 0 THEN BEGIN
              omega_m = self.omega_m
              whomega = WHERE(omeganames EQ 'omega_m')
              defomega[whomega] = 1
              ndefomega = ndefomega+1
           ENDIF ELSE BEGIN
              MESSAGE, "missing values for density parameters"
           ENDELSE
        ENDELSE
     ENDELSE
  ENDWHILE
  
  ;; Check consistency between density parameters
  ;;----------------------------------------------

  CASE ndefomega OF
     nomega: BEGIN
        ok = (self.omega_m + self.omega_rad + self.omega_de + self.omega_k) EQ 1
        IF NOT ok THEN MESSAGE, "incompatible input values for density parameters"
     END
     (nomega-1): BEGIN
        wh = WHERE(defomega EQ 0, complement=whdef)
        FOR i=0, nomega-2 DO BEGIN
           ok = EXECUTE('omegavalue[whdef[i]] = '+omeganames[whdef[i]])
           IF NOT ok THEN MESSAGE, "An error occured while defining density parameters"
        ENDFOR
        ok = EXECUTE(omeganames(wh[0])+' = 1-TOTAL(omegavalue)')
        IF NOT ok THEN MESSAGE, "An error occured while defining density parameters"
     END
     ELSE: MESSAGE, "missing values for density parameters"
  ENDCASE

  IF DEFINED(omega_m) NE 0 THEN BEGIN
     self.omega_m = omega_m
     defomega[0] = 1
  ENDIF
  IF DEFINED(omega_rad) NE 0 THEN BEGIN
     self.omega_rad = omega_rad
     defomega[1] = 1
  ENDIF
  IF DEFINED(omega_de) NE 0 THEN BEGIN
     self.omega_de = omega_de
     defomega[2] = 1
  ENDIF
  IF DEFINED(omega_k) NE 0 THEN BEGIN
     self.omega_k = omega_k
     defomega[3] = 1
  ENDIF

  ;; Baryons
  ;;---------
  IF DEFINED(omega_b) NE 0 THEN BEGIN
     self.omega_b = omega_b
  ENDIF

  ;; Light neutrinos
  ;;-----------------
  IF DEFINED(omega_nu) NE 0 THEN BEGIN
     self.omega_nu = omega_nu
  ENDIF

  ok = ((self.omega_b + self.omega_nu) LE self.omega_m)
  IF NOT ok THEN MESSAGE, 'Incompatible density parameters'

  ;; Dark Energy or cosmological constant (?)
  ;;------------------------------------------
  IF DEFINED(w_de) NE 0 THEN self.w_de = w_de
;  cosmo.lambda = cosmo.omega_de

  ;; Initial power spectrum
  ;;------------------------
  IF DEFINED(n_scal) NE 0 THEN self.n_s = n_scal
  IF DEFINED(n_s_running) NE 0 THEN self.n_s_running = n_s_running
  IF DEFINED(n_tens) NE 0 THEN self.n_t = n_tens
  IF DEFINED(scalar_amplitude) NE 0 THEN self.scalar_amplitude = scalar_amplitude
  IF DEFINED(r_tens_scal) NE 0 THEN self.r = r_tens_scal

  ;; Reionisation
  ;;--------------
  IF DEFINED(tau_reion) NE 0 THEN self.tau_reion = tau_reion

  ;; Helium fraction
  ;;-----------------
  IF DEFINED(fraction_he) NE 0 THEN self.he_fraction = fraction_he

  ;; Neutrino types
  ;;----------------
  IF DEFINED(n_massless_nu) NE 0 THEN self.n_massless_nu = n_massless_nu
  IF DEFINED(n_massive_nu) NE 0 THEN self.n_massive_nu = n_massive_nu
  
  RETURN, 1

END

;;-------------------------------------------------------------------------------------------
;; DEFINITION OF COSMO OBJECT
;;-------------------------------------------------------------------------------------------
PRO COSMO__DEFINE

  dg = {cosmo_dg, z:findgen(1001)/100, value:FLTARR(100)}

  cosmo = {cosmo, $
           T_CMB:2.725, $                ; CMB temperature
           h:0.704, $                    ; Hubble parameter
           Omega_m:0.272, $              ; Omega matter
           Omega_rad:0., $               ; Omega radiation
           Omega_de:FLOAT('NaN'), $      ; Omega dark energy (for w not necessarily equal to -1)
           Omega_k:0., $                 ; Omega curvature
           Omega_b:0.0456, $             ; Omega baryon
           Omega_nu:0., $                ; Omega neutrino
           Sigma_8:0.809, $              ; sigma_8 (for LSS power normalisation, use in SZ)
           n_s: 0.963, $                 ; scalar spectral index
           n_s_running: 0., $            ; running of the scalar spectral index
           n_t: 0., $                    ; tensor spectral index
           r:0.00, $                     ; tensor to scalar ratio
           tau_reion:0.087, $            ; reionisation optical depth
;          z_reion:11., $                ; reionisation redshift
           he_fraction:0.24, $           ; helium fraction by mass
           n_massless_nu:3.04, $         ; number of effective massless neutrinos species
           n_massive_nu:0., $            ; number of massive neutrino species
           w_de: -1., $                  ; dark energy equation of state
           scalar_amplitude: 2.441e-9, $ ; scalar amplitude
           linear_growth_factor: dg $    ; linear growth factor
          }

END
