;FUNCTION COSMO_PK, k, cosmo=cosmo, cluster=cluster, nonorm=nonorm, bbks=bbks
;+
; NAME:
;    COSMO_PK
; ONE LINE HELP:
;    Compute P(k), for CDM-like models
; PURPOSE:
;    Given a wavenumber k (Mpc^(-1.)), this routine returns the corresponding
;    value of the power spectrum P(k) (Mpc^3).
;    The general form of the power spectrum is taken from Eisenstein & Hu (1998) except if bbks is set.  
; CALLING SEQUENCE:
;    pk = COSMO_PK(k, cosmo=cosmo, cluster=cluster, nonorm=nonorm, bbks=bbks)
; 
; INPUT:
;    k : the wavenumber(s) (can be an array) in Mpc^(-1.)
; OUTPUT:
;    pk : the corresponding P(k) in Mpc^3
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
;    cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;    nonorm : return a no normalized spectrum
;    bbks : set this keyword if you want to use the BBKS (1986) function instead of the one from Eisenstein & Hu (1998)
; EXAMPLES:
;
; RESTRICTIONS:
;    Report any bugs to Jean-Baptiste Melin
;    jean-baptiste.melin@cea.fr
;
; PROCEDURES CALLED:
;    COSMO_PK_NORM_PK
; REVISION HISTORY
;    Written from cdm_sigma, Jean-Baptiste Melin, May 25 2001
;    Normalisation fixed, JBM and JD, 28 may 2001
;    Minor changes in the header, JBM, May 29 2001
;    CDM_PK becomes COSMO_PK, JBM, September 6 2001
;    Keyword cosmo introduced, JBM, Sept 27 2001
;    cluster keyword added, JBM, Feb 6 2006
;    spectral index ns added, JBM, Sep 21 2006
;    now use the cosmo_tk function, JBM, July 28 2009
;-

;-----------------------------------------------------------------------
FUNCTION COSMO_PK_I_SIGMA, x
;---------------------------

COMMON COSMO_PK_NORM_PK_COSMO, norm_pk_cosmo, norm_pk_cluster, norm_pk_bbks
COMMON COSMO_PK_NORM_PK_SCALE, norm_pk_r

k = x/norm_pk_r
ns = norm_pk_cosmo.n_s

Tk = COSMO_TK(k, cosmo=norm_pk_cosmo, cluster=norm_pk_cluster, bbks=norm_pk_bbks)

Pk  =  k^ns*Tk^2

IF (x LT 0.001) THEN Wk = 1. ELSE Wk  = 3./x^2*(sin(x)/x - cos(x))  

RETURN, k^2*Pk*Wk^2
END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION COSMO_PK_NORM_PK, cosmo=cosmo, cluster=cluster, bbks=bbks
;-----------------------------------------------------------------

x1 = 0.0                       ; roots of Wth(x)
x2 = 4.496
x3 = 7.723
x4 = 10.902
x5 = 14.066
x6 = 17.221
x7 = 20.371

COMMON COSMO_PK_NORM_PK_COSMO, norm_pk_cosmo, norm_pk_cluster, norm_pk_bbks
COMMON COSMO_PK_NORM_PK_SCALE, norm_pk_r
norm_pk_cosmo = cosmo
norm_pk_cluster = cluster
IF KEYWORD_SET(bbks) THEN norm_pk_bbks = 1 ELSE norm_pk_bbks = 0

; Normalization to sig8

norm_pk_r = 8./norm_pk_cosmo.h
sig1 = qromo('COSMO_PK_I_SIGMA',x1,x2)
sig2 = qromo('COSMO_PK_I_SIGMA',x2,x3)
sig3 = qromo('COSMO_PK_I_SIGMA',x3,x4)
sig4 = qromo('COSMO_PK_I_SIGMA',x4,x5)
sig5 = qromo('COSMO_PK_I_SIGMA',x5,x6)
if (sig5/sig1 gt 0.01) then stop, 'sig not converging'

RETURN,1./(2.*!pi^2)*(1./norm_pk_r)*(sig1+sig2+sig3+sig4+sig5)
END
;------------------------------------------------------------------------------

FUNCTION COSMO_PK, k, cosmo=cosmo, cluster=cluster, nonorm=nonorm, bbks=bbks

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
	PRINT,'  CALL IS : pk = COSMO_PK(k)'
	PRINT,'  Keywords : cosmo=cosmo, cluster=cluster, nonorm=nonorm, bbks=bbks'
     GOTO, closing
ENDIF

ns = cosmo.n_s

Tk = COSMO_TK(k, cosmo=cosmo, cluster=cluster, bbks=bbks)

Pk  =  k^ns*Tk^2

IF NOT KEYWORD_SET(nonorm) THEN BEGIN
        bias = 1./(cosmo.sigma_8)
        normpk = COSMO_PK_NORM_PK(cosmo=cosmo,cluster=cluster,bbks=bbks) 
	RETURN, Pk/normpk*(1./bias^2.)
ENDIF ELSE BEGIN
	RETURN, Pk
ENDELSE

closing:
END
