;FUNCTION EVRARD, delta_c, sigma, z, cosmo=cosmo
;
;+
; NAME:
;  EVRARD
; ONE LINE HELP:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Evrard et al. 2001 fitting
; PURPOSE:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Evrard et al. 2001 fitting
; CALLING SEQUENCE:
;  result = EVRARD(delta_c, sigma, z, cosmo=cosmo)
;
; INPUT:
;  delta_c : the critical density contrast at redshift z
;  sigma : fluctuation amplitude for scale M and redshift z
;  z : the redshift of computation
; OUTPUT:
;  M/rho0*dNclu/(d_covolume*dln(sigma^(-1)))
;    
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
; EXAMPLES:
;  
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  No procedure called 
; REVISION HISTORY
;  Written, Jean-Baptiste Melin, November 21 2001
;-

PRO JENKINS_PARAMETERS, A, B, e, omega_m

; Fit mass function parameters given in Evrard et al. 2001
; "Galaxy clusters in Hubble volume simulations" page 8

x = (1-omega_m)/0.7
A1 = 0.27
B1 = 0.65
e1 = 3.77
A03 = 0.22
B03 = 0.73
e03 = 3.86

A = (1-x)*A1+x*A03
B = (1-x)*B1+x*B03
e = (1-x)*e1+x*e03

END

FUNCTION EVRARD, delta_c, sigma, z, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
    print,'  CALL IS : EVRARD(delta_c, sigma, z)'
    print,'  KEYWORD : cosmo=cosmo'
    GOTO, closing
ENDIF

H_z = HUBBLE_AT_Z(z, cosmo=cosmo)
omega_m_z = cosmo.omega_m*(1.+z)^3.*(100.*cosmo.h/H_z)^2.
JENKINS_PARAMETERS, A, B, e, omega_m_z

RETURN, A*EXP(-1*(ABS(ALOG(1./sigma)+B))^e)

closing:
END
