;+
; NAME:
;    FAST_CONVERT_MASS_RADIUS
; ONE LINE HELP:
;    Convert masses and radii from an overdensity to another given a NFW profile (fast version)
; PURPOSE:
;    Convert masses and radii from an overdensity to another given a NFW profile (fast version)
; CALLING SEQUENCE:
;    FAST_CONVERT_MASS_RADIUS, mass_in, z_in, big_delta_in, big_delta_out, c_nfw, $
;                              r_in, r_out, mass_out, r_s=r_s, $
;                              cosmo=cosmo, in_mean=in_mean, out_mean=out_mean, five_hundred=five_hundred, bullock=bullock, gao=gao
; INPUT:
;    mass_in : the cluster mass you want to convert (10^15 mass solar unit)
;    z_in : the cluster redshift
;    big_delta_in : the overdensity defining the input cluster (compared to rho_crit by default
;                     or rho_mean if the in_mean keyword is set)
;    big_delta_out : the overdensity defining the output cluster (compared to rho_crit by default
;                      or rho_mean if the out_mean keyword is set)
;    c_nfw : the NFW concentration parameter
;            c_nfw=r200/rs where r200 is defined compared to rho_crit (see Navarro, Frenk and White 1996)
;            the procedure uses c_nfw=5 if not defined
; OUTPUT:
;    r_s : scale radius as an output if set
;    r_in : the input radius (in Mpc) corresponding to mass_in and big_delta_in
;    r_out : the output radius (in Mpc) corresponding to c_nfw and big_delta_out
;    mass_out : the output mass (in 10^15 Msol unit) corresponding to c_nfw and big_delta_out
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
;    in_mean : set this keyword if you give big_delta_in in rho_mean unit
;              instead of rho_crit unit
;    out_mean : set this keyword if you give big_delta_out in rho_mean unit
;               instead of rho_crit unit
;    bullock : if set then c_nfw is not used; the Bullock et al. 2001 formula c_vir=9.*(1+z)^(-1)*(M_vir/M_star)^(-0.13) is used
;    gao :  if set then c_nfw is not used; the Gao et al. 2008 fit is used (Table 1)
;    five_hundred : if set c_nfw is r500/rs and not r200/rs
; EXAMPLES:
;
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jean-Baptiste Melin
;    jean-baptiste.melin@cea.fr
;
; PROCEDURES CALLED:
;
; REVISION HISTORY
;    Adapted from convert_mass_radius.pro (faster version), Antoine Chamballu, September 2 2006
;    cosmetic changes + five_hundred keyword added, Jean-Baptiste Melin, July 3 2008
;    bullock and gao keywords added, JBM, July 4 2008
;    r_s keyword added and c500=3.3 (r500) equivalent to c200=5, JBM, July 8 2008
;    clu_num=0 changed for clu_num=0l for /bullock and /gao +
;      precision increased in the dichotomies, AC, August 29 2008
;    Returns use_c_200 in c_nfw for /gao + doesn't compute M_200 
;      twice if big_delta_out = 200. (in /gao), AC, September 10 2008
;    Adapted to wCDM models, Matthieu Roman, April 2012
;-

;--------------------------------------------------------------------------------------------------------
PRO FAST_CONVERT_MASS_RADIUS, mass_in, z_in, big_delta_in, big_delta_out, c_nfw, $
                              r_in, r_out, mass_out, r_s=r_s, $
                              cosmo=cosmo, in_mean=in_mean, out_mean=out_mean, $
                              five_hundred=five_hundred, bullock=bullock, gao=gao
;--------------------------------------------------------------------------------------------------------

nclu = N_ELEMENTS(mass_in)

IF (KEYWORD_SET(bullock) EQ 0) AND (KEYWORD_SET(gao) EQ 0) THEN BEGIN
  IF (NOT DEFINED(c_nfw)) THEN BEGIN
    IF KEYWORD_SET(five_hundred) THEN BEGIN
      PRINT, 'NFW concentration parameter is not defined (c_nfw=r500/rs). I take c_nfw=3.3'
      c_nfw = REPLICATE(3.3, nclu)
    ENDIF ELSE BEGIN
      PRINT, 'NFW concentration parameter is not defined (c_nfw=r200/rs). I take c_nfw=5.'
      c_nfw = REPLICATE(5., nclu)
    ENDELSE
  ENDIF
  use_c_nfw = FLOAT(c_nfw)
ENDIF

IF KEYWORD_SET(in_mean) THEN BEGIN
    omega_m_z = cosmo.omega_m*(1+z_in)^3.*(HUBBLE_AT_Z(z_in, cosmo=cosmo)/100./cosmo.h)^(-2.)
    use_big_delta_in = big_delta_in*omega_m_z
ENDIF ELSE BEGIN
    use_big_delta_in = big_delta_in
ENDELSE

IF KEYWORD_SET(out_mean) THEN BEGIN
    omega_m_z = cosmo.omega_m*(1+z_in)^3.*(HUBBLE_AT_Z(z_in, cosmo=cosmo)/100./cosmo.h)^(-2.)
    use_big_delta_out = big_delta_out*omega_m_z
ENDIF ELSE BEGIN
    use_big_delta_out = big_delta_out
ENDELSE

; Compute r_in (Same result as in the CLUSTER_MODEL procedure)
r_in = 1.69*mass_in^(1/3.)*(178./use_big_delta_in)^(1/3.)*(cosmo.h)^(-2/3.)* $
       (HUBBLE_AT_Z(z_in, cosmo=cosmo)/100./cosmo.h)^(-2/3.)


IF (KEYWORD_SET(bullock) EQ 0) AND (KEYWORD_SET(gao) EQ 0) THEN BEGIN

  IF KEYWORD_SET(five_hundred) THEN BEGIN
    ; Compute the normalisation of the NFW profile given c_nfw=r500/rs
    delta_c = 500/3.*use_c_nfw^3/(ALOG(1+use_c_nfw)-use_c_nfw/(1+use_c_nfw))
  ENDIF ELSE BEGIN    
    ; Compute the normalisation of the NFW profile given c_nfw=r200/rs
    delta_c = 200/3.*use_c_nfw^3/(ALOG(1+use_c_nfw)-use_c_nfw/(1+use_c_nfw))
  ENDELSE

  mass_out = 0.*mass_in
  r_out = 0.*r_in
  r_s = 0.*r_in

  FOR clu_num=0l, nclu-1 DO BEGIN

    ; Compute x_in=r_in/rs
    x1 = 10.
    x2 = 20.
    while abs(x2-x1) gt 1d-4 do begin
        fx1 = use_big_delta_in[clu_num]/3.*x1^3-delta_c[clu_num]*(ALOG(1+x1)-x1/(1+x1))
        fx2 = use_big_delta_in[clu_num]/3.*x2^3-delta_c[clu_num]*(ALOG(1+x2)-x2/(1+x2))
        
        a = (fx2-fx1)/(x2-x1)
        b = fx1-a*x1
        x1 = x2
        x2 = -b/a
    endwhile
    x_in = x1

    ; caracteristic scale of the NFW profile in Mpc
    rs = r_in[clu_num]/x_in
    r_s[clu_num] = rs
    
    ; Compute x_out=r_out/rs
    x1 = 10.
    x2 = 20.
    while abs(x2-x1) gt 1d-4 do begin
        fx1 = use_big_delta_out[clu_num]/3.*x1^3-delta_c[clu_num]*(ALOG(1+x1)-x1/(1+x1))
        fx2 = use_big_delta_out[clu_num]/3.*x2^3-delta_c[clu_num]*(ALOG(1+x2)-x2/(1+x2))
        
        a = (fx2-fx1)/(x2-x1)
        b = fx1-a*x1
        x1 = x2
        x2 = -b/a
    endwhile
    x_out = x2
    
    ; compute r_out
    r_out[clu_num] = x_out*rs

    ; compute mass_out
    mass_out[clu_num] = (x_out/x_in)^3*use_big_delta_out[clu_num]/use_big_delta_in[clu_num]*mass_in[clu_num]

  ENDFOR

ENDIF ELSE BEGIN

  IF KEYWORD_SET(bullock) THEN BEGIN

    ; Hu & Kravtsov 2003 approach

    mass_star = 1.5*1d-2*(cosmo.h)^(-1)

    ;* Mvir computation first *

    use_big_delta_c = BDELTA_C(z_in, cosmo=cosmo)

    delta_mass_out = 1d-2 ; 1% accuracy in mass_vir computation

    mass_vir = 0.*mass_in

    FOR clu_num=0l, nclu-1 DO BEGIN

      mass_out = mass_in[clu_num]

      old_mass_out = [-1.]

      WHILE ABS((old_mass_out-mass_out)/old_mass_out) GE delta_mass_out DO BEGIN

        ; Bullock et al. 2001
        use_c_vir = 9.*(1.+z_in[clu_num])^(-1)*(mass_out/mass_star)^(-0.13)

        old_mass_out = mass_out

        ; Compute the normalisation of the NFW profile
        delta_c = use_big_delta_c[clu_num]/3.*use_c_vir^3/(ALOG(1+use_c_vir)-use_c_vir/(1+use_c_vir))

        ; Compte x_in=r_in/rs
        x1 = 10.
        x2 = 20.
        while abs(x2-x1) gt 1d-4 do begin
          fx1 = use_big_delta_in[clu_num]/3.*x1^3-delta_c*(ALOG(1+x1)-x1/(1+x1))
          fx2 = use_big_delta_in[clu_num]/3.*x2^3-delta_c*(ALOG(1+x2)-x2/(1+x2))
        
          a = (fx2-fx1)/(x2-x1)
          b = fx1-a*x1
          x1 = x2
          x2 = -b/a
        endwhile
        x_in = x1

        ; caracteristic scale of the NFW profile in Mpc
        rs = r_in[clu_num]/x_in

        ; Compte x_out=r_out/rs
        x1 = 10.
        x2 = 20.
        while abs(x2-x1) gt 1d-4 do begin
          fx1 = use_big_delta_c[clu_num]/3.*x1^3-delta_c*(ALOG(1+x1)-x1/(1+x1))
          fx2 = use_big_delta_c[clu_num]/3.*x2^3-delta_c*(ALOG(1+x2)-x2/(1+x2))
        
          a = (fx2-fx1)/(x2-x1)
          b = fx1-a*x1
          x1 = x2
          x2 = -b/a
        endwhile
        x_out = x2

        ; compute r_out
        r_out = x_out*rs

        ; compute mass_out
        mass_out = (x_out/x_in)^3*use_big_delta_c[clu_num]/use_big_delta_in[clu_num]*mass_in[clu_num]
    
      ENDWHILE
   
      mass_vir[clu_num] = mass_out

    ENDFOR

    ;* then Mout computation *

    use_c_vir = 9.*(1.+z_in)^(-1)*(mass_vir/mass_star)^(-0.13)

    ; Compute the normalisation of the NFW profile
    delta_c = use_big_delta_c/3.*use_c_vir^3/(ALOG(1+use_c_vir)-use_c_vir/(1+use_c_vir))

    mass_out = 0.*mass_in
    r_out = 0.*r_in
    r_s = 0.*r_in

    FOR clu_num=0l, nclu-1 DO BEGIN

      ; Compute x_in=r_in/rs
      x1 = 10.
      x2 = 20.
      while abs(x2-x1) gt 1d-4 do begin
        fx1 = use_big_delta_in[clu_num]/3.*x1^3-delta_c[clu_num]*(ALOG(1+x1)-x1/(1+x1))
        fx2 = use_big_delta_in[clu_num]/3.*x2^3-delta_c[clu_num]*(ALOG(1+x2)-x2/(1+x2))
        
        a = (fx2-fx1)/(x2-x1)
        b = fx1-a*x1
        x1 = x2
        x2 = -b/a
      endwhile
      x_in = x1

      ; caracteristic scale of the NFW profile in Mpc
      rs = r_in[clu_num]/x_in
      r_s[clu_num] = rs

      ; Compute x_out=r_out/rs
      x1 = 10.
      x2 = 20.
      while abs(x2-x1) gt 1d-4 do begin
        fx1 = use_big_delta_out[clu_num]/3.*x1^3-delta_c[clu_num]*(ALOG(1+x1)-x1/(1+x1))
        fx2 = use_big_delta_out[clu_num]/3.*x2^3-delta_c[clu_num]*(ALOG(1+x2)-x2/(1+x2))
        
        a = (fx2-fx1)/(x2-x1)
        b = fx1-a*x1
        x1 = x2
        x2 = -b/a
      endwhile
      x_out = x2

      ; compute r_out
      r_out[clu_num] = x_out*rs

      ; compute mass_out
      mass_out[clu_num] = (x_out/x_in)^3*use_big_delta_out[clu_num]/use_big_delta_in[clu_num]*mass_in[clu_num]

    ENDFOR

  ENDIF
  
  IF KEYWORD_SET(gao) THEN BEGIN

     ; Hu & Kravtsov 2003 method with Gao et al. 2008 fit

     ztab = [0., 0.5, 1., 2., 3.]
     atab = [-0.138,-0.125,-0.092,-0.031,-0.004]
     btab = [2.646,2.372,1.891,0.985,0.577]

     a_in = INTERPOL(atab,ztab,z_in)
     b_in = INTERPOL(btab,ztab,z_in)

     ;* M200 computation first *

     delta_mass_out = 1d-3      ; 0.1% accuracy in mass_vir computation

     mass_200 = 0.*mass_in

     FOR clu_num=0l, nclu-1 DO BEGIN

        mass_out = mass_in[clu_num]
        
        old_mass_out = [-1.]
      
        WHILE ABS((old_mass_out-mass_out)/old_mass_out) GE delta_mass_out DO BEGIN
           ; Gao et al. 2008
           use_c_200 = 10^(a_in[clu_num]*ALOG10(mass_out*1d15)+b_in[clu_num])

           old_mass_out = mass_out

           ; Compute the normalisation of the NFW profile
           delta_c = 200/3.*use_c_200^3/(ALOG(1+use_c_200)-use_c_200/(1+use_c_200))
        
           ; Compte x_in=r_in/rs
           x1 = 10.
           x2 = 20.
           while abs(x2-x1) gt 1d-4 do begin
              fx1 = use_big_delta_in[clu_num]/3.*x1^3-delta_c*(ALOG(1+x1)-x1/(1+x1))
              fx2 = use_big_delta_in[clu_num]/3.*x2^3-delta_c*(ALOG(1+x2)-x2/(1+x2))
        
              a = (fx2-fx1)/(x2-x1)
              b = fx1-a*x1
              x1 = x2
              x2 = -b/a
           endwhile
           x_in = x1

           ; caracteristic scale of the NFW profile in Mpc
           rs = r_in[clu_num]/x_in

           ; Compte x_out=r_out/rs
           x1 = 10.
           x2 = 20.
           while abs(x2-x1) gt 1d-4 do begin
              fx1 = 200/3.*x1^3-delta_c*(ALOG(1+x1)-x1/(1+x1))
              fx2 = 200/3.*x2^3-delta_c*(ALOG(1+x2)-x2/(1+x2))
        
              a = (fx2-fx1)/(x2-x1)
              b = fx1-a*x1
              x1 = x2
              x2 = -b/a
           endwhile
           x_out = x2

           ; compute r_out
           r_out = x_out*rs

           ; compute mass_out
           mass_out = (x_out/x_in)^3*200./use_big_delta_in[clu_num]*mass_in[clu_num]
    
        ENDWHILE
   
        mass_200[clu_num] = mass_out
        
     ENDFOR
     
     ;* then Mout computation *
     use_c_200 = 10^(a_in*ALOG10(mass_200*1d15)+b_in)
     c_nfw = use_c_200

     ; Compute the normalisation of the NFW profile
     delta_c = 200/3.*use_c_200^3/(ALOG(1+use_c_200)-use_c_200/(1+use_c_200))

     mass_out = 0.*mass_in
     r_out = 0.*r_in
     r_s = 0.*r_in

     FOR clu_num=0l, nclu-1 DO BEGIN

        if use_big_delta_out[clu_num] eq 200. then begin
           mass_out[clu_num] = mass_200[clu_num]
        endif else begin

      ; Compute x_in=r_in/rs
           x1 = 10.
           x2 = 20.
           while abs(x2-x1) gt 1d-4 do begin
              fx1 = use_big_delta_in[clu_num]/3.*x1^3-delta_c[clu_num]*(ALOG(1+x1)-x1/(1+x1))
              fx2 = use_big_delta_in[clu_num]/3.*x2^3-delta_c[clu_num]*(ALOG(1+x2)-x2/(1+x2))
              
              a = (fx2-fx1)/(x2-x1)
              b = fx1-a*x1
              x1 = x2
              x2 = -b/a
           endwhile
           x_in = x1
           
           ; caracteristic scale of the NFW profile in Mpc
           rs = r_in[clu_num]/x_in
           r_s[clu_num] = rs

           ; Compute x_out=r_out/rs
           x1 = 10.
           x2 = 20.
           while abs(x2-x1) gt 1d-4 do begin
              fx1 = use_big_delta_out[clu_num]/3.*x1^3-delta_c[clu_num]*(ALOG(1+x1)-x1/(1+x1))
              fx2 = use_big_delta_out[clu_num]/3.*x2^3-delta_c[clu_num]*(ALOG(1+x2)-x2/(1+x2))
              
              a = (fx2-fx1)/(x2-x1)
              b = fx1-a*x1
              x1 = x2
              x2 = -b/a
           endwhile
           x_out = x2

           ; compute r_out
           r_out[clu_num] = x_out*rs

           ; compute mass_out
           mass_out[clu_num] = (x_out/x_in)^3*use_big_delta_out[clu_num]/$
                               use_big_delta_in[clu_num]*mass_in[clu_num]
        endelse
     ENDFOR
     
  ENDIF
  
ENDELSE

END
