;FUNCTION GET_CLUSTERING_BIAS, mass, redshift, cosmo=cosmo, cluster=cluster
;+
; NAME:
;    GET_CLUSTERING_BIAS
; ONE LINE HELP:
;    called in CREATE_SZ_SIMULATION to compute the clustering bias of the cluster catalog
; PURPOSE:
;    This procedure is called in CREATE_SZ_SIMULATION in order to compute
;    the clustering bias of the cluster catalog using Mo & White linear
;    bias (1996)
; CALLING SEQUENCE
;    result = GET_CLUSTERING_BIAS(mass, redshift, cosmo=cosmo, cluster=cluster)
; INPUT:
;    mass : an array of masses
; OUTPUT:
;    
;    
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
;    cluster :
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@apc.univ-paris7.fr
;
; PROCEDURES CALLED:
;    MATTIG, HUBBLE_AT_Z
;
; REVISION HISTORY
;    Written, JBM, April 2001
;    cluster keyword added, JBM, February 3 2006
;    Removed explicit call to isarray.pro (JHUAPL), Jacques Delabrouille, 01 feb 2007
;-

FUNCTION GET_CLUSTERING_BIAS, mass, redshift, cosmo=cosmo, cluster=cluster
; according to formula (20) of Mo and White (1996)
; enter mass in 10^15 solar masses

m = mass[UNIQ(mass,SORT(mass))]
nm = N_ELEMENTS(m)
granddelta1 = FLTARR(nm)

use_cosmo = cosmo
use_cluster = cluster

FOR i=0L, nm-1 DO BEGIN
  ma = m[i]
  PK_SIGMA, ma, gd, al, cosmo=use_cosmo, cluster=use_cluster
  granddelta1[i] = gd
ENDFOR

z = redshift[UNIQ(redshift,SORT(redshift))]
nz = N_ELEMENTS(z)
GROWTH_FACTOR, z, D_g, f, cosmo=cosmo
;delta_c = 1.686	;Einstein-De Sitter Universe only
delta_c = INTERPOL(cluster.small_delta_c_array, cluster.z_array, z)
delta1 = delta_c/D_g

nmass = N_ELEMENTS(mass)

result = FLTARR(nmass)

FOR i=0L, nmass-1 DO BEGIN
  whm = WHERE(m EQ mass[i])
  whz = WHERE(z EQ redshift[i])
  v1 = delta1[whz]/ $
        granddelta1[whm]
  result[i] = 1.+(v1^2-1)/delta1[whz] ;+0.6/delta1[whz]/(1.+0.9*v1^0.6)
ENDFOR

;IF NOT ISARRAY(mass) THEN result=result[0]
IF n_elements(mass) EQ 0 THEN BEGIN
  m_is_an_array = 0
ENDIF ELSE BEGIN
  s = size(mass)
  m_is_an_array = s[0] NE 0
END
IF NOT m_is_an_array THEN result=result[0]

RETURN, result

END
