;PRO GROWTH_FACTOR, z_array, D_g_array, f_array, cosmo=cosmo, slope=slope
;+
; NAME:
;  GROWTH_FACTOR
; ONE LINE HELP:
;  Compute the linear growth factor D_g and f = d(lnD_g)/d(lna) if
;  slope is set for a QCDM cosmology
; PURPOSE:
;  Compute the linear growth factor D_g and f = d(lnD_g)/d(lna) if slope is set
;  (a is the expansion parameter) for a QCDM cosmology
; CALLING SEQUENCE:
;  GROWTH_FACTOR, z_array, D_g_array, f_array, cosmo=cosmo, /slope
; INPUT:
;  z_array : an array of redshift values
;
; OUTPUT:
;  D_g_array : the array of D_g values corresponding to z_array values
;  f_array : the array of f=d(lnD_g)/d(lna) values if slope is set
;    
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters (cf. DEFINE_COSMO)
;  slope : if you want to compute f = d(lnD_g)/d(lna)
; EXAMPLES:
;  
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@apc.univ-paris7.fr
;
; PROCEDURES CALLED:
;  SUBFUNC_GROWTH_FACTOR, HUBBLE_AT_Z, DERIV_HUBBLE_AT_Z, DERIV
; REVISION HISTORY
;  Written from GROWTH_FACTOR, Jean-Baptiste Melin, July 1 2001
;  Heart (1977) cannot be used, bug fixed, JBM, August 26 2005
;  Fixed an apparent probem of numerical precision for some
;  realisations of z_array, Jacques Delabrouille, 5 May 2012
;-

;------------------------------------------------------------------------------
FUNCTION SUBFUNC_GROWTH_FACTOR_NO_QUINTESSENCE, a
;-------------------------------------------------
  COMMON GROWTH_FACTOR_COSMO, growth_factor_cosmo
  x = a/(100.*growth_factor_cosmo.h)* $
      HUBBLE_AT_Z(1./a-1., cosmo=growth_factor_cosmo)
  RETURN, x^(-3.)
END
;------------------------------------------------------------------------------
PRO GROWTH_FACTOR_NO_QUINTESSENCE, z_array, D_g_array, f_array, cosmo=cosmo, slope=slope
; taken from Lahav 1991 and Bartlett 1997

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : GROWTH_FACTOR, z_array, D_g_array, f_array'
        print,'Keywords : cosmo=cosmo, slope=slope'
     GOTO, closing
ENDIF

; giving the values of common variables
COMMON GROWTH_FACTOR_COSMO, growth_factor_cosmo
growth_factor_cosmo = cosmo

x_array = 1./(100.*growth_factor_cosmo.h*(1.+z_array))* $
          HUBBLE_AT_Z(z_array,cosmo=growth_factor_cosmo)
integrale_array = QROMO('subfunc_growth_factor_no_quintessence',0.,1./(1.+z_array))
; compute the normalisation factor for D_g
; In fact, D_g <-> D_g(z)/D_g(0)
norm_array = QROMO('subfunc_growth_factor_no_quintessence',0.,1.)

D_g_array = x_array*(1.+z_array)*integrale_array/norm_array

IF KEYWORD_SET(slope) THEN BEGIN
 z_sup = MAX(z_array)
 step_z = 0.01
 z = FINDGEN((z_sup+1)/step_z)*step_z
 lna = ALOG(1/(1.+z))
 x_array = 1./(100.*growth_factor_cosmo.h*(1.+z))* $
          HUBBLE_AT_Z(z,cosmo=growth_factor_cosmo)
 integrale_array = QROMO('subfunc_growth_factor_no_quintessence',0.,1./(1.+z))
 D_g = x_array*(1.+z)*integrale_array
 lnDg = ALOG(D_g)
 f_array = INTERPOL(DERIV(lna,lnDg),z,z_array)
ENDIF

closing:
END

;------------------------------------------------------------------------------
FUNCTION SUBFUNC_GROWTH_FACTOR, X, Y
;-------------------------------------
  COMMON GROWTH_FACTOR_COSMO, growth_factor_cosmo
  omega_m = growth_factor_cosmo.omega_m
  h = growth_factor_cosmo.h
  ea = HUBBLE_AT_Z(1./X-1., cosmo=growth_factor_cosmo)/(100.*h)
  dea = DERIV_HUBBLE_AT_Z(1./X-1., cosmo=growth_factor_cosmo)/(100.*h)
  ; JBM, see your notebook p. 112 or Percival 2005
  RETURN, [Y[1], 1./(ea*X)^2.*(3./2.*omega_m/X^3.*Y[0]-ea*X*(3*ea+X*dea)*Y[1])]
END
;------------------------------------------------------------------------------


;PRO GROWTH_FACTOR, z_array, D_g_array, f_array, cosmo=cosmo, slope=slope
PRO GROWTH_FACTOR, z_input, D_g_output, f_output, cosmo=cosmo, slope=slope   ; quick fix for numerical precision issues

  IF ( N_PARAMS() EQ 0 ) THEN BEGIN
     print,'CALL IS : GROWTH_FACTOR, z_input, D_g_output, f_output'
     print,'Keywords : cosmo=cosmo, slope=slope'
     GOTO, closing
  ENDIF

  IF cosmo.w_dark_energy EQ -1 THEN BEGIN
     GROWTH_FACTOR_NO_QUINTESSENCE, z_input, D_g_output, f_output, cosmo=cosmo, slope=slope
  ENDIF ELSE BEGIN
     ;; giving the values of common variables
     COMMON GROWTH_FACTOR_COSMO, growth_factor_cosmo
     growth_factor_cosmo = cosmo
     
     ;; ----> quick fix for numerical precision issues  
     z_array = FINDGEN(2001)/100
     WHILE MAX(z_input) GE MAX(z_array) DO BEGIN
        add2z = (FINDGEN(2000)+1.)/100 + MAX(z_array)
        z_array = [z_array,add2z]
     ENDWHILE
     ;; <----
     
     z = [0., z_array, 100000.]
     nz = N_ELEMENTS(z)
     a = 1./(1.+z)
     X_ini = a[nz-1]
     Y_ini = [1.,1.]
     
     Dg = DBLARR(nz)
     FOR i=0, nz-1 DO BEGIN
        H = a[i]-X_ini
        result = LSODE(Y_ini,X_ini,H,'subfunc_growth_factor')
        Dg[i]=result[0]
     ENDFOR
     
     D_g_array = Dg[1:nz-2]/Dg[0]
     
     ;; ----> quick fix for numerical precision issues  
     D_g_output = INTERPOL(D_g_array,z_array,z_input,/spline)   
     ;; <----
     
     IF KEYWORD_SET(slope) THEN BEGIN
        result_check = CHECK_MATH()
        f = DERIV(ALOG(a[1:nz-2]),ALOG(Dg[1:nz-2]))
        result_check = CHECK_MATH()
        IF result_check NE 0 THEN BEGIN
           MESSAGE, 'Numerical error in calculation of derivatives -- Please report bug to PSM coordinators'
        ENDIF
        ;; ----> quick fix for numerical precision issues  
        ;;f_array = f
        f_output = INTERPOL(f,z_array,z_input,/spline)
        ;; <----
     ENDIF
  ENDELSE

closing:
END
