;PRO INT_MASS_FUNCTION, z_list, mass_list, int_mf, $
;                       cosmo=cosmo, cluster=cluster, $
;                       mass_function=mass_function, $
;                       redshift=redshift
;+
; NAME:
;  INT_MASS_FUNCTION
; ONE LINE HELP:
;  give the value of integrated mass function on redshifts or masses
; PURPOSE:
;  give the value of integrated mass function on redshifts or masses (default : Press_schechter function)
; CALLING SEQUENCE:
;  INT_MASS_FUNCTION, z_list, mass_list, int_mf, $
;                     cosmo=cosmo, cluster=cluster, $
;                     mass_function=mass_function, $
;                     redshift=redshift
; INPUT:
;  
; OUTPUT:
;  z_list : the redshift list
;  mass_list : the list of masses
;  int_mf : the value of the integrated mass function on redshifts or on masses
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;  mass_function : the choosen mass function 
;                   (e.g. Sheth-Tormen, Jenkins or Press-Schechter (default))
;  redshift : set this keyword to integrate on redshifts
;             if not set, the mass function will be integrated on masses
; EXAMPLES:
;
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  MF_DENSITY
;  
; REVISION HISTORY
;  Written, JBM, September 11 2001 
;  Keyword cosmo introduced, JBM, Sept 27 2001
;  cluster keyword added, JBM, Feb 2 2006
;-

PRO INT_MASS_FUNCTION, z_list, mass_list, int_mf, $
                       cosmo=cosmo, cluster=cluster, $
                       mass_function=mass_function, $
                       redshift=redshift

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
 PRINT,'  CALL IS : INT_MASS_FUNCTION, z_list, mass_list, int_mf'
 PRINT,'  Keywords :'
 PRINT,'  cosmo=cosmo, cluster=cluster'
 PRINT,'  mass_function=mass_function'
 PRINT,'  redshift=redshift'
 GOTO, closing
ENDIF

IF KEYWORD_SET(redshift) THEN BEGIN
  M_inf=0.1
  ; choose the step in log10 for M (e.g. 200. steps between 10^43 and 10^44)
  step_log10 = 20.
  ; compute the step in ln for M
  delta_lnM = ALOG(10)/step_log10
  ; max value for M in M_array
  M_sup = 10. ; in 10^15 solar mass units
  ; compute the number of elements for M_array
  log10_m = (FINDGEN(CEIL((ALOG10(M_sup)-ALOG10(M_inf))*step_log10)+1)/step_log10)+ALOG10(M_inf)
  mass_list = EXP(ALOG(10)*log10_m)
  nb_elts_mass_list = N_ELEMENTS(mass_list)
  
  MF_DENSITY, zl, mass_list, mfd, $
  cosmo=cosmo, cluster=cluster, mass_function=mass_function
  
  z_list = zl
  int_mf = FLTARR(nb_elts_mass_list)
  nb_elts_z_list = N_ELEMENTS(z_list)
  delta_z = z_list[1:nb_elts_z_list-1] - z_list[0:nb_elts_z_list-2]
  delta_z = [delta_z,delta_z[nb_elts_z_list-2]]
  
  FOR i=0,nb_elts_z_list-1 DO BEGIN
    int_mf = int_mf+mfd[i,*]*delta_z[i]
  ENDFOR  
ENDIF ELSE BEGIN
  z_sup= 4.
  z_inf = 0.01
  step_z = 0.05
  z_list = (FINDGEN(CEIL((z_sup-z_inf)/step_z)+1)*step_z)+z_inf
  nb_elts_z_list = N_ELEMENTS(z_list)
  
  MF_DENSITY, z_list, ml, mfd, $
  cosmo=cosmo, cluster=cluster, mass_function=mass_function
  
  mass_list = ml
  nb_elts_mass_list = N_ELEMENTS(mass_list)
  int_mf = FLTARR(nb_elts_z_list)
  delta_lnM = ALOG(mass_list[1:nb_elts_mass_list-1])-ALOG(mass_list[0:nb_elts_mass_list-2]) 
  delta_lnM = [delta_lnM,delta_lnM[nb_elts_mass_list-2]]
  
  FOR i=0,nb_elts_mass_list-1 DO BEGIN
    int_mf = int_mf+mfd[*,i]*delta_lnM[i]
  ENDFOR
ENDELSE
closing:
END
