;PRO MF_DENSITY, z_list, mass_list, mf_dens, $
;                cosmo=cosmo, cluster=cluster, $
;                mass_function=mass_function
;+
; NAME:
;  MF_DENSITY
; ONE LINE HELP:
;  Compute the cluster density dN/(dz*dlnM) given by a mass_function for 1 square degree.
; PURPOSE:
;  Compute the cluster density dN/(dz*dlnM) given by a mass_function
;  with dOmega = 1 squaredegree (default : Press_schechter function)
; CALLING SEQUENCE:
;          MF_DENSITY, z_list, mass_list, mf_dens, $
;                      cosmo=cosmo, cluster=cluster, $
;                      mass_function=mass_function
; INPUT:
;  z_list : the redshift list
;  mass_list : the list of masses
; OUTPUT:
;  mf_dens : the mass function density as a function of z and mass
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;  mass_function : the choosen mass function 
;                   (e.g. Sheth-Tormen, Jenkins or Press-Schechter (default))
;  
; EXAMPLES:
;
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  GROWTH_FACTOR, COVOLUME,
;  PK_SIGMA
; REVISION HISTORY
;  Written, JBM, September 11 2001 
;  Optimised, JBM,  Sept 24 2001
;  Keyword cosmo introduced, JBM, Sept 27 2001
;  cluster keyword added, JBM, Feb 2 2006
;-

PRO MF_DENSITY, z_list, mass_list, mf_dens, $
                cosmo=cosmo, cluster=cluster, $
                mass_function=mass_function

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
 PRINT,'  CALL IS : MF_DENSITY, z_list, mass_list, mf_dens'
 PRINT,'  Keywords : cosmo=cosmo, cluster=cluster, mass_function=mass_function'
 GOTO, closing
ENDIF

; default mass_function
IF NOT KEYWORD_SET(mass_function) THEN BEGIN
  PRINT, 'Using the Press-Schechter function...'
  mass_function = 'press_schechter'
ENDIF

IF NOT DEFINED(z_list) THEN BEGIN
  ; choose the max and min values for z
  z_sup = 5.
  z_inf = 0.001
  ; chose the step for z
  delta_z = 0.01
  ; z_list building
  z_list = (FINDGEN(CEIL((z_sup-z_inf)/delta_z)+1)*delta_z)+z_inf
ENDIF
n_elts_z_list = N_ELEMENTS(z_list)


IF NOT DEFINED(mass_list) THEN BEGIN
  M_inf=0.1
  ; choose the step in log10 for M (e.g. 100. steps between 10^43 and 10^44)
  step_log10 = 200.
  ; compute the step in ln for M
  delta_lnM = ALOG(10)/step_log10
  ; max value for M in M_array
  M_sup = 10. ; in 10^15 solar mass units
  ; compute the number of elements for M_array
  log10_m = (FINDGEN(CEIL((ALOG10(M_sup)-ALOG10(M_inf))*step_log10)+1)/step_log10)+ALOG10(M_inf)
  mass_list = EXP(ALOG(10)*log10_m)
ENDIF

n_elts_mass_list = N_ELEMENTS(mass_list)

; compute the critical linear over-density
delta_c_array = FLTARR(n_elts_z_list)
;delta_c_array[*] = 1.68 ; very small variations with z and the cosmology
delta_c_array = INTERPOL(cluster.small_delta_c_array, cluster.z_array, z_list)

; compute the linear growth factor
GROWTH_FACTOR, z_list, D_g_array, f_array, cosmo=cosmo

;compute the covolumes
cov_array = COVOLUME(z_list, cosmo=cosmo)

; compute sigma and the slope for the M values of mass_list
sigma_slope_array = DBLARR(2,n_elts_mass_list)
mass_func = DBLARR(n_elts_z_list, n_elts_mass_list)
FOR i=0,n_elts_mass_list-1 DO BEGIN
  PK_SIGMA, mass_list[i], sigma, alpha, cosmo=cosmo, cluster=cluster, /slope
  sigma_slope_array[0,i] = sigma
  sigma_slope_array[1,i] = alpha
  ; compute the mass function for z and mass
  mass_func[*,i] = CALL_FUNCTION(mass_function,delta_c_array,sigma_slope_array[0,i]*D_g_array, z_list, cosmo=cosmo)
ENDFOR

; compute rho0
Mpc2m = 3.0856775807d+22 ; in m
rho0 = Mpc2m^3*cosmo.omega_m*(cosmo.h)^2*1.88*1d-26 ; in kg/Mpc^3

M_sun = 1.9889*1d30 ; in kg
solid_angle = (!dtor)^2
fact = rho0/(1d15*M_sun)*solid_angle

mf_dens = DBLARR(n_elts_z_list,n_elts_mass_list)

mf_limit=1d-50

; create the catalog
FOR k = 0, n_elts_z_list-1 DO BEGIN   ; loop on redshift
    i=0
    lim=10.
    WHILE (i LT n_elts_mass_list) AND (lim GT mf_limit) DO BEGIN ; loop on cluster masses
    ; compute the average number of clusters using the mass function
    lim = mass_func[k,i]*fact/mass_list[i]* $
                   sigma_slope_array[1,i]*cov_array[k]
    mf_dens[k,i] = lim
    i = i+1
   ENDWHILE
ENDFOR

closing:
END
