;FUNCTION PK_SIGMA_VELOCITY, scale, z_array, cosmo=cosmo, cluster=cluster
;
;+
; NAME:
;    PK_SIGMA_VELOCITY
; ONE LINE HELP:
;    Compute integrated P(k) for 3D velocity field.
; PURPOSE:
;    Given the scale "scale" (in Mpc), the redshifts z_array, 
;    the bias, h, omega_m and lambda, this routine returns
;    the 3 dimensional rms velocity for z_array in km/s,
;    i.e. a(z)*H(z)*f(z)*D_g(z)*SQRT(1./(2*!pi^2)*int(P(k)*W(k*scale)^2))
; 
; CALLING SEQUENCE:
;    sig_vec = PK_SIGMA_VELOCITY(scale, z_array, cosmo=cosmo, cluster=cluster)
; 
; INPUT:
;    scale : scale in Mpc
; OUTPUT:
;    sig_vec  : the 3D rms velocity for the choosen scale at redshifts z_array in km/s
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
;    cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
; EXAMPLES:
;
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;    COSMO_PK
;    GROWTH_FACTOR
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, September 7 2001
;    Keyword cosmo introduced, JBM, Sept 27 2001
;    cluster keyword introduced, JBM, Feb 6 2006
;    silent keyword removed in call of cosmo_pk, JBM, Jul 28 2009
;-

;------------------------------------------------------------------------------
function PK_SIGMA_VELOCITY_I_VEC, x
;----------------------------------
common PK_SIGMA_VELOCITY_COSMO, pk_sigma_velocity_cosmo, pk_sigma_velocity_cluster
common PK_SIGMA_VELOCITY_SCALE, pk_sigma_velocity_r

k = x/pk_sigma_velocity_r
Pk = COSMO_PK(k, cosmo=pk_sigma_velocity_cosmo, cluster=pk_sigma_velocity_cluster, /nonorm)

if (x lt 0.001) then Wk = 1. else Wk  = 3./x^2*(sin(x)/x - cos(x))

return, Pk*Wk^2
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


FUNCTION PK_SIGMA_VELOCITY, scale, z_array, cosmo=cosmo, cluster=cluster

common PK_SIGMA_VELOCITY_COSMO, pk_sigma_velocity_cosmo, pk_sigma_velocity_cluster
common PK_SIGMA_VELOCITY_SCALE, pk_sigma_velocity_r

pk_sigma_velocity_cosmo = cosmo
pk_sigma_velocity_cluster = cluster

x1 = 0.0                       ; roots of Wth(x)
x2 = 4.496
x3 = 7.723
x4 = 10.902
x5 = 14.066
x6 = 17.221
x7 = 20.371

; compute the spectrum normalization 
norm = COSMO_PK(1., cosmo = pk_sigma_velocity_cosmo, cluster=pk_sigma_velocity_cluster) / COSMO_PK(1., cosmo = pk_sigma_velocity_cosmo, cluster=pk_sigma_velocity_cluster, /nonorm)
pk_sigma_velocity_r  = scale

vec1 = qromo('PK_SIGMA_VELOCITY_I_VEC',x1,x2)
vec2 = qromo('PK_SIGMA_VELOCITY_I_VEC',x2,x3)
vec3 = qromo('PK_SIGMA_VELOCITY_I_VEC',x3,x4)
vec4 = qromo('PK_SIGMA_VELOCITY_I_VEC',x4,x5)
vec5 = qromo('PK_SIGMA_VELOCITY_I_VEC',x5,x6)
vec6 = qromo('PK_SIGMA_VELOCITY_I_VEC',x6,x7)
if (vec6/vec1 gt 0.1) then stop, 'vec not converging'
vec = norm/(2.*!pi^2)*(1./pk_sigma_velocity_r)*(vec1+vec2+vec3+vec4 $
        +vec5+vec6)
GROWTH_FACTOR, z_array, D_g_array, f_array, cosmo=cosmo, /slope
RETURN, HUBBLE_AT_Z(z_array, cosmo=cosmo)/(1.+z_array)* $
        f_array*SQRT(vec)*D_g_array ; in km/s

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
