;FUNCTION PK_VELOCITY_AUTOCORRELATION, scale, z_array, cosmo=cosmo, cluster=cluster, smooth_size=smooth_size
;
;+
; NAME:
;    PK_VELOCITY_AUTOCORRELATION
; ONE LINE HELP:
;    Compute integrated P(k) for scalar xsi_v.
; PURPOSE:
;    Given the scales "scale" (in Mpc) and the corresponding redshifts z_array, 
;    this routine returns the scalar velocity autocorrelation function,
;    i.e. a(z)^2*H(z)^2*f(z)^2*D_g(z)^2*SQRT(1./(2*!pi^2)*int(P(k)*SIN(kr)/(kr)))
;    BE CAREFUL : scale and z_array must be arrays of same size !
; 
; CALLING SEQUENCE:
;    xsi_v = PK_VELOCITY_AUTOCORRELATION(scale, z_array, cosmo=cosmo, cluster=cluster, smooth_size=smooth_size)
; 
; INPUT:
;    scale : scales in Mpc
;    z_array : the corresponding reshifts
; OUTPUT:
;    xsi_v  : the velocity autocorrelation function for the choosen scales and redshifts.
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
;    cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;    smooth_size : the smoothing scale for the powe spectrum
; EXAMPLES:
;
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;    COSMO_PK
;    GROWTH_FACTOR
;    HUBBLE_AT_Z
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, February 2002
;    cluster keyword added, JBM, Feb 6 2006 
;-

;-----------------------------------------------------------------------------------------------------------------
FUNCTION ZERO_BESSEL, x_max, step_x
;----------------------------------

x = (DINDGEN(x_max/step_x)+1)*step_x

v = sin(x)/x
whv = WHERE(v GE 0)
y = x[whv]+step_x
w = sin(y)/y
whw = WHERE(w LT 0, nwhw)
z1 = y[whw]-step_x


whv = WHERE(v LE 0)
y = x[whv]+step_x
w = sin(y)/y
whw = WHERE(w GT 0, nwhw)

z2 = y[whw]-step_x

z = [z1, z2]
z = z[UNIQ(z, SORT(z))]

RETURN, z

END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
function PK_VELOCITY_AUTOCORRELATION_I_VEC, x
;--------------------------------------------
common PK_VELOCITY_AUTOCORRELATION_COSMO, pk_velocity_autocorrelation_cosmo, pk_velocity_autocorrelation_cluster
common PK_VELOCITY_AUTOCORRELATION_SCALE, pk_velocity_autocorrelation_r, pk_velocity_autocorrelation_s 

k = x/pk_velocity_autocorrelation_r
Pk = COSMO_PK(k, cosmo=pk_velocity_autocorrelation_cosmo, cluster=pk_velocity_autocorrelation_cluster, /silent, /nonorm)
;Pk = EFSPK(k, cosmo=pk_velocity_autocorrelation_cosmo, cluster=pk_velocity_autocorrelation_cluster, /nonorm)

y = k*pk_velocity_autocorrelation_s
if (y lt 0.001) then Wk = 1. else Wk  = 3./y^2*(sin(y)/y - cos(y))

j0 = SIN(x)/x

return, Pk*j0*Wk^2.
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


FUNCTION PK_VELOCITY_AUTOCORRELATION, scale, z_array, cosmo=cosmo, cluster=cluster, smooth_size=smooth_size

common PK_VELOCITY_AUTOCORRELATION_COSMO, pk_velocity_autocorrelation_cosmo, pk_velocity_autocorrelation_cluster
common PK_VELOCITY_AUTOCORRELATION_SCALE, pk_velocity_autocorrelation_r, pk_velocity_autocorrelation_s 

IF NOT DEFINED(smooth_size) THEN smooth_size=0.

pk_velocity_autocorrelation_s = smooth_size
pk_velocity_autocorrelation_cosmo = cosmo
pk_velocity_autocorrelation_cluster = cluster

; compute the spectrum normalization 
norm = COSMO_PK(1., cosmo = pk_velocity_autocorrelation_cosmo, cluster=pk_velocity_autocorrelation_cluster, $
                /silent) / COSMO_PK(1., cosmo = pk_velocity_autocorrelation_cosmo, cluster=pk_velocity_autocorrelation_cluster, /silent, /nonorm)
;norm = EFSPK(1., cosmo = pk_velocity_autocorrelation_cosmo, cluster=pk_velocity_autocorrelation_cluster) / $
;       EFSPK(1., cosmo = pk_velocity_autocorrelation_cosmo, cluster=pk_velocity_autocorrelation_cluster, /nonorm)

x_max = 3000. ; OK for scale <= 1000 Mpc
step_x = 0.01

x = ZERO_BESSEL(x_max, step_x)
x = [0., x]
nx = N_ELEMENTS(x)
vec = DBLARR(nx-1)

scale_list = scale[UNIQ(scale, SORT(scale))]
nscale = N_ELEMENTS(scale_list)
tot_vec = DBLARR(nscale)
integ = DBLARR(N_ELEMENTS(scale))

FOR j=0, nscale-1 DO BEGIN
  vec[*] = 0
  pk_velocity_autocorrelation_r  = scale_list[j]
  vec[0] = QROMO('PK_VELOCITY_AUTOCORRELATION_I_VEC',x[0],x[1])
  i = 0
  WHILE (i LE nx-3) AND (ABS(vec[i]/TOTAL(vec[0:i])) GE 0.001) DO BEGIN
    i = i+1
    vec[i] = QROMB('PK_VELOCITY_AUTOCORRELATION_I_VEC',x[i],x[i+1], /DOUBLE)
  ENDWHILE
  tot_vec[j] = TOTAL(vec) 
  wh = WHERE(pk_velocity_autocorrelation_r EQ scale)
  integ[wh] = norm/(2.*!pi^2)*(1./pk_velocity_autocorrelation_r)*tot_vec[j]
ENDFOR

GROWTH_FACTOR, z_array, D_g_array, f_array, cosmo=cosmo, /slope
RETURN, (HUBBLE_AT_Z(z_array, cosmo=cosmo)/(1.+z_array)* $
        f_array)^2.*integ*D_g_array^2. ; in km^2s^-2

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
