;FUNCTION SDELTA_C, z_array, cosmo=cosmo
;+
; NAME:
;    SDELTA_C
; ONE LINE HELP:
;    Compute the value of the linear critical density contrast delta_c at redshift z (quintessence model).
; PURPOSE:
;    Compute the value of the linear critical density contrast delta_c at
;    redshift z as fast as possible (quintessence model)
; CALLING SEQUENCE:
;    s_delta_c = SDELTA_C(z_array, cosmo=cosmo)
; INPUT:
;    z_array : an array of redshift values
;
; OUTPUT:
;    s_delta_c : the array of linear critical density contrast corresponding to z_array
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jean-Baptiste Melin
;    jean-baptiste.melin@cea.fr
;
; PROCEDURES CALLED:
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, March 14 2006
;    Merged from SMALL_BIG_DELTA_C and SMALL_BIG_DELTA_C_Q, Matthieu Roman, April 2012
;-

FUNCTION SDELTA_C, z_array, cosmo=cosmo

; Approximation taken from Kitayama & Suto 1996, ApJ 469

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
   print,'CALL IS : delta_c = SDELTA_C(z_array)'
   print,'Keyword : cosmo=cosmo'
   GOTO, closing
ENDIF

;small_delta_c_array = REPLICATE(1.68d0,N_ELEMENTS(z_array))

omega_m = cosmo.omega_m
omega_q = cosmo.omega_de
h = cosmo.h

IF omega_m EQ 1 AND omega_q EQ 0 THEN BEGIN
  small_delta_c_array = REPLICATE(3d0*(12d0*!dpi)^(2d0/3)/20d0, N_ELEMENTS(z_array))
ENDIF ELSE BEGIN
  IF omega_m LT 1 AND omega_q EQ 0 THEN BEGIN
     omega_m_z = omega_m*(1+z_array)^3/(HUBBLE_AT_Z(z_array,cosmo=cosmo)/100./h)^2
     xz = 2./omega_m_z-1
     eta_f_z = ALOG(xz+SQRT(xz^2-1))
     small_delta_c_array = 3d0/2d0*(3d0*SINH(eta_f_z)*(SINH(eta_f_z)-eta_f_z)/(COSH(eta_f_z)-1)^2-2)* $
                           (1+(2*!dpi/(SINH(eta_f_z)-eta_f_z))^(2d0/3))
  ENDIF ELSE BEGIN
     nz = N_ELEMENTS(z_array)
     delta_eds = 3d0*(12d0*!dpi)^(2d0/3)/20d0
     small_delta_c_array = REPLICATE(delta_eds, nz)
  ENDELSE
ENDELSE

RETURN, small_delta_c_array

closing:
END
