; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Provides a wrapper to the FIRB simulation code
;
; @categories pipe
;
; @keyword test {in}{optional}{type=boolean} If set, no computation
; done, just check if the set of parameters are ok
; @keyword debug {in}{optional}{type=boolean} If set no catch on error
; done, subdirectory components/firb is not removed when an error occurs
;
; @uses 
;
; @history <p>22/09/2009: First version, Jacques Delabrouille </p>
; @copyright Jacques Delabrouille, CNRS
;-

PRO FIRB_PIPE, test=test, debug=debug

  COMMON psm, param
  COMMON sky, model

;; Define parameters for convenience
;;-----------------------------------
  gp = param.global
  firbp = model.firb
  names = param.names
  runp = param.run
  pixp = model.mapping.pix

;; Error handling : remove ps component directory when catching an error
;;----------------------------------------------------------------------
  componentdir = GETPSMDIR('firb') 
  IF ((NOT KEYWORD_SET(debug))) THEN BEGIN 
     CATCH, error_status
     IF error_status NE 0 THEN BEGIN
        PRINT, 'An error was encoutered while computing FIRB maps.'
        PRINT, 'Removing subdirectory ' +componentdir
        SPAWN, 'rm -r '+componentdir
        CATCH, /CANCEL
        MESSAGE, !ERR_STRING
     ENDIF
  ENDIF

  IF ((NOT KEYWORD_SET(test)) AND (STRLOWCASE(firbp.model) NE 'no_firb')) THEN BEGIN
     firbdirexists = FILE_TEST(componentdir, /directory)
     IF NOT firbdirexists THEN SPAWN, 'mkdir '+componentdir
  ENDIF

;; Generation of the FIRB model
;;------------------------------
  
  ;; IR BACKGROUND
  ;;---------------
  IF STRLOWCASE(firbp.model) NE 'no_firb' THEN BEGIN
     
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "MODELING INFRARED BACKGROUND"      
     SET_PSM_SEEDCHAIN, 'firb-pipe'

     ;; Create the directory for the FIRB model
     firbdir = GETPSMDIR('firb')
     dir_exists = FILE_TEST(firbdir)
     IF dir_exists THEN BEGIN
        ;; check that it is a directory
        dir_exists = FILE_TEST(firbdir,/directory)
        IF dir_exists THEN BEGIN
           ;; Empty directory if needed
           CASE STRLOWCASE(runp.clear_all) OF 
              'yes': BEGIN
                 fitsfiles = FILE_SEARCH(firbdir+'*.fits')
                 FOR i=0, N_ELEMENTS(fitsfiles)-1 DO BEGIN
                    IF FILE_BASENAME(fitsfiles(i)) NE '' THEN FILE_DELETE, fitsfiles(i), /quiet
                 ENDFOR
              END
              'no': 
           ENDCASE
        ENDIF ELSE BEGIN
           MESSAGE, firbdir + ' exists and is not a directory'
        ENDELSE
     ENDIF ELSE BEGIN
        SPAWN, 'mkdir '+firbdir
     ENDELSE
     
     ;; Various options for FIRB
     ;;--------------------------
     CASE STRLOWCASE(firbp.model) OF

        ;; Model of Joaquin Gonzalez-Nuevo et al. (2005)
        ;;-----------------------------------------------
        'jgn2005': BEGIN
           IF KEYWORD_SET(test) THEN RETURN
           
           IF gp.fields NE 'T' THEN polarised = 1 ELSE polarised = 0 
           
           ;; Define list of useful frequencies
           ;;-----------------------------------
           
           ;; Define a set of frequencies at which the FIRB is written
           ;; The density (in log scale) is bigger in the 100-1000 GHz range, where the FIRB is likely to be observed by Planck
           ;; freq = [1.,5.,25.,100.,250.,500.,1000.,3000.]*1e9
           
           freq = [30.,44.,70.,100.,143.,217.,353.,545.,857.]*1e9
           
           ;; Produce a FIRB map at all requested frequencies
           ;;-------------------------------------------------
           Slim = 1e32  ;; very high to avoid writing a catalog.
           i = 0

           ;; Generate a data ID number
           PSM_ID_KEYGEN, firb_key

          ;; Do the FIRB maps
           MESSAGE, 'WARNING: Update the writing of the FIRB maps', /info
           f = FLOAT(freq/1e9) ;in GHz

           ;; Generate a new map and catalogue if i=0 and if a new FIRB is requested
           IF ((firbp.new_firb EQ 'yes') AND (i EQ 0)) THEN $
              PSM_PS_IRB, pixp.nside, f, Slim, /new_ls_cat, /new_sph_sim $
           ELSE $
              PSM_PS_IRB, pixp.nside, f, Slim
                                    
           ;; Bibliography
           ;;--------------
           PSM_ADD2CITE, 'Cosmic Infrared Background anisotropies are simulated with the method of Gonzalez-Nuevo, Toffolatti and Argueso (2005).'
           PSM_ADD2BIB, '2005ApJ...621....1G'

        END

        'pep_cl': BEGIN
           ;; Get CIB Cl
           ;;------------
           clpep = PEP_FIRB_CL(lmax=SKY_LMAX(), correl=correl, poisson=poisson)
           freq = [100e9,143e9,217e9,353e9,545e9,857e9,!phyconst.c/160e-6,!phyconst.c/100e-6]

           ;; Make maps at reference frequencies and emission laws
           ;;------------------------------------------------------
           STOP
           
           PSM_PEP_FIRB, pixp.nside, freq, clpep

           ;; Bibliography
           ;;--------------
           PSM_ADD2CITE, 'Cosmic Infrared Background anisotropies are simulated on the basis of the CIB spectrum measured by the Planck collaboration (2011).'
           PSM_ADD2BIB, '2011A&A...536A..18P'

        END
    
        'castex2012': BEGIN
           FIRB_GAL_TO_SHELLS, cosmo=model.cosmo
        END

        'pep-castex2012': BEGIN
           MAKE_FIRB_MIX
	END

     ENDCASE

  ENDIF

closing: RETURN
END
