; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Reads the specific parameters used for the generation of the Point Sources in the PSM
;
; @categories pipe
;   
; @param table {in}{required}{type=strarr(*,2)} vector of couple (keyword, value)
; @param ps_seed {in}{required}{type=long} point source seed value
; @param psp {out}{required}{type=struct} Point Source code specific parameters
; @keyword carefulness {in}{optional}{type=integer} level of carefulness in handling missing or repeated keywords in the table. 
;
; @uses fill_struct
; 
; @history <p>06/02/2009: First version, Jacques Delabrouille </p>
; @history <p>28/04/2009: Removed unnecessary intermediate 'model' structure, Jacques Delabrouille </p>
; @history <p>29/04/2009: Directory and file names for ps included here now, Jacques Delabrouille </p>
; @history <p>19/05/2009: Bug in the no_ps model fixed, Jean-Baptiste Melin </p>
; @history <p>22/05/2009: Keyword for carefulness added, improved handling of missing or repeated keywords, Jacques Delabrouille</p>
; @history <p>18/09/2009: Now is a function + restore keyword added, Jacques Delabrouille</p>
; @copyright <p>Jacques Delabrouille, CNRS, 2009</p>
;-
FUNCTION READ_FIRB_PARAM, table, carefulness=carefulness, restore=restore

  ;; File where parameters will be written
  ;;---------------------------------------
  file = GETPSMDIR('private',/create)+'firb_parameters.sav'

  which_firb = {firb_model: 'no_firb'}  

  IF NOT KEYWORD_SET(restore) THEN BEGIN
     FILL_STRUCT, table, which_firb, tags=['FIRB_MODEL'], carefulness=carefulness
     
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;; FIRB pipeline parameters
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

     CASE which_firb.firb_model OF

        ;; Model of Joaquin Gonzalez-Nuevo et al. (2005)
        ;;-----------------------------------------------
        'jgn2005': BEGIN
           firb_files={ $
                      cl_sph_input:FIX_SEPARATOR(!PSMROOT)+'data/ancillary/models/firb/JGN-2005/cl_sph_input.fits', $
                      lapi_353GHz:FIX_SEPARATOR(!PSMROOT)+'data/ancillary/models/firb/JGN-2005/lapi_353GHz.dat', $
                      sph_extrapol_table_353GHz:FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/models/firb/JGN-2005/sph_extrapol_table_353GHz.dat', $
                      lensing_alpha:FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/models/firb/JGN-2005/lensing_alpha.dat', $
                      intermediate_lensing_cat:FIX_SEPARATOR(!PSMROOT) + 'Data/ancillary/models/firb/JGN-2005/intermediate_lensing_cat.fits', $
                      Tmap353:FIX_SEPARATOR(!PSMROOT)+'Data/ancillary/models/firb/JGN-2005/Tmap353GHz.fits' $
                      }
           firbp = { $
                   model:which_firb.firb_model, $         ;
                   new_firb:'no', $                       ;
                   save_firb_alm:'yes', $                 ;
                   save_firb_map:'yes', $                 ;
                   firb_files:firb_files $                ;
                   }
           tags = ['NEW_FIRB','SAVE_FIRB_ALM','SAVE_FIRB_MAP']
           FILL_STRUCT, table, firbp, tags=tags, carefulness=carefulness
           CASE firbp.new_firb OF
              'yes': BEGIN
                 firbp.firb_files.Tmap353 = PSM_TMPFILENAME()
                 firbp.intermediate_lensing_cat = PSM_TMPFILENAME()
              END
              'no': ; do nothing
           ENDCASE
        END

        ;; Model of Guillaume Castex et al. (2012)
        ;;-----------------------------------------
        'castex2012': BEGIN
           firbp = { $
                   model:which_firb.firb_model, $         ;
                   save_firb_alm:'yes', $                 ;
                   save_firb_map:'yes' $                  ;
                   }
           tags = ['SAVE_FIRB_ALM','SAVE_FIRB_MAP']
           FILL_STRUCT, table, firbp, tags=tags, carefulness=carefulness
        END

        ;; Model from multivariate Cl matching Planck Early Paper
        ;;--------------------------------------------------------
        'pep_cl': BEGIN
           firbp = { $
                   model:which_firb.firb_model, $         ;
                   save_firb_alm:'yes', $                 ;
                   save_firb_map:'yes' $                  ;
                   }
           tags = ['SAVE_FIRB_ALM','SAVE_FIRB_MAP']
           FILL_STRUCT, table, firbp, tags=tags, carefulness=carefulness
        END

        ;; Model of Guillaume Castex et al. (2012) modified using PEP
        ;;-----------------------------------------------------------
        'pep-castex2012': BEGIN
           firbp = { $
                   model:which_firb.firb_model, $         ;
                   add_firb_component:'no',$              ;
                   add_firb_percent_power:FLTARR(6),$     ;
                   save_firb_alm:'yes', $                 ;
                   save_firb_map:'yes' $                  ;
                   }
           tags = ['SAVE_FIRB_ALM','SAVE_FIRB_MAP','ADD_FIRB_COMPONENT','ADD_FIRB_PERCENT_POWER']
           FILL_STRUCT, table, firbp, tags=tags, carefulness=carefulness
        END
        ;; No Far Infrared Background
        ;;----------------------------
        'no_firb': BEGIN
           firbp = { model:which_firb.firb_model}
        END

        ELSE: MESSAGE, 'FAR INFRARED BACKGROUND MODEL NOT ALLOWED'

     ENDCASE

     ;; Write the parameters
     ;;----------------------
     SAVE, filename=file, firbp
  
  ENDIF ELSE BEGIN
     RESTORE, filename=file
  ENDELSE
     
  RETURN, firbp

END
