; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jacques Delabrouille
;   Marc-Antoine Miville-Deschenes
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ADD_SMALLSCALE
;
; PURPOSE:
;	Function to add small scale fluctuations to a low resolution template of
;	galactic emission.
;
; CATEGORY:
;	PSM pipeline; sky model (Galaxy)
;
; CALLING SEQUENCE:
;	Result =	ADD_SMALLSCALE(Inputmap, Struct, SEED=, PLANCKRES=, MASK=, $
;			ALPHA=, BETA=, GAMMA=, RESOLUTION=, NL=, ORDERING=, SCALING=)
;
; INPUTS:
;	Inputmap:	Input lower resolution map to which small
;			scale structure is to be added.
;
; OPTIONAL INPUTS:
;	Struct:		Input structure with tags including
;			RESOLUTION, ALPHA, BETA, GAMMA, that can be
;			used instead of the corresponding keywords.
;	
; KEYWORD PARAMETERS:
;	SEED:		The seed to be used for random number generation. 
;			If not specified, a call to the GETSEED function
;			is made (by the GENERATE_X_LM procedure).
;
;	PLANCKRES:	The target resolution of the output map with
;			small scale fluctuations added. If not
;			defined, it is read from the !sky_model
;			structure (if defined), else it is set to 5
;			arcmin.
;
;	MASK:		A Healpix map of same size and ordering as the output.
;			If set to a non-zero value, small scale
;			fluctuations are added only where MASK=1.
;	ALPHA:		
;	BETA:		
;	GAMMA:		
;	RESOLUTION:	Resolution of the input map.	
;
;	NL:		Number of harmonic ell modes (=lmax+1).	
;
;	ORDERING:	Ordering of the input HEALPix map (default=NESTED).
;	
;	SCALING:	A Healpix map of same size and ordering as the output.
;			This is the map used to scale the added
;			Gaussian fluctuations to match the local level
;			of emission. By default, the input map itself
;			is used (positive values only).
;
; OUTPUTS:
;	Result is the map with small scale fluctuations added.
;
; COMMON BLOCKS:
;	None
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Generate a Gaussian random field with appropriate power
;	spectrum (set by the difference in resolutions of the input
;	and output maps), scale it locally, and add it ot the input map.
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	2007:		Marc-Antoine Miville-Deschenes, First version.
; 	01/06/2010:	Jacques Delabrouille, Use call to
; 			GENERATE_X_LM instead of SYNFAST.
; 	12/09/2011:	MAMD, General clean-up.
; 	07/10/2011:	JD, added keyword for ordering of input map.
; 	07/11/2011:	MAMD, changes related to the new thermal dust polarization model
; 	 	 	and additionnal clean up in preparation for the public release.
; 	19/09/2012:	JD, doc header.
;-

function add_smallscale, inputmap, struct, seed=seed, planckres=planckres, $
                         mask=mask, beta=beta, alpha=alpha, gamma=gamma, $
                         resolution=resolution, nl=nl, ordering=ordering, $
                         scaling=scaling

  if not keyword_set(planckres) then planckres = (*!sky_model).planckresolution
  if (planckres le 0) then planckres = 5.             ; arcmin

  if not keyword_set(resolution) then resolution=0
  if not keyword_set(alpha) then alpha=0
  if not keyword_set(beta) then beta=0
  if not keyword_set(gamma) then gamma=0
  if not keyword_set(mask) then mask=0

  IF NOT KEYWORD_SET(ordering) THEN ordering='NESTED'

; extract Ancillary data information from structure
  if keyword_set(struct) then begin
     tags = tag_names(struct)
     for i=0, n_elements(tags)-1 do begin
        case tags(i) of
           'RESOLUTION': resolution = struct.resolution
           'ALPHA': alpha = struct.alpha
           'BETA': beta = struct.beta
           'GAMMA': gamma = struct.gamma
           else :
        endcase
     endfor
  endif

; make sure all parameters are non zero
  check_param = alpha*beta*gamma*resolution 
  if (check_param eq 0) then begin
     print, "could not add small scales, parameter missing"
     return, inputmap
  endif

; make sure resolution of template is not higher than planckres
  if (resolution le planckres) then begin
     PSM_INFO_MESSAGE, "Could not add small scales: resolution of template higher than requested resolution"
     return, inputmap
  endif

; nside of input map
  npix = (SIZE(inputmap))[1]
  nside = npix2nside(npix)

; create Gaussian random field
  IF NOT KEYWORD_SET(nl) THEN nl = (*!sky_model).lmax + 1
  l = (findgen(nl-1)+1)
  sigma_planck = planckres/60./2.354*!pi/180.    ; sigma of Planck in radian
  sigma_input = resolution/60./2.354*!pi/180.    ; sigma of input map in radian

  cl = l^(gamma) * ( exp(-l^2 * sigma_planck^2) - exp(-l^2 * sigma_input^2) )

  l = [0,l]
  cl = [0,cl]
  GENERATE_X_LM, l, cl, xlm, my_seed=seed
  PSM_ALM2MAP, xlm, map, ordering=ordering, hpx_nside=nside, /nohdr, /overwrite
  
  map = map / stddev(map)
  map = map - avg(map)

; modulate small scale fluctuation by large scale intensity
; remove minimum from input map to make sure only positive values are
; exponentiated
;
  if not keyword_set(scaling) then scaling = (inputmap>0)
  map = map*alpha*scaling^beta

; add small scale to original map 
; deal with mask if provided (add structure only where mask is equal to 1)
  if keyword_set(mask) then begin
     read_fits_map, struct.mask, tempo, hdr, exthdr
     if (n_elements(exthdr) gt 1) then order_in = sxpar(exthdr, 'ORDERING') else order_in='RING'
     ud_grade, tempo, mask, nside_out=nside, order_in=order_in, order_out=ordering
     tempo=0
     outputmap = inputmap
     ind = where(mask eq 1, nbind)
     if (nbind gt 1) then outputmap(ind) = inputmap(ind)+map(ind)
  endif else begin
     outputmap = inputmap+map
  endelse

  return, outputmap

end
