; -----------------------------------------------------------------------------
;
; Copyright (C) 2007, 2012
;   Anthony J. Banday
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	R2FF
;
; PURPOSE:
;	 Function to compute the scale factor to convert the H-Alpha
;	 intensity in Rayleighs to the free-free brightness
;	 temperature in uK_RJ at a specified frequency in GHz. This
;	 function implements relations (11) and (8) in Dickinson et
;	 al. (2003) MNRAS 341, 369. 
;
; CATEGORY:
;	Sky model (galaxy)
;
; CALLING SEQUENCE:
;	result = R2FF(Freq_ghz)
;
; INPUTS:
;	Freq_ghz:	Radio frequency (between 1 and 1000 GHz) with
;	                which to convert the Halpha values (R) to
;	                free-free emission (uK_RJ). 
;
; KEYWORD PARAMETERS:
;	T_E:    	Electron temperature in units of K [default 7000K]
;	VERBOSE:	If set, run-time statements are printed to screen.
;
; OUTPUTS:
;	The result of this function is the scale factor to be used to
;	convert a given H-alpha emission in Rayleighs into expected
;	free-free emission in uK_RJ.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	January 2007:	Anthony J. Banday, First version.
; 	08/01/2012:	JD, doc and PSM license header.
;-

FUNCTION R2FF, freq_ghz, T_E = t_e, VERBOSE = verbose

  if n_elements(t_e) gt 0 then begin
     T_e = double(T_e)
     T4 = double(t_e)/1.d4
  endif else begin
     T_e = 0.7d4                ; assume T=7000 K
     T4 = double(0.7) 
     if keyword_set(verbose) then $
        print, '*** Assumed electron temperature, T =', round(T4*10000), 'K'
  endelse
  
  freq_ghz = double(freq_ghz)
  
; atomic fraction
  Z = 1.0d0
  
; calculation of gaunt correction factor (equation 8 in Dickinson et al.)
  factor = ( alog(4.995d-2 * freq_ghz^(-1)) + 1.5d0 * alog(T_e) ) 
  gaunt_correct = 0.366d0 * freq_ghz^(0.1) * T_e^(-0.15) * factor
  if keyword_set(verbose) then $
     print, '*** Gaunt correction factor (a) =', gaunt_correct
  
; Now calulate tb_ha
  tb_ha = 8.396d3 * gaunt_correct * freq_ghz^(-2.1) * T4^(0.667) * 10.d0^(0.029/T4) * (1.08) * (Z^2)
  
; Exit
  return, tb_ha
end
