; -----------------------------------------------------------------------------
;
; Copyright (C) 2004-2012
;   Jacques Delabrouille
;   Jean-Philippe Bernard
;   Guillaume Castex
;   Marc-Antoine Miville-Deschenes
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	GAL_MODEL_INIT
;
; PURPOSE:
;	Initialize system variables used by Planck Sky Model
;
; CATEGORY:
;	Sky model (galaxy)
;
; CALLING SEQUENCE:
;	GAL_MODEL_INIT, Nside
;
; INPUTS:
;	Nside:	HEALPix nside parameter for sky pixelisation.
;
; KEYWORD PARAMETERS:
;	THERMAL_DUST:	Thermal dust model: 
;                       	0 = No thermal dust
;                       	1 = SFD model
;                       	2 = TLS model (presently inactivated)
;                       	3 = BBM model (presently inactivated)
;                       	4 = SFD model with H-II regions cut out
;	SPINNING_DUST:	...
;	FREE_FREE:	...
;	SYNCHROTRON:	...
;	SMALLSCALE:	Set this keyword to generate random small
;	                scale fluctuations in the case where the
;	                ancillary data templates are at lower
;	                resolution that the target resolution (defined
;	                with the PLANCKRES keyword, see below).
;	POLARISATION:	Set this keyword for polarised galactic emission.
;	PLANCKRES:	Target resolution of the galactic maps.
;	LMAX:   	Target maximum harmonic mode of the galactic maps.
;
; OUTPUTS:
;	!sky_model IDL system variable is created (array of pointers)
;
; SIDE EFFECTS:
;	Modification of system variables
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	14/01/2004:	Jean-Philippe Bernard, First version
; 	                (originally SKY_MODEL_INIT).
; 	13/07/2004:	JPB, Include Synchrotron polar templates.
; 	05/03/2007:	MAMD, Set dir_save to new PSM architecture.
; 	14/03/2007:	MAMD, Major rework.
; 	24/05/2007:	MAMD, Add polarisation and modify use of idl-save files keyword.
; 	25/08/2009:	Jacques Delabrouille, Add LMAX keyword.
; 	15/11/2009:	JD, Renamed GAL_MODEL_INIT, removed some small
; 	                scales (needs fixing).
; 	10/07/2010:	JD, Fix some memory leaks.
; 	29/11/2010:	JD + Guillaume Castex, Commented-out TLS and BBM dust models
; 	                (needs fixing).
; 	06/05/2011:	JD, added 7th component to spinning dust mixture.
; 	21/07/2011:	JD, Reactivate some small scales (with revised generation).
; 	09/09/2011:	MAMD, Add smallscale keyword to some free-free and spinning dust models.
; 	19/09/2011:	JD, Reactivate small scales for spinning dust.
; 	09/09/2011:	MAMD, changes related to the new thermal dust
; 	                polarization model; start clean up in
; 	                preparation for the public release. 
; 	08/01/2012:	JD, doc and PSM license header.
; 	14/03/2012:	JD, added dust_intrinsic_pol keyword.
;-
PRO GAL_MODEL_INIT,nside, $
                   thermal_dust=thermal_dust, $
                   spinning_dust=spinning_dust, $
                   free_free=free_free, $
                   synchrotron=synchrotron, $
                   reset=reset, $
                   smallscale=smallscale, $
                   polarisation=polarisation, $
                   planckres=planckres, $
                   lmax=lmax, freefree_te=freefree_te, $
                   dust_intrinsic_pol=dust_intrinsic_pol

  IF n_params() EQ 0 THEN BEGIN
     print,'gal_model_init,nside[,thermal_dust=]'
     print,'                    [,synchrotron=]'
     print,'                    [,free_free=]'
     print,'                    [,spinning_dust=]'
     print,'                    [,/reset]'
     print,'                    [,/smallscale]'
     print,'                    [,/polarisation]'
     print,'                    [,planckres=]'
     print,'                    [,lmax=]'
     print,'                    [,freefree_te=]'
     goto,sortie
  ENDIF

; Keyword initialisation
  if not keyword_set(smallscale) then smallscale=0
  if not keyword_set(polarisation) then polarisation=0
  if not keyword_set(planckres) then planckres=5.
  IF NOT KEYWORD_SET(freefree_te) THEN freefree_te=7000.

;check if !indef syst variable is defined
  defsysv,'!indef',exist=ex
  if (ex eq 0) then defsysv, '!indef', -32768.

;check if !sky_model syst variable already exist and if it is valid
  defsysv,'!sky_model', exist=ex
  IF ex THEN ex = PTR_VALID(!sky_model)

  IF ex EQ 0 THEN BEGIN
     message,'Initializing the PSM syst variable.',/info
     initpsm = 1
  ENDIF ELSE BEGIN
     initpsm = 0
     IF Nside NE (*!sky_model).Nside THEN BEGIN
        message,'Warning: change of NSIDE keyword',/info
        reset = 1
     ENDIF
     IF smallscale NE (*!sky_model).smallscale THEN BEGIN
        message,'Warning: change of SMALLSCALE keyword',/info
        reset = 1
     ENDIF
     IF polarisation NE (*!sky_model).polarisation THEN BEGIN
        message,'Warning: change of POLARISATION keyword',/info
        reset = 1
     ENDIF
     IF (smallscale and planckres NE (*!sky_model).planckresolution) THEN BEGIN
        message,'Warning: change of PLANCKRES keyword',/info
        reset = 1
     ENDIF
  ENDELSE

  IF keyword_set(reset) THEN message,'Resetting the PSM syst variable.',/info $
  else reset = 0

  IF (reset OR initpsm) THEN BEGIN
     IF ex THEN HEAP_FREE, !sky_model
     gal_sys_var_init, Nside, smallscale=smallscale, polarisation=polarisation, planckres=planckres, lmax=lmax
  ENDIF

;--------------------------------------
;----- THERMAL DUST -------------------
;--------------------------------------
  IF keyword_set(thermal_dust) THEN BEGIN
     modnum = strtrim(thermal_dust,2)

     case thermal_dust of
        1: begin                ;=== SFD Dust
           psm_sfd_init, 'SFD_I100', smallscale=smallscale, polarfraction=dust_intrinsic_pol
;           (*!sky_model).thermal_dust.I100 = file2ptr('SFD_I100')
;           (*!sky_model).thermal_dust.SFD_RATIO = file2ptr('SFD_RATIO')           
        END

        2 : begin               ;=== TLS Dust
           PSM_DEBUG_MESSAGE, 'TLS Dust model not correctly implemented'
           stop
           ;psm_tls_define_cstes
           ;(*!sky_model).thermal_dust.I100 = file2ptr('SFD_I100')
           ;tempo = file2ptr('SFD_RATIO')
           ;d = alog(*tempo)
           ;T2=exp(2.9992896+0.26268873*d+0.034024256*d^2+0.0079240288*d^3+0.0018007412*d^4+0.00018900522*d^5)
           ;d = 0
           ;(*!sky_model).thermal_dust.TLS_TEMP=ptr_new(T2)            
        END
        
        3: begin                ;==== dust BBM
           PSM_DEBUG_MESSAGE, 'BBM Dust model not correctly implemented'
           stop
           ;read_kappas,all_kappas,freqs,Ts
           ;(*!sky_model).thermal_dust.bbm_st.kappas=ptr_new(all_kappas)
           ;(*!sky_model).thermal_dust.bbm_st.freqs=ptr_new(freqs)
           ;(*!sky_model).thermal_dust.bbm_st.Ts=ptr_new(Ts)
           ;IF not ptr_valid((*!sky_model).thermal_dust.I100) THEN BEGIN
           ;   (*!sky_model).thermal_dust.I100 = file2ptr('SFD_I100')
           ;ENDIF
           ;IF not ptr_valid((*!sky_model).thermal_dust.sfd_T2) THEN BEGIN
           ;   tempo = file2ptr('SFD_RATIO')
           ;   d = alog(*tempo)
           ;   T2=exp(2.872+0.2407*d+2.962e-2*d^2+4.719e-3*d^3+9.320e-4*d^4+1.109e-4*d^5)
           ;   d = 0
           ;   (*!sky_model).thermal_dust.sfd_T2=ptr_new(T2)
           ;ENDIF
        END      

        4: begin                ;=== SFD Dust no HII regions
           psm_sfd_init, 'SFD_I100_NO_HII', smallscale=smallscale, polarfraction=dust_intrinsic_pol
;           (*!sky_model).thermal_dust.I100 = file2ptr('SFD_I100_NO_HII')
;           (*!sky_model).thermal_dust.SFD_RATIO = file2ptr('SFD_RATIO')           
        END
        5: BEGIN
           ok = TEST_PSM_DATA(GETPSMDIR('psmdata')+'ancillary/models/gal/SFD/FFP6-JD_i100_healpix_2048_v1.fits')
           IF NOT ok THEN BEGIN
              PSM_INFO_MESSAGE, 'This dust model requires data that is private to the Planck collaboration.'
              PSM_INFO_MESSAGE, "Please download the dust template file before running the PSM."
              PSM_INFO_MESSAGE, "Use the command: GET_PSM_DATA, datafile='ancillary/models/gal/SFD/FFP6-JD_i100_healpix_2048_v1.fits', /private"
              MESSAGE, 'aborting the PSM run'
           ENDIF
           psm_sfd_init, 'FFP6_I100_JD', smallscale=smallscale, polarfraction=dust_intrinsic_pol
        END
        else : print, "Unknown model for thermal dust : ", modnum
     endcase
     (*!sky_model).thermal_dust.modelnumber = thermal_dust

  endif

;--------------------------------------
;----- SYNCHROTRON  -------------------
;--------------------------------------
  IF keyword_set(synchrotron) THEN BEGIN
     modnum = strtrim(synchrotron,2)

     case synchrotron of
        1: begin                ;=== GIARDINO et al. 2002
           (*!sky_model).synchrotron.t408 = file2ptr('GIARDINO_408')
           (*!sky_model).synchrotron.beta = file2ptr('GIARDINO_INDEX')
        END
        2: begin                ;=== LAMBDA 408 MHz map
           (*!sky_model).synchrotron.t408 = file2ptr('LAMBDA_408')
           (*!sky_model).synchrotron.beta = file2ptr('GIARDINO_INDEX')
        END
        3: begin                ;=== LAMBDA 408 MHz map and Giardino2002 Beta map
           (*!sky_model).synchrotron.t408 = file2ptr('LAMBDA_408')
           (*!sky_model).synchrotron.beta = file2ptr('GIARDINO2002_INDEX')
        END
        4: begin                ;=== LAMBDA 408 MHz map and MAMD2007 Beta map
           (*!sky_model).synchrotron.t408 = file2ptr('LAMBDA_408')
           (*!sky_model).synchrotron.beta = file2ptr('MAMD2007_BETA_SYNC')
        END
        5: begin                ;=== LAMBDA 408 MHz map and Beta = -3.0
           (*!sky_model).synchrotron.t408 = file2ptr('LAMBDA_408')
           (*!sky_model).synchrotron.beta = ptr_new(-3.0)
        END
        6: begin                ;=== LAMBDA 408 MHz map and Beta model 4 of mamd et al. (2008)
           (*!sky_model).synchrotron.t408 = file2ptr('LAMBDA_408', struct, nside, smallscale=smallscale)
           (*!sky_model).synchrotron.beta = file2ptr('MAMD2008_BETA_SYNC_MODEL4')
        END
        7: begin ;=== LAMBDA 408 MHz map and Beta model 4 of mamd et al. (2008) but with simple synchrotron curvature model (CD)
           (*!sky_model).synchrotron.t408 = file2ptr('LAMBDA_408')
           (*!sky_model).synchrotron.beta = file2ptr('MAMD2008_BETA_SYNC_MODEL4')
           (*!sky_model).synchrotron.curvature = file2ptr('SYNCH_CURVATURE_MAP1')
        END
        else : print, "Unknown model for synchrotron : ", modnum
     endcase

     (*!sky_model).synchrotron.modelnumber = synchrotron

     if keyword_set(polarisation) then begin
        if keyword_set(smallscale) then begin ; scale small scales with intensity                                
           (*!sky_model).polarisation = 0
           scaling = psm_synchrotron(lindgen(nside2npix(nside)), 23.)
           (*!sky_model).polarisation = 1
        endif else begin
           scaling=0
        endelse
        (*!sky_model).synchrotron.polQ23GHz = file2ptr('WMAP_POLQ_23GHZ', smallscale=smallscale, scaling=scaling)
        (*!sky_model).synchrotron.polU23GHz = file2ptr('WMAP_POLU_23GHZ', smallscale=smallscale, scaling=scaling)
     endif

  endif

;--------------------------------------
;------- FREE FREE  -------------------
;--------------------------------------
  IF keyword_set(free_free) THEN BEGIN
     modnum = strtrim(free_free,2)

     case free_free of
        1: begin                ;=== Dickinson et al. model and Halpha map
           (*!sky_model).free_free.halpha = file2ptr('DICKINSON_HALPHA', smallscale=smallscale)
           (*!sky_model).free_free.ebv = file2ptr('SFD_EBV')
           (*!sky_model).free_free.f_d = 0.33    ; dust fraction (0-1)
           (*!sky_model).free_free.limit = 1.0   ; limit at which to stop correcting (mag.)
           (*!sky_model).free_free.T_e = freefree_te   ; electron temperature
        END
        
        2: begin                ;=== Dickinson et al. model & Finkbeiner Halpha map
           (*!sky_model).free_free.halpha = file2ptr('FINKBEINER_HALPHA')
           (*!sky_model).free_free.ebv = file2ptr('SFD_EBV')
           (*!sky_model).free_free.f_d = 0.33    ; dust fraction (0-1)
           (*!sky_model).free_free.limit = 1.0   ; limit at which to stop correcting (mag.)
           (*!sky_model).free_free.T_e = freefree_te   ; electron temperature
        END
        
        3: begin                ; Free_Free_BM
           (*!sky_model).free_free.wmap_mem = file2ptr('FREE_FREE_WMAP_MEM',keep=[0], smallscale=smallscale)
           (*!sky_model).free_free.T_e = freefree_te   ; electron temperature
        END

        4: begin                ; Free_Free_BM
           (*!sky_model).free_free.halpha = file2ptr('DICKINSON_HALPHA')
           (*!sky_model).free_free.ebv = file2ptr('DOBASHI_AV')
           (*!sky_model).free_free.T_e = freefree_te   ; electron temperature
        END

        5: begin                ;=== Miville-Deschenes et al. (2008)
           (*!sky_model).free_free.halpha = file2ptr('DICKINSON_HALPHA', smallscale=smallscale)
           (*!sky_model).free_free.ebv = file2ptr('SFD_EBV', smallscale=smallscale)
           (*!sky_model).free_free.wmap_mem = file2ptr('FREE_FREE_WMAP_MEM', smallscale=smallscale)
           (*!sky_model).free_free.f_d = 0.33    ; dust fraction (0-1)
           (*!sky_model).free_free.limit = 1.0   ; limit at which to stop correcting (mag.)
           (*!sky_model).free_free.T_e = freefree_te   ; electron temperature
        END

        6: begin                ; Free_Free MAMD et al. 2008
           (*!sky_model).free_free.mamd2008 = file2ptr('FREE_FREE_MAMD2008', struct, nside, smallscale=smallscale)
           (*!sky_model).free_free.T_e = freefree_te   ; electron temperature
        END

        else : print, "Unknown model for free free : ", modnum
     endcase
     (*!sky_model).free_free.modelnumber = free_free
     
  endif


;--------------------------------------
;--------- Spinning Dust --------------
;--------------------------------------
  IF keyword_set(spinning_dust) THEN BEGIN
     modnum = strtrim(spinning_dust,2)
     dl98_spin_dust_init  
     
     case spinning_dust of
        1: begin                ;=== Juvela et al.
           (*!sky_model).spinning_dust.WCO = file2ptr('WCO_DAME')
           (*!sky_model).spinning_dust.NHI = file2ptr('NHI_DICKEY_LOCKMAN')
           (*!sky_model).spinning_dust.halpha = file2ptr('FINKBEINER_HALPHA')
        END
        2 : BEGIN               ; === Draine & Lazarian (1998) model scaled with E(B-V)
           (*!sky_model).spinning_dust.sfd_ebv = file2ptr('SFD_EBV')
           (*!sky_model).spinning_dust.composition=[0.0,0.35,0,0.0,0,0,0]
        END
        3 : BEGIN               ; === Draine & Lazarian (1998) model with mamd2008 model 2
           (*!sky_model).spinning_dust.SPIN23GHZ = file2ptr('SPINNING_23_MAMD2008_MODEL2')
           (*!sky_model).spinning_dust.composition = [0.,0.962,0,0.,0.,0.038,0]
        END
        4 : BEGIN               ; === Draine & Lazarian (1998) model with mamd2008 model 4
           (*!sky_model).spinning_dust.SPIN23GHZ = file2ptr('SPINNING_23_MAMD2008_MODEL4', smallscale=smallscale)
           (*!sky_model).spinning_dust.composition = [0.,0.962,0,0.,0.,0.038,0]
        END 
        5 : BEGIN               ; === DL model with user-defined composition
           (*!sky_model).spinning_dust.SPIN23GHZ = file2ptr('SPINNING_23_MAMD2008_MODEL4', smallscale=smallscale)
           COMMON sky, model
           (*!sky_model).spinning_dust.composition = model.gal.spindust.emlaw.composition
        END
        else : print, "Unknown model for spinning dust : ", modnum
     endcase

     (*!sky_model).spinning_dust.modelnumber = spinning_dust
  endif

  message,'Galactic Planck Sky Model succesfully initialized',/info

sortie:

END
