; -----------------------------------------------------------------------------
;
; Copyright (C) 2004-2012
;   Jacques Delabrouille
;   Jean-Philippe Bernard
;   Maude Le Jeune
;   Marc-Antoine Miville-Deschenes
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	GAL_SYS_VAR_INIT
;
; PURPOSE:
;	Initialisation of system variables and file names for ancillary data used to build
;	the galactic sky model.
;
; CATEGORY:
;	Sky model (galaxy)
;
; CALLING SEQUENCE:
;	GAL_SYS_VAR_INIT, Nside
;
; INPUTS:
;	Nside:	HEALPix nside parameter for sky pixelisation.
;
; KEYWORD PARAMETERS:
;	SMALLSCALE:	Set this keyword to generate random small
;	                scale fluctuations in the case where the
;	                ancillary data templates are at lower
;	                resolution that the target resolution (defined
;	                with the PLANCKRES keyword, see below).
;	POLARISATION:	Set this keyword for polarised galactic emission.
;	PLANCKRES:	Target resolution of the galactic maps.
;	LMAX:   	Target maximum harmonic mode of the galactic maps.
;
; OUTPUTS:
;	None (system variables are modified)
;
; COMMON BLOCKS:
;	None
;
; SIDE EFFECTS:
;	Generates a lot of system variables
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	dd/mm/yyyy:	???, First version.
; 	13/07/2004:	???, Modified to include Synchrotron polar fields.
; 	08/06/2006:	Marc-Antoine Miville-Deschenes, include files
; 	                in global structure and add smallscale keyword. 
; 	18/01/2007:	Jean-Philippe Bernard, include entry for Dobashi Av map used in free_BM.
; 	05/03/2007:	MAMD, change dir to fit WG2 architecture. add smallscale to
;                       sky_model structure.
; 	30/05/2007:	MAMD, add polarisation and planckres keywords.
; 	28/02/2008:	MAMD, update synchrotron polarisation model
;                       (now based on WMAP 23 GHz U and Q).
; 	dd/mm/2008:	MAMD, use nside=2048 templates when needed.
; 	dd/mm/2008:	Maude Le Jeune, add set_ancilfile function to check whether
;                       2048 ancillary file are present.
; 	25/08/2009:	Jacques Delabrouille, added lmax keyword.
; 	02/12/2009:	JD, former name psm_sys_var_init changed to
; 	                gal_sys_var_init; changes in data paths due to
; 	                reorganisation of data directory. 
; 	22/04/2010:	JD, more reorganisation.
; 	27/05/2010:	JD, still more reorganisation.
; 	14/09/2010:	JD, still more reorganisation.
; 	30/03/2011:	MAMD, modification of 4 templates (2 models
; 	                for dust polarisation and Q and U 23 GHz of WMAP-7yrs.
; 	                - models/gal/MAMD-2009/gmap_thermaldust_3deg_512_v2.fits
; 	                - models/gal/MAMD-2009/delta_gamma_dust_20deg_512_v2.fits
; 	                - models/gal/MAMD-2009/wmap7yr_polu_23GHz_3deg_512.fits
; 	                - models/gal/MAMD-2009/wmap7yr_polq_23GHz_3deg_512.fits
; 	06/05/2011:	JD, added 7th component to spinning dust mixture.
; 	30/08/2011:	JD, Now reads new processed maps for the freefree_mamd2008 model.
; 	September 2011:	JD, Clean high Nside cases.
; 	07/11/2011:	MAMD, changes related to the new thermal dust
; 	                polarization model; more cleaning up in
; 	                preparation for the public release. 
; 	08/01/2012:	JD, doc and PSM license header.
;       07/02/2012:     MAMD, add back the synchrotron spectral index
;                       map of Giardino et al. (2002)
;-


FUNCTION SET_ANCILFILE, nside, nside_lowres, template_lowres, template_highres

  IF (nside LE nside_lowres) THEN file = template_lowres ELSE file = template_highres
  ok = TEST_PSM_DATA(file, /get)
  IF NOT FILE_TEST(template_highres) THEN BEGIN
     template_highres = template_lowres
     file = template_lowres
     ok = TEST_PSM_DATA(file, /get)
  ENDIF
  IF NOT file_test(file) THEN PRINT, 'WARNING, file '+ file + ' does not exist.'
  RETURN, file
END


PRO GAL_SYS_VAR_INIT, Nside, smallscale=smallscale, polarisation=polarisation, planckres=planckres, lmax=lmax

;-----------------------------------------------
;----------- set Global variables--- -----------
;-----------------------------------------------

; check what is defined
defsysv, '!PSM_DIR', exists=ex_psm_dir
defsysv, '!PSM_TEMPLATE_DIR', exists=ex_psm_template_dir
defsysv, '!PSM_ANCILLARY_DIR', exists=ex_psm_ancillary_dir
defsysv, '!PSMROOT', exists=ex_psmroot

; define PSM code directory
if not keyword_set(ex_psm_dir) and keyword_set(ex_psmroot) then $
   defsysv, '!PSM_DIR', FIX_SEPARATOR(!PSMROOT)+'galactic/'

; define directory for templates (.sav files)
if not keyword_set(ex_psm_template_dir) then begin
   psm_template_dir = getenv('PSM_TEMPLATE_DIR')
   if not keyword_set(psm_template_dir) and keyword_set(ex_psmroot) then $
      defsysv,'!PSM_TEMPLATE_DIR', FIX_SEPARATOR(!PSMROOT)+'data/Intermediate/galactic/' $
   else defsysv,'!PSM_TEMPLATE_DIR', psm_template_dir
endif

; Set filename for ancillary data
if not keyword_set(ex_psm_ancillary_dir) then begin
   dir = getenv('PSM_ANCILLARY_DIR')
   if not keyword_set(dir) and keyword_set(ex_psmroot) then $
      dir = FIX_SEPARATOR(!PSMROOT)+'data/Ancillary'         ; WG2 definition
   if not keyword_set(dir) then dir = getenv('TEMPLATE_DIR') ; backward compatibility (obsolete)
   defsysv, '!PSM_ANCILLARY_DIR', dir
endif

;-----------------------------------------------
;----------- set SKY_MODEL structure -----------
;-----------------------------------------------

eptr=ptr_new()  ;Empty pointer

;--------- thermal dust -----------
sfd_st={alpha1:1.5,alpha2:2.6,f1:0.0309,qratio:11.2,T1_av:9.6,T2_av:16.4}  ;FDS model #7
bbm_st={kappas:eptr,freqs:eptr,Ts:eptr}
thermal_dust = {modelnumber:0, $
                sfd_st:sfd_st, $
                SFD_TEMP:[eptr, eptr], $
                SFD_ALPHA:[0.,0.], $
                SFD_NORM:eptr, $
                SFD_NORM_PREFACTOR:[0.,0.], $
                POLAR_FRACTION:[0.,0.], $
                POLAR_Q_PREFACTOR:eptr, $
                POLAR_U_PREFACTOR:eptr, $
                bbm_st:bbm_st, $
                TLS_TEMP:eptr}

;-------- spinning dust ------------
dl98={jnu_cnm:eptr,jnu_wnm:eptr, jnu_wim:eptr, $
      jnu_mol:eptr,jnu_drk:eptr, jnu_rn:eptr, jnu_extra:eptr, jnu_freqs:eptr}
spinning_dust = {modelnumber:0, $
                 WCO:eptr, $
                 NHI:eptr, $
                 Halpha:eptr, $
                 Halpha_mask:eptr, $
                 SPIN23GHZ:eptr, $
                 dl98:dl98, $
                 composition:[0.,0.,0.,0.,0.,0.,0.], $
                 SFD_EBV:eptr}

;--------- free-free -------------
free_free = {modelnumber:0, $
             T_e:7000., $         ; electron temperature
             f_d:0.33, $          ; dust fraction (0-1)
             limit:1.0, $         ; limit (mag.) at which to stop correction
             Halpha:eptr, $
             Halpha_mask:eptr, $
             Ebv:eptr, $
             wmap_mem:eptr, $
             mamd2008:eptr}

;--------- synchrotron ------------
synchrotron = {modelnumber:0, $
               T408:eptr, $
               beta:eptr, $
               polQ23GHz:eptr, $
               polU23GHz:eptr, $
               curvature:eptr}


;----------------------------------------------
;--------- Definition of all template used -----
;----------------------------------------------

dir = GETPSMDIR('psmdata')+'ancillary/'

;------------ Thermal dust ------------
file = SET_ANCILFILE(nside, 1024,dir+'models/gal/SFD/FINK_Rmap_healpix_1024.fits' , $
                     dir+'models/gal/SFD/FINK_Rmap_healpix_2048.fits')
SFD_RATIO = {file:file, resolution:7*60., alpha:0.08, beta:1.0, gamma:-3.0, dataptr:eptr}
; in effect the resolution of this map is variable, from ~40 arcmin in
; the plane to 7 degrees at high latitudes.

file = set_ancilfile (nside, 1024,dir+'models/gal/SFD/SFD_i100_healpix_1024.fits', $
                      dir+'models/gal/SFD/SFD_i100_healpix_2048_2.fits')
SFD_I100 =  {file:file, resolution:9., alpha:0.1, beta:1.0, gamma:-3.0, dataptr:eptr}

file = SET_ANCILFILE(nside, 1024,dir+'models/gal/SFD/SFD_i100_healpix_1024_no_uchii.fits', $
                     dir+'models/gal/SFD/SFD_i100_healpix_2048_no_uchii_4.fits')
SFD_I100_NO_HII =  {file:file, resolution:9., alpha:0.1, beta:1.0, gamma:-3.0, dataptr:eptr}

file = SET_ANCILFILE(nside, 2048,dir+'models/gal/SFD/FFP6-JD_i100_healpix_2048_v2.fits', $
                     dir+'models/gal/SFD/FFP6-JD_i100_healpix_2048_v1.fits')
FFP6_I100_JD =  {file:file, resolution:4., alpha:0.1, beta:1.0, gamma:-3.0, dataptr:eptr}

;file = SET_ANCILFILE(nside, 256, dir+'models/gal/SFD/onedeg_SFD_Ebv_h256.fits' , $
;                     dir+'models/gal/SFD/onedeg_SFD_Ebv_h2048.fits')
file = SET_ANCILFILE(nside, 256, dir+'models/gal/SFD/onedeg_SFD_Ebv_JDprocessed_256.fits' , $
                     dir+'models/gal/SFD/onedeg_SFD_Ebv_JDprocessed_2048.fits')
SFD_EBV =  {file:file, resolution:60., alpha:0.2, beta:1.0, gamma:-3.0, dataptr:eptr}

;--------- Thermal dust polarization ---------
file = SET_ANCILFILE(nside, 512, dir+'models/gal/MAMD-2009/Q_DUST_PREFACTOR_3deg_512.fits', $
                     dir+'models/gal/MAMD-2009/Q_DUST_PREFACTOR_3deg_2048_apodized.fits')
Q_DUST_PREFACTOR = {file:file, resolution:3*60., alpha:0.06, beta:1., gamma:-2.8, dataptr:eptr}

file = SET_ANCILFILE(nside, 512, dir+'models/gal/MAMD-2009/U_DUST_PREFACTOR_3deg_512.fits', $
                     dir+'models/gal/MAMD-2009/U_DUST_PREFACTOR_3deg_2048_apodized.fits')
U_DUST_PREFACTOR = {file:file, resolution:3*60., alpha:0.06, beta:1., gamma:-2.8, dataptr:eptr}


;----- Spinning dust -----
file = SET_ANCILFILE(nside, 512, dir+'models/gal/MAMD-2009/spin23_mamd2008_model2-JDprocessed_512.fits', $
                     dir+'models/gal/MAMD-2009/spin23_mamd2008_model2-JDprocessed_2048.fits')
SPINNING_23_MAMD2008_MODEL2 = {file:file, resolution:1.1*60., alpha:0.3, beta:1.1, gamma:-3.0, dataptr:eptr}

file = SET_ANCILFILE(nside, 512, dir+'models/gal/MAMD-2009/spin23_mamd2008_model4_v2-JDprocessed_512.fits', $
                     dir+'models/gal/MAMD-2009/spin23_mamd2008_model4_v2-JDprocessed_2048.fits')
SPINNING_23_MAMD2008_MODEL4 = {file:file, resolution:1.1*60., alpha:0.3, beta:1.1, gamma:-3.0, dataptr:eptr}

;-------- Synchrotron -------
file = set_ancilfile (nside, 256, dir+'models/gal/MAMD-2009/sync_beta_model_4_n256_v2_nested.fits', $
                      dir+'models/gal/MAMD-2009/sync_beta_model_4_n2048_v2_nested.fits')
MAMD2008_BETA_SYNC_MODEL4 = {file:file, resolution:5*60., alpha:0., beta:0.0, gamma:0.0, dataptr:eptr}

;; file = set_ancilfile (nside, 512, dir+'models/gal/Synchrotron-Curvature/curvature_minus0.3_512.fits', $
;;                       dir+'models/gal/Synchrotron-Curvature/curvature_minus0.3_2048.fits')
;; SYNCH_CURVATURE_MAP1 = {file:file, resolution:60., alpha:0., beta:0.0, gamma:0.0, dataptr:eptr}

file = set_ancilfile (nside, 512, dir+'observations/Haslam/lambda_haslam408_dsds_zerolevel-JDprocessed_512.fits', $
                      dir+'observations/Haslam/lambda_haslam408_dsds_zerolevel-JDprocessed_2048.fits')
LAMBDA_408 = {file:file, resolution:60., alpha:0.05, beta:1.0, gamma:-3.0, dataptr:eptr}

file = set_ancilfile (nside, 512,dir+'models/gal/GIARDINO-2002/SpectralIndex_Giardino2002.fits', $
                      dir+'models/gal/GIARDINO-2002/SpectralIndex_Giardino2002_2048.fits')
GIARDINO2002_INDEX = {file:file, resolution:10*60., alpha:0.0, beta:0.0, gamma:0.0, dataptr:eptr}

file = set_ancilfile (nside, 512,dir+'models/gal/MAMD-2009/wmap7yr_polq_23GHz_3deg_512_v3.fits', $
                      dir+'models/gal/MAMD-2009/wmap7yr_polq_23GHz_3deg_2048_v3.fits')
WMAP_POLQ_23GHZ = {file:file, resolution:3*60., alpha:5.0, beta:1.0, gamma:-2.8, dataptr:eptr}

file = set_ancilfile (nside, 512,dir+'models/gal/MAMD-2009/wmap7yr_polu_23GHz_3deg_512_v3.fits', $
                      dir+'models/gal/MAMD-2009/wmap7yr_polu_23GHz_3deg_2048_v3.fits')
WMAP_POLU_23GHZ = {file:file, resolution:3*60., alpha:2.0, beta:1.0, gamma:-2.8, dataptr:eptr}

;-------- Free-Free ---------
file = SET_ANCILFILE(nside, 256, dir+'observations/Halpha/onedeg_diff_halpha_JDprocessed_256.fits', $
                     dir+'observations/Halpha/onedeg_diff_halpha_JDprocessed_2048.fits')
DICKINSON_HALPHA = {file:file, resolution:60., alpha:0.3, beta:1.1, gamma:-2.6, dataptr:eptr}

file = SET_ANCILFILE(nside, 256, dir+'observations/WMAP/MEM/wmap_k_mem_freefree_3yr_v2_JDprocessed_256.fits', $
                     dir+'observations/WMAP/MEM/wmap_k_mem_freefree_3yr_v2_JDprocessed_2048.fits')
FREE_FREE_WMAP_MEM = {file:file, resolution:60., alpha:0.1, beta:1.1, gamma:-2.6, dataptr:eptr}

file = set_ancilfile (nside, 256, dir+'models/gal/MAMD-2009/free_free_mamd2008_v2-JDprocessed_256.fits', $
                      dir+'models/gal/MAMD-2009/free_free_mamd2008_v2-JDprocessed_2048.fits')
FREE_FREE_MAMD2008 = {file:file, resolution:70., alpha:0.3, beta:1.1, gamma:-2.6, dataptr:eptr}

;-------- CO -------
WCO_DAME = {file:dir+'observations/CO/CO_Dame_data_512.fits', resolution:30., $
            mask:dir+'observations/CO/CO_Dame_mask_512.fits', dataptr:eptr} 
           ; resolution is variable : 0.125 to 0.5 degree.

;-------- HI -------
NHI_DICKEY_LOCKMAN = {file:dir+'observations/HI/Dickey_Lockman_data_512.fits', $
                      resolution:[30., 60.], $
                      mask:dir+'observations/HI/Dickey_Lockman_mask_512.fits' ,dataptr:eptr}
;-------- AV -------
DOBASHI_AV = {file:dir+'/Extinction/Dobashi/Av_512_NESTED_G_d_MAP.fits', resolution:0, dataptr:eptr}


;-------------------
; Structures
;-------------------

ANCILLARY = {SFD_RATIO:SFD_RATIO, $
             SFD_I100:SFD_I100, $
             SFD_I100_NO_HII:SFD_I100_NO_HII, $
             FFP6_I100_JD:FFP6_I100_JD, $
             SFD_EBV:SFD_EBV, $
             Q_DUST_PREFACTOR:Q_DUST_PREFACTOR, $
             U_DUST_PREFACTOR:U_DUST_PREFACTOR, $
             MAMD2008_BETA_SYNC_MODEL4:MAMD2008_BETA_SYNC_MODEL4, $
             GIARDINO2002_INDEX:GIARDINO2002_INDEX, $
;             SYNCH_CURVATURE_MAP1:SYNCH_CURVATURE_MAP1, $
             LAMBDA_408:LAMBDA_408, $
             WMAP_POLQ_23GHZ:WMAP_POLQ_23GHZ, $
             WMAP_POLU_23GHZ:WMAP_POLU_23GHZ, $
             DICKINSON_HALPHA:DICKINSON_HALPHA, $
             FREE_FREE_WMAP_MEM:FREE_FREE_WMAP_MEM, $
             FREE_FREE_MAMD2008:FREE_FREE_MAMD2008, $
             SPINNING_23_MAMD2008_MODEL2:SPINNING_23_MAMD2008_MODEL2, $
             SPINNING_23_MAMD2008_MODEL4:SPINNING_23_MAMD2008_MODEL4, $
             WCO_DAME:WCO_DAME, $
             NHI_DICKEY_LOCKMAN:NHI_DICKEY_LOCKMAN, $
             DOBASHI_AV:DOBASHI_AV}

if not keyword_set(nside) then begin
   nside=128
   print, 'Nside not defined, use default value : ', nside
endif
if not keyword_set(lmax) then begin
   lmax=3*nside
   print, 'Lmax not defined, use default value (3xnside)'
endif

if not keyword_set(smallscale) then smallscale=0
if not keyword_set(polarisation) then polarisation=0
if not keyword_set(planckres) then planckres=5.

sm={thermal_dust:thermal_dust, $
    spinning_dust:spinning_dust, $
    free_free:free_free, $
    synchrotron:synchrotron, $
    Nside:fix(nside), $
    lmax:fix(lmax), $
    smallscale:smallscale, $
    planckresolution:planckres, $
    polarisation:polarisation, $
    ancillary:ancillary}

defsysv,'!sky_model',ptr_new(sm)

heap_gc   ;remove unused pointers

END


; unused Templates

;file = set_ancilfile (nside, 1024, dir+'/Lambda/H-alpha/Halpha_data_fwhm06_1024.fits',dir+'/Lambda/H-alpha/Halpha_data_fwhm06_2048.fits')
;FINKBEINER_HALPHA = {file:file, resolution:[5., 60.], alpha:0.3, beta:1.0, gamma:-2.6, $
;                     mask:dir+'/Lambda/H-alpha/Halpha_mask_fwhm06_1024.fits', dataptr:eptr}

;file = set_ancilfile (nside, 512, dir+'/Synchrotron/Giardino/NewTemplateSFDNs512.fits', dir+'/Synchrotron/Giardino/NewTemplateSFDNs2048.fits')
;GIARDINO_408 = {file:file, resolution:0., alpha:0, beta:0, gamma:0, dataptr:eptr}

;file = set_ancilfile (nside, 512,dir+'/Synchrotron/Giardino/BetaV1N512_MEM_NoFF.fits',dir+'/Synchrotron/Giardino/BetaV1N2048_MEM_NoFF.fits')
;GIARDINO_INDEX = {file:file, resolution:2*60., alpha:0.02, beta:1.0, gamma:-3.0, dataptr:eptr}

;file = set_ancilfile (nside, 512, dir+'/Synchrotron/gmap_sync23_3deg_512.fits',  dir+'/Synchrotron/gmap_sync23_3deg_2048.fits')
;GSYNC_23GHZ = {file:file, resolution:3*60., alpha:0.17, beta:0.5, gamma:-2.0, dataptr:eptr}

;file = set_ancilfile (nside, 512, dir+'/SFD/phi_polar_dust_20deg_512.fits', dir+'/SFD/phi_polar_dust_20deg_2048.fits')
;PHI_POLAR_DUST = {file:file, resolution:20*60., alpha:0.0, beta:0.0, gamma:0.0, dataptr:eptr}

;file = set_ancilfile (nside, 512, dir+'/Synchrotron/beta_sync_mamd_512.fits', dir+'/Synchrotron/beta_sync_mamd_2048.fits')
;MAMD2007_BETA_SYNC = {file:file, resolution:3*60., alpha:0., beta:0.0, gamma:0.0, dataptr:eptr}

;file = set_ancilfile (nside, 512, dir+'models/gal/MAMD-2009/delta_gamma_dust_20deg_512_v2.fits', dir+'/Models/gal/MAMD-2009/delta_gamma_dust_20deg_2048.fits')
;DELTA_GAMMA_POLAR_DUST = {file:file, resolution:20*60., alpha:0.0, beta:0.0, gamma:0.0, dataptr:eptr}

;file = SET_ANCILFILE(nside, 512, dir+'models/gal/MAMD-2009/E_DUST_PREFACTOR_3deg_512.fits', $
;                     dir+'models/gal/MAMD-2009/E_DUST_PREFACTOR_3deg_2048.fits')
;E_DUST_PREFACTOR = {file:file, resolution:3*60., alpha:0.08, beta:1., gamma:-2.3, dataptr:eptr}

;file = SET_ANCILFILE(nside, 512, dir+'models/gal/MAMD-2009/B_DUST_PREFACTOR_3deg_512.fits', $
;                     dir+'models/gal/MAMD-2009/B_DUST_PREFACTOR_3deg_2048.fits')
;B_DUST_PREFACTOR = {file:file, resolution:3*60., alpha:0.08, beta:1., gamma:-2.3, dataptr:eptr}

;file = SET_ANCILFILE(nside, 512, dir+'models/gal/MAMD-2009/gmap_thermaldust_3deg_512_v3.fits', $
;                     dir+'models/gal/MAMD-2009/gmap_thermaldust_3deg-JDprocessed_2048.fits')
;GMAP_POLAR_DUST = {file:file, resolution:3*60., alpha:0.04, beta:1, gamma:-2.3, dataptr:eptr}

;file = set_ancilfile (nside, 512, dir+'models/gal/MAMD-2009/gamma_thermaldust_3deg_512_v3.fits', $
;                      dir+'models/gal/MAMD-2009/gamma_thermaldust_3deg_2048_v3.fits')
;GAMMA_POLAR_DUST = {file:file, resolution:3*60., alpha:0.2, beta:1, gamma:-2.7, dataptr:eptr}
