; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jacques Delabrouille
;   Marc Betoule
;   Guillaume Castex
;   Lauranne Fauvet
;   Samuel Leach
;   Maude Le Jeune
;   Marc-Antoine Miville-Deschenes
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	GALACTIC_PIPE
;
; PURPOSE:
;	Generate a model of galactic diffuse emission.
;
; CATEGORY:
;	Sky model (galaxy)
;
; CALLING SEQUENCE:
;	GALACTIC_PIPE
;
; INPUTS:
;	None
;
; KEYWORD PARAMETERS:
;	DEBUG:	Set this keyword for debug mode (for PSM developers
;	essentially). If set no catch on error is done, subdirectory
;	components/gal is not removed when an error occurs. 
;
; OUTPUTS:
;	None (products are written to files in the PSM output durectory)
;
; COMMON BLOCKS:
;	PSM:	contains the param structure describing PSM run parameters
;	SKY:	contains the model structure describing sky parameters
;
; SIDE EFFECTS:
;	Writes (and possibly overwrites) files in the PSM output directory.
;
; RESTRICTIONS:
;       Must be launched from within a PSM simulation (do not use it
;       as a standalone routine).
;
;       Report any bugs to Jacques Delabrouille 
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	31/01/2007:	Marc Betoule, First version.
; 	05/02/2007:	MB, Pipe modification : split prediction and simulation.
; 	06/02/2007:	MB, Model structure creation (parameters
; 	                reading becomes conditional.
; 	08/03/2007:	MB, Add the TEST keyword.
; 	25/03/2007:	MB, Passing data using pointers.
; 	12/07/2007:	Maude Le Jeune, Seed is an input parameter and
;               	is set @ PSM_MAIN. 
; 	17/10/2007:	Samuel Leach, '&&' changed to 'AND' for IDL 5.5 compatibility.
; 	18/01/2008:	Marc-Antoine Miville-Deschenes, Change units to MJy/sr.
; 	27/02/2008:	MAMD, Change sign of U.
; 	04/03/2008:	MAMD, Add mamd2008 model.
; 	24/04/2009:	MAMD, Revision and simplification for PSM v1.7.
; 	28/04/2009:	JD, Now write the model maps.
; 	15/12/2009:	JD, Major reorganisation, now keep only two
; 	                input keywords, everything else if exchanged
; 	                via disk I/O (and COMMON blocks).
; 	06/07/2010:	JD, add CO molecular lines.
; 	10/07/2010:	JD, try to fix memory leaks.
; 	11/09/2010:	JD, Get PSM data from the data ftp repository if needed.
; 	14/09/2010:	JD, Better handling of precision (single vs. double).
; 	09/11/2010:	JD, Removed TEST keyword.
; 	09/03/2011:	JD, Implementation of automated visualisation of maps.
; 	16/03/2011:	JD, Implementation of automated visualisation
; 	                of emission laws.
; 	05/04/2011:	Guillaume Castex, Fix small bug in call to GET_PSM_DATA.
; 	08/04/2011:	JD, Better handling of maximum sky harmonic mode.
; 	19/05/2011:	JD, Free memory pointed at in !SKY_MODEL only
; 	                if it defined. Upgrade CO template map if
; 	                output sky nside bigger than 512. Force CO map
; 	                to positive values only. 
; 	07/07/2011:	JD, Implementation of automated visualisation
; 	                of CO lines.
; 	09/09/2011:	JD, Turned off small scales in spinning dust
; 	                (needs to be revised).
; 	29/09/2011:	JD, Added generation of some bibliographic
; 	                references at each PSM run.
; 	15/11/2011:	JD, Modifications to the method for generating polarised dust.
; 	08/01/2012:	JD, doc and PSM license header.
; 	14/03/2012:	JD, fixed a bug for dust polarisation
; 	                (intrinsic polar forgotten!).
; 	14/03/2012:	JD, and Lauranne Fauvet, implemented model for
; 	                galactic magnetic field.
;-
PRO GALACTIC_PIPE, debug=debug

  COMMON psm, param
  COMMON sky, model

;; Define parameters for convenience
;;-----------------------------------
  gp = param.global
  galp = model.gal
  runp = param.run
  pixp = SKY_PIX()

  isdouble = STRLOWCASE(gp.precision) EQ 'double'

;; Error handling : remove galactic component directory when catching an error
;;----------------------------------------------------------------------
  IF ((NOT KEYWORD_SET(debug))) THEN BEGIN 
     CATCH, error_status
     IF error_status NE 0 THEN BEGIN
        PRINT, 'An error was encoutered while computing galactic maps.'
        CATCH, /CANCEL
        MESSAGE, !ERR_STRING
     ENDIF
  ENDIF

;; Create the output directory if needed
;;---------------------------------------
  componentdir = GETPSMDIR('components',exists=exists)
  IF (NOT exists) THEN FILE_MKDIR, componentdir

;; Should we reorder the data? input maps are nested
;;---------------------------------------------------
  reorder_needed = (pixp.ordering EQ 'RING')
  ipring = LINDGEN(NSIDE2NPIX(pixp.nside))
  IF reorder_needed THEN BEGIN
     MESSAGE, 'Original template ordering for galactic emission assumed to be nested. Reordering needed.', /info
                                ;pix_param = {pix_type:'HEALPIX',healpix_ordering:'RING',healpix_coordsys:'G'}

     RING2NEST, pixp.nside, TEMPORARY(ipring), pix_num
  ENDIF ELSE BEGIN
     pix_num = TEMPORARY(ipring)
  ENDELSE

;; MAIN PART OF THE GALACTIC PIPE
;;-------------------------------
  PRINT, 'COMPUTING GALACTIC FOREGROUNDS ...'

  ;; Sky beam
  ;;----------
  sky_beam=GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin')

  IF STRLOWCASE(gp.fields) NE 't' THEN polar = 1 ELSE polar = 0

  ;; Biblio
  ;;--------
  text = ''
  CASE (galp.synchrotron.model) OF
     6: BEGIN 
        text = 'The model of galactic synchrotron'
        CASE galp.freefree.model OF
           6: BEGIN
              IF galp.spindust.model EQ 4 THEN BEGIN
                 text = text + ', free-free and spinning dust emissions are based on the decomposition of WMAP data by Miville-Deschenes et al. (2008).'
              ENDIF ELSE BEGIN
                 text = text + 'and free-free emissions are based on the decomposition of WMAP data by Miville-Deschenes et al. (2008).'
              ENDELSE
           END
           ELSE: BEGIN
              IF galp.spindust.model EQ 4 THEN BEGIN
                 text = text + ' and spinning dust emissions are based on the decomposition of WMAP data by Miville-Deschenes et al. (2008).'
              ENDIF ELSE BEGIN
                 text = text + 'is based on the decomposition of WMAP data by Miville-Deschenes et al. (2008).'
              ENDELSE
           END
           ELSE: 
        ENDCASE
     END
     ELSE: BEGIN
        CASE galp.freefree.model OF
           6: BEGIN
              IF galp.spindust.model EQ 4 THEN BEGIN
                 text = 'Free-free and spinning dust emissions are based on the decomposition of WMAP data by Miville-Deschenes et al. (2008).'
              ENDIF ELSE BEGIN
                 text = 'Free-free emission is based on the decomposition of WMAP data by Miville-Deschenes et al. (2008).'
              ENDELSE
           END
           ELSE: BEGIN
              IF galp.spindust.model EQ 4 THEN BEGIN
                 text = 'Spinning dust emissions is based on the decomposition of WMAP data by Miville-Deschenes et al. (2008).'
              ENDIF ELSE BEGIN
                 
              ENDELSE
           END
           ELSE: 
        ENDCASE
     END
  ENDCASE 
  IF STRLEN(text) NE 0 THEN PSM_ADD2CITE, text

  IF ((galp.synchrotron.model EQ 6) OR (galp.freefree.model EQ 6) or (galp.spindust.model EQ 4)) THEN PSM_ADD2BIB, '2008A&A...490.1093M'
  IF (galp.thermaldust.model EQ 4) THEN BEGIN
     PSM_ADD2CITE, 'Thermal dust emission in intensity is based on Model 7 of Finkbeiner, Davis & Schlegel (1999).'
     PSM_ADD2BIB, '1999ApJ...524..867F'
  ENDIF

;;----------------------------------------------------------------------------------------------------
;; POLARISATION FOR FAUVET 2011 MODEL
;;----------------------------------------------------------------------------------------------------

  IF ((galp.galpol.gal_polar_model EQ 'fauvet2011') AND (polar) AND ((galp.synchrotron.model NE 0) OR (galp.thermaldust.model NE 0))) THEN BEGIN

     A = galp.galpol.gal_bfield_turb_ampl
     pdeg = galp.galpol.gal_bfield_pitch_angle
     
     IF ((A EQ .2) AND (pdeg EQ -30.)) THEN BEGIN
        reffile = GETPSMDIR('psmdata')+'ancillary/models/gal/B-FIELD-FAUVET-2011/reference_A0.2_P30_nside32.sav'
        ok = TEST_PSM_DATA(reffile, /get)
        
        RESTORE, file=reffile

     ENDIF ELSE BEGIN
        bturbfile = GETPSMDIR('psmdata')+'ancillary/models/gal/B-FIELD-FAUVET-2011/B_turb_x.sav'
        ok = TEST_PSM_DATA(bturbfile, /get)
        
        nsideb = 32L
        npixb = NSIDE2NPIX(nsideb)
        
        RESTORE, file=bturbfile
        p = pdeg / !radeg
        make_templates_polar, nsideb, B_turb_x, A, p, sync_ref, dust_ref, g_sync, g_d
     ENDELSE

     PSM_ADD2CITE, 'The modelling of polarised galactic emission uses the model of Fauvet et al. (2011).'
     PSM_ADD2BIB, '2011A&A...526A.145F'

  ENDIF 
  
;;----------------------------------------------------------------------------------------------------
;; SYNCHROTRON
;;----------------------------------------------------------------------------------------------------
  HEAP_GC
  IF (galp.synchrotron.model) NE 0 THEN BEGIN  
        
     ;; Create the synchrotron directory
     ;;----------------------------------
     componentdir = GETPSMDIR('synchrotron',exists=exists)
     IF (NOT exists) THEN FILE_MKDIR, componentdir
     
     smallscale = (galp.synchrotron.smallscale EQ 'yes')
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "MODELING SYNCHROTRON"
     
     ;; Create galactic sky model structure for synchrotron only
     ;;----------------------------------------------------------
     SET_PSM_SEEDCHAIN, 'synchrotron-pipe'
     GAL_MODEL_INIT, pixp.nside, synchrotron=galp.synchrotron.model, free_free=0, thermal=0, spinning=0, $
                     smallscale=smallscale, planckres=SKY_RESOLUTION(), polarisation=polar, lmax=model.global.sky_lmax, /reset
     ;; Define PSM component structure for synchrotron
     ;;------------------------------------------------
     PSM_ID_KEYGEN, id
     IF model.gal.synchrotron.model EQ 7 THEN BEGIN
        synchrotron = COMPONENT_STRUCT('curvpowerlaw',name=param.names.synchrotron_basename,id=id)
     ENDIF ELSE BEGIN
        synchrotron = COMPONENT_STRUCT( model.gal.synchrotron.emlaw.synchrotron_emission_law,name=param.names.synchrotron_basename,id=id)
     ENDELSE
     *synchrotron.e1.numin.value = 0.
     *synchrotron.e1.numax.value = FLOAT('INF')

     ;; Units for synchrotron amplitude
     ;;---------------------------------
     synchrotron.e1.ampl.unit = 'MJy/sr'
        ;; nuref
        *synchrotron.e1.nuref.value = 2.3e10
        
        ;; File name for amplitude
        *synchrotron.e1.ampl.value = GETPSMDIR('synchrotron')+'synchrotron_ampl.fits'
        PSM_ID_KEYGEN, syncampl_id
        
        ;; Polarised amplitude
        IF polar THEN BEGIN
           CASE galp.galpol.gal_polar_model OF
              'mamd2008': BEGIN
                 tstokes = PSM_SYNCHROTRON(pix_num, 23., polq=qstokes, polu=ustokes)
              END
              'fauvet2011': BEGIN
                 IF PRECISION_ISDOUBLE() THEN tstokes = DOUBLE(PSM_SYNCHROTRON(pix_num, 23.)) $
                 ELSE tstokes = FLOAT(PSM_SYNCHROTRON(pix_num, 23.))

                 map = [sync_ref.i, sync_ref.q/sync_ref.i, sync_ref.u/sync_ref.i] & map = REFORM(map,N_ELEMENTS(sync_ref.i),3)
                 UD_GRADE, map, mmm, nside_out=128, order_out='RING', order_in='RING'
                 smoothmap = mmm
                 FOR it=0,3 DO BEGIN & $
                    k=0 & HPX_PIXSMOOTHING, mmm[*,k], smmm, maxremote=1, ordering='RING', /silent & smoothmap[*,k] = smmm & $
                    k=1 & HPX_PIXSMOOTHING, mmm[*,k], smmm, maxremote=1, ordering='RING', /silent & smoothmap[*,k] = smmm & $
                    k=2 & HPX_PIXSMOOTHING, mmm[*,k], smmm, maxremote=1, ordering='RING', /silent & smoothmap[*,k] = smmm & $
                    ;;smoothmap[0:3,1:2] = mmm[0:3,1:2] & smoothmap[196603:196607,1:2] = mmm[196603:196607,1:2]
                    mmm = smoothmap & $
                 ENDFOR
                    
                 PSM_MAP2ALM, mmm, alm, iter=1
                 PSM_ALM2MAP, TEMPORARY(alm), map, hpx_nside = SKY_NSIDE(), /overwrite

                 p_sync=3.0d0                              ; synchrotron spectral index		
                 PI_sync=(p_sync+1.)/(p_sync+7./3.)        ; synchrotron polarisation fraction
                 
                 qstokes = map[*,1]*PI_sync * (tstokes) ; map[*,1] is Q/I for the reference simple synchrotron template
                 ustokes = map[*,2]*PI_sync * (tstokes) ; map[*,2] is U/I for the reference simple synchrotron template

                 DESTROY_DATA, map
              END
           ENDCASE

           IF PRECISION_ISDOUBLE() THEN BEGIN
              WRITE_HPXMAP, (*synchrotron.e1.ampl.value), tstokes=DOUBLE(TEMPORARY(tstokes)), $
                            qstokes = DOUBLE(TEMPORARY(qstokes)), $
                            ustokes = DOUBLE(TEMPORARY(ustokes)), $
                            datatype ='COMP', psm_dtid=syncampl_id, beam=sky_beam, $
                            ordering='RING', units=synchrotron.e1.ampl.unit, lmax=SKY_LMAX(), coordsys='G'
           ENDIF ELSE BEGIN
              WRITE_HPXMAP, (*synchrotron.e1.ampl.value), tstokes=FLOAT(TEMPORARY(tstokes)), $
                            qstokes = FLOAT(TEMPORARY(qstokes)), $
                            ustokes = FLOAT(TEMPORARY(ustokes)), $
                            datatype ='COMP', psm_dtid=syncampl_id, beam=sky_beam, $
                            ordering='RING', units=synchrotron.e1.ampl.unit, lmax=SKY_LMAX(), coordsys='G'
           ENDELSE
           
        ;; Unpolarised amplitude               
        ENDIF ELSE BEGIN
           IF PRECISION_ISDOUBLE() THEN tstokes = DOUBLE(PSM_SYNCHROTRON(pix_num, 23.)) $
           ELSE tstokes = FLOAT(PSM_SYNCHROTRON(pix_num, 23.))
           WRITE_HPXMAP, (*synchrotron.e1.ampl.value), $
                         tstokes=tstokes, datatype ='COMP', psm_dtid=syncampl_id, beam=sky_beam, $
                         ordering='RING', units=synchrotron.e1.ampl.unit, lmax=SKY_LMAX(), coordsys='G'
           *((*!sky_model).synchrotron.t408) = 0B
        ENDELSE


        ;; Spectral index
        *synchrotron.e1.specind.value = GETPSMDIR('synchrotron')+'synchrotron_specind.fits'
        PSM_ID_KEYGEN, syncspecind_id
        IF PRECISION_ISDOUBLE() THEN WRITE_HPXMAP, (*synchrotron.e1.specind.value), $
                      tstokes=DOUBLE((*(*!sky_model).synchrotron.beta)[pix_num]+2), datatype ='COMP', psm_dtid=syncspecind_id, beam=sky_beam, $
                      ordering='RING', units='unitless', lmax=SKY_LMAX(), coordsys='G' $
        ELSE WRITE_HPXMAP, (*synchrotron.e1.specind.value), $
                      tstokes=FLOAT((*(*!sky_model).synchrotron.beta)[pix_num]+2), datatype ='COMP', psm_dtid=syncspecind_id, beam=sky_beam, $
                      ordering='RING', units='unitless', lmax=SKY_LMAX(), coordsys='G'
        *((*!sky_model).synchrotron.beta) = 0B
        ;synchrotron.e1.specind.form = 'map'        

        ;; If model with curvature
        IF model.gal.synchrotron.model EQ 7 THEN BEGIN
           *synchrotron.e1.curvnuref.value = 2.3e10
           *synchrotron.e1.curvampl.value = GETPSMDIR('synchrotron')+'synchrotron_curvampl.fits'
           PSM_ID_KEYGEN, synccurvampl_id
           IF PRECISION_ISDOUBLE() THEN WRITE_HPXMAP, (*synchrotron.e1.curvampl.value), $
                                                      tstokes=DOUBLE(*(*!sky_model).synchrotron.curvature), $
                                                      datatype ='COMP', psm_dtid=synccurvampl_id, beam=sky_beam, $
                                                      ordering='RING', units=synchrotron.e1.curvampl.unit, lmax=SKY_LMAX(), coordsys='G' $
           ELSE WRITE_HPXMAP, (*synchrotron.e1.curvampl.value), $
                              tstokes=FLOAT((*!sky_model).synchrotron.curvature), datatype ='COMP', psm_dtid=synccurvampl_id, beam=sky_beam, $
                              ordering='RING', units=synchrotron.e1.curvampl.unit, lmax=SKY_LMAX(), coordsys='G' 
           *((*!sky_model).synchrotron.curvature) = 0B
           ;synchrotron.e1.curvampl.form = 'map'
        ENDIF ELSE BEGIN
           CASE model.gal.synchrotron.emlaw.synchrotron_emission_law OF
              'curvpowerlaw': BEGIN
                 *synchrotron.e1.curvnuref.value = model.gal.synchrotron.emlaw.synchrotron_curv_freq
                 *synchrotron.e1.curvampl.value = model.gal.synchrotron.emlaw.synchrotron_curv_ampl
              END
              'powerlaw': BEGIN
                 ;; do nothing
              END
           ENDCASE
        ENDELSE

        PSM_DEBUG_MESSAGE, 'Still have to fix maps so that polarisation fraction remains less than 1', routine='GALACTIC_PIPE', infolimit=2, stoplimit=3
        
     ;; Free memory (this will leave a bunch of dangling references...)
     ;;-----------------------------------------------------------------
        HEAP_FREE, (*!sky_model).synchrotron.t408
        HEAP_FREE, (*!sky_model).synchrotron.beta
        IF polar THEN BEGIN
           HEAP_FREE, (*!sky_model).synchrotron.polQ23GHz
           HEAP_FREE, (*!sky_model).synchrotron.polU23GHz
        ENDIF

     ;; Assign polarisation info
     ;;--------------------------
        synchrotron.polarised = polar

     ;; Write save file for the synchrotron component model
     ;;-----------------------------------------------------
        IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE, 'Writing the synchrotron emission model'
        SAVE_COMP, synchrotron
        PRTSTRUCT, synchrotron, file=GETPSMDIR('synchrotron')+'synchrotron.txt', name='SYNCHROTRON'

     ;; Free memory
     ;;--------------
        DESTROY_DATA, synchrotron

  ENDIF ELSE BEGIN
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "NO SYNCHROTRON GENERATED"
  ENDELSE
  
  DEFSYSV, '!sky_model', exists=exists_sky_model
  IF exists_sky_model THEN HEAP_FREE, !sky_model

;;----------------------------------------------------------------------------------------------------
;; FREE FREE
;;----------------------------------------------------------------------------------------------------
  HEAP_GC
  IF (galp.freefree.model) NE 0 THEN BEGIN

     ;; Create the free-free directory
     ;;--------------------------------
     componentdir = GETPSMDIR('freefree',exists=exists)
     IF (NOT exists) THEN FILE_MKDIR, componentdir

     smallscale = (galp.freefree.smallscale EQ 'yes')
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "MODELING FREE-FREE"

     ;; Create galactic sky model structure for free-free only
     ;;--------------------------------------------------------
     SET_PSM_SEEDCHAIN, 'freefree-pipe'
     GAL_MODEL_INIT, pixp.nside, synchrotron=0, free_free=galp.freefree.model, thermal=0, spinning=0, freefree_te=galp.freefree.freefree_e_temp, $
                     smallscale=smallscale, planckres=SKY_RESOLUTION(), polarisation=polar, lmax=SKY_LMAX(), /reset
     ;; Define PSM component structure for free-free
     ;;----------------------------------------------
     PSM_ID_KEYGEN, id
     PSM_ID_KEYGEN, freefreeampl_id
     freefree = COMPONENT_STRUCT('freefree',name=param.names.freefree_basename,id=id)
     *(freefree.e1.numin.value) = 0.
     *(freefree.e1.numax.value) = FLOAT('INF')
     *(freefree.e1.nuref.value) = 2.3e10
     *(freefree.e1.temp.value) = (*!sky_model).free_free.T_e
     *(freefree.e1.ampl.value) = GETPSMDIR('freefree')+'freefree_ampl.fits'
     freefree.e1.ampl.unit = 'MJy/sr'
     ;freefree.e1.ampl.form = 'map'

     IF PRECISION_ISDOUBLE() THEN WRITE_HPXMAP, (*freefree.e1.ampl.value), $
                   tstokes = DOUBLE(PSM_FREE_FREE(pix_num,23d0)), datatype ='COMP', psm_dtid=freefreeampl_id, beam=sky_beam, $
                   ordering='RING', units=freefree.e1.ampl.unit, lmax=SKY_LMAX(), coordsys='G' $
     ELSE WRITE_HPXMAP, (*freefree.e1.ampl.value), $
                   tstokes = FLOAT(PSM_FREE_FREE(pix_num,23.)), datatype ='COMP', psm_dtid=freefreeampl_id, beam=sky_beam, $
                   ordering='RING', units=freefree.e1.ampl.unit, lmax=SKY_LMAX(), coordsys='G'


     ;; Free memory 
     ;;-------------
     HEAP_FREE, (*!sky_model).free_free.halpha
     HEAP_FREE, (*!sky_model).free_free.halpha_mask
     HEAP_FREE, (*!sky_model).free_free.ebv
     HEAP_FREE, (*!sky_model).free_free.wmap_mem
     HEAP_FREE, (*!sky_model).free_free.mamd2008

     ;; REMARKS
     ;;---------
     ;; temperature ---------
     ;;;freqref = 23.                                 ; GHz
     ;;;amp_freefree = psm_free_free(pix_num,freqref) ;in MJy/sr
     ;;;freq = 90.                                    ; this should be the frequency at which we want to compute the freefree
     ;;;freefree_temp = freefree_index(freq, freqref, (*!sky_model).free_free.T_e)
   
                                ; pour etre completement
                                ; propre: l'indice spectral du
                                ; free-free varie tres lentement avec nu,
                                ; on peut utiliser la fonction
                                ; freefree_spectrum.pro pour calculer
                                ; le spectre (en unite de K_CMB)
     ;;;TKCMB = freefree_spectrum(nu, nuref=FREQref, Te=(*!sky_model).free_free.T_e, amp=1.)
     ;; no polarization ------

     SAVE_COMP, freefree
     PRTSTRUCT, freefree, file=GETPSMDIR('freefree')+'freefree.txt', name='FREEFREE'
     DESTROY_DATA, freefree

  ENDIF ELSE BEGIN
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "NO FREE-FREE GENERATED"
  ENDELSE

  DEFSYSV, '!sky_model', exists=exists_sky_model
  IF exists_sky_model THEN HEAP_FREE, !sky_model

;;----------------------------------------------------------------------------------------------------
;; SPINNING DUST
;;----------------------------------------------------------------------------------------------------
  ; ce qui suit fonctionne pour tous les modeles de spinning dust sauf
  ; le modele #1, qui n'est plus utilise. Ca pourrait changer, ce
  ; qui impliquera une plus grande complexite : dans ce modele l'emission spinning
  ; dust est la somme de 3 templates (HI, CO et Halpha) chacun extrapole
  ; avec un spectre different..... a prevoir.

  HEAP_GC
  IF (galp.spindust.model) NE 0 THEN BEGIN

     ;; Create the spinning dust directory
     ;;------------------------------------
     componentdir = GETPSMDIR('spindust',exists=exists)
     IF (NOT exists) THEN FILE_MKDIR, componentdir

     smallscale = (galp.spindust.smallscale EQ 'yes')
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "MODELING SPINNING DUST"

     ;; Create galactic sky model structure for spinning dust only
     ;;------------------------------------------------------------
     SET_PSM_SEEDCHAIN, 'spindust-pipe'
     ;; addition of small scale structure to spinning dust needs revision
     GAL_MODEL_INIT, pixp.nside, synchrotron=0, free_free=0, $
                     thermal=0, spinning=galp.spindust.model, $
                     smallscale=smallscale, planckres=SKY_RESOLUTION(), polarisation=polar, lmax=SKY_LMAX(), /reset

     ;; Define PSM component structure for spinning dust
     ;;--------------------------------------------------
     PSM_ID_KEYGEN, id
     PSM_ID_KEYGEN, spindustampl_id
     spindust = COMPONENT_STRUCT('spindust',name=param.names.spindust_basename,id=id)
     *spindust.e1.numin.value = 0.
     *spindust.e1.numax.value = FLOAT('INF')
     *(spindust.e1.nuref.value) = 2.3e10
     *(spindust.e1.dlcomp.value) = (*!sky_model).spinning_dust.composition
     *(spindust.e1.ampl.value) = GETPSMDIR('spindust')+'spindust_ampl.fits'
     spindust.e1.ampl.unit = 'MJy/sr'
     ;spindust.e1.ampl.form = 'map'

     ;; Write the spinning dust map at 23 GHz
     ;;---------------------------------------
     IF PRECISION_ISDOUBLE() THEN WRITE_HPXMAP, (*spindust.e1.ampl.value), $
                   tstokes = DOUBLE(PSM_SPINNING_DUST(pix_num,23d0)), datatype ='COMP', psm_dtid=spindustampl_id, beam=sky_beam, $
                   ordering='RING', units=spindust.e1.ampl.unit, lmax=SKY_LMAX(), coordsys='G' $
     ELSE WRITE_HPXMAP, (*spindust.e1.ampl.value), $
                   tstokes = FLOAT(PSM_SPINNING_DUST(pix_num,23d0)), datatype ='COMP', psm_dtid=spindustampl_id, beam=sky_beam, $
                   ordering='RING', units=spindust.e1.ampl.unit, lmax=SKY_LMAX(), coordsys='G'

     ;; Free memory 
     ;;-------------
     HEAP_FREE, ((*!sky_model).spinning_dust).spin23GHz

     ;; REMARKS
     ;;---------
     ;; temperature ----------
     ;;;freqref = 23.                                     ; GHz
     ;;;amp_spinning = psm_spinning_dust(pix_num,freqref) ;in MJy/sr

     ; spinning dust spectrum (MJy/sr)
     ;;;IMJYsr = dl98_spectrum(nu, nuref=freqref, composition=(*!sky_model).spinning_dust.composition)
   
     ;; no polarization ------

     SAVE_COMP, spindust   
     PRTSTRUCT, spindust, file=GETPSMDIR('spindust')+'spindust.txt', name='SPINDUST'
     DESTROY_DATA, spindust

  ENDIF ELSE BEGIN
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "NO SPINNING DUST GENERATED"
  ENDELSE

  DEFSYSV, '!sky_model', exists=exists_sky_model
  IF exists_sky_model THEN HEAP_FREE, !sky_model
        
;;----------------------------------------------------------------------------------------------------
;; THERMAL DUST
;;----------------------------------------------------------------------------------------------------
  HEAP_GC
  IF (galp.thermaldust.model) NE 0 THEN BEGIN

     ;; Create the thermal dust directory
     ;;-----------------------------------
     componentdir = GETPSMDIR('thermaldust',exists=exists)
     IF (NOT exists) THEN FILE_MKDIR, componentdir

     smallscale = (galp.thermaldust.smallscale EQ 'yes')
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "MODELING THERMAL DUST"

     ;; Create galactic sky model structure for thermal dust only
     ;;-----------------------------------------------------------
     SET_PSM_SEEDCHAIN, 'thermaldust-pipe'
     IF galp.thermaldust.i100 EQ 'FFP6-JD' THEN GAL_MODEL_INIT, pixp.nside, synchrotron=0, free_free=0, thermal=galp.thermaldust.model, spinning=0, dust_intrinsic_pol=galp.thermaldust.dust_intrinsic_pol, $
        smallscale=0, planckres=4.39, polarisation=polar, lmax=model.global.sky_lmax, /reset $
     ELSE GAL_MODEL_INIT, pixp.nside, synchrotron=0, free_free=0, thermal=galp.thermaldust.model, spinning=0, dust_intrinsic_pol=galp.thermaldust.dust_intrinsic_pol, $
                          smallscale=smallscale, planckres=SKY_RESOLUTION(), polarisation=polar, lmax=model.global.sky_lmax, /reset
     
     nuref = !phyconst.c/1e-4   ;; 100 microns = 2.9979247 THz
     Wm2srHz_to_MJysr = CONVERSION_FACTOR('W/m2/sr/Hz','MJy/sr',nuref)

     ;; Define PSM component structure for thermal dust
     ;;-------------------------------------------------
     PSM_ID_KEYGEN, id
     thermaldust = COMPONENT_STRUCT('greybody','greybody', name=param.names.thermaldust_basename,id=id)
     *thermaldust.e1.numin.value = 0.
     *thermaldust.e1.numax.value = FLOAT('INF')
     *thermaldust.e2.numin.value = 0.
     *thermaldust.e2.numax.value = FLOAT('INF')

     ;; Just to compare
     ;;-----------------
     ;; dust = PSM_SFD(pix_num,3000)

     ;; Amplitude, temperature, nuref and spectral index of first greybody
     ;;--------------------------------------------------------------------
     *(thermaldust.e1.specind.value) = (*!sky_model).thermal_dust.sfd_alpha[0]
     *(thermaldust.e1.nuref.value) = nuref
     *(thermaldust.e1.temp.value) = GETPSMDIR('thermaldust')+'thermaldust_temp1.fits'
     PSM_ID_KEYGEN, dusttemp1_id
     ;thermaldust.e1.temp.form = 'map'
     IF PRECISION_ISDOUBLE() THEN WRITE_HPXMAP, (*thermaldust.e1.temp.value), $
                   tstokes=DOUBLE((*(*!sky_model).thermal_dust.sfd_temp[0])[pix_num]), datatype ='COMP', psm_dtid=dusttemp1_id, beam=sky_beam, $
                   ordering='RING', units='unitless', lmax=SKY_LMAX(), coordsys='G' $
     ELSE WRITE_HPXMAP, (*thermaldust.e1.temp.value), $
                   tstokes=FLOAT((*(*!sky_model).thermal_dust.sfd_temp[0])[pix_num]), datatype ='COMP', psm_dtid=dusttemp1_id, beam=sky_beam, $
                   ordering='RING', units='unitless', lmax=SKY_LMAX(), coordsys='G'
     *(thermaldust.e1.ampl.value) = GETPSMDIR('thermaldust')+'thermaldust_ampl1.fits'
     PSM_ID_KEYGEN, dustampl1_id
     ;thermaldust.e1.ampl.form = 'map'
     thermaldust.e1.ampl.unit = 'MJy/sr'

     I1 = ((*!sky_model).thermal_dust.sfd_norm_prefactor[0] * Wm2srHz_to_MJysr) $
          * (nuref/1e9)^(*(thermaldust.e1.specind.value)) $
          * PLANCK_BNU(nuref, temp=(*(*!sky_model).thermal_dust.sfd_temp[0])[pix_num])
     IF PRECISION_ISDOUBLE() THEN $
        I1 = TEMPORARY(I1) * (*(*!sky_model).thermal_dust.sfd_norm)[pix_num] $
     ELSE $
        I1 = FLOAT(TEMPORARY(I1) * (*(*!sky_model).thermal_dust.sfd_norm)[pix_num])

     IF galp.thermaldust.i100 EQ 'FFP6-JD' THEN BEGIN
        Itemporary = TEMPORARY(I1)
        CHANGE_MAP_RESOLUTION, TEMPORARY(Itemporary), I1, beam_in=4.39, beam_out=SKY_RESOLUTION(), iter=1, lmax=SKY_LMAX()
     ENDIF

     ;HEAP_FREE, ((*!sky_model).thermal_dust.sfd_temp[0])
     ;I1 = (*!sky_model).thermal_dust.sfd_norm_prefactor[0] * (*(*!sky_model).thermal_dust.sfd_norm)[pix_num]


     IF polar THEN BEGIN
        CASE galp.galpol.gal_polar_model OF
           'mamd2008': BEGIN
                                ;Pdust1 = (*(*!sky_model).thermal_dust.polar_gmap)[pix_num] * I1 * (*!sky_model).thermal_dust.polar_fraction[0] 
                                ;Q1 = Pdust1 * COS(2* (*(*!sky_model).thermal_dust.polar_angle)[pix_num])
                                ;U1 = TEMPORARY(Pdust1) * SIN(2*
                                ;(*(*!sky_model).thermal_dust.polar_angle)[pix_num])
              Q1 = I1*(*(*!sky_model).thermal_dust.polar_q_prefactor)[pix_num] * (*!sky_model).thermal_dust.polar_fraction[1]
              U1 = I1*(*(*!sky_model).thermal_dust.polar_u_prefactor)[pix_num] * (*!sky_model).thermal_dust.polar_fraction[1]
           END
           'fauvet2011': BEGIN
              map = [dust_ref.i, dust_ref.q/dust_ref.i, dust_ref.u/dust_ref.i] & map = REFORM(map,N_ELEMENTS(dust_ref.i),3)
              UD_GRADE, map, mmm, nside_out=128, order_out='RING', order_in='RING'
              smoothmap = mmm
              FOR it=0,3 DO BEGIN & $
                 k=0 & HPX_PIXSMOOTHING, mmm[*,k], smmm, maxremote=1, ordering='RING', /silent & smoothmap[*,k] = smmm & $
                 k=1 & HPX_PIXSMOOTHING, mmm[*,k], smmm, maxremote=1, ordering='RING', /silent & smoothmap[*,k] = smmm & $
                 k=2 & HPX_PIXSMOOTHING, mmm[*,k], smmm, maxremote=1, ordering='RING', /silent & smoothmap[*,k] = smmm & $
                 ;;smoothmap[0:3,1:2] = mmm[0:3,1:2] & smoothmap[196603:196607,1:2] = mmm[196603:196607,1:2]
                 mmm = smoothmap & $
              ENDFOR
              
              PSM_MAP2ALM, mmm, alm, iter=1, lmax=383
              PSM_ALM2MAP, TEMPORARY(alm), map, hpx_nside = SKY_NSIDE(), /overwrite
              
              PI_dust = galp.thermaldust.dust_intrinsic_pol    ; dust polarisation fraction
              
              Q1 = map[*,1]*PI_dust * I1 ; map[*,1] is Q/I for the reference simple dust template
              U1 = map[*,2]*PI_dust * I1 ; map[*,2] is U/I for the reference simple dust template
           END
        ENDCASE

        IF NOT PRECISION_ISDOUBLE() THEN BEGIN
           Q1 = FLOAT(TEMPORARY(Q1))
           U1 = FLOAT(TEMPORARY(U1))
        ENDIF
        WRITE_HPXMAP, (*thermaldust.e1.ampl.value), $
                      tstokes=TEMPORARY(I1), qstokes=TEMPORARY(Q1), ustokes=TEMPORARY(U1), $
                      datatype ='COMP', psm_dtid=dustampl1_id, beam=sky_beam, $
                      ordering='RING', units='MJy/sr', lmax=SKY_LMAX(), coordsys='G'
     ENDIF ELSE BEGIN
        WRITE_HPXMAP, (*thermaldust.e1.ampl.value), $
                      tstokes=TEMPORARY(I1), datatype ='COMP', psm_dtid=dustampl1_id, beam=sky_beam, $
                      ordering='RING', units='MJy/sr', lmax=SKY_LMAX(), coordsys='G'
     ENDELSE

     ;; Amplitude, temperature, nuref and spectral index of second greybody
     ;;---------------------------------------------------------------------
     *(thermaldust.e2.specind.value) = (*!sky_model).thermal_dust.sfd_alpha[1]
     *(thermaldust.e2.nuref.value) = nuref
     *(thermaldust.e2.temp.value) = GETPSMDIR('thermaldust')+'thermaldust_temp2.fits'
     PSM_ID_KEYGEN, dusttemp2_id
     ;thermaldust.e2.temp.form = 'map'
     IF PRECISION_ISDOUBLE() THEN WRITE_HPXMAP, (*thermaldust.e2.temp.value), $
                   tstokes=DOUBLE((*(*!sky_model).thermal_dust.sfd_temp[1])[pix_num]), datatype ='COMP', psm_dtid=dusttemp2_id, beam=sky_beam, $
                   ordering='RING', units='unitless', lmax=SKY_LMAX(), coordsys='G' $
     ELSE WRITE_HPXMAP, (*thermaldust.e2.temp.value), $
                   tstokes=FLOAT((*(*!sky_model).thermal_dust.sfd_temp[1])[pix_num]), datatype ='COMP', psm_dtid=dusttemp2_id, beam=sky_beam, $
                   ordering='RING', units='unitless', lmax=SKY_LMAX(), coordsys='G'
     *(thermaldust.e2.ampl.value) = GETPSMDIR('thermaldust')+'thermaldust_ampl2.fits'
     PSM_ID_KEYGEN, dustampl2_id
     ;thermaldust.e2.ampl.form = 'map'
     thermaldust.e2.ampl.unit = 'MJy/sr'

     I2 = ((*!sky_model).thermal_dust.sfd_norm_prefactor[1] * Wm2srHz_to_MJysr) $
          * (nuref/1e9)^(*(thermaldust.e2.specind.value)) $
          * PLANCK_BNU(nuref, temp=(*(*!sky_model).thermal_dust.sfd_temp[1])[pix_num])
     IF PRECISION_ISDOUBLE() THEN I2 = DOUBLE(TEMPORARY(I2) * (*(*!sky_model).thermal_dust.sfd_norm)[pix_num]) $
     ELSE I2 = FLOAT(TEMPORARY(I2) * (*(*!sky_model).thermal_dust.sfd_norm)[pix_num])
     IF galp.thermaldust.i100 EQ 'FFP6-JD' THEN BEGIN
        Itemporary = TEMPORARY(I2)
        CHANGE_MAP_RESOLUTION, TEMPORARY(Itemporary), I2, beam_in=4.39, beam_out=SKY_RESOLUTION(), iter=1, lmax=SKY_LMAX()
     ENDIF

     HEAP_FREE, ((*!sky_model).thermal_dust.sfd_temp[1])
     ;I2 = (*!sky_model).thermal_dust.sfd_norm_prefactor[1] * (*(*!sky_model).thermal_dust.sfd_norm)[pix_num]

     IF polar THEN BEGIN
        CASE galp.galpol.gal_polar_model OF
           'mamd2008': BEGIN
                                ;Pdust2 = (*(*!sky_model).thermal_dust.polar_gmap)[pix_num] * I2 * (*!sky_model).thermal_dust.polar_fraction[1] 
                                ;Q2 = Pdust2 * COS(2* (*(*!sky_model).thermal_dust.polar_angle)[pix_num])
                                ;U2 = TEMPORARY(Pdust2) * SIN(2* (*(*!sky_model).thermal_dust.polar_angle)[pix_num])
              Q2 = I2*(*(*!sky_model).thermal_dust.polar_q_prefactor)[pix_num] * (*!sky_model).thermal_dust.polar_fraction[1]
              U2 = I2*(*(*!sky_model).thermal_dust.polar_u_prefactor)[pix_num] * (*!sky_model).thermal_dust.polar_fraction[1]
           END
           'fauvet2011': BEGIN
              Q2 = map[*,1]*PI_dust * I2
              U2 = map[*,2]*PI_dust * I2          
           END
        ENDCASE

        IF NOT PRECISION_ISDOUBLE() THEN BEGIN
           Q2 = FLOAT(TEMPORARY(Q2))
           U2 = FLOAT(TEMPORARY(U2))
        ENDIF
        WRITE_HPXMAP, (*thermaldust.e2.ampl.value), $
                      tstokes=TEMPORARY(I2), qstokes=TEMPORARY(Q2), ustokes=TEMPORARY(U2), $
                      datatype ='COMP', psm_dtid=dustampl2_id, beam=sky_beam, $
                      ordering='RING', units='MJy/sr', lmax=SKY_LMAX(), coordsys='G'
     ENDIF ELSE BEGIN
        WRITE_HPXMAP, (*thermaldust.e2.ampl.value), $
                      tstokes=TEMPORARY(I2), datatype ='COMP', psm_dtid=dustampl2_id, beam=sky_beam, $
                      ordering='RING', units='MJy/sr', lmax=SKY_LMAX(), coordsys='G'
     ENDELSE
     HEAP_FREE, (*!sky_model).thermal_dust.sfd_norm

     ;; Assign polarisation info
     ;;--------------------------
        thermaldust.polarised = polar

     ;; Write save file for the thermal dust component model
     ;;-----------------------------------------------------
        IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE, 'Writing the thermal dust emission model'
        SAVE_COMP, thermaldust
        PRTSTRUCT, thermaldust, file=GETPSMDIR('thermaldust')+'thermaldust.txt', name='THERMALDUST'
        DESTROY_DATA, thermaldust

  ENDIF ELSE BEGIN
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "NO THERMAL DUST GENERATED"
  ENDELSE

  DEFSYSV, '!sky_model', exists=exists_sky_model
  IF exists_sky_model THEN HEAP_FREE, !sky_model

;;----------------------------------------------------------------------------------------------------
;; CO LINES
;;----------------------------------------------------------------------------------------------------
  HEAP_GC

  IF (galp.co.model) NE 0 THEN BEGIN

     ;; Create the CO directory
     ;;-------------------------
     componentdir = GETPSMDIR('co',exists=exists)
     IF ((NOT exists) AND (NOT KEYWORD_SET(test))) THEN FILE_MKDIR, componentdir

     smallscale = (galp.co.smallscale EQ 'yes')
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "MODELING CO LINE EMISSION"

     ;; Create galactic sky model structure for CO only
     ;;-------------------------------------------------
     SET_PSM_SEEDCHAIN, 'co-pipe'

     ;; Define PSM component structure for CO
     ;;---------------------------------------
     PSM_ID_KEYGEN, id
     co = COMPONENT_STRUCT('dirac','dirac','dirac',name=param.names.co_basename,id=id)
     *(co.e1.nuref.value) = 1.15271d11
     *(co.e1.numin.value) = 1.152d11
     *(co.e1.numax.value) = 1.153d11
     *(co.e1.ampl.value) = GETPSMDIR('co')+'co_ampl10.fits'
     co.e1.ampl.unit = 'MJy/sr'
     *(co.e2.nuref.value) = 2.30538d11
     *(co.e2.ampl.value) = GETPSMDIR('co')+'co_ampl21.fits'
     *(co.e2.numin.value) = 2.305d11
     *(co.e2.numax.value) = 2.306d11
     co.e2.ampl.unit = 'MJy/sr'
     *(co.e3.nuref.value) = 3.45796d11
     *(co.e3.ampl.value) = GETPSMDIR('co')+'co_ampl32.fits'
     *(co.e3.numin.value) = 3.457d11
     *(co.e3.numax.value) = 3.459d11
     co.e3.ampl.unit = 'MJy/sr'

     ;; Amplitude of CO line for all transitions
     ;;------------------------------------------
     beta = 1d3/!phyconst.c
     dnu1 = *(co.e1.nuref.value) * beta
     amp1 = 1d0 * dnu1 * CONVERSION_FACTOR('K_RJ','MJy/sr',*(co.e1.nuref.value), /double) ; for 1K_RJ km/s, in MJy/sr, integrated in band
     dnu2 = *(co.e2.nuref.value) * beta
     r21_10 = 0.65d0         ; ratio 2-1 to 1-0 from Franois Boulanger
     amp2 = r21_10 * dnu2 * CONVERSION_FACTOR('K_RJ','MJy/sr',*(co.e2.nuref.value), /double) ; for 1K_RJ km/s, in MJy/sr, integrated in band
     dnu3 = *(co.e3.nuref.value) * beta
     r32_21 = 0.5d0          ; ratio 3-2 to 2-1 from Franois Boulanger
     amp3 = r21_10 * r32_21 * dnu3 * CONVERSION_FACTOR('K_RJ','MJy/sr',*(co.e3.nuref.value), /double) ; for 1K_RJ km/s, in MJy/sr, integrated in band

     nu1= *(co.e1.nuref.value)
     nu2= *(co.e2.nuref.value)
     nu3= *(co.e3.nuref.value)
     IF NOT PRECISION_ISDOUBLE() THEN BEGIN
        amp1 = FLOAT(amp1)
        amp2 = FLOAT(amp2)
        amp3 = FLOAT(amp3)
     ENDIF

     ;; Read CO template, process it, and write it in the sky model
     ;;-------------------------------------------------------------
     comap = GETPSMDIR('psmdata')+'ancillary/observations/CO/CO_Dame_data_512.fits'
     ok = FILE_TEST(comap)
     IF NOT ok THEN GET_PSM_DATA, datafile=comap
     READ_MAP, GETPSMDIR('psmdata')+'ancillary/observations/CO/CO_Dame_data_512.fits', map, signal='T', ordering=ord
     map = map>0

     IF SKY_NSIDE() GT 512 THEN BEGIN
        UD_GRADE, TEMPORARY(map), mapout, nside_out=SKY_NSIDE(), order_in='NESTED', order_out='RING'
        map = TEMPORARY(mapout)
     ENDIF ELSE BEGIN
        npixco = N_ELEMENTS(map)
        iring = LINDGEN(npixco)
        RING2NEST, NPIX2NSIDE(npixco), iring, inest
        map = map[inest]
     ENDELSE

     ;; For the moment simply smooth to sky resolution
     CHANGE_MAP_RESOLUTION, map, mapsky, lmax=SKY_LMAX(), $
                            beam_in = GET_BEAM_STRUCT('GAUSSIAN',beamsize=0.,sizeunits='arcmin'), $
                            beam_out= GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin'), $
                            nside_out = SKY_NSIDE(), double = PRECISION_ISDOUBLE(), $
                            ordering_in='RING', ordering_out='RING', /nohdr
     mapsky = mapsky>0

     PSM_ID_KEYGEN, coampl_id
     WRITE_HPXMAP, (*co.e1.ampl.value), $
                   tstokes=amp1*mapsky, datatype ='COMP', psm_dtid=coampl_id, beam=sky_beam, $
                   ordering='RING', units='MJy/sr', lmax=SKY_LMAX(), coordsys='G'
     PSM_ID_KEYGEN, coampl_id
     WRITE_HPXMAP, (*co.e2.ampl.value), $
                   tstokes=amp2*mapsky, datatype ='COMP', psm_dtid=coampl_id, beam=sky_beam, $
                   ordering='RING', units='MJy/sr', lmax=SKY_LMAX(), coordsys='G'
     PSM_ID_KEYGEN, coampl_id
     WRITE_HPXMAP, (*co.e3.ampl.value), $
                   tstokes=amp3*mapsky, datatype ='COMP', psm_dtid=coampl_id, beam=sky_beam, $
                   ordering='RING', units='MJy/sr', lmax=SKY_LMAX(), coordsys='G'

     SAVE_COMP, co
     PRTSTRUCT, co, file=GETPSMDIR('co')+'co.txt', name='CO'
     DESTROY_DATA, co
  ENDIF ELSE BEGIN
     IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "NO CO-LINE EMISSION GENERATED"
  ENDELSE

  ;; Visu, if requested
  ;;--------------------
  IF param.run.visu NE 0 THEN VISU_COMP, 'galaxy', level=param.run.visu, output=param.run.output_visu

  ;; Make figure of emission laws
  ;;------------------------------
  IF param.run.visu GE 2 THEN BEGIN
     nu = ((FINDGEN(3990)+1.)/10.+1.)*1e9
     inusd = PSM_EMLAW(nu, emission_name='SPINDUST', nuref=2.3e10)
     inuff = PSM_EMLAW(nu, emission_name='FREEFREE', nuref=2.3e10)
     CASE param.run.output_visu OF
        'ps':BEGIN
           oldp=!P
           SET_PLOT, 'ps'
           DEVICE, /encapsulated, filename=FIX_SEPARATOR(GETPSMDIR('visu'))+'spindust_emlaw.eps', xsize=16, ysize=10
           ;;DEVICE, /encapsulated, filename='spindust_freefree_CO_emlaws.eps', xsize=16, ysize=10
        END
        'png': WINDOW, /free
        'screen': WINDOW, /free
        ELSE: ;do nothing
     ENDCASE
     PLOT, nu, inusd, /xlog, /ylog, xrange=[8e8,5e11], yrange=[1e-3,100], /xstyle, /ystyle, $
           title='Spinning dust, free-free and CO emission laws', xtitle='Frequency (Hz)', ytitle='Intensity', $
           thick=2, xthick=2, ythick=2, line=1 ;, xcharsize=1.2, ycharsize=1.2, charsize=1.5
     OPLOT, nu, inuff*2, thick=2, line=3
     IF DEFINED(nu1) THEN OPLOT, [nu1,nu1], [1e-10,1], thick=2
     IF DEFINED(nu2) THEN OPLOT, [nu2,nu2], [1e-10,amp2/amp1], thick=2
     IF DEFINED(nu3) THEN OPLOT, [nu3,nu3], [1e-10,amp3/amp1], thick=2
     LEGEND, /top, /left, linestyle=[3,1,0], ['free-free','spinning dust','CO molecular lines']
     CASE param.run.output_visu OF
        'ps':BEGIN
           DEVICE, /close
           SET_PLOT, 'x'
           !P=oldp
        END
        ELSE: ;do nothing
     ENDCASE
  ENDIF
  
  
  HEAP_GC

END

