FUNCTION PSM_SYNCHROTRON, pix_num, freq, index=index_map, polQ=polQ, polU=polU

;+
; NAME:
;    psm_synchrotron
; CATEGORY:
;    Planck Sky Model
; PURPOSE:
;    Computes Synchrotron emission at given position(s) and frequency and 
;    polarization Q and U if keyword is specified
; CALLING SEQUENCE:
;    vec=psm_synchrotron(pix_num,freq[,index=][,PolQ=][,PolU=]
; INPUT:
;    pix_num = array of pixel numbers in the Nside used during
;              initialization (see sky_model_init).
;              Should be in same scheme (NESTED is the default)
;    freq    = frequency at which calculation is to be done (GHz)
; OUTPUT:
;    vec     = Synchrotron contribution at given pixels and frequency (MJy/sr)
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    index   = if set, the map of synchrotron index map is given as input
; OUTPUT KEYWORDS:
;    POLQ = retrieves polarization Q
;    POLU = retrieves polarization U
; SIDE EFFECTS:
;    None
; EXAMPLES:
;     Nside=256
;     vec=psm_synchrotron(nside2npix(Nside),100.)
; RESTRICTIONS:
;    The sky_model_init routine must have been executed before
; PROCEDURES CALLED:
;    None
; REVISION HISTORY:
;    Created by Jean-Philippe Bernard 16/01/2004 Uses input from G. Giardino
;    Modified Sep 29 04 JPB to replace minmax by min and max
;    Modified Jun 25 04 by Roberta Paladini to include polarization
;    Modified Mar 15 07 by MAMD to use (*!sky_model).synchrotron.modelnumber
;    Modified May 29 07 by MAMD to use (*!sky_model).polarisation
;    Modified Feb 26 08 by MAMD : accomodate constant spectral index
;    Modified Feb 28 08 by MAMD : use WMAP Q and U maps at 23 GHz for polarisation 
;    Modified Sep 15 08 by MB : Quick but Dirty fix for P > 0.75 (or
;    even 1)
;    Modified Dec 01 09 by JD : now uses CONVERSION_FACTOR to convert units, + use TEMPORARY() to reduce memory used
;-

IF n_params() EQ 0 THEN BEGIN
  print,'vec=psm_synchrotron(pix_num,freq[,index=][PolQ=][PolU=])'
  goto,exit
ENDIF

nu0=0.408                                                      ;frequency of the Haslam template (GHz)
nu1 = 23.   ; reference frequency for curvature parameter 

sync_template=(*(*!sky_model).synchrotron.t408)(pix_num)       ;This is K antenna
IF NOT keyword_set(index_map) THEN BEGIN
   index_map=(*(*!sky_model).synchrotron.beta)
   if (n_elements(index_map) gt 1) then index_map = index_map(pix_num)
endif

modnum = (*!sky_model).synchrotron.modelnumber
; add curvature term if required
nu1 = 23.   ; reference frequency for curvature parameter
if (modnum eq 7) then begin
   curvature_map = (*(*!sky_model).synchrotron.curvature)
   index_map = index_map + (curvature_map*alog10(freq/nu1))
   PSM_DEBUG_MESSAGE, 'Check pix_num for curvature term', routine='PSM_SYNCHROTRON'
endif

; define curvature term
; IF NOT keyword_set(curvature) THEN curvature_map = index_map * 0.0 ELSE curvature_map = (*(*!sky_model).synchrotron.curvature) 

sync_map = TEMPORARY(sync_template) * (freq/nu0)^index_map ;in K Antenna

;transform into MJy/sr 
sync_map = TEMPORARY(sync_map) * CONVERSION_FACTOR('K_RJ','MJy/sr',freq*1e9)

; POLARISATION
IF keyword_set((*!sky_model).polarisation) THEN BEGIN
   sig_mjy = CONVERSION_FACTOR('mK_RJ','MJy/sr',freq*1e9) * CONVERSION_FACTOR('K_CMB','K_RJ',23e9)

   polQ=(*(*!sky_model).synchrotron.polQ23GHz)(pix_num)
   polQ = polQ*(freq/23.)^index_map*sig_mjy
   polU=(*(*!sky_model).synchrotron.polU23GHz)(pix_num)
   polU = polU*(freq/23.)^TEMPORARY(index_map)*sig_mjy

   ;This short patch fixes P>PI_Synchrotron
   P = sqrt(polQ^2 + polU^2) / abs(sync_map)
   F = (0.75*tanh(P/0.75))/TEMPORARY(P)
   polQ = F * TEMPORARY(polQ)
   polU = TEMPORARY(F) * TEMPORARY(polU)

   ;;;;;;;;;;;;;;;;;;;;;;;;;;
ENDIF

HEAP_GC

RETURN, sync_map
exit:

END
