FUNCTION psm_sfd,pix_num,freq,polQ=polQ,polU=polU

;+
; NAME:
;    psm_sfd
; CATEGORY:
;    Planck Sky Model
; PURPOSE:
;    Computes SFD dust emission at given position(s) and frequency and 
;    polarization Q and U if keyword is specified
; CALLING SEQUENCE:
;    vec=psm_sfd(pix_num,freq)
; INPUT:
;    pix_num = array of pixel numbers in the Nside used during
;              initialization (see sky_model_init).
;              Should be in same scheme (NESTED is the default)
;    freq    = frequency at which calculation is to be done (GHz)
; OUTPUT:
;    vec     = SFD contribution at given pixels and frequency (MJy/sr)
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    polQ    = if set, the map of Q Stokes parameter is output
;    polU    = if set, the map of U Stokes parameter is output
;    no_polar= if set, no polarization is calculated
; OUTPUT KEYWORDS:
;    POLQ = retrieves polarization Q
;    POLU = retrieves polarization U
; SIDE EFFECTS:
;    None
; EXAMPLES:
;     Nside=256
;     vec=psm_sfd(lindgen(12L*Nside^2),100.)
; RESTRICTIONS:
;    The sky_model_init routine must have been executed before
; PROCEDURES CALLED:
;    None
; REVISION HISTORY:
;    Created by Jean-Philippe Bernard 16/01/2004
;    Modified by Roberta Paladini 18/09/2005 to include polarization
;    (5% of total intensity)
;    Modified by Marc-Antoine Miville-Deschenes (12/06/2007) to use less memory
;    Modified by Marc-Antoine Miville-Deschenes (18/01/2008) : output in MJy/sr
;    Modified by Jacques Delabrouille (04/12/2009) : uses planck_bnu
;    instead of psm_planck_function
;    Modified by M.-A. Miville-Deschenes (07/11/2011) : use of new
;    templates for dust polarization
;-

IF n_params() EQ 0 THEN BEGIN
  print,'vec=psm_sfd(pix_num,freq [,PolQ=][,PolU=][,/no_polar])'
  goto,exit
ENDIF

;computes intensity map
bt1 = PLANCK_BNU(freq*1e9,temp = (*(*!sky_model).thermal_dust.SFD_TEMP[0])[pix_num],/megajysr)
I1 = (*!sky_model).thermal_dust.SFD_NORM_PREFACTOR[0] * $
     (*(*!sky_model).thermal_dust.SFD_NORM)[pix_num] * $
     freq^(*!sky_model).thermal_dust.SFD_ALPHA[0] * $
     TEMPORARY(BT1)

bt2 = PLANCK_BNU(freq*1e9,temp = (*(*!sky_model).thermal_dust.SFD_TEMP[1])[pix_num],/megajysr)
I2 = (*!sky_model).thermal_dust.SFD_NORM_PREFACTOR[1] * $
     (*(*!sky_model).thermal_dust.SFD_NORM)[pix_num] * $
     freq^(*!sky_model).thermal_dust.SFD_ALPHA[1] * $
     TEMPORARY(BT2)

; polarization
IF keyword_set((*!sky_model).polarisation) THEN BEGIN
   polQ = (*(*!sky_model).thermal_dust.POLAR_Q_PREFACTOR)[pix_num] * $
           ( I1 * (*!sky_model).thermal_dust.POLAR_FRACTION[0] + $
             I2 * (*!sky_model).thermal_dust.POLAR_FRACTION[1] )
   polU = (*(*!sky_model).thermal_dust.POLAR_U_PREFACTOR)[pix_num] * $
           ( I1 * (*!sky_model).thermal_dust.POLAR_FRACTION[0] + $
             I2 * (*!sky_model).thermal_dust.POLAR_FRACTION[1] )   
endif

; output is the sum of the two component
output = temporary(I1) + temporary(I2)

RETURN, output
exit:


END
