PRO psm_sfd_init, i100mapname, smallscale=smallscale, polarfraction=polarfraction

;+
; NAME:
;    psm_sfd
; CATEGORY:
;    Planck Sky Model
; PURPOSE:
;    Computes SFD dust emission at given position(s) and frequency and 
;    polarization Q and U if keyword is specified
; CALLING SEQUENCE:
;    vec=psm_sfd(pix_num,freq)
; INPUT:
;    pix_num = array of pixel numbers in the Nside used during
;              initialization (see sky_model_init).
;              Should be in same scheme (NESTED is the default)
;    freq    = frequency at which calculation is to be done (GHz)
; OUTPUT:
;    vec     = SFD contribution at given pixels and frequency (MJy/sr)
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    polQ    = if set, the map of Q Stokes parameter is output
;    polU    = if set, the map of U Stokes parameter is output
;    no_polar= if set, no polarization is calculated
; OUTPUT KEYWORDS:
;    POLQ = retrieves polarization Q
;    POLU = retrieves polarization U
; SIDE EFFECTS:
;    None
; EXAMPLES:
;     Nside=256
;     vec=psm_sfd(lindgen(12L*Nside^2),100.)
; RESTRICTIONS:
;    The sky_model_init routine must have been executed before
; PROCEDURES CALLED:
;    None
; REVISION HISTORY:
;    Created by Jean-Philippe Bernard 16/01/2004
;    Modified by Roberta Paladini 18/09/2005 to include polarization
;    (5% of total intensity)
;    Modified by Marc-Antoine Miville-Deschenes (12/06/2007) to use less memory
;    Modified by Marc-Antoine Miville-Deschenes (18/01/2008) : output in MJy/sr
;    Modified by Marc-Antoine Miville-Deschenes (30/03/2011) :
;    polarization fraction set to 15%
;    Modified by Marc-Antoine Miville-Deschenes (01/11/2011) :
;    Modified by Jacques Delabrouille (14/03/2012): Added polarfraction keyword
;    
;-

IF n_params() EQ 0 THEN BEGIN
  print,'psm_sfd_init, i100mapname, smallscale=, polarfraction='
  goto,exit
ENDIF

IF NOT DEFINED(polarfraction) THEN polarfraction=0.15

; model parameters
;=== Finkbeiner et al. 1999 - Model #7
alpha1=(*!sky_model).thermal_dust.sfd_st.alpha1
alpha2=(*!sky_model).thermal_dust.sfd_st.alpha2
f1=(*!sky_model).thermal_dust.sfd_st.f1
qratio=(*!sky_model).thermal_dust.sfd_st.qratio ;q1/q2
freq0=3000.  ;ref freq I100 (GHz)
lamb0=3.e14/freq0/1.e9  ;microns

; compute T1 and T2
SFD_RATIO_ptr = file2ptr('SFD_RATIO')
d = alog(1.d*(*SFD_RATIO_PTR))
ptr_free, SFD_RATIO_PTR

                                ;parameters below are from Finkbeiner et al. 1999, 524 867, Eq B2
                                ;T2=exp(2.872+0.2407*d+2.962e-2*d^2+4.719e-3*d^3+9.320e-4*d^4+1.109e-4*d^5)
                                ;derived by JPB for Model 7
T2=float(exp(2.8883246+0.23886849*d+0.029290881*d^2+0.0053778112*d^3+0.0010268379*d^4+0.00010812753*d^5))
d = 0
                                ;=== Model #7 of Finkbeiner et al.
                                ;T1=(T1_av^(4.+alpha1)/T2_av^(4.+alpha2)*T2^(4.+alpha2))^(1./(4.+alpha1))
hvk=6.626e-34/1.38e-23*freq0*1.e9
                                ;have to compute the integral ratio
                                ;this way, since the zeta-Rieman function does not seem to exist in
                                ;IDL (while the gamma function does). See Eq 16 in Finkbeiner et al.
Zfact=qromo('sfd_int_func2',0.,/midexp)/qromo('sfd_int_func1',0.,/midexp)
T1=(1./qratio*Zfact*hvk^(alpha1-alpha2)*T2^(4.+alpha2))^(1./(4.+alpha1))
   
BT10 = PLANCK_BNU(freq0*1d9,temp=T1)*1d20
BT20 = PLANCK_BNU(freq0*1d9,temp=T2)*1d20
I100_PTR = file2ptr(i100mapname, smallscale=smallscale)
norm = (*I100_PTR) / ( ( f1*temporary(BT10) + 1./qratio*(1.-f1)*temporary(BT20) ) * kfactor(alpha2,T2) )

ptr_free, I100_PTR
BT10 = 0
BT20 = 0

factor = [f1/freq0^alpha1, (1-f1) / qratio / freq0^alpha2]

(*!sky_model).thermal_dust.SFD_TEMP = [ptr_new(T1), ptr_new(T2)]
(*!sky_model).thermal_dust.SFD_ALPHA = [alpha1, alpha2]
(*!sky_model).thermal_dust.SFD_NORM = ptr_new(norm)
(*!sky_model).thermal_dust.SFD_NORM_PREFACTOR = factor



     if keyword_set((*!sky_model).polarisation) then begin
                                ; the small scales are not scaled
                                ; (scaling=1; simple Gaussian randomn field)
                                ; because Q/U = Q/U_PREFACTOR * I * POLAR_FRACTION.
                                ; In effect, the small scale
                                ; fluctuations added in Q/U_PREFACTOR
                                ; will then be multiplied by I in the
                                ; construction of Q and U.
           (*!sky_model).thermal_dust.POLAR_Q_PREFACTOR = file2ptr('Q_DUST_PREFACTOR', smallscale=smallscale, scaling=1.)
           (*!sky_model).thermal_dust.POLAR_U_PREFACTOR = file2ptr('U_DUST_PREFACTOR', smallscale=smallscale, scaling=1.)
           (*!sky_model).thermal_dust.POLAR_FRACTION = [polarfraction, polarfraction]

;        gamma_ptr = file2ptr('GAMMA_POLAR_DUST', struct, smallscale=smallscale, scaling=1.)
;        gmap_ptr = file2ptr('GMAP_POLAR_DUST', struct, smallscale=smallscale, scaling=1.)
;        (*!sky_model).thermal_dust.POLAR_ANGLE = gamma_ptr
;        (*!sky_model).thermal_dust.POLAR_GMAP = gmap_ptr
;        (*!sky_model).thermal_dust.POLAR_FRACTION = [0.15, 0.15]

;        Edust = file2ptr('E_DUST_PREFACTOR', smallscale=smallscale, scaling=1)
;        Bdust = file2ptr('B_DUST_PREFACTOR', smallscale=smallscale, scaling=1)  
;        eb2qu, (*Edust), (*Bdust), Qdust, Udust, /nest
;        (*!sky_model).thermal_dust.POLAR_Q_PREFACTOR = ptr_new(Qdust)
;        (*!sky_model).thermal_dust.POLAR_U_PREFACTOR = ptr_new(Udust)
;        (*!sky_model).thermal_dust.POLAR_FRACTION = [0.15, 0.15]

; old code
        ;; Q23_ptr = file2ptr('WMAP_POLQ_23GHZ', /polar, smallscale=smallscale)
        ;; U23_ptr = file2ptr('WMAP_POLU_23GHZ', /polar, smallscale=smallscale)
        ;; tempo = 0.5*atan( (*U23_ptr), (*Q23_ptr))
        ;; ptr_free, Q23_ptr
        ;; ptr_free, U23_ptr         
        ;; delta_gamma_ptr = file2ptr('DELTA_GAMMA_POLAR_DUST') ; , smallscale=smallscale);, /polar)
        ;; polar_angle = temporary(tempo) + (*delta_gamma_ptr)
        ;; tempo = 0
        ;; ptr_free, delta_gamma_ptr
        ;; ind = where(polar_angle gt !pi/2, nbind)
        ;; if (nbind gt 0) then polar_angle(ind) = !pi-polar_angle(ind)
        ;; ind = where(polar_angle lt -1.*!pi/2, nbind)
        ;; if (nbind gt 0) then polar_angle(ind) = !pi+polar_angle(ind)
        ;; (*!sky_model).thermal_dust.POLAR_ANGLE = ptr_new(polar_angle)        
        ;; (*!sky_model).thermal_dust.POLAR_FRACTION = [0.15, 0.15]
        ;; (*!sky_model).thermal_dust.POLAR_GMAP = file2ptr('GMAP_POLAR_DUST', min=1.e-6, max=1., smallscale=smallscale)
     endif


exit:

END
