; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	EXISTING_COMPONENT_LIST
;
; PURPOSE:
;	This function returns the list of existing PSM  component
;	names in a given PSM run.
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	result = EXISTING_COMPONENT_LIST()
;
; INPUTS:
;	None
;
; KEYWORD PARAMETERS:
; 	NCOMP:	Number of existing components (Output keyword)
;
; OUTPUTS:
;	a string array with the name of all existing PSM 
;	component names.
;
; COMMON BLOCKS:
;	SKY:	model structure
;	PSM:	param structure
;
; PROCEDURE
;	The software checks either for the definition of the model
;	used for each of the components (in the model structure of the
;	SKY common block), or for existing component directories in the
;	PSM output directory, or checks for existing files in the PSM
;	output directory.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	18/12/2009:	JD, Change way to find information for SZ,
; 	                output NCOMP keyword implemented.
; 	27/05/2010:	JD, Far Infrared Background now an independent component.
; 	21/06/2010:	JD, Point source components split into faint
; 	                and strong.
; 	06/07/2010:	JD, CO lines added ('co' component).
; 	01/09/2010:	Guillaume Castex, WMAP sources added.
; 	07/04/2011:	JD, Renamed strong and faint sources
; 	                (underscore '_' removed in names). 
; 	22/04/2011:	GC, ERCSC sources added.
; 	06/01/2012:	JD, doc and PSM license header.
;-
FUNCTION EXISTING_COMPONENT_LIST, ncomp=ncomp

  COMMON sky, model
  COMMON psm, param
  
  list_comp = PSM_ALLCOMP()
  ;; dipole cmb synchrotron freefree thermaldust spindust thermalsz kineticsz polarsz irps radiops uchii wmap coldcore firb zle
  nel = N_ELEMENTS(list_comp)
  exists_comp = BYTARR(nel)
  
  ;; Dipole and CMB anisotropies
  ;;-----------------------------
  IF DIPOLE_MODEL() NE 'no_dipole' THEN exists_comp[WHERE(list_comp EQ 'dipole')] = 1B  
  IF CMB_MODEL() NE 'no_cmb' THEN exists_comp[WHERE(list_comp EQ 'cmb')] = 1B

  ;; Galaxy
  ;;--------
  IF model.gal.synchrotron.model NE 0 THEN exists_comp[WHERE(list_comp EQ 'synchrotron')] = 1B
  IF model.gal.freefree.model NE 0 THEN exists_comp[WHERE(list_comp EQ 'freefree')] = 1B
  IF model.gal.thermaldust.model NE 0 THEN exists_comp[WHERE(list_comp EQ 'thermaldust')] = 1B
  IF model.gal.spindust.model NE 0 THEN exists_comp[WHERE(list_comp EQ 'spindust')] = 1B
  IF model.gal.co.model NE 0 THEN exists_comp[WHERE(list_comp EQ 'co')] = 1B

  ;; SZ
  ;;----
  IF SZ_MODEL() NE 'no_sz' THEN BEGIN
     th_ok = FILE_TEST(GETPSMDIR('sz')+'thermalsz_map.fits') OR FILE_TEST(GETPSMDIR('sz')+'relatsz0_map.fits')
     IF th_ok THEN exists_comp[WHERE(list_comp EQ 'thermalsz')] = 1B
     kin_ok = FILE_TEST(GETPSMDIR('sz')+'kineticsz_map.fits')
     IF kin_ok THEN exists_comp[WHERE(list_comp EQ 'kineticsz')] = 1B
     pol_ok = FILE_TEST(GETPSMDIR('sz')+'polarsz_map.fits')
     IF pol_ok THEN exists_comp[WHERE(list_comp EQ 'polarsz')] = 1B
  ENDIF

  ;; Point sources
  ;;---------------
  IF PS_MODEL() NE 'no_ps' THEN BEGIN
     strong_ir_ok = FILE_TEST(GETPSMDIR('ps')+'strongirps.sav')
     IF strong_ir_ok THEN exists_comp[WHERE(list_comp EQ 'strongirps')] = 1B
     faint_ir_ok = FILE_TEST(GETPSMDIR('ps')+'faintirps.sav')
     IF faint_ir_ok THEN exists_comp[WHERE(list_comp EQ 'faintirps')] = 1B
     strong_radio_ok = FILE_TEST(GETPSMDIR('ps')+'strongradiops.sav')
     IF strong_radio_ok THEN exists_comp[WHERE(list_comp EQ 'strongradiops')] = 1B
     faint_radio_ok = FILE_TEST(GETPSMDIR('ps')+'faintradiops.sav')
     IF faint_radio_ok THEN exists_comp[WHERE(list_comp EQ 'faintradiops')] = 1B
     strong_uchii_ok = FILE_TEST(GETPSMDIR('ps')+'stronguchii.sav')
     IF strong_uchii_ok THEN exists_comp[WHERE(list_comp EQ 'stronguchii')] = 1B
     faint_uchii_ok = FILE_TEST(GETPSMDIR('ps')+'faintuchii.sav')
     IF faint_uchii_ok THEN exists_comp[WHERE(list_comp EQ 'faintuchii')] = 1B
     strong_wmap_ok = FILE_TEST(GETPSMDIR('ps')+'strongwmapps.sav')
     IF strong_wmap_ok THEN exists_comp[WHERE(list_comp EQ 'strongwmapps')] = 1B
     faint_wmap_ok = FILE_TEST(GETPSMDIR('ps')+'faintwmapps.sav')
     IF faint_wmap_ok THEN exists_comp[WHERE(list_comp EQ 'faintwmapps')] = 1B
     strong_ercsc_ok = FILE_TEST(GETPSMDIR('ps')+'strongercscps.sav')
     IF strong_ercsc_ok THEN exists_comp[WHERE(list_comp EQ 'strongercscps')] = 1B
     faint_ercsc_ok = FILE_TEST(GETPSMDIR('ps')+'faintercscps.sav')
     IF faint_ercsc_ok THEN exists_comp[WHERE(list_comp EQ 'faintercscps')] = 1B
  ENDIF

  ;; FIRB
  ;;------

  IF FIRB_MODEL() NE 'no_firb' THEN BEGIN
     firb_ok = FILE_TEST(GETPSMDIR('firb'))
     IF firb_ok THEN exists_comp[WHERE(list_comp EQ 'firb')] = 1B
  ENDIF

  whex = WHERE(exists_comp EQ 1B,ncomp)
  IF ncomp NE 0 THEN RETURN, list_comp[whex] ELSE RETURN, 0

END
