; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	READ_SKY_PARAM
;
; PURPOSE:
;	Make the SKY COMMON block from the input table of parameter
;	names and values.
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	READ_SKY_PARAM, Table
;
; INPUTS:
;	Table:	A table of keyword parameters and values, read from a
;	PSM input file
;
; OPTIONAL INPUTS:
;	None
;	
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	None
;
; COMMON BLOCKS:
;	PSM:	contains the param structure describing PSM run parameters
;	SKY:	contains the model structure describing sky parameters
;
; SIDE EFFECTS:
;	Creates or updates the COMMON SKY block.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	24/11/2009:	JD, some reorganisation of the SKY model structure.
; 	18/12/2009:	JD, modifications to allow updating the sky
; 	                model in a PSM run.
; 	22/12/2009:	JD, correction of thermal SZ units ('y_sz'
; 	                instead of 'ysz'.
; 	18/01/2010:	JD, more modifications to allow updating the sky
; 	                model in a PSM run.
; 	21/06/2010:	JD, CMB temperature now from function CMB_TEMPERATURE
; 	21/06/2010:	JD, Added sky window function, possibility to
;  	                update the sky model inactivated (not really
;  	                useful, too complicated) 
; 	27/01/2011:	JD, Remove Z_reion (redshift of reionisation)
; 	                to use only the reionisation optical depth tau_reion.
; 	04/05/2011:	JD, CMB temperature now set here (always
; 	                changing my mind, but after all only idiots never do).
; 	05/09/2011:	JD, Write sky resolution into a system variable
; 	06/10/2011:	JD, Added a check that sky_lmax is less than
; 	                (3 sky_nside - 1)
; 	20/10/2011:	JD, Added flexibility for scalar amplitude
; 	24/11/2011:	JD, Modified default cosmological parameters
; 	                to match the WMAP best fit (WMAP7 Lambda CDM +
; 	                H0 + BAO).
; 	24/11/2011:	JD, Corrected default Omega Baryon.
; 	06/01/2012:	JD, doc and PSM license header.
;-
FUNCTION READ_SKY_PARAM, table

  COMMON psm, param
  COMMON sky, model
  
  IF DEFINED(table) THEN BEGIN
     dummy = {sky_sim:'new'}
     FILL_STRUCT, table, dummy, carefulness=psm_carefulness()
     sky_sim = dummy.sky_sim
  
;;-----------------------------------------------------------------------------------
;;    Sky model parameters for a new sky realisation
;;-----------------------------------------------------------------------------------
  

  ;; READ PARAMETERS IN THE PSM FILE
  ;;---------------------------------

        ;; Sky simulation info 
        ;;---------------------
        gp = { $                                                 ;
             sky_precision:param.global.precision, $             ; {'simple','double'}
             sky_fields:param.global.fields, $                   ;
             sky_resolution:15., $                               ;
             sky_lmax:1536, $                                    ;
             sky_pixwindow:1, $                                  ;
             run_camb:'no', $                                    ;
             run_class:'no' $                                    ;
             }
        tags = ['SKY_RESOLUTION','SKY_LMAX','SKY_PIXWINDOW','RUN_CAMB','RUN_CLASS']
        FILL_STRUCT, table, gp, tags=tags, carefulness=psm_carefulness()
        DEFSYSV, '!SKY_RESOLUTION', FLOAT(gp.sky_resolution)
        
        ;; Sky mapping parameters
        ;;------------------------
        pixp = READ_PIX_PARAM(table, carefulness=psm_carefulness())
        mapping = {type:'MAP', $
                   beam:GET_BEAM_STRUCT('GAUSSIAN',beamsize=gp.sky_resolution,sizeunits='arcmin'), $
                   lmin:0L, $
                   lmax:gp.sky_lmax, $
                   pix:pixp, $
                   sky_pixwindow:1 $
                  }
        IF gp.sky_lmax GE (pixp.nside * 3) THEN BEGIN
           PSM_INFO_MESSAGE, 'Value of lmax for the PSM simulation is greater than 3nside-1'
           PSM_INFO_MESSAGE, 'lmax is set to 3nside-1='+NUMBER2STRING(pixp.nside*3 - 1)+' by the PSM.'
           gp.sky_lmax = (pixp.nside*3 - 1)
        ENDIF

        ;; Component parameters
        ;;----------------------
        dipole = READ_DIPOLE_PARAM(table, carefulness=psm_carefulness())
        cmbp = READ_CMB_PARAM(table, carefulness=psm_carefulness())
        galp = READ_GAL_PARAM(table, carefulness=psm_carefulness())
        szp = READ_SZ_PARAM(table, carefulness=psm_carefulness())
        psp = READ_PS_PARAM(table, carefulness=psm_carefulness())
        firbp = READ_FIRB_PARAM(table, carefulness=psm_carefulness())
        zlep = {zle_model:'no_zle'}
        
        ;; Cosmological parameters
        ;;-------------------------

        ;; We define the structure template for the cosmological parameters.
        ;; This defines the default values
        c = DEFINE_COSMO(/silent)
       
        ;; We fill the structure on the basis of the input parameters
        tags = 'OMEGA_'+['M','B','K','NU']
        tags = [tags, 'T_CMB', 'H', 'SIGMA_8', 'N_S', 'N_S_RUNNING', 'N_T', 'N_T_RUNNING', 'R', 'TAU_REION', $
                'HE_FRACTION','N_MASSIVE_NU','N_MASSLESS_NU','W_DARK_ENERGY', 'K_PIVOT', 'SCALAR_AMPLITUDE']
        FILL_STRUCT, table, c, tags=tags, carefulness=psm_carefulness()
        IF c.omega_nu NE 0 THEN IF c.n_massive_nu EQ 0 THEN MESSAGE, 'Erroneous cosmological parameters, number of massive neutrinos missing'
        
        ;; This below serves to define consistently omega_de so that
        ;; omega_de + omega_m + omega_k=1
        cosmop = DEFINE_COSMO(t_cmb=c.t_cmb, h=c.h, omega_m=c.omega_m, omega_b=c.omega_b, omega_k=c.omega_k, omega_nu=c.omega_nu, $
                              sigma_8=c.sigma_8, n_scal=c.n_s, n_s_running=c.n_s_running, n_tens=c.n_t, n_t_running=c.n_t_running, r_tens_scal=c.r, $
                              tau_reion=c.tau_reion, fraction_he=c.he_fraction, n_massless_nu=c.n_massless_nu, $
                              w_dark_energy=c.w_dark_energy, k_pivot=c.k_pivot, scalar_amplitude=c.scalar_amplitude, /silent)
        IF PSM_VERBOSITY() GE 1 THEN PRTSTRUCT, cosmop, name='COSMO'

        CASE cmbp.model OF
           'nongaussian_fnl': BEGIN
              CASE cmbp.readjust_ng_spectrum OF
                 'no': BEGIN
                    ;; Overwrite the cosmological parameters to match those
                    ;; used for the non-gaussian simulation
                    PSM_INFO_MESSAGE, 'WARNING: Overwriting cosmological parameters to match those of the non-gaussian CMB simulation'
                    cosmop.h = 0.704
                    cosmop.omega_m = 0.272
                    cosmop.omega_b = 0.046
                    cosmop.omega_k = 0
                    cosmop.omega_nu = 0
                    cosmop.n_s = 0.963
                    cosmop.n_s_running = 0
                    cosmop.tau_reion = 0.087
                    cosmop.he_fraction = 0.24
                    cosmop.n_massless_nu = 3.04
                    cosmop.n_massive_nu = 0
                    cosmop.w_dark_energy = -1
                    cosmop.k_pivot = 0.002
                    cosmop.scalar_amplitude = 2.441e-9
                 END
                 'yes': BEGIN
                    ;; Do nothing here
                 END
              ENDCASE
              ;; CASE cmbp.cmb_lensing OF 
              ;;    'ilens': MESSAGE, "'ilens' CMB lensing not implemented for non-gaussian CMB"
              ;;    'cl': MESSAGE, "'cl' CMB lensing not implemented for non-gaussian CMB"
              ;;    ELSE:  ; do nothing
              ;; ENDCASE
           END
           ;; 'gaussian': BEGIN
           ;;    CASE cmbp.cmb_cl_source OF
           ;;       'standard-LCDM': BEGIN
           ;;          CASE cmbp.cmb_lensing OF 
           ;;             'ilens': MESSAGE, "'ilens' CMB lensing not implemented for non-gaussian CMB"
           ;;             ELSE:    ; do nothing
           ;;          ENDCASE
           ;;       END
           ;;       'CAMB': BEGIN
           ;;       END
           ;;       ELSE: ;; Do nothing
           ;;    ENDCASE
           ;; END
           ELSE: ;; Do nothing
        ENDCASE
        cosmop.omega_de = 1 - cosmop.omega_m - cosmop.omega_k

        ;; Units
        ;;-------
        units = {cmb:'uK_CMB',gal:'MJy/sr',thermalsz:'y_sz',kineticsz:'uK_CMB',ps:'mJy',firb:'MJy/sr'}
             
        CASE SKY_TASK() OF
           'new': BEGIN
              parameters = {global:gp, mapping:mapping, cosmo:cosmop, dipole:dipole, cmb:cmbp, gal:galp, sz:szp, ps:psp, firb:firbp, zle:zlep, units:units}
           END

           'update': BEGIN        

              ;; This case is inactivated (too complicated, and not
              ;; really useful)

              MESSAGE, "sky task is not implemented, choose between 'new' and 'restore'" 
              RESTORE_SKY_MODEL, old_model
              
              ;; Component parameters
              ;;----------------------
              dipole = READ_DIPOLE_PARAM(table, carefulness=psm_carefulness())
              IF dipole.model EQ 'existing' THEN dipole = READ_DIPOLE_PARAM(table, carefulness=psm_carefulness(), /restore)
              cmbp = READ_CMB_PARAM(table, carefulness=psm_carefulness())
              IF cmbp.model EQ 'existing' THEN cmbp = READ_CMB_PARAM(table, carefulness=psm_carefulness(), /restore)
              galp = READ_GAL_PARAM(table, carefulness=psm_carefulness())
              ;;;IF galp.model EQ 'existing' THEN galp = READ_GAL_PARAM(table, carefulness=psm_carefulness(), /restore)
              szp = READ_SZ_PARAM(table, carefulness=psm_carefulness())
              IF szp.model EQ 'existing' THEN szp = READ_SZ_PARAM(carefulness=psm_carefulness(), /restore)
              psp = READ_PS_PARAM(table, carefulness=psm_carefulness())
              IF psp.model EQ 'existing' THEN psp = READ_PS_PARAM(carefulness=psm_carefulness(), /restore)
              firbp = READ_FIRB_PARAM(table, carefulness=psm_carefulness())
              IF firbp.model EQ 'existing' THEN firbp = READ_FIRB_PARAM(table, carefulness=psm_carefulness(), /restore)
              zlep = {zle_model:'no_zle'}

              parameters = {global:old_model.global, mapping:old_model.mapping, cosmo:old_model.cosmo, dipole:dipole, cmb:cmbp, gal:galp, sz:szp, ps:psp, firb:firbp, zle:zlep, units:units}

           END 
     
           'restore': BEGIN              
              RESTORE_SKY_MODEL, model              
           END

           ELSE: MESSAGE, "sky task is not implemented, choose between 'new' and 'restore'"

        ENDCASE
        
     ENDIF ELSE BEGIN
        RESTORE_SKY_MODEL, model
     ENDELSE
     
     ;; Save the model in all cases
     SAVE_SKY_MODEL
     
     RETURN, parameters
        
END


