; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; Function to create a catalogue of thermal sources, with its header
; 
; @categories ps
; 
; @param name_cat {in}{required}{type=string} Name pf the catalogue
; @param nsources {in}{required}{type=long64} Number of sources in the catalogue 
; @param ngreybody {in}{required}{type=int} Number of greybody for the thermal emission of each source 
; @keyword units {in}{optional}{type=string} Units used for the fluxes in the catalogue (default 'mJy' for millijanskys)
; @keyword polarised {in}{optional}{type=boolean} Set this keyword to make a catalogue of polarised sources (default = not polarised)
; @keyword usenan {in}{optional}{type=boolean} Set this keyword to use IEEE 'NaN' and 'Inf' as initialisation values (default = use 0 values)
; @keyword psm_hdr {out}{optional}{type=structure} PSM header of the catalogue
;
; @history <p>30/03/2009: First version, Jacques Delabrouille</p>
; @history <p>17/04/2009: polar added in the header, Jacques Delabrouille</p>
; @copyright Jacques Delabrouille, CNRS (2009)
;-
FUNCTION CREATE_THERMALSRC_MODEL_CAT, name_cat, nsources, ngreybody, units=units, polarised=polarised, usenan=usenan, psm_hdr=psm_hdr

nmodel = ngreybody

IF NOT KEYWORD_SET(usenan) THEN BEGIN
   db = 0d0 
   fla = FLTARR(nmodel)
   maxi = FLTARR(nmodel)
ENDIF ELSE BEGIN
   db = DOUBLE('NaN')
   fla = REPLICATE(FLOAT('NaN'),nmodel)
   maxi = REPLICATE(FLOAT('Inf'),nmodel)
ENDELSE

IF NOT KEYWORD_SET(units) THEN units='mJy'

  IF NOT KEYWORD_SET(polarised) THEN BEGIN
;     source = { $
;              name:'', $
;              glon_rad:db, $
;              glat_rad:db, $
;              nuref:fla, $
;              valueref:fla, $
;              specind:fla, $
;              temp:fla, $
;              minrange:fla, $
;              maxrange:fla $
;              }
     source = { $
              name:'', $
              glon_rad:db, $
              glat_rad:db, $
              nuref:fla, $
              valueref:fla, $
              specind:fla, $
              temp:fla $
              }
  ENDIF ELSE BEGIN
;     source = { $
;              name:'', $
;              glon_rad:db, $
;              glat_rad:db, $
;              nuref:fla, $
;              valueref:fla, $
;              specind:fla, $
;              temp:fla, $
;              pol_frac: fla, $
;              pol_angle_rad: fla, $
;              minrange:FLTARR(nmodel), $
;              maxrange:maxi $
;              }
     source = { $
              name:'', $
              glon_rad:db, $
              glat_rad:db, $
              nuref:fla, $
              valueref:fla, $
              specind:fla, $
              temp:fla, $
              pol_frac: fla, $
              pol_angle_rad: fla $
              }
  ENDELSE

  IF NOT KEYWORD_SET(polarised) THEN polar=0B ELSE polar=1B
  psm_hdr = {datatype:'PS_MODEL_CAT',cat_name:name_cat,nsources:LONG64(nsources), nmodel:FIX(nmodel), em_law:REPLICATE('GREYBODY',nmodel),units:units,polar:BYTE(polar)}
  RETURN, REPLICATE(source,nsources) ;;

END


  
  

