; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jacques Delabrouille
;   Marc Betoule
;   Guillaume Castex
;   Gianfranco de Zotti
;   Joaquin Gonzalez-Nuevo
;   Samuel Leach
;   Maude Le Jeune
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; Group all the postprocessing done to the catalog compilation from Dave Clements. This includes :
;   - Conversion from RA DEC to galactic coordinates
;   - Sky coverage homogeneisation
;   - Suppression of nonthermal emission from sources (obtained from the radio catalog)
;   - Suppression of extended objects
;   - Add polarisation to the infrared sources if needed 
;   - Naming of the sources simulated to homogeneize sky coverage
; 
; @categories ps
; 
; @param param {in}{required}{type=structure} Parameters of the PSM run
; @keyword amplitude_polar {in}{required}{type=double} polarisation fraction on average, between 0 and 1. (typical values 0.01) 
; @keyword show {in}{optional}{type=boolean} if set plot some histograms
;
; @history <p>24/04/2007: First version, Marc Betoule</p>
; @history <p>26/04/2008: IR PS code now removes extended sources from the catalogue, Marc Betoule</p>
; @history <p>12/07/2007: Now reads catalog version 2.5 (with H-II regions removed) instead of 2.2, Jacques Delabrouille</p>
; @history <p>07/11/2007: Remove useless strongdec keyword, Maude Le Jeune</p>
; @history <p>18/03/2008: New input catalog, Maude Le Jeune</p>
; @history <p>07/04/2009: Major rewriting for PSM v1.7, Jacques Delabrouille</p>
;-

PRO POSTPROCESSIRASCAT, amplitude_polar=amplitude_polar, show=show

  COMMON psm, param
  COMMON sky, model

  gp = param.global
  runp = param.run
  names = param.names

;; Initial seed for infrared point source pipeline
;;-------------------------------------------------
  SET_PSM_SEEDCHAIN, 'ps-pipe-ir'

;; Reading the Ancillary catalog
;;-------------------------------

  IF runp.verbosity GE 2 THEN PSM_INFO_MESSAGE, "Reading input catalogue of IRAS thermal sources"
  dir = GETPSMDIR('psmdata')+'ancillary/observations/IRAS/' 
  iraspsfile = dir+'planck_input_therm_cat1.0.fits'
  ok = FILE_TEST(iraspsfile)
  IF NOT ok THEN GET_PSM_DATA, datafile=iraspsfile
  cat = MRDFITS(iraspsfile,1)
  ;; Identification of extended sources
  index = LONARR(n_elements(cat))
  i = WHERE( STRTRIM(cat.name,2) eq 'SMC')
  if i ge 0 then index [i] = 1 ;;Extended nearby galaxy
  i = WHERE( STRTRIM(cat.name,2) eq 'NGC 6729')
  if i ge 0 then index [i] = 1 ;;resolved ? (Diffuse galactic nebula)
  i = WHERE( STRTRIM(cat.name,2) eq 'NGC 7023')
  if i ge 0 then index [i] = 1 ;;resolved ? (Galactic Open Cluster)
  i = WHERE( STRTRIM(cat.name,2) eq 'IRAS 06084-0611')
  if i ge 0 then index [i] = 1 ;; Bright galactic IRAS  source about 5'x10'
  i = WHERE( STRTRIM(cat.name,2) eq 'MESSIER 031')
  if i ge 0 then index [i] = 1 ;;Extended nearby galaxy
  i = WHERE( STRTRIM(cat.name,2) eq 'MESSIER 033')
  if i ge 0 then index [i] = 1 ;;Extended nearby galaxy
  i = WHERE( STRTRIM(cat.name,2) eq 'IRAS 06581-0848')
  if i ge 0 then index [i] = 1 ;;Bright galactic HII region 5'x10'
  i = WHERE( STRTRIM(cat.name,2) eq 'LMC')
  if i ge 0 then index [i] = 1 ;;Extended nearby galaxy
  i = WHERE( STRTRIM(cat.name,2) eq 'PKS 0605-06')
  if i ge 0 then index [i] = 1 ;;Extended galactic HII region 7'x10'
  i = WHERE( STRTRIM(cat.name,2) eq 'NGC 6744')
  if i ge 0 then index [i] = 1 ;;resolved ? Bright nearby galaxy 
  i = WHERE( STRTRIM(cat.name,2) eq '2MASX J05304609+3347536')
  if i ge 0 then index [i] = 1 ;; resolved ?? (extended galactic source)
  i = WHERE( STRTRIM(cat.name,2) eq 'NGC 2071 NED01')
  if i ge 0 then index [i] = 1 ;; galactic reflection nebula
  i = WHERE( STRTRIM(cat.name,2) eq 'MESSIER 082')
  if i ge 0 then index [i] = 1 ;; nearby galaxy
  i = WHERE( STRTRIM(cat.name,2) eq 'NGC 1333 NED01') 
  if i ge 0 then index [i] = 1 ;; galactic reflection nebula
  i = WHERE( STRTRIM(cat.name,2) eq 'IRAS 18583-3657')
  if i ge 0 then index [i] = 1 ;; bright iras source near NGC6729
  
  extcat = cat[WHERE(index EQ 1)]                                         ;; extended sources

  i = WHERE((cat._60micron LE 0.) AND (cat._100micron LE 0.), nwh)        ;; nonthermal sources
  IF nwh NE 0 THEN index[i] = 1

  ;; Keep only non-extended sources which have a thermal component (observed by IRAS at 60 or 100 microns)
  cat = cat[WHERE(index EQ 0)]                                            ;; other sources
  index = 0                                                               ;; free memory

  ;; Find temperature outliers and fix them
  whcrap = WHERE(cat.thermal_fit_temp GT 200, nwh) 
  IF nwh NE 0 THEN cat[whcrap].thermal_fit_temp = 200.


;; Do the IRAS DATA CATALOGUE
;;----------------------------

  IF runp.verbosity GE 2 THEN PSM_INFO_MESSAGE, "Putting catalogue of IRAS thermal sources in appropriate data format"

  iras = LOAD_INSTRUMENT('IRAS_IDEAL')

  freqiras = [iras.channel_1.band.nu_c,iras.channel_2.band.nu_c]
  bandiras = [iras.channel_1.band,iras.channel_2.band]

  DESTROY_DATA, iras

  nsources = N_ELEMENTS(cat)
  units = 'mJy'                                                           ;; warning, this is assumed later, DO NOT CHANGE IT

  iras_data_cat = CREATE_PS_DATA_CAT(nsources, bandiras, name='IRAS Ancillary PS catalog')

  ;; RA-Dec (2000) to galactic
  dtor = !DPI/180D0
  EULER, cat.ra*dtor, cat.dec*dtor, glon_rad, glat_rad, 1, /radian

  *(iras_data_cat.lon.value) = TEMPORARY(glon_rad)                                  
  *(iras_data_cat.lat.value) = TEMPORARY(glat_rad)                                  
  *(iras_data_cat.objectname) = cat.name

  ;; Assumed errors in %
  flux60_error_percentage = 5.                                            ;; assume 5% error
  flux100_error_percentage = 5.                                           ;; assume 5% error
  specind_error_percentage = 10.                                          ;; assume 10% error
  temp_error_percentage = 10.                                             ;; assume 10% error

  ;; Assign data values for sources with a 60 micron flux
  wh60ok = WHERE(cat._60micron GT 0., nwh60ok)

  *(iras_data_cat.obs2.iflux) = REPLICATE(FLOAT('NAN'),nsources)
  *(iras_data_cat.obs2.ierr) = REPLICATE(FLOAT('NAN'),nsources)
  (*(iras_data_cat.obs2.iflux))[wh60ok] = cat[wh60ok]._60micron * 1e29           ;; flux converted from W/m2/Hz to mJy
  (*(iras_data_cat.obs2.ierr))[wh60ok] = (*(iras_data_cat.obs2.iflux))[wh60ok] * (flux60_error_percentage/100.)     

  ;; Assign data values for sources with a 100 micron flux
  wh100ok = WHERE(cat._100micron GT 0., nwh100ok)

  *(iras_data_cat.obs1.iflux) = REPLICATE(FLOAT('NAN'),nsources)
  *(iras_data_cat.obs1.ierr) = REPLICATE(FLOAT('NAN'),nsources)
  (*(iras_data_cat.obs1.iflux))[wh100ok] = cat[wh100ok]._100micron * 1e29        ;; flux converted from W/m2/Hz to mJy
  (*(iras_data_cat.obs1.ierr))[wh100ok] = (*(iras_data_cat.obs1.iflux))[wh100ok] * (flux100_error_percentage/100.)   

  ;; Assign units (mJy)
  iras_data_cat.obs1.unit = 'mJy'
  iras_data_cat.obs2.unit = 'mJy'

;; Do the MODEL CATALOGUE
;;------------------------

  ngreybody = 1

  IF runp.verbosity GE 2 THEN PSM_INFO_MESSAGE, "Modelling thermal sources"
  ;;Allocate the output catalog
  IF NOT keyword_set(amplitude_polar) THEN BEGIN
     thermal_model_cat = CREATE_THERMALSRC_MODEL_CAT('IR PS Model Catalog', nsources, ngreybody, units=units, /usenan, psm_hdr=thermal_model_header)
  ENDIF ELSE BEGIN
     thermal_model_cat = CREATE_THERMALSRC_MODEL_CAT('IR PS Model Catalog', nsources, ngreybody, units=units, /polarised, /usenan, psm_hdr=thermal_model_header)
  ENDELSE

  ;;Assign model catalogue values for real sources from data catalogue
  thermal_model_cat[0:nsources-1].name = *(iras_data_cat.objectname)                 
  thermal_model_cat[0:nsources-1].glon_rad = (*iras_data_cat.lon.value)                 
  thermal_model_cat[0:nsources-1].glat_rad = (*iras_data_cat.lat.value)             
  thermal_model_cat[wh60ok].nuref = freqiras[1]
  thermal_model_cat[wh60ok].valueref[0] = cat[wh60ok]._60micron * 1e29         ;; flux converted from W/m2/Hz to mJy
  thermal_model_cat[wh60ok].specind[0] = cat[wh60ok].thermal_fit_beta          ;; 
  thermal_model_cat[wh60ok].temp[0] = cat[wh60ok].thermal_fit_temp             ;; 

  wh100only = WHERE((cat._60micron LE 0.) AND (cat._100micron GT 0.), nwh100only)
  thermal_model_cat[wh100only].nuref = freqiras[0]
  thermal_model_cat[wh100only].valueref[0] = cat[wh100only]._100micron * 1e29  ;; flux converted from W/m2/Hz to mJy
  thermal_model_cat[wh100only].specind[0] = cat[wh100only].thermal_fit_beta    ;; 
  thermal_model_cat[wh100only].temp[0] = cat[wh100only].thermal_fit_temp       ;; 


  ;; Different models: prediction or simulation
  ;;--------------------------------------------
  CASE model.ps.model OF
     
     ;;---------------------------------------------------------------------------------------------------
     ;; SIMULATION
     ;;---------------------------------------------------------------------------------------------------

     'simulation': BEGIN

     ;; Create and add fake sources for coverage homogeneisation (if needed)
     ;;----------------------------------------------------------------------
  

        ;; Make bins in flux with fixed number of sources, unless fluxes are too widely spread, in which case make bins within fixed flux ratio
  
        ;; First, limit the catalogue of considered sources to those below some limit flux
        ;; Above limitfluxmjy the IRAS catalogue at 60 microns is assumed complete
        
        IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "COVERAGE HOMOGENEISATION"
        
        limitfluxmjy = 1000.          
        
        wh_flux_low_enough = WHERE((*iras_data_cat.obs2.iflux)[wh60ok] LE limitfluxmjy)
        temp_valueref = thermal_model_cat[wh60ok[wh_flux_low_enough]].valueref
        temp_specind = thermal_model_cat[wh60ok[wh_flux_low_enough]].specind
        temp_temp = thermal_model_cat[wh60ok[wh_flux_low_enough]].temp
        temp_theta = !DPI/2D0 - thermal_model_cat[wh60ok[wh_flux_low_enough]].glat_rad
        temp_phi = thermal_model_cat[wh60ok[wh_flux_low_enough]].glon_rad
        
        ;; Compute source density in bins of flux
        numbermax = 50000
        valuefactormax = 2
        bins = MAKE_SORTED_BINS(temp_valueref, numbermax, valuefactormax, nbins=nbins, n_in_bin=n_in_bin)
        
        ;; For each bin, except those with few sources, create fake sources to homogeneize the coverage
        
        numbermin = 1000
        
        FOR b=0, nbins-1 DO BEGIN
           IF n_in_bin[b] GE numbermin THEN BEGIN
              
              ;; Find distribution of sources on the sky (number of sources per pixel at nside_dens)
              nside_dens = 32L
              wh = WHERE(bins EQ b)
              flux = temp_valueref[wh]
              specind = temp_specind[wh]
              temp = temp_temp[wh]
              IF runp.verbosity GE 2 THEN PSM_INFO_MESSAGE, 'IRAS 60 microns catalogue: computing source density for coverage homogeneisation between '+ $
                 NUMBER2STRING(MIN(flux))+' and '+ NUMBER2STRING(MAX(flux))+' mJy'
              theta_i_rad = temp_theta[wh]
              phi_i_rad = temp_phi[wh]
            ;;; stat = HEALPIX_DENSITY(theta_i_rad, phi_i_rad, nside_dens, threshold=100, /visu)
              stat = HEALPIX_DENSITY(theta_i_rad, phi_i_rad, nside_dens, threshold=100)
              
              ;; Define the maximum value of stat
              ;; Use the max of stat, above 30 degrees galactic latitude, and at longitudes between 0 and 180
              PIX2ANG_NEST, nside_dens, LINDGEN(N_ELEMENTS(stat)), theta, phi
              truc = TEMPORARY(stat)*(TEMPORARY(theta) LE 45.*!DTOR) ;*(TEMPORARY(phi) LE !PI)
              avgdens = MAX(TEMPORARY(truc))
              mean_source_n = avgdens
              
              ;; Compute the source number count at nside = nside_dens
              stat2 = HEALPIX_DENSITY(theta_i_rad, phi_i_rad, nside_dens, threshold=1)
              
              ;; Draw a number of sources on the sky according to this density per pixel
              npix_dens = nside_dens^2 * 12L
              n2add = RANDOMU(GETSEED(), npix_dens, poisson=mean_source_n, /double)
              n2add = (n2add - TEMPORARY(stat2)) > 0
              
              ;; To distribute the extra sources, use the nested property of healpix
              ;; Get nside for coordinates based on a finer healpix grid
              
              nside_extra = LONG64(SKY_NSIDE())*4LL
              ;; resol_extra = model.global.sky_resolution / 5. 
              ;; npix_extra = 4*!PI / !DTOR^2 * 3600 / resol_extra^2
              ;; nside_extra = 2L^CEIL(ALOG(SQRT(npix_extra/12.))/ALOG(2.))
              
              ;; Distribute randomly drawn sources in each pixel at nside_dens onto pixels at nside_extra
              npix_per_cell = ((nside_extra/nside_dens)^2) > 1LL
              total2add = TOTAL(DOUBLE(n2add))
              pixnum = LON64ARR(total2add)
              currentnum = 0L
              randomu_draw = RANDOMU(GETSEED(), total2add)
              FOR p=0LL, npix_dens-1 DO BEGIN
                 nn = n2add[p]
                 IF nn NE 0 THEN BEGIN
                    pixnum[currentnum:currentnum+nn-1] = p * npix_per_cell + LONG64(npix_per_cell * randomu_draw[currentnum:currentnum+nn-1])
                    currentnum = currentnum + nn
                 ENDIF
              ENDFOR
              randomu_draw = 0B ;free memory
              
              ;; Convert pixel number to galactic coordinates to get
              ;; coordinates of simulated sources
              PIX2ANG_NEST, nside_extra, TEMPORARY(pixnum), glat_rad, glon_rad
              glat_rad = !DPI/2D0 - TEMPORARY(glat_rad)
              
              ;; Now make the catalogue of extra sources
              IF NOT keyword_set(amplitude_polar) THEN BEGIN
                 extra_model_cat = CREATE_THERMALSRC_MODEL_CAT('IR PS Model Catalog', total2add, ngreybody, units=units, /usenan, psm_hdr=extra_model_header)
              ENDIF ELSE BEGIN
                 extra_model_cat = CREATE_THERMALSRC_MODEL_CAT('IR PS Model Catalog', total2add, ngreybody, units=units, /polarised, /usenan, psm_hdr=extra_model_header)
              ENDELSE
              
              ;; Fill in coordinates
              extra_model_cat.glon_rad = glon_rad                 
              extra_model_cat.glat_rad = glat_rad 
              
              
              ;; Copy properties from existing sources in bin
              which_source = LONG(RANDOMU(GETSEED(),total2add) * n_in_bin[b])
              extra_model_cat.nuref = freqiras[1]
              extra_model_cat.valueref[0] = flux[which_source]                   ;; flux converted from W/m2/Hz to mJy
              extra_model_cat.specind[0] = specind[which_source]                 ;; 
              extra_model_cat.temp[0] = temp[which_source]                       ;; 
              
              ;; Add extra_model_cat to existing thermal_model_cat
              thermal_model_cat = [thermal_model_cat,extra_model_cat]           
              
              ;; Update number of point sources in the thermal_model_cat header
              thermal_model_header.nsources = thermal_model_header.nsources + extra_model_header.nsources
              
           ENDIF
        ENDFOR
        
        ;; Fill in the model catalogue
        
        ;; Give a name to all extra sources 
        n_extra = thermal_model_header.nsources - iras_data_cat.nsources
        thermal_model_cat[iras_data_cat.nsources : iras_data_cat.nsources+n_extra-1L].name = 'IR_DUPL_'+NUMBER2STRING(LINDGEN(n_extra))
        
        ;; add uncertainties: flux at 60 microns
        wh60ok = WHERE(FLOAT(thermal_model_cat.nuref) EQ FLOAT(freqiras[1]), nwh60ok)
        draw = RANDOMN(GETSEED(), nwh60ok)
        thermal_model_cat[wh60ok].valueref = thermal_model_cat[wh60ok].valueref * (1. + flux60_error_percentage/100.)^draw
        
        ;; add uncertainties: flux at 100 microns
        draw = RANDOMN(GETSEED(), nwh100only)
        thermal_model_cat[wh100only].valueref = thermal_model_cat[wh100only].valueref * (1.+flux100_error_percentage/100.)^draw
        
        ;; add uncertainties: spectral indices
        draw = RANDOMN(GETSEED(), thermal_model_header.nsources)
        thermal_model_cat.specind[0] = thermal_model_cat.specind[0] * (1. + specind_error_percentage/100.)^draw
        
        ;; add uncertainties: temperatures
        draw = RANDOMN(GETSEED(), thermal_model_header.nsources)
        thermal_model_cat.temp = thermal_model_cat.temp * (1. + temp_error_percentage/100.)^draw
        
        ;; add polarisation properties if necessary
        IF KEYWORD_SET(amplitude_polar) THEN BEGIN
           thermal_model_cat[*].pol_frac[0] = ((amplitude_polar * (RANDOMN(GETSEED(),thermal_model_header.nsources))^2) < 1.) *100.
           thermal_model_cat[*].pol_angle_rad[0] = !PI*RANDOMU(GETSEED(),thermal_model_header.nsources)
        ENDIF
     
     END
     
     ;;---------------------------------------------------------------------------------------------------
     ;; PREDICTION
     ;;---------------------------------------------------------------------------------------------------

     'prediction': BEGIN
        
        ;; Do nothing
     END

   
  ENDCASE
  ;;---------------------------------------------------------------------------------------------------------------------------

     
;; Create the new model catalogue
;;-------------------------------
   
  ;; Select which sources are strong and which ones are faint
  ;;----------------------------------------------------------

  nfreq = N_ELEMENTS(model.ps.strong_ps_limit_freq_ghz)
  source_is_strong = BYTARR(N_ELEMENTS(thermal_model_cat))
  
  FOR i=0, nfreq-1 DO BEGIN
     nu_c = 1e9*model.ps.strong_ps_limit_freq_ghz[i]
     fluxlimitmJy = 1000*model.ps.strong_ps_limit_flux_jy[i]
     result = PSM_EMLAW(nu_c,emission_name='greybody',nuref=(thermal_model_cat.nuref)[*],$
                       temp=(thermal_model_cat.temp)[*],specind=(thermal_model_cat.specind)[*])
     result = TEMPORARY(result) * (thermal_model_cat.valueref)[*]
     wh = WHERE(result GE fluxlimitmJy, nwh)
     IF nwh NE 0 THEN source_is_strong[wh] = 1B
  ENDFOR

  whstrong = WHERE(source_is_strong, nstrong, complement=whfaint, ncomplement=nfaint)   
  
  ;; Use same nuref for all sources in the model catalogue
  ;; Compute flux at that reference frequency
  ;;-------------------------------------------------------
  nuref = 1e12
  valueref = PSM_EMLAW(nuref,emission_name='greybody',nuref=(thermal_model_cat.nuref)[*],$
                       temp=(thermal_model_cat.temp)[*],specind=(thermal_model_cat.specind)[*])
  valueref = TEMPORARY(valueref) * (thermal_model_cat.valueref)[*]
  
  ;; Strong sources
  ;;----------------
  IF nstrong NE 0 THEN BEGIN
     newcat1 = CREATE_PS_MODEL_CAT(nstrong, ['greybody'], name='strong'+names.irps_basename, id=id, polarised=KEYWORD_SET(amplitude_polar))
     *(newcat1.lon.value) = (thermal_model_cat.glon_rad)[whstrong]
     *(newcat1.lat.value) = (thermal_model_cat.glat_rad)[whstrong]
     *newcat1.objectname = (thermal_model_cat.name)[whstrong]
     *newcat1.e1.numin.value = 0.
     *newcat1.e1.numax.value = FLOAT('INF')
     newcat1.e1.ampl.unit = 'mJy'
     *newcat1.e1.nuref.value = nuref
     *newcat1.e1.specind.value = REFORM((thermal_model_cat.specind)[0,whstrong])
     *newcat1.e1.temp.value = REFORM((thermal_model_cat.temp)[0,whstrong])
     IF NOT KEYWORD_SET(amplitude_polar) THEN BEGIN
        *newcat1.e1.ampl.value = valueref[whstrong]
     ENDIF ELSE BEGIN
        apol = REFORM(thermal_model_cat.pol_frac)/100
;       CASE newcat1.polar_model OF 
;          'GLOBAL': BEGIN
        *newcat1.pol_fraction = apol[whstrong]
        *newcat1.pol_angle = (thermal_model_cat.pol_angle_rad)[whstrong]
;          END
;          'PER_EMISSION': BEGIN
        *newcat1.e1.ampl.value = [[valueref[whstrong]], $
                                  [valueref[whstrong]]*[apol[whstrong]*REFORM(COS(2.*(thermal_model_cat.pol_angle_rad)[whstrong]))], $
                                  [valueref[whstrong]]*[apol[whstrong]*REFORM(SIN(2.*(thermal_model_cat.pol_angle_rad)[whstrong]))]]
;          END
;       ENDCASE
        apol=0      
     ENDELSE
     filename = GETPSMDIR('ps') + 'strong'+names.irps_basename + '.sav'
     WRITE_CATALOG, newcat1, filename, /sav, name='comp'
     DESTROY_DATA, newcat1
  ENDIF
  
  ;; Faint sources
  ;;---------------
  IF nfaint NE 0 THEN BEGIN
     newcat2 = CREATE_PS_MODEL_CAT(nfaint, ['greybody'], name='faint'+names.irps_basename, id=id, polarised=KEYWORD_SET(amplitude_polar))
     *(newcat2.lon.value) = (thermal_model_cat.glon_rad)[whfaint]
     *(newcat2.lat.value) = (thermal_model_cat.glat_rad)[whfaint]
     *newcat2.objectname = (thermal_model_cat.name)[whfaint]
     *newcat2.e1.numin.value = 0.
     *newcat2.e1.numax.value = FLOAT('INF')
     newcat2.e1.ampl.unit = 'mJy'
     *newcat2.e1.nuref.value = nuref    
     *newcat2.e1.specind.value = REFORM((thermal_model_cat.specind)[0,whfaint])
     *newcat2.e1.temp.value = REFORM((thermal_model_cat.temp)[0,whfaint])
     IF NOT KEYWORD_SET(amplitude_polar) THEN BEGIN
        *newcat2.e1.ampl.value = valueref[whfaint]
     ENDIF ELSE BEGIN
        apol = REFORM(thermal_model_cat.pol_frac)/100
;       CASE newcat2.polar_model OF 
;          'GLOBAL': BEGIN
        *newcat2.pol_fraction = apol
        *newcat2.pol_angle = (thermal_model_cat.pol_angle_rad)[whstrong]
;          END
;          'PER_EMISSION': BEGIN
        *newcat2.e1.ampl.value = [[valueref[whfaint]], $
                                  [valueref[whfaint]]*[apol[whfaint]*REFORM(COS(2.*(thermal_model_cat.pol_angle_rad)[whfaint]))], $
                                  [valueref[whfaint]]*[apol[whfaint]*REFORM(SIN(2.*(thermal_model_cat.pol_angle_rad)[whfaint]))]]
;          END
;       ENDCASE
        apol=0      
     ENDELSE
     filename = GETPSMDIR('ps') + 'faint'+names.irps_basename + '.sav'
     WRITE_CATALOG, newcat2, filename, /sav, name='comp'
     DESTROY_DATA, newcat2
  ENDIF
  
  IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE, "Catalogues of infrared sources written: "+filename
  
;; Write the IRAS ancillary catalogue if needed
;;----------------------------------------------
  
  IF param.tasks.write_ancillary EQ 'yes' THEN BEGIN
     dir = GETPSMDIR('ancillary')
     dir_exists = FILE_TEST(dir, /directory)
     IF NOT dir_exists THEN FILE_MKDIR, dir
     filename = FIX_SEPARATOR(dir) + names.irps_basename + '_ancil_cat.sav'
     WRITE_CATALOG, iras_data_cat, filename, /sav
     IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE, 'Ancillary infrared source data catalogue written: '+filename
  ENDIF
  
END
