; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jacques Delabrouille
;   Marc Betoule
;   Guillaume Castex
;   Maude Le Jeune
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; Provides a wrapper to the ps simulation code
;
; @categories pipe
;
; @param param {in}{required}{type=structure} Parameters of the psm run
; @keyword test {in}{optional}{type=boolean} If set, no computation
; done, just check if the set of parameters are ok
; @keyword debug {in}{optional}{type=boolean} If set no catch on error
; done, subdirectory components/ps is not removed when an error occurs
;
; @history <p>06/02/2007: First version, Marc Betoule </p>
; @history <p>08/03/2007: Add the test keyword, Marc Betoule </p>
; @history <p>12/07/2007: Seed is an input parameter and is set at psm_main, Maude le Jeune </p>
; @history <p>06/11/2007: Remove old ps models, Maude le Jeune </p>
; @history <p>28/03/2008: Add catalog of strong sources if exists, Maude le Jeune </p>
; @history <p>21/01/2009: PSM observation parameters now an input, Jacques Delabrouille </p>
; @history <p>15/04/2009: Complete revision for v1.7, Jacques Delabrouille </p>
; @copyright Marc Betoule, Jacques Delabrouille, Maude le Jeune, CNRS, Observatoire de Paris
;-

PRO PS_PIPE, debug=debug

  COMMON psm, param
  COMMON sky, model

;; Define parameters for convenience
;;-----------------------------------
  gp = param.global
  psp = model.ps
  names = param.names
  runp = param.run
  pixp = model.mapping.pix

;; Error handling : remove ps component directory when catching an error
;;----------------------------------------------------------------------
  componentdir = GETPSMDIR('ps') 
  IF ((NOT KEYWORD_SET(debug))) THEN BEGIN 
     CATCH, error_status
     IF error_status NE 0 THEN BEGIN
        PRINT, 'An error was encoutered while computing ps maps.'
        PRINT, 'Removing subdirectory ' +componentdir
        SPAWN, 'rm -r '+componentdir
        CATCH, /CANCEL
        MESSAGE, !ERR_STRING
     ENDIF
  ENDIF

  IF (STRMID(STRLOWCASE(psp.model),0,3) ne 'no_') THEN BEGIN
     psdirexists = FILE_TEST(componentdir, /directory)
     IF NOT psdirexists THEN SPAWN, 'mkdir '+componentdir
  ENDIF

;; Generation of the point source model
;;--------------------------------------
  
  CASE STRLOWCASE(psp.model) OF
   'prediction': BEGIN
      ;MESSAGE, '"' + psp.model + '" Point Source prediction model not implemented, sorry'      
      IF KEYWORD_SET(test) THEN RETURN

      IF gp.fields NE 'T' THEN polarised = 1 ELSE polarised = 0 

      ;; RADIO SOURCES
      ;;---------------
      IF STRLOWCASE(psp.include_radio_sources) EQ 'yes' THEN BEGIN
         IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CATALOGUE OF RADIO SOURCES"
         PSM_PS_MULTIFREQ_RADIO, /new_cat, polarised=polarised
      ENDIF

      ;; IR SOURCES
      ;;------------
      IF STRLOWCASE(psp.include_ir_sources) EQ 'yes' THEN BEGIN
         IF gp.fields EQ 'TP' THEN amplitude_polar = psp.mean_ir_polar_degree
         ;; Creation of the iras data and ir model catalogs
         IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CATALOGUE OF INFRARED SOURCES"         
         POSTPROCESSIRASCAT, amplitude_polar = amplitude_polar
      ENDIF

      ;; H-II REGIONS
      ;;--------------
      IF STRLOWCASE(psp.include_uchii_sources) EQ 'yes' THEN BEGIN
         IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CATALOGUE OF ULTRACOMPACT H-II REGIONS"         
         INTERPOLATE_HII_CATALOG
      ENDIF

      ;; ERCSC SOURCES
      ;;--------------
      IF STRLOWCASE(psp.include_ercsc_sources) EQ 'yes' THEN BEGIN
         IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CATALOGUE OF ERCSC"
         PSM_ERCSC_SOURCES, polarised=polarised
      ENDIF

   END
   'simulation': BEGIN
      IF KEYWORD_SET(test) THEN RETURN
      
      IF gp.fields NE 'T' THEN polarised = 1 ELSE polarised = 0 
   
      ;; RADIO SOURCES
      ;;---------------
      IF STRLOWCASE(psp.include_radio_sources) EQ 'yes' THEN BEGIN
         IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CATALOGUE OF RADIO SOURCES"
         PSM_PS_MULTIFREQ_RADIO, /new_cat, polarised=polarised
      ENDIF

      ;; IR SOURCES
      ;;------------
      IF STRLOWCASE(psp.include_ir_sources) EQ 'yes' THEN BEGIN
         IF gp.fields EQ 'TP' THEN amplitude_polar = psp.mean_ir_polar_degree
         ;; Creation of the iras data and ir model catalogs
         IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CATALOGUE OF INFRARED SOURCES"         
         POSTPROCESSIRASCAT, amplitude_polar = amplitude_polar
      ENDIF

      ;; H-II REGIONS
      ;;--------------
      IF STRLOWCASE(psp.include_uchii_sources) EQ 'yes' THEN BEGIN
         IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CATALOGUE OF ULTRACOMPACT H-II REGIONS"         
         INTERPOLATE_HII_CATALOG
      ENDIF

      ;; ERCSC SOURCES
      ;;--------------
      IF STRLOWCASE(psp.include_ercsc_sources) EQ 'yes' THEN BEGIN
         IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "GENERATING CATALOGUE OF ERCSC"
         PSM_ERCSC_SOURCES, polarised=polarised
      ENDIF

      ;; IR BACKGROUND
      ;;---------------
;;       IF STRLOWCASE(psp.include_firb) EQ 'yes' THEN BEGIN

;;          IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "MODELING INFRARED BACKGROUND" 

;;          SET_PSM_SEEDCHAIN, 'ps-pipe-firb'
;;          ;; Define list of useful frequencies
;;          ;;-----------------------------------

;;            ;; Define a set of frequencies at which the FIRB is written
;;            ;; The density (in log scale) is bigger in the 100-1000 GHz range, where the FIRB is likely to be observed by Planck
;;            ;; freq = [1.,5.,25.,100.,250.,500.,1000.,3000.]*1e9

;;            freq = [30.,44.,70.,100.,143.,217.,353.,545.,857.]*1e9

;;            ;; Produce a FIRB map at all requested frequencies
;;            ;;-------------------------------------------------
;;            Slim = 1e32  ;; very high to avoid writing a catalog.
;;            i = 0

;;            ;; Generate a data ID number
;;            PSM_ID_KEYGEN, firb_key

;;            ;; Create the directory for the FIRB model
;;            firbdir = GETPSMDIR('firb')
;;            dir_exists = FILE_TEST(firbdir)
;;            IF dir_exists THEN BEGIN
;;               ;; check that it is a directory
;;               dir_exists = FILE_TEST(firbdir,/directory)
;;               IF dir_exists THEN BEGIN
;;                  ;; Empty directory if needed
;;                  CASE STRLOWCASE(runp.clear_all) OF 
;;                     'yes': BEGIN
;;                        fitsfiles = FILE_SEARCH(firbdir+'*.fits')
;;                        FOR i=0, N_ELEMENTS(fitsfiles)-1 DO BEGIN
;;                           IF FILE_BASENAME(fitsfiles(i)) NE '' THEN FILE_DELETE, fitsfiles(i), /quiet
;;                        ENDFOR
;;                     END
;;                     'no': 
;;                  ENDCASE
;;               ENDIF ELSE BEGIN
;;                  MESSAGE, firbdir + ' exists and is not a directory'
;;               ENDELSE
;;            ENDIF ELSE BEGIN
;;               SPAWN, 'mkdir '+firbdir
;;            ENDELSE

;;            ;; Do the FIRB maps
;;            MESSAGE, 'WARNING: Update the writing of the FIRB maps', /info
;;            f = FLOAT(freq/1e9) ;in GHz

;;            ;; Generate a new map and catalogue if i=0 and if a new FIRB is requested
;;            IF ((psp.new_firb EQ 'yes') AND (i EQ 0)) THEN $
;;               PSM_PS_IRB, pixp.healpix_nside, f, Slim, /new_ls_cat, /new_sph_sim $
;;            ELSE $
;;               PSM_PS_IRB, pixp.healpix_nside, f, Slim
                                    
;;       ENDIF



;; OBSOLETE
;;----------


      ;; Save point source parameters
      ;;------------------------------
      ;;IF psp.model NE 'no_ps' THEN SAVE, psp, filename=GETPSMDIR('ps')+'ps_model_param.sav'

    
   END
ENDCASE

closing: RETURN
END
