; -----------------------------------------------------------------------------
;
; Copyright (C) 2007, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CLUSTER_TEMP2MASS
;
; PURPOSE:
;	Compute the mass of a galaxy cluster given its temperature.
;	This function is the reverse function of CLUSTER_MASS2TEMP. 
;
; CATEGORY:
;	Sky model (clusters)
;
; CALLING SEQUENCE:
;	result  = CLUSTER_TEMP2MASS(temp, z)
;
; INPUTS:
;	Temp:	The cluster temperature in keV (can be an array).
;	Z:	The cluster redshift ( can be an array of same
;	        dimension as Temp).
;
; KEYWORD PARAMETERS:
;	COSMO:   	Structure containing the cosmological parameters.
;	CLUSTER:	Structure containing the structure formation parameters (cf. DEFINE_CLUSTER).
;
; OUTPUTS:
;	The output is the cluster mass, in units of 10^15 solar masses.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille and Jean-Baptiste Melin
;       delabrouille@apc.univ-paris7.fr
;       jean-baptiste.melin@cea.fr
;
; MODIFICATION HISTORY:
; 	08/03/2007:	Jean-Baptiste Melin, First version, included in
; 	                the ANCILLARY2INTERMEDIATE_CLUSTER_CATALOG routine.
; 	08/01/2012:	Jacques Delabrouille, now standalone routine, doc and PSM license header.
;-

FUNCTION CLUSTER_TEMP2MASS, temp, z, cosmo=cosmo, cluster=cluster

  nz = N_ELEMENTS(z)
  mass = FLTARR(nz)
  
  step_log10 = 1.
  M_inf=0.001                   ; in 10^15 solar mass units
  M_sup = 1000.                 ; in 10^15 solar mass units
  log10_m = (FINDGEN(CEIL((ALOG10(M_sup)-ALOG10(M_inf))*step_log10)+1)/step_log10)+ALOG10(M_inf)
  M_array = EXP(ALOG(10)*log10_m)
  nm = N_ELEMENTS(M_array)
  
  FOR i=0, nz-1 DO BEGIN
     z_array = REPLICATE(z[i], nm)
     cat = CLUSTER_MODEL(z_array, M_array, cosmo=cosmo, cluster=cluster, /Te)
     wh = where(cat.te LE 0, nwh)
     IF nwh GE 1 THEN print, 'Te', i
     wh = where(temp LE 0, nwh)
     IF nwh GE 1 THEN print, 'temp', i
     mass[i] = EXP(INTERPOL(ALOG(M_array),ALOG(cat.te),ALOG(temp[i])))
  ENDFOR
  
  RETURN, mass
  
END

