; -----------------------------------------------------------------------------
;
; Copyright (C) 2002, 2006, 2008, 2010
;   Alexandre Boucaud
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;  FAST_INT_Y
;
; PURPOSE:
;  compute quickly the integrated Y of the clusters of masses M and redshift z
;
; CALLING SEQUENCE:
;  result = FAST_INT_Y(z_array, M_array)
;
; INPUTS:
;  Z_array : the array containing cluster redshifts z
;  M_array : the array containing cluster masses M (in 1d15*M_sun units)
;
; KEYWORDS:
;  COSMO :               the structure containing the cosmological parameters
;  CLUSTER :             the structure containing the structure formation
;                        parameters (cf. DEFINE_CLUSTER)
;  CHANDRA :             set this keyword if you want the experimental profile
;                        from chandra survey
;  CHANDRA, XMM :        if set, don't use the beta profile but the nfw profile of Chandra or XMM
;  NORM :                if set, use the normalisation found in the observations for cluster profiles
;  NFW_A, NFW_B, NFW_C : parameters of the NFW generalized profile (you must set (norm or nr500) and xmm/chandra keywords to use these keywords)
;  C500 :                concentration rs/r500 of the pressure profile (you must set (norm or nr500) and xmm/chandra keywords to use this keyword)
;  P0 :                  normalisation of the pressure profile (you must set (norm or nr500) and xmm/chandra keywords to use this keyword)
;  CM500 :               concentration rs/r500 of the matter profile (you must set (norm or nr500) and xmm/chandra keywords to use this keyword)
;  NR500 :               integration is performed up to n*r500 instead of rvir (you must set the xmm or chandra keyword to use this keyword)
;  NSTD :                non standard XMM profile (you must set the xmm to use this keyword)
;  CCO :                 cool core profile (if nstd keyword is set)
;  MDI :                 morphologically disturbed profile (if nstd keyword is set)
;
; OUTPUTS:
;  The integrated Y parameter (arcmin^2)
;
; EXAMPLES:
;
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  BETA_PROFILE, ANG_DIST, HUBBLE_AT_Z
; REVISION HISTORY
;  Written, Jean-Baptiste Melin, March 19 2002
;  cluster keyword added, Feb 3 2006
;  Big Delta_c introduced, JBM, March 7 2006
;  chandra, xmm, norm keywords added, JBM & Alexandre Boucaud June 26 2008
;  cosmetic improvements, JBM, July 1 2008
;  norm case coded, JBM, July 24 2008
;  nfw_abc, c500, P0 and cm500 keywords added, JBM, Oct. 6 2008
;  code structure rewritten for norm case, JBM, Oct. 9 2008
;  nr500 keyword added, JBM,  Dec 2 2008
;  nr500 keyword can be used without the norm keyword, JBM, Dec 7 2008
;  nstd keyword added, JBM, Sept 13 2010
;  cco and mdi keyword added, JBM, Oct 20 2010
;-
FUNCTION FAST_INT_Y, z_array, M_array, cosmo=cosmo, cluster=cluster, $
                     chandra=chandra, xmm=xmm, norm=norm, $
                     nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0, cm500=cm500, nr500=nr500, $
                     nstd=nstd, cco=cco, mdi=mdi

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
    print,'CALL IS : result = FAST_INT_Y(z_array, M_array)'
    print,'Keyword : cosmo=cosmo, cluster=cluster'
    print,'chandra=chandra, xmm=xmm, norm=norm'
    print,'nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0, cm500=cm500, nr500=nr500'
    print,'nstd=nstd, cco=cco, mdi=mdi'
    GOTO, closing
ENDIF
 
step_z = 0.01
z_sup = 1d2
z_inf = 1d-5
z_list = (FINDGEN(CEIL((z_sup-z_inf)/step_z)+1)*step_z)+z_inf
nz = N_ELEMENTS(z_list)

IF KEYWORD_SET(norm) OR KEYWORD_SET(nr500) THEN BEGIN ; if not then Y can be computed with the beta profile, the result is the same

  IF NOT KEYWORD_SET(chandra) AND NOT KEYWORD_SET(xmm) THEN BEGIN
    PRINT, 'chandra or xmm not specified. I use xmm...'
    xmm=1
  ENDIF

  z_0 = z_array[0]
  m_0 = M_array[0]  

  clus0 = CLUSTER_MODEL(z_0, m_0, /rs, $
                        cosmo=cosmo, cluster=cluster, r_cut=r_cut, $
                        chandra=chandra, xmm=xmm, norm=norm, $
                        nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi)

  IF KEYWORD_SET(nr500) THEN BEGIN
    r_cut=nr500*c500
  ENDIF

  nb_steps = 1000.
  r_array = MAX(r_cut)/nb_steps*(FINDGEN(nb_steps)+1)

  cat = NFW_PROFILE(z_0, m_0, r_array, $
                    cosmo=cosmo, cluster=cluster, $
                    chandra=chandra, xmm=xmm, norm=norm, $
                    nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0, cm500=cm500, nr500=nr500, nstd=nstd, cco=cco, mdi=mdi)

  s = SIZE(cat.rad_array)
  nr = s[1]

  delta_rad = cat.rad_array[1:nr-1]-cat.rad_array[0:nr-2]
  delta_rad = [delta_rad,delta_rad[nr-2,*]]

  result = 2*!pi*TOTAL(cat.y_array*cat.rad_array*delta_rad,1)
  ad = ANG_DIST(z_0, cosmo=cosmo)
  result = result/ad^2
  result = result/(!dtor)^2*60.^2
  res0 = result[0]

  COMMON MASS2DENS_NFW_PARAMETER, mass2dens_a, mass2dens_b, mass2dens_c
  mass2dens_a = nfw_a
  mass2dens_b = nfw_b
  mass2dens_c = nfw_c

  nza = N_ELEMENTS(z_array)

  IF nza GE nz THEN BEGIN
    bdc1 = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_list)
    FAST_CONVERT_MASS_RADIUS, REPLICATE(1., nz), z_list, bdc1, REPLICATE(500.,nz), REPLICATE(cm500, nz), $
                              rvir, r500, m500, $
                              cosmo=cosmo, /five_hundred
    rs1 = r500/c500
    cvir1 = rvir/rs1
    int_prof1 = DBLARR(nz)
    int_prof1[nz-1] = QROMB('subfunc_mass2dens_nfw',0.,cvir1[nz-1])
    FOR i=1, nz-1 DO BEGIN
      int_prof1[nz-1-i]=int_prof1[nz-i]+QROMB('subfunc_mass2dens_nfw',cvir1[nz-i],cvir1[nz-1-i])
    ENDFOR
    ad1 = ANG_DIST(z_list, cosmo=cosmo)
    ad = INTERPOL(ad1, z_list, z_array)
    ez = HUBBLE_AT_Z(z_array, cosmo=cosmo)
    bdc = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_array)
    cvir = INTERPOL(cvir1, z_list, z_array)
    int_prof = INTERPOL(int_prof1, z_list, z_array)
  ENDIF ELSE BEGIN
    bdc = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_array)
    FAST_CONVERT_MASS_RADIUS, REPLICATE(1., nza), z_array, bdc, REPLICATE(500.,nza), REPLICATE(cm500, nza), $
                              rvir, r500, m500, $
                              cosmo=cosmo, /five_hundred
    rs = r500/c500
    cvir = rvir/rs
    so=SORT(z_array)
    int_prof = DBLARR(nza)
    int_prof[so[nza-1]] = QROMB('subfunc_mass2dens_nfw',0.,cvir[so[nza-1]])
    FOR i=1, nza-1 DO BEGIN
      int_prof[so[nza-1-i]]=int_prof[so[nza-i]]+QROMB('subfunc_mass2dens_nfw',cvir[so[nza-i]],cvir[so[nza-1-i]])
    ENDFOR
    ad = ANG_DIST(z_array, cosmo=cosmo)
    ez = HUBBLE_AT_Z(z_array, cosmo=cosmo)
    bdc = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_array)
  ENDELSE 

  IF KEYWORD_SET(norm) THEN BEGIN ; case norm
    IF KEYWORD_SET(nr500) THEN BEGIN ; case norm+nr500
      int_prof[*] = 1.
    ENDIF
    result = res0*(M_array/M_array[0])^(5./3)*(ez/ez[0])^(2./3)*(bdc/bdc[0])^(-5./3)*(ad/ad[0])^(-2.)* $
             (int_prof/int_prof[0])*(cvir/cvir[0])^(-5.)
  ENDIF ELSE BEGIN ; case no norm+nr500
    result = res0*(M_array/M_array[0])^(5./3)*(ez/ez[0])^(2./3)*(bdc/bdc[0])^(1./3)*(ad/ad[0])^(-2.)* $
             (int_prof/int_prof[0])^(-1.)
  ENDELSE

ENDIF ELSE BEGIN

  r_cut=10. ; The result does not depend on r_cut value !
  beta=2./3 ; The result does not depend on beta value !

  nb_steps = 1000.
  r_array = r_cut/nb_steps*FINDGEN(nb_steps)

  z_0 = z_array[0]
  m_0 = M_array[0]

  cat = BETA_PROFILE(z_0, m_0, r_array, $
                     cosmo=cosmo, cluster=cluster, $
                     r_cut=r_cut, beta=beta)
  s = SIZE(cat.rad_array)
  nr = s[1]

  delta_rad = cat.rad_array[1:nr-1]-cat.rad_array[0:nr-2]
  delta_rad = [delta_rad,delta_rad[nr-2,*]]

  result = 2*!pi*TOTAL(cat.y_array*cat.rad_array*delta_rad,1)
  ad = ANG_DIST(z_0, cosmo=cosmo)
  result = result/ad^2
  result = result/(!dtor)^2*60.^2
  res0 = result[0]

  IF N_ELEMENTS(z_array) GE nz THEN BEGIN
    D1 = ANG_DIST(z_list, cosmo=cosmo)
    D=INTERPOL(D1,z_list,z_array)
  ENDIF ELSE BEGIN  
    D = ANG_DIST(z_array, cosmo=cosmo)
  ENDELSE
  E = HUBBLE_AT_Z(z_array, cosmo=cosmo)/100./cosmo.h
  bdc = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_array)

  result = res0*(M_array/M_array[0])^(5./3)*(E/E[0])^(2./3)*(bdc/bdc[0])^(1./3)*(D/D[0])^(-2.)

ENDELSE

IF KEYWORD_SET(xmm) AND KEYWORD_SET(nstd) THEN result = result*(M_array/M_array[0])^0.115865

RETURN, result

closing:
END
