; -----------------------------------------------------------------------------
;
; Copyright (C) 2002-2006
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;     FAST_TCCY2ZM
;
; PURPOSE:
;     Compute quickly z and mass if y0 and theta_c are given or the opposite if reverse is set.
;
; CALLING SEQUENCE:
;     FAST_Y0TC2ZM, y0, theta_c, z, mass, cosmo=cosmo, cluster=cluster
;
; INPUTS:
;     Theta_c : theta core array (in arcmin)
;     Y0 : y0 array
;        OR
;     Z : redshift
;     Mass : the mass in 10^15 solar units
;
; KEYWORDS:
;     COSMO : the structure containing the cosmological parameters.
;     CLUSTER : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;     REVERSE : to compute theta_c and y0 from z and mass
;
; OUTPUTS:
;     Z : redshift
;     Mass : the mass in 10^15 solar units
;        OR
;     Theta_c : theta core array (in arcmin)
;     Y0 : y0 array
;
; EXAMPLES:
;     
; SIDE EFFECTS:
;     
; RESTRICTIONS:
;     Report any bugs to Jacques Delabrouille
;     delabrouille@apc.univ-paris7.fr
;
; REVISION HISTORY
;     Written, Jean-Baptiste Melin, June 18 2002
;     cluster structure added, JBM, February 3 2006
;     Big Delta_c introduced, JBM, March 7 2006
;     interpolation redshift list (z_list) extended, JBM, March 15 2006 
;-
PRO FAST_Y0TC2ZM, y0, theta_c, z, mass, cosmo=cosmo, cluster=cluster, reverse=reverse


IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        PRINT, 'CALL IS : FAST_Y0TC2ZM, y0, theta_c, z, mass'
        PRINT, 'Keywords : cosmo=cosmo, cluster=cluster, reverse=reverse'
     GOTO, closing
ENDIF

IF KEYWORD_SET(reverse) THEN BEGIN
  step_z = 0.01
  z_sup = 1d2 ;10.
  z_inf = 1d-5 ;0.01
  z_list = (FINDGEN(CEIL((z_sup-z_inf)/step_z)+1)*step_z)+z_inf

  r_cut=10. ; The result does not depend on r_cut value !
  beta=2./3 ; The result does not depend on beta value !

  z_0 = z[0]
  m_0 = mass[0]

  E1 = HUBBLE_AT_Z(z_list, cosmo=cosmo)/100./cosmo.h
  E = INTERPOL(E1, z_list, z)
  D1 = ANG_DIST(z_list, cosmo=cosmo)
  D = INTERPOL(D1, z_list, z)
  bdc = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z)

  cat = CLUSTER_MODEL(z_0, m_0, cosmo=cosmo, cluster=cluster, $
                      /rc, /y0, r_cut=r_cut, beta=beta)
  r_c = cat.rc
  ANG2SIZE, coreang, r_c, z_0, cosmo=cosmo, /reverse
  res0b = 60.*coreang

  theta_c = res0b*(mass/mass[0])^(1./3)*(E/E[0])^(-2./3)*(bdc/bdc[0])^(-1./3)*(D/D[0])^(-1.)

  res0 = cat.y0
  
  y0 = res0*(mass/mass[0])*(E/E[0])^2.*(bdc/bdc[0])

ENDIF ELSE BEGIN

   step_z = 0.01
   z_sup = 1d2 ;10.
   z_inf = 1d-5 ;0.01
   z_list = (FINDGEN(CEIL((z_sup-z_inf)/step_z)+1)*step_z)+z_inf

  ; step redshift
    step_log10_z = 500.
  ; step m
    step_log10 = 500.

  ; mass limits
    M_inf = 0.01
    M_sup = 10.
  ; z limit
    z_sup = 10.
    z_inf = 0.01
 
  ; building redshift array
    log10_z = (FINDGEN(CEIL((ALOG10(z_sup)-ALOG10(z_inf))*step_log10_z)+1)/step_log10_z)+ALOG10(z_inf)
    redshift = EXP(ALOG(10)*log10_z)
    
  ; building mass array in 10^15 M_sun units
    log10_m = (FINDGEN(CEIL((ALOG10(M_sup)-ALOG10(M_inf))*step_log10)+1)/step_log10)+ALOG10(M_inf)
    m = EXP(ALOG(10)*log10_m)

    nm = N_ELEMENTS(m)
    nz = N_ELEMENTS(redshift)
    
    in_do = INTO_DOMAIN_TC_Y0(theta_c, y0, z_inf, z_sup, M_inf, M_sup, cosmo=cosmo, cluster=cluster)

    fz = REPLICATE(1.,nm)
    fm = REPLICATE(1.,nz)
    z_array = redshift#TRANSPOSE(fz)
    M_array = fm#TRANSPOSE(m)
    
    r_cut=10. ; The result does not depend on r_cut value !
    beta=2./3 ; The result does not depend on beta value !

    E1 = HUBBLE_AT_Z(z_list, cosmo=cosmo)/100./cosmo.h
    E = INTERPOL(E1, z_list, z_array)
    D1 = ANG_DIST(z_list, cosmo=cosmo)
    D = INTERPOL(D1, z_list, z_array)
    bdc = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_array)

    z_0 = z_array[0]
    m_0 = M_array[0]

    cat = CLUSTER_MODEL(z_0, m_0, cosmo=cosmo, cluster=cluster, /rc, /y0, $
          r_cut=r_cut, beta=beta)
    r_c = cat.rc
    ANG2SIZE, coreang, r_c, z_0, cosmo=cosmo, /reverse
    res0b = 60.*coreang

    big_theta_c = res0b*(M_array/M_array[0])^(1./3)*(E/E[0])^(-2./3)*(bdc/bdc[0])^(-1./3)*(D/D[0])^(-1.)

    res0 = cat.y0

    big_y0 = res0*(M_array/M_array[0])*(E/E[0])^2.*(bdc/bdc[0])

    ratio = big_theta_c[*,0]/big_y0[*,0]^(1./3)
    r = theta_c/y0^(1./3)
    nelts = N_ELEMENTS(y0)
    indm = LONARR(nelts)
    indz = LONARR(nelts)
    FOR i=0L, nelts-1 DO BEGIN
      IF in_do[i] THEN BEGIN
        wh = WHERE(ratio GE r[i], nwh)
        IF nwh EQ 0 THEN indz[i] = ratio[nz-1] ELSE indz[i] = MAX(wh)
        wh = WHERE(big_y0[indz[i],*] LE y0[i], nwh)
        IF nwh EQ 0 THEN indm[i] = big_y0[indz[i],nm-1] ELSE indm[i]= MAX(wh)
      ENDIF ELSE BEGIN
        indz[i]=0
        indm[i]=0
      ENDELSE
    ENDFOR
    z = redshift[indz]
    mass = m[indm]
    wh = WHERE(in_do EQ 0, nwh)
    IF (nwh GE 1) THEN BEGIN
      z[wh] = -1
      mass[wh] = -1
    ENDIF
ENDELSE
closing:
END
