; -----------------------------------------------------------------------------
;
; Copyright (C) 2002-2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	INTO_DOMAIN_TC_CY
;
; PURPOSE:
;	This function returns 1 if cluster parameters (theta_c, Y)
;	correspond to a cluster in the interval [z_inf,z_sup], [M_inf,M_sup], 0 otherwise.
;
; CATEGORY:
;	Sky model (Clusters)
;
; CALLING SEQUENCE:
;	Result = INTO_DOMAIN_TC_CY(Theta_c, Cy, Z_inf, Z_sup, M_inf, M_sup, COSMO=, CLUSTER=)
;
; INPUTS:
;	Theta_c:	Angular core radius of a galaxy cluster (in arcmin) - can be an array
;	Y:		Integrated Compton parameter of a given cluster (in arcmin^2) - can be an array
;	Z_inf:		Minimum value of redshift in z,M plane
;	Z_sup:		Maximum value of redshift in z,M plane
;	M_inf:		Minimum value of mass in z,M plane (units of 10e15 solar masses)
;	M_sup:		Maximum value of mass in z,M plane (units of 10e15 solar masses)
;
; KEYWORD PARAMETERS:
;	COSMO:		a structure containing cosmological model (parameters),
;			created with DEFINE_COSMO
;	CLUSTER:	a structure containing clustering (structure formation) parameters
;			created with DEFINE_CLUSTER
;
; OUTPUTS:
;	The result is 0 or 1 (same dimension as input Theta_c and Y).
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	19/03/2002:	Jean-Baptiste Melin, First version.
; 	03/02/2006:	JBM, CLUSTER keyword added
; 	11/01/2012:	Jacques Delabrouille, update the handling of cosmo structure
; 	14/09/2012:	JD, update doc and license header
;-
FUNCTION INTO_DOMAIN_TC_CY, theta_c, cy, z_inf, z_sup, M_inf, M_sup, cosmo=cosmo, cluster=cluster


IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        PRINT, 'CALL IS : in_do = INTO_DOMAIN(theta_c, cy, z_inf, z_sup, M_inf, M_sup)'
        PRINT, 'Keyword : cosmo=cosmo, cluster=cluster'
     GOTO, closing
ENDIF

step_log10_z = 10.
step_log10 = 10.

log10_z = (FINDGEN(CEIL((ALOG10(z_sup)-ALOG10(z_inf))*step_log10_z)+1)/step_log10_z)+ALOG10(z_inf)
z = EXP(ALOG(10)*log10_z)
;z = (FINDGEN(CEIL((z_sup-z_inf)/step_z)+1)*step_z)+z_inf

log10_m = (FINDGEN(CEIL((ALOG10(M_sup)-ALOG10(M_inf))*step_log10)+1)/step_log10)+ALOG10(M_inf)
m = EXP(ALOG(10)*log10_m)

nz = N_ELEMENTS(z)
nm = N_ELEMENTS(m)
    
lim_z_theta_c = FLTARR(nm, 2)
lim_z_cy = FLTARR(nm, 2)
lim_m_theta_c = FLTARR(nz, 2)
lim_m_cy = FLTARR(nz, 2)

redshift = REPLICATE(MIN(z), nm)
lim_z_cy[*,0] = FAST_INT_Y(redshift, m, cosmo=cosmo, cluster=cluster)
cat = CLUSTER_MODEL(redshift, m, cosmo=cosmo, cluster=cluster, /rc)
r_c = cat.rc
ANG2SIZE, coreang, r_c, redshift, cosmo=cosmo, /reverse
lim_z_theta_c[*,0] = 60.*coreang

redshift = REPLICATE(MAX(z), nm)
lim_z_cy[*,1] = FAST_INT_Y(redshift, m, cosmo=cosmo, cluster=cluster)
cat = CLUSTER_MODEL(redshift, m, cosmo=cosmo, cluster=cluster, /rc)
r_c = cat.rc
ANG2SIZE, coreang, r_c, redshift, cosmo=cosmo, /reverse
lim_z_theta_c[*,1] = 60.*coreang

mass = REPLICATE(MIN(m), nz)
lim_m_cy[*,0] = FAST_INT_Y(z, mass, cosmo=cosmo, cluster=cluster)
cat = CLUSTER_MODEL(z, mass, cosmo=cosmo, cluster=cluster, /rc)
r_c = cat.rc
ANG2SIZE, coreang, r_c, z, cosmo=cosmo, /reverse
lim_m_theta_c[*,0] = 60.*coreang

mass = REPLICATE(MAX(m), nz)
lim_m_cy[*,1] = FAST_INT_Y(z, mass, cosmo=cosmo, cluster=cluster)
cat = CLUSTER_MODEL(z, mass, cosmo=cosmo, cluster=cluster, /rc)
r_c = cat.rc
ANG2SIZE, coreang, r_c, z, cosmo=cosmo, /reverse
lim_m_theta_c[*,1] = 60.*coreang

nelts = N_ELEMENTS(theta_c)
in_do = INTARR(nelts)

tc_min = MIN(lim_z_theta_c)
tc_max = MAX(lim_z_theta_c)
cy_min = MIN(lim_m_cy)
cy_max = MAX(lim_m_cy)
theta_c_1 = MIN(lim_z_theta_c[*,1])
theta_c_2 = MIN(lim_z_theta_c[*,0])
theta_c_3 = MAX(lim_z_theta_c[*,0])
theta_c_4 = MAX(lim_z_theta_c[*,1])
cy_1 = MIN(lim_z_cy[*,1])
cy_2 = MIN(lim_z_cy[*,0])
cy_3 = MAX(lim_z_cy[*,0])
cy_4 = MAX(lim_z_cy[*,1])

;plot, lim_z_theta_c[*,0], lim_z_cy[*,0], /ylog, xrange=[tc_min, tc_max], yrange=[cy_min, cy_max], psym=1
;oplot, lim_z_theta_c[*,1], lim_z_cy[*,1], psym=1
;oplot, lim_m_theta_c[*,1], lim_m_cy[*,1], psym=1
;oplot, lim_m_theta_c[*,0], lim_m_cy[*,0], psym=1



FOR i=0L, nelts-1 DO BEGIN
  tc = theta_c[i]
  y = cy[i]
  IF (tc GE tc_min) AND (tc LE tc_max) AND (y GE cy_min) AND (y LE cy_max) THEN BEGIN
    IF (tc LE theta_c_2) THEN BEGIN
      ok = MIN(ABS(lim_m_theta_c[*,0]-tc),ind)
      IF (lim_m_cy[ind,0] LE y) THEN BEGIN
        IF (tc LE theta_c_4) THEN BEGIN
          ok = MIN(ABS(lim_z_theta_c[*,1]-tc),ind)
          IF (lim_z_cy[ind,1] GE y) THEN in_do[i] = 1
        ENDIF ELSE BEGIN
          ok = MIN(ABS(lim_m_theta_c[*,1]-tc),ind)
          IF (lim_m_cy[ind,1] GE y) THEN in_do[i] = 1
        ENDELSE
      ENDIF
    ENDIF ELSE BEGIN
      ok = MIN(ABS(lim_z_theta_c[*,0]-tc),ind)
      IF (lim_z_cy[ind,0] LE y) THEN BEGIN
        IF (tc LE theta_c_4) THEN BEGIN
          ok = MIN(ABS(lim_z_theta_c[*,1]-tc),ind)
          IF (lim_z_cy[ind,1] GE y) THEN in_do[i] = 1
        ENDIF ELSE BEGIN
          ok = MIN(ABS(lim_m_theta_c[*,1]-tc),ind)
          IF (lim_m_cy[ind,1] GE y) THEN in_do[i] = 1
        ENDELSE
      ENDIF
    ENDELSE
  ENDIF
ENDFOR

RETURN, in_do

closing:
END
