; -----------------------------------------------------------------------------
;
; Copyright (C) 2008
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;     CLUSTER_FIR_SOURCES_LTF
; ONE LINE HELP:
;     Draw the luminosity, the type, the flux and the cluster parent number of the cluster far infrared sources.
; PURPOSE:
;     Draw the luminosity, the type, the flux and the cluster parent number of the cluster far infrared sources.
; CALLING SEQUENCE:
;     fir_sources_ltf = CLUSTER_FIR_SOURCES_LTF(z_array, M_array, frequencies, fir_total_flux, cosmo=cosmo, iseed=iseed, dir=dir, flim=flim, verbose=verbose)
; INPUT:
;     z_array : the redshift array of the cluster catalogue
;     M_array : the mass array of the cluster catalogue (10^15 Msol unit)
;     frequencies : the frequencies in GHz
; OUTPUT:
;     fir_total_flux : DBLARR(nband,N_ELEMENTS(z_array)) Total cluster fir source flux for each PSM frequency
;     fir_sources_ltf : a structure with sources having flux>flim for at least a PSM frequency with
;        * fir_sources_ltf[i].cluster : the number of the cluster (cluster parent number) that contains the radio source i
;        * fir_sources_ltf[i].lum : the luminosity of the radio source i (10^12 Lsol unit) 
;        * fir_sources_ltf[i].type : 'normal' or 'starburst'
;        * fir_sources_ltf[i].flux : DBLARR(nband) Flux in MJy for each PSM frequency
; KEYWORDS:
;     cosmo : the cosmology
;     iseed : the random init seed
;     dir : the directory where the luminosity functions are (Lagache, Dole & Puget)
;     verbose : to get comments
; EXAMPLES:
;     
; ALGORITHM:
;     
; SIDE EFFECTS:
;     
; RESTRICTIONS:
;     Report any bugs to Jean-Baptiste Melin
;     jean-baptiste.melin@cea.fr
; PROCEDURES CALLED:
;     HOD, CLUSTER_FIR_SOURCES_TAB_FLUX
; REVISION HISTORY
;     Written from cluster_fir_sources_luminosity.pro, Jean-Baptiste Melin, October 21 2008
;     frequencies are now an input, JBM, October 28 2008
;-
FUNCTION CLUSTER_FIR_SOURCES_LTF, z_array, M_array, frequencies, fir_total_flux, cosmo=cosmo, iseed=iseed, dir=dir, flim=flim, verbose=verbose


IF NOT KEYWORD_SET(dir) THEN dir = dialog_pickfile(/dir)
dir = FIX_SEPARATOR(dir)

IF NOT KEYWORD_SET(iseed) THEN iseed = LONG(systime(1))

IF NOT KEYWORD_SET(flim) THEN flim=2d-7 ; MJy

IF KEYWORD_SET(verbose) THEN print, 'Computing the tabulated source fluxes...'
flux = CLUSTER_FIR_SOURCES_TAB_FLUX(frequencies, cosmo=cosmo, dir=dir)
nband = N_ELEMENTS(frequencies)

fctlum = MRDFITS(dir+'fctlum.fits', 1, /DSCALE, /SILENT)
n_lum = N_ELEMENTS(fctlum.loglum)
int_lum = DBLARR(n_lum)
int_psb = DBLARR(n_lum)

step_log10 = 200.
delta_log10_lum = ALOG(10)/step_log10
lum_min = 10^(MIN(fctlum.loglum)-12.) ; 10^12 Lsol unit
lum_max = 10^(MAX(fctlum.loglum)-12.)
log10_lum = (FINDGEN(CEIL((ALOG10(lum_max)-ALOG10(lum_min))*step_log10)+1)/step_log10)+ALOG10(lum_min)
lum = EXP(ALOG(10)*log10_lum)

z_list = fctlum.z
n_z_list = N_ELEMENTS(z_list)

cluster_number = N_ELEMENTS(M_array)

indz = INTARR(cluster_number)
FOR i=0L, cluster_number-1 DO BEGIN
  ok = MIN(ABS(z_array[i]-z_list), whmin)
  indz[i] = whmin
ENDFOR

fir_galaxy_average_number = HOD(M_array*1d15, cosmo=cosmo, /SILENT)

node = {cluster:0L, lum:0d, type:'normal', flux:DBLARR(nband)}
fir_galaxy_number = INTARR(cluster_number)
fir_total_flux = DBLARR(nband,cluster_number)

IF KEYWORD_SET(verbose) THEN print, 'Drawing the source luminosities and types...'
FOR i=0, n_z_list-1 DO BEGIN
  ;print, i, n_z_list-1
  which_clusters = WHERE(indz EQ i, ncluster)
  IF ncluster GE 1 THEN BEGIN
    FOR j=0, ncluster-1 DO BEGIN
      firgn = RANDOMN(iseed, POISSON=fir_galaxy_average_number[which_clusters[j]], /DOUBLE)
      fir_galaxy_number[which_clusters[j]] = FIX(firgn)
    ENDFOR
    int_lum=(fctlum.logfctlum_tot)[*,i]
    int_psb=(fctlum.prob_sb)[*,i]
    prob_lum = EXP(ALOG(10)*INTERPOL(int_lum, fctlum.loglum, ALOG10(lum)+12))
    prob_psb = INTERPOL(int_psb, fctlum.loglum, ALOG10(lum)+12)
    tot_num = TOTAL(fir_galaxy_number[which_clusters])
    IF tot_num GE 1 THEN BEGIN
      pix = RANDOM_DRAW(prob_lum, tot_num, /silent, iseed=iseed)
      is_sb = BYTARR(tot_num)
      upix = pix[UNIQ(pix,SORT(pix))]
      nupix = N_ELEMENTS(upix)
      FOR j=0, nupix-1 DO BEGIN
        which_sources = WHERE(pix EQ upix[j], nsource)
        is_sb[which_sources]=RANDOM_DRAW([1.-prob_psb[upix[j]],prob_psb[upix[j]]],nsource,iseed=iseed)
      ENDFOR
      whsb = WHERE(is_sb EQ 1, nwhsb)
      whnsb = WHERE(is_sb EQ 0, nwhnsb)
      sflux=DBLARR(nband,tot_num)
      FOR j=0, nband-1 DO BEGIN
        IF nwhnsb GE 1 THEN sflux[j,whnsb]=INTERPOL((flux.flux)[0,j,i,*],(flux.lum),lum[pix[whnsb]])
        IF nwhsb GE 1 THEN sflux[j,whsb]=INTERPOL((flux.flux)[1,j,i,*],(flux.lum),lum[pix[whsb]])
      ENDFOR
      istart = 0
      numcluster = LONARR(tot_num)
      FOR k=0, ncluster-1 DO BEGIN
        IF TOTAL(fir_galaxy_number[which_clusters[k]]) GE 1 THEN BEGIN
          iend = istart+TOTAL(fir_galaxy_number[which_clusters[k]])-1
          numcluster[istart:iend] = which_clusters[k]
          FOR j=0, nband-1 DO BEGIN
            fir_total_flux[j,which_clusters[k]]=TOTAL(sflux[j,istart:iend])
          ENDFOR      
          istart=iend+1
        ENDIF
      ENDFOR
      whlim = WHERE(sflux GE flim, nwhlim)
      IF nwhlim GE 1 THEN BEGIN
        indices = ARRAY_INDICES(sflux, whlim)
        IF tot_num EQ 1 THEN reform_indices = 0 ELSE reform_indices = REFORM(indices[1,*])
        uind = reform_indices[UNIQ(reform_indices,SORT(REFORM(reform_indices)))]
        cat = REPLICATE(node,N_ELEMENTS(uind))
        cat.cluster = numcluster[uind]
        cat.lum = lum[pix[uind]]
        whs = WHERE(is_sb[uind] EQ 1, nwhs)
        IF nwhs GE 1 THEN cat[whs].type = 'starburst'
        cat.flux = sflux[*,uind]
        IF DEFINED(scat) EQ 0 THEN scat=cat ELSE scat=[scat,cat]
      ENDIF
    ENDIF
  ENDIF
ENDFOR

RETURN, scat

END
