; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2002, 2006-2010
;   Jacques Delabrouille
;   Guillaume Castex
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CREATE_SZ_SIMULATION
;
; PURPOSE:
;	Create simulated SZ effect maps.
;
; CALLING SEQUENCE:
;	CREATE_SZ_SIMULATION, Map_hd, Catalog, y_map, Taubeta_map, Taubeta2_q_map, Taubeta2_u_map
;
; INPUTS:
;	map_hd: 	The header of the map.
;
; KEYWORDS:
;	MAP_TYPE:	The type of map ('small-map' only for the moment).
;	COSMO:  	The structure containing the cosmological parameters.
;	CLUSTER:	The structure containing the structure formation parameters (cf. DEFINE_CLUSTER).
;	CHANDRA, XMM:	If set, don't use the beta profile but the nfw profile of Chandra or XMM.
;	NORM:   	If set, use the normalisation found in the observations for cluster profiles.
;	MASS_FUNCTION:	The mass function (default : Press_Schechter).
;	ISEED:  	The seed used for random number generation.
;	NR500:  	Integration is performed up to n*r500 instead
;	                of rvir for cluster flux (you must set the xmm
;	                or chandra keyword to use this keyword). 
;
; OUTPUTS:
;	catalog:  	The cluster catalog.
;	y_map:    	The map of the y parameter.
;	taubeta_map:	The map of tau x beta (optical depth times velocity/c).
;	taubeta2_q_map:	The map of tau x beta^2 (optical depth times
;	                velocity^2/c^2), polar Q component. 
;	taubeta2_u_map:	The map of tau x beta^2 (optical depth times
;	                velocity^2/c^2), polar U component.
;
; EXAMPLES:
;
; ALGORITHM:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	j.delabrouille@cdf.in2p3.fr
; PROCEDURES CALLED:
;
; REVISION HISTORY
;	Written, Jacques Delabrouille and Jean-Baptiste Melin, May 2001
;	mass_function added, JBM, Sept 11 2001
;	Keyword cosmo introduced, JBM, Sept 27 2001
;	DRAW_LONLAT introduced, JBM, Aug 6 2002
;	cluster keyword added, JBM, Feb 6 2006
;	"theta_c" field changed as "tc" to be consistent with the cluster extraction algorithm, JBM, March 21 2007
;	z_inf changed from 0.001 to 0.01, JBM, March 21 2007
;	rv (virial radius) and tv (theta virial) included in the catalog, JBM, July 17 2007
;	chandra, xmm and norm keywords added, JBM, June 23 2008
;	cosmetic changes, JBM, July 1 2008
;	nr500 keyword added, JBM, Dec 4 2008
;	cm500 included in DO_CLUSTER_CATALOG, JBM, Dec 11 2008
;-

PRO DO_CLUSTER_CATALOG, z_list, box, D_g_box, cell_size, map_hd, mf_cat, $
	     		map_type=map_type, $
                        cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500

IF NOT KEYWORD_SET(iseed) THEN iseed=LONG(systime(1))

IF STRLOWCASE(map_type) EQ 'small-map' THEN BEGIN
	nx=SXPAR(map_hd,'NAXIS1')
	ny=SXPAR(map_hd,'NAXIS2')
	map_ang_size_x = nx*ABS(SXPAR(map_hd,'CDELT1'))
	map_ang_size_y =  ny*ABS(SXPAR(map_hd,'CDELT2'))
        lon_map = (FINDGEN(nx)-(nx-1.)/2.)#REPLICATE(ABS(SXPAR(map_hd,'CDELT1')),ny)
        ;lat_map = REPLICATE(ABS(SXPAR(map_hd,'CDELT2')),nx)#(FINDGEN(ny)-(ny-1.)/2.)
        solid_angle = map_ang_size_x * map_ang_size_y * !dtor^2
ENDIF
IF STRLOWCASE(map_type) EQ 'half-sky' THEN solid_angle = 2.*!pi
IF STRLOWCASE(map_type) EQ 'all-sky' THEN solid_angle = 4.*!pi
dimensions = [N_ELEMENTS(box(*,0,0)),N_ELEMENTS(box(0,*,0)),N_ELEMENTS(box(0,0,*))]
deltarho_rho = 0.*z_list

z_cut = 5. ; be careful redondancy with z_sup in DO_DENSITY_CONTRAT_BOX
boxpix = cell_size/!dtor/((1+z_cut)*ANG_DIST(z_cut, cosmo=cosmo))
mappix = lon_map[1,0]-lon_map[0,0] ; lon_map[1,0]-lon_map[0,0]=lat_map[0,1]-lat_map[0,0]
underpix_pix = FIX(boxpix/mappix)>1
s = SIZE(lon_map)
map_dim_x = s[1]
map_dim_y = s[2]
underpix_pix = PREVIOUS_EVEN(underpix_pix) ; nx and ny must be both even
underpix_mappix = underpix_pix*mappix
underpix_map_dim_x = CEIL(map_dim_x/FLOAT(underpix_pix))
underpix_map_dim_y = CEIL(map_dim_y/FLOAT(underpix_pix))
underpix_lon_map = (FINDGEN(underpix_map_dim_x)-(underpix_map_dim_x-1.)/2.)# $
                   REPLICATE(underpix_mappix,underpix_map_dim_y)
underpix_lat_map = REPLICATE(underpix_mappix,underpix_map_dim_x)# $
                   (FINDGEN(underpix_map_dim_y)-(underpix_map_dim_y-1.)/2.)
delta_map = FLTARR(underpix_map_dim_x,underpix_map_dim_y)

FOR i=0L, N_ELEMENTS(z_list)-1 DO BEGIN
        redshift = z_list[i]
        COMOVING2OBSERVABLE, x_map, y_map, z_map, underpix_lon_map, underpix_lat_map, redshift, $
                               cosmo=cosmo, /reverse
	BOX_PIX2COORD, px, py, pz, x_map, y_map, z_map, dimensions, cell_size, $
                         map_type='small-map',/reverse
	delta_map = box(px,py,pz)*D_g_box(px,py,pz)
        deltarho_rho(i) = AVG(delta_map)
ENDFOR

M_inf = 0.1
mf_cat = CLUSTER_CATALOG(solid_angle, M_inf, z_list=z_list, deltarho_rho=deltarho_rho, $
	  	   cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500)
	; LINEAR BIAS TAKEN INTO ACCOUNT IN CLUSTER_CATALOG

;mf_cat = CLUSTER_CATALOG(solid_angle, M_inf, z_list=z_list, deltarho_rho=0.*z_list, $
;			   cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed)

END

;-------------------------------------------------------------------------------------------------------

PRO CREATE_SZ_SIMULATION, map_hd, catalog, y_map, taubeta_map, taubeta2_q_map, taubeta2_u_map, $
			     map_type=map_type, chandra=chandra, norm=norm, xmm=xmm, nr500=nr500, $
			     cosmo=cosmo, cluster=cluster, $
                             mass_function=mass_function, iseed=iseed

IF NOT KEYWORD_SET(iseed) THEN iseed=LONG(systime(1))

IF NOT KEYWORD_SET(mass_function) THEN BEGIN
   print, 'Choosing Press-Schechter mass function...'
   mass_function='press_schechter'
ENDIF

IF DEFINED(map_type) EQ 0 THEN map_type = 'small-map'
IF DEFINED(map_hd) EQ 0 THEN map_hd=GENERATE_HEADER(nx=360, ny=360, lon_c=0, lat_c=0, $
		pixdeg=1./120., angle=0., coord='G')

IF STRLOWCASE(map_type) EQ 'small-map' THEN BEGIN
	nx=SXPAR(map_hd,'NAXIS1')
	ny=SXPAR(map_hd,'NAXIS2')
        IF (NOT (nx/2 EQ nx/2.)) OR (NOT (ny/2 EQ ny/2.)) THEN BEGIN
           PRINT, 'nx and ny must be both even !'
           ; see the repixelisation in add_positions_to_catalog
           GOTO, closing
        ENDIF
	map_ang_size_x = nx*ABS(SXPAR(map_hd,'CDELT1'))
	map_ang_size_y =  ny*ABS(SXPAR(map_hd,'CDELT2'))
	lon_map = (FINDGEN(nx)-(nx-1.)/2.)#REPLICATE(ABS(SXPAR(map_hd,'CDELT1')),ny)
	lat_map = REPLICATE(ABS(SXPAR(map_hd,'CDELT2')),nx)#(FINDGEN(ny)-(ny-1.)/2.)
ENDIF


;we generate a density contrast box of appropriate dimensions, in comoving coordinates
;-------------------------------------------------------------------------------------

PRINT, '  GENERATING THE DENSITY CONTRAST BOX'

DO_DENSITY_CONTRAST_BOX, map_ang_size_x, map_ang_size_y, dimensions, cell_size, $
			 box, D_g_box, f_box, lon_box, lat_box, redshift_box, $
			 map_type = map_type, $
			 cosmo=cosmo, cluster=cluster, iseed=iseed ;, smooth_size=5*cosmo.h

lon_box=0B
lat_box=0B ; to save memory space

HELP, box
HELP, cell_size

z_inf = 0.01 ; if z_inf LE 0.01 they might be some big cluster at very low redshift
z_sup = 5. ; be careful redondancy with z_sup in DO_DENSITY_CONTRAT_BOX
;z_step = 0.001
z_step = 0.01
z_list = (FINDGEN(CEIL((z_sup-z_inf)/z_step)+1)*z_step)+z_inf

;generate the cluster catalog
;----------------------------

PRINT,''
PRINT, '  GENERATING THE CLUSTER CATALOG'

IF KEYWORD_SET(xmm) OR KEYWORD_SET(chandra) THEN BEGIN
  ; Get the value of cm500 to do the mass conversion M_jenkins -> M_vir in cluster_catalog.pro
  get_cm500 = CLUSTER_MODEL(0., 1., /rs, xmm=xmm, chandra=chandra, norm=norm, cosmo=cosmo, cluster=cluster, cm500=cm500)
ENDIF

DO_CLUSTER_CATALOG, z_list, box, D_g_box, cell_size, map_hd, mf_cat, $
		      map_type=map_type, $
                      cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500

;add cluster positions
;---------------------

bias = GET_CLUSTERING_BIAS(mf_cat.mass, mf_cat.z, cosmo=cosmo, cluster=cluster)

show=0
ADD_POSITIONS_TO_CATALOG, mf_cat, lon_map, lat_map, box, D_g_box, cell_size, bias, show=show, $
		          cosmo=cosmo, map_type=map_type, iseed=iseed

CATALOG_DRAW_LONLAT, mf_cat, map_hd=map_hd, iseed=iseed

;get cluster velocities
;----------------------
PRINT,''
PRINT, '  GENERATING VELOCITIES'
;PRINT, '  WARNING, STILL NEED TO FIX REDSHIFT DEPENDENCE'
GET_VELOCITY_BOXES, box, cell_size, f_box, D_g_box, redshift_box, vx_box, vy_box, vz_box, $
	              cosmo=cosmo

box=0B
f_box=0B
redshift_box=0B
D_g_box=0B

COMOVING2OBSERVABLE, x, y, z, mf_cat.lon, mf_cat.lat, mf_cat.z, $
                       cosmo=cosmo, /reverse
BOX_PIX2COORD, px, py, pz, x, y, z, dimensions, cell_size, map_type=map_type, /reverse

vc_x = vx_box(px,py,pz)
vc_y = vy_box(px,py,pz)
vc_z = vz_box(px,py,pz)

vx_box=0B
vy_box=0B
vz_box=0B

;add velocities to mf_cat
;------------------------
catalog = CREATE_STRUCT(mf_cat(0),'v_radial',0d,'v_transverse',0d,'angle',0.)
catalog = REPLICATE(catalog,N_ELEMENTS(mf_cat))

tagnames = TAG_NAMES(mf_cat)
nnn = N_ELEMENTS(tagnames)
FOR i =0, nnn-1 DO catalog.(i) = mf_cat.(i)

catalog.v_radial = vc_x
catalog.v_transverse = sqrt(vc_y^2+vc_z^2)
catalog.angle = ATAN(vc_z,vc_y)

mf_cat = catalog


;add cluster parameters
;----------------------
PRINT,''
PRINT, '  MODELLING THE CLUSTERS'
model = CLUSTER_MODEL(mf_cat.z, mf_cat.mass, cosmo=cosmo, cluster=cluster, $
	/Te, /ne0, /tau0, /y0, chandra=chandra, norm=norm, xmm=xmm, c500=c500, cm500=cm500)
;catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rc',0d,'tau0',0d,'y0',0d,'theta_c',0.,'cy',0d)
;catalog =
;CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rc',0d,'tau0',0d,'y0',0d,'tc',0.,'cy',0d)

IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  IF KEYWORD_SET(nr500) THEN BEGIN
    catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rs',0d,'rv',0d,'tau0',0d,'y0',0d,'ts',0.,'tv',0.,'cy',0d,'r500',0d,'t500',0d,'mass500',0d,'cy500',0d, $
                            'rn500',0d,'tn500',0d,'massn500',0d,'cyn500',0d)
  ENDIF ELSE BEGIN
    catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rs',0d,'rv',0d,'tau0',0d,'y0',0d,'ts',0.,'tv',0.,'cy',0d)
  ENDELSE
ENDIF ELSE BEGIN
  catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rc',0d,'rv',0d,'tau0',0d,'y0',0d,'tc',0.,'tv',0.,'cy',0d)
ENDELSE
catalog = REPLICATE(catalog,N_ELEMENTS(mf_cat))

tagnames = TAG_NAMES(mf_cat)
nnn = N_ELEMENTS(tagnames)
FOR i=0, nnn-1 DO catalog.(i) = mf_cat.(i)
tagnames = TAG_NAMES(model)
FOR i=0, N_ELEMENTS(tagnames)-1 DO catalog.(i+nnn) = model.(i)

;ANG2SIZE, ang, catalog.rc, catalog.z, /reverse, cosmo=cosmo
;catalog.tc = ang*60.
;catalog.theta_c = ang*60.
da = ANG_DIST(catalog.z, cosmo=cosmo)

IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  catalog.ts = catalog.rs/!dtor/da*60.
ENDIF ELSE BEGIN
  catalog.tc = catalog.rc/!dtor/da*60.
ENDELSE
catalog.tv = catalog.rv/!dtor/da*60.

catalog.cy = FAST_INT_Y(catalog.z, catalog.mass, cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm)

IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
  catalog.r500 = c500*catalog.rs
  catalog.t500 = c500*catalog.ts
  FAST_CONVERT_MASS_RADIUS, catalog.mass, catalog.z, INTERPOL(cluster.big_delta_c_array, cluster.z_array, catalog.z), REPLICATE(500.,N_ELEMENTS(catalog)), $
                            REPLICATE(cm500,N_ELEMENTS(catalog)),rvir,r500,m500,cosmo=cosmo,/five_hundred
  catalog.mass500 = m500
  catalog.cy500 = FAST_INT_Y(catalog.z, catalog.mass, cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=1.)
  catalog.rn500 = nr500*catalog.r500
  catalog.tn500 = nr500*catalog.t500
  catalog.massn500 = (ALOG(1.+nr500*cm500)-nr500*cm500/(1.+nr500*cm500))/(ALOG(1.+cm500)-cm500/(1.+cm500))*catalog.mass500
  factn500 = FAST_INT_Y(catalog[0].z, catalog[0].mass, cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=nr500)/catalog[0].cy500
  catalog.cyn500=factn500[0]*catalog.cy500
ENDIF


mf_cat = catalog

;make SZ maps
;------------
PRINT, ''
PRINT, '  GENERATING THE SZ MAPS'
z_limits = [0.,10.]
which = WHERE( (mf_cat.z GE z_limits(0)) AND (mf_cat.z LE z_limits(1)) )
catalog = mf_cat(which)
;CREATE_Y_TAUBETA_MAP, map_hd, catalog, y_map, taubeta_map, $
;                          taubeta2_q_map, taubeta2_u_map, $
;                          cosmo=cosmo, cluster=cluster, $
;                          /polar

CREATE_Y_TAUBETA_MAP, map_hd, catalog, y_map, taubeta_map, $
                          taubeta2_q_map, taubeta2_u_map, $
                          cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, $
                          /polar

;visualisation
;-------------
;simplevisu, y_map, mag=2, /no
;simplevisu, taubeta_map, mag=2, /no
;radial(catalog.pixel) = vc_x(which)
;simplevisu, radial, mag=2, /no

;maxlength = 12	;length of mark in pixels
;factor = FLOAT(maxlength)/MAX(catalog.v_transverse)/[nx,ny]
;FOR i=0, N_ELEMENTS(catalog)-1 DO BEGIN & $
;	veloc =[catalog[i].v_transverse*COS(catalog[i].angle),catalog[i].v_transverse*SIN(catalog[i].angle)] & $
;	start = [(catalog[i].pixel MOD nx)/FLOAT(nx),(catalog[i].pixel/nx)/FLOAT(ny)] & $
;	ending = start+veloc*factor & $
;	xxx = [start(0),ending(0)] & $
;	yyy = [start(1),ending(1)] & $
;	plot, xxx,yyy,/noerase,position=[0,0,1,1],xrange=[0,1],yrange=[0,1] & $
;ENDFOR
;
;simplevisu, taubeta2_q_map, mag=2, /no
;simplevisu, taubeta2_u_map, mag=2, /no

closing:
END
