; -----------------------------------------------------------------------------
;
; Copyright (C) 2003, 2006-2009
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;  CREATE_Y_TAUBETA_MAP_ALLSKY
; ONE LINE HELP:
;  create y map, tau*beta map and polarised maps for the whole sky
; PURPOSE:
;  create a Compton parameter map,
;  an optical depth * radial_velocity/c_light map,
;  an optical depth * (transverse_velocity/c_light)^2 map
;  
; CALLING SEQUENCE:
;  CREATE_Y_TAUBETA_MAP_ALLSKY, nside, catalog, y_map, taubeta_map,
;                               taubeta2_q_map, taubeta2_u_map,
;                               cosmo=cosmo, cluster=cluster,
;                               r_cut=r_cut, beta=beta, polar=polar, verbose=verbose,
;                               chandra=chandra, norm=norm, xmm=xmm, nr500=nr500,
;                               cm500=cm500, P0=P0, c500=c500, nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, nstd=nstd
;
; INPUT:
;  nside : the nside of HEALPix map you want
;  catalog : the catalog of clusters
; OUTPUT:
;  y_map : the y cluster map
;  taubeta_map : the tau*beta cluster map
;  If keyword polar is set,
;   taubeta2_q_map : the local Stokes parameter Q tau*beta^2 map
;   taubeta2_u_map : the local Stokes parameter U tau*beta^2 map
;   The frame for Q and U is taken from the HEALPix convention :
;                            (x,y) where x is along the local cluster meridian towards south
;                                    and y is along the local cluster parallel towards east
;   Note : For each cluster of the generated catalog, catalog[cluster].angle gives the angle between south
;          (x axis) and the transverse velocity towards east (y axis)
;          (i.e. counterclockwise, z pointing outwards the sphere)
;   Q and U are defined as :
;     Q = Ix-Iy
;     U = Ix'-Iy' where (x',y') = ROT((x,y), !pi/4, counterclockwise)
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters (cf. DEFINE_COSMO)
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;  r_cut : the radius (in number of rc) where you stop your beta model
;          default : r_cut = 10.
;          OR
;          the radius (in number of rs) where you stop your nfw profile if candra or xmm is set.
;          default : r_cut = r_vir/rs
;  beta : the factor beta in the beta model (e.g. 0.66)
;         default : beta = 2./3
;  polar : set this keyword to compute polarized maps
;
;  verbose : print comments
;  chandra, xmm : if set, don't use the beta profile but the nfw profile of Chandra or XMM
;  norm : if set, use the normalisation found in the observations for cluster profiles
;  nr500 : integration is performed up to n*r500 instead of rvir for cluster flux (you must set the xmm or chandra keyword to use this keyword)
;  nfw_a, nfw_b, nfw_c : parameters of the NFW generalized profile (you must set norm and xmm/chandra keywords to use these keywords)
;  c500 : concentration rs/r500 of the pressure profile (you must set norm and xmm/chandra keywords to use this keyword)
;  P0 : normalisation of the pressure profile (you must set norm and xmm/chandra keywords to use this keyword)
;  cm500 : concentration rs/r500 of the matter profile (you must set norm and xmm/chandra keywords to use this keyword)
;  nstd : non standard XMM profile (you must set the xmm to use this keyword)
;
; EXAMPLES:
;
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  SXPAR, CLUSTER_MODEL, ANG2SIZE,
;  BETA_PROFILE, BETA_MODEL_MAP, PLACE_CLUSTER
;  POLAR_NEARBY_ANGLE_ON_SPHERE
; REVISION HISTORY
;  Written from CREATE_Y_TAUBETA_MAP, Jean-Baptiste Melin, December 2003
;  cluster keyword added, JBM, Feb 6 2006
;  bug for a print function in second loop fixed, JBM, May 29 2006
;  polar maps result checked and procedure header clarified, JBM, June 13 2006
;  healpix convention is used now, JBM, July 4 2006
;  "theta_c" field changed as "tc" to be coherent with the extraction algorithm, JBM, March 21 2007
;  verbose keyword added, JBM, March 22 2007
;  Now computes the polarisation angle for all pixels of a resolved cluster (solves problems at poles), JD, April 2 2007
;  chandra, xmm and norm keywords added, JBM, July 24 2008
;  nr500 keyword added, JBM, Dec 8 2008
;  bug in call with beta profile and setting nr500 keyword fixed [Thank you Maude!], JBM, Jan 5 2009
;  bug in call with r_cut and chandra/xmm profile fixed, JBM, Jan 6 2009
;  inclusive keyword added to query_disc and '(npack NE nwhr/FLOAT(nelts)', JBM, May 26 2009
;  cm500, P0, c500, nfw_abc keywords added, JBM, Feb 20 2012
;  implemented first order relativistic thermal effect, JD, Mar 10 2012
;  nstd keyword added, JBM, June 13 2012
;-

PRO CREATE_Y_TAUBETA_MAP_ALLSKY, nside, catalog, y_map, taubeta_map, $
                                 taubeta2_q_map, taubeta2_u_map, $
                                 ythetae_map = ythetae_map, ythetae2_map=ythetae2_map, ythetae3_map=ythetae3_map, ythetae4_map=ythetae4_map, $
                                 relativistic=relativistic, $
                                 cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, $
                                 r_cut=r_cut, beta=beta, polar=polar, verbose=verbose, $
                                 cm500=cm500, P0=P0, c500=c500, nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, nstd=nstd

  IF ( N_PARAMS() EQ 0 ) THEN BEGIN
     print,'CALL IS : CREATE_Y_TAUBETA_MAP_ALLSKY, nside, catalog, y_map, taubeta_map,'
     print,'                                       taubeta2_q_map, taubeta2_u_map,'
     print,'Output keyword for 1st order relativistic correction : ythetae_map=ythetae_map'
     print,'Keywords : relativistic=relativistic, cosmo=cosmo, cluster=cluster'
     print,'           r_cut=r_cut, beta=beta, polar=polar, verbose=verbose'
     print,'           chandra=chandra, xmm=xmm, norm=norm, nr500=nr500'
     print,'           cm500=cm500, P0=P0, c500=c500, nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, nstd=nstd'
     GOTO, closing
  ENDIF

  ;; useful physical units
  c_light = 2.9979246d+08       ; in m/s
  c_light = c_light/1e3         ; in km/s
  
  npix = LONG(12*nside^2.)
  hpix = SQRT(4*!pi/npix)/!dtor ; in deg
  
  y_map = fltarr(npix)
  taubeta_map = fltarr(npix)
  IF KEYWORD_SET(relativistic) NE 0 THEN BEGIN
     IF relativistic GE 1 THEN ythetae_map = fltarr(npix)
     IF relativistic GE 2 THEN ythetae2_map = fltarr(npix)
     IF relativistic GE 3 THEN ythetae3_map = fltarr(npix)
     IF relativistic GE 4 THEN ythetae4_map = fltarr(npix)
  ENDIF ELSE BEGIN
     relativistic = 0
  ENDELSE
  IF KEYWORD_SET(polar) THEN BEGIN
     taubeta2_q_map = fltarr(npix)
     taubeta2_u_map = fltarr(npix)
  ENDIF
  
  ;; define r_cut and beta default values
  IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
     ;; If r_cut is defined, calling cluster_model.pro does not change the value 
     clus = CLUSTER_MODEL(catalog.z,catalog.mass, /rs, $
                          cosmo=cosmo, cluster=cluster, r_cut=r_cut, $
                          chandra=chandra, xmm=xmm, norm=norm, c500=c500, nstd=nstd)
  ENDIF ELSE BEGIN
     IF NOT KEYWORD_SET(r_cut) THEN r_cut=10.
     IF NOT KEYWORD_SET(beta) THEN beta=2./3
  ENDELSE
  
  IF DEFINED(catalog) THEN cluster_nb = N_ELEMENTS(catalog) ELSE cluster_nb=0
  
  electron_mass = 0.51099906*1d3
  
  IF (cluster_nb GT 0) THEN BEGIN
     
     IF KEYWORD_SET(verbose) THEN BEGIN
        PRINT, cluster_nb, ' cluster(s) in the catalog.'
     ENDIF
     
     IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
        ;; IF KEYWORD_SET(nr500) THEN BEGIN -> bug if beta model and nr500 is set
        diam_nr500 = 2*catalog.tn500/60.
        unresolved = (diam_nr500 LT hpix)
     ENDIF ELSE BEGIN
        diam_vir = 2*catalog.tv/60.
        unresolved = (diam_vir LT hpix)
     ENDELSE
     
     ;; Placing the resolved clusters on the map
     ;;------------------------------------------
     
     ;; y_map[*] = 0
     ;; taubeta_map[*] = 0
     ;; taubeta2_q_map[*]=0
     ;; taubeta2_u_map[*]=0
     
     whr = WHERE(unresolved EQ 0, nwhr)
     
     IF KEYWORD_SET(verbose) THEN BEGIN
        PRINT, 'Resolved clusters' 
     ENDIF
     
     t0 = LONG(systime(1))
     nb_steps = 100.
     IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
        ;; IF KEYWORD_SET(nr500) THEN BEGIN -> bug if beta model and nr500 is set
        r_array = nr500*c500/FLOAT(nb_steps)*(FINDGEN(nb_steps)+1)
     ENDIF ELSE BEGIN
        r_array = MAX(r_cut)/FLOAT(nb_steps)*(FINDGEN(nb_steps)+1)
     ENDELSE
     r_sup = MAX(r_array)
     nelts = LONG(5000.)
     npack = nwhr/nelts
     IF npack GE 1 THEN BEGIN
        FOR i=0,npack-1 DO BEGIN
           ;; print, i, npack
           jstart = i*nelts
           cat = catalog[whr[jstart:jstart+nelts-1]]
           IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) THEN BEGIN
              y_and_tau = NFW_PROFILE([cat.z], [cat.mass], r_array, $
                                      cosmo=cosmo, cluster=cluster, r_cut=r_cut[whr[jstart:jstart+nelts-1]], $
                                      chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, cm500=cm500, P0=P0, $
                                      c500=c500, nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, nstd=nstd)
           ENDIF ELSE BEGIN
              y_and_tau = BETA_PROFILE([cat.z], [cat.mass], r_array, $
                                       cosmo=cosmo, cluster=cluster, $
                                       beta=beta, r_cut=r_cut)
           ENDELSE
           FOR j=0, nelts-1 DO BEGIN
              ;; if j/100. EQ j/100 then print, i, npack, j, nelts
              IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) THEN BEGIN
                 thc_rad = catalog[whr[jstart+j]].ts / 60.*!dtor ; in rad
              ENDIF ELSE BEGIN
                 thc_rad = catalog[whr[jstart+j]].tc / 60.*!dtor ; in rad
              ENDELSE
              thc_array = r_array*thc_rad
              thc_sup = r_sup*thc_rad
              ANG2VEC, catalog[whr[jstart+j]].lat, catalog[whr[jstart+j]].lon, vector, /Astro
              QUERY_DISC, nside, vector, thc_sup, listpix, nlist, /inclusive
              ;; QUERY_DISC, nside, vector, thc_sup, listpix, nlist
              IF nlist GE 2 THEN BEGIN
                 PIX2ANG_RING, nside, listpix, theta_list, phi_list
                 ANG2VEC, theta_list, phi_list, vector_list
                 dist_list = ACOS((TOTAL(vector_list*(REPLICATE(1,nlist)#vector),2))<1)
                 
                 y_val_pix = (INTERPOL(y_and_tau[j].y_array, thc_array, dist_list))>0
                 IF TOTAL(y_val_pix) GT 0 THEN BEGIN
                    IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
                                ;IF KEYWORD_SET(nr500) THEN BEGIN -> bug if beta model and nr500 is set
                       cluster_cy = catalog[whr[jstart+j]].cyn500
                    ENDIF ELSE BEGIN
                       cluster_cy = catalog[whr[jstart+j]].cy
                    ENDELSE
                    y_val_pix = y_val_pix*cluster_cy/(TOTAL(y_val_pix)*(hpix*60.)^2.)
                    y_map[listpix] = y_map[listpix]+y_val_pix
                    t_val_pix = (INTERPOL(y_and_tau[j].tau_array, thc_array, dist_list))>0
                    t_val_pix = t_val_pix*cluster_cy/(TOTAL(t_val_pix)*(hpix*60.)^2.)* $
                                electron_mass/catalog[whr[jstart+j]].Te
                    tb_val_pix = t_val_pix*(catalog[whr[jstart+j]].v_radial)/c_light
                    taubeta_map[listpix] = taubeta_map[listpix]+tb_val_pix

                    ;; relativistic SZ effect
                    IF KEYWORD_SET(relativistic) THEN BEGIN
                       IF relativistic GE 1 THEN ythetae_map[listpix] = ythetae_map[listpix] + y_val_pix*catalog[whr[jstart+j]].te/electron_mass
                       IF relativistic GE 2 THEN ythetae2_map[listpix] = ythetae2_map[listpix] + y_val_pix*(catalog[whr[jstart+j]].te/electron_mass)^2
                       IF relativistic GE 3 THEN ythetae3_map[listpix] = ythetae3_map[listpix] + y_val_pix*(catalog[whr[jstart+j]].te/electron_mass)^3
                       IF relativistic GE 4 THEN ythetae4_map[listpix] = ythetae4_map[listpix] + y_val_pix*(catalog[whr[jstart+j]].te/electron_mass)^4
                    ENDIF

                    ;; polarised SZ effect
                    IF KEYWORD_SET(polar) THEN BEGIN
                       tb2_val_pix = ((catalog[whr[jstart+j]].v_transverse)/c_light)^2*t_val_pix
                       ;; here correction needed (important for extended clusters near the poles)
                       ;; tb2q_val_pix = tb2_val_pix*COS(2*catalog[whr[jstart+j]].angle)
                       ;; tb2u_val_pix = tb2_val_pix*SIN(2*catalog[whr[jstart+j]].angle)
                       theta_in = (90d0 - catalog[whr[jstart+j]].lat) * !DTOR
                       phi_in = catalog[whr[jstart+j]].lon * !DTOR
                       POLAR_NEARBY_ANGLE_ON_SPHERE, theta_in, phi_in, catalog[whr[jstart+j]].angle,theta_list, phi_list, angle_out
                       tb2q_val_pix = tb2_val_pix*COS(2*angle_out)
                       tb2u_val_pix = tb2_val_pix*SIN(2*angle_out)
                       ;;-------------------------------------------------------------------------
                       taubeta2_q_map[listpix] = taubeta2_q_map[listpix]+tb2q_val_pix
                       taubeta2_u_map[listpix] = taubeta2_u_map[listpix]+tb2u_val_pix
                    ENDIF

                 ENDIF ELSE BEGIN
                    unresolved[whr[jstart+j]] = 1
                 ENDELSE
              ENDIF ELSE BEGIN
                 unresolved[whr[jstart+j]] = 1
              ENDELSE
           ENDFOR
        ENDFOR
     ENDIF
     ;; IF NOT (npack EQ nwhr/FLOAT(nelts)) THEN BEGIN
     IF (npack NE nwhr/FLOAT(nelts)) THEN BEGIN
        jstart = npack*nelts
        cat = catalog[whr[jstart:nwhr-1]]
        IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) THEN BEGIN
           y_and_tau = NFW_PROFILE([cat.z], [cat.mass], r_array, $
                                   cosmo=cosmo, cluster=cluster, r_cut=r_cut[whr[jstart:nwhr-1]], $
                                   chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, cm500=cm500, P0=P0, c500=c500, $
                                   nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, nstd=nstd)
        ENDIF ELSE BEGIN
           y_and_tau = BETA_PROFILE([cat.z], [cat.mass], r_array, $
                                    cosmo=cosmo, cluster=cluster, $
                                    beta=beta, r_cut=r_cut)
        ENDELSE
        sub_nelts = nwhr-jstart
        FOR j=0, sub_nelts-1 DO BEGIN
           ;; if j/100. EQ j/100 then print, j, sub_nelts
           IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) THEN BEGIN
              thc_rad = catalog[whr[jstart+j]].ts / 60.*!dtor ; in rad
           ENDIF ELSE BEGIN
              thc_rad = catalog[whr[jstart+j]].tc / 60.*!dtor ; in rad
           ENDELSE
           thc_array = r_array*thc_rad
           thc_sup = r_sup*thc_rad
           ANG2VEC, catalog[whr[jstart+j]].lat, catalog[whr[jstart+j]].lon, vector, /Astro
           QUERY_DISC, nside, vector, thc_sup, listpix, nlist, /inclusive
           ;; QUERY_DISC, nside, vector, thc_sup, listpix, nlist
           IF nlist GE 2 THEN BEGIN
              PIX2ANG_RING, nside, listpix, theta_list, phi_list
              ANG2VEC, theta_list, phi_list, vector_list
              dist_list = ACOS((TOTAL(vector_list*(REPLICATE(1,nlist)#vector),2))<1)
              
              y_val_pix = (INTERPOL(y_and_tau[j].y_array, thc_array, dist_list))>0
              IF TOTAL(y_val_pix) GT 0 THEN BEGIN
                 IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
                    ;; IF KEYWORD_SET(nr500) THEN BEGIN -> bug if beta model and nr500 is set
                    cluster_cy = catalog[whr[jstart+j]].cyn500
                 ENDIF ELSE BEGIN
                    cluster_cy = catalog[whr[jstart+j]].cy
                 ENDELSE
                 y_val_pix = y_val_pix*cluster_cy/(TOTAL(y_val_pix)*(hpix*60.)^2.)
                 y_map[listpix] = y_map[listpix]+y_val_pix
                 t_val_pix = (INTERPOL(y_and_tau[j].tau_array, thc_array, dist_list))>0
                 t_val_pix = t_val_pix*cluster_cy/(TOTAL(t_val_pix)*(hpix*60.)^2.)* $
                             electron_mass/catalog[whr[jstart+j]].Te
                 tb_val_pix = t_val_pix*(catalog[whr[jstart+j]].v_radial)/c_light
                 taubeta_map[listpix] = taubeta_map[listpix]+tb_val_pix
                 
                 ;; relativistic SZ effect
                 IF KEYWORD_SET(relativistic) THEN BEGIN
                    IF relativistic GE 1 THEN ythetae_map[listpix] = ythetae_map[listpix] + y_val_pix*catalog[whr[jstart+j]].te/electron_mass
                    IF relativistic GE 2 THEN ythetae2_map[listpix] = ythetae2_map[listpix] + y_val_pix*(catalog[whr[jstart+j]].te/electron_mass)^2
                    IF relativistic GE 3 THEN ythetae3_map[listpix] = ythetae3_map[listpix] + y_val_pix*(catalog[whr[jstart+j]].te/electron_mass)^3
                    IF relativistic GE 4 THEN ythetae4_map[listpix] = ythetae4_map[listpix] + y_val_pix*(catalog[whr[jstart+j]].te/electron_mass)^4
                 ENDIF
                 
                 ;; polarised SZ effect
                 IF KEYWORD_SET(polar) THEN BEGIN
                    tb2_val_pix = ((catalog[whr[jstart+j]].v_transverse)/c_light)^2*t_val_pix
                    ;; here correction needed (important for extended clusters near the poles)
                    ;; tb2q_val_pix = tb2_val_pix*COS(2*catalog[whr[jstart+j]].angle)
                    ;; tb2u_val_pix = tb2_val_pix*SIN(2*catalog[whr[jstart+j]].angle)
                    theta_in = (90d0 - catalog[whr[jstart+j]].lat) * !DTOR
                    phi_in = catalog[whr[jstart+j]].lon * !DTOR
                    POLAR_NEARBY_ANGLE_ON_SPHERE, theta_in, phi_in, catalog[whr[jstart+j]].angle,theta_list, phi_list, angle_out
                    tb2q_val_pix = tb2_val_pix*COS(2*angle_out)
                    tb2u_val_pix = tb2_val_pix*SIN(2*angle_out)
                    ;;-------------------------------------------------------------------------
                    taubeta2_q_map[listpix] = taubeta2_q_map[listpix]+tb2q_val_pix
                    taubeta2_u_map[listpix] = taubeta2_u_map[listpix]+tb2u_val_pix
                 ENDIF
                 
              ENDIF ELSE BEGIN
                 unresolved[whr[jstart+j]] = 1
              ENDELSE
           ENDIF ELSE BEGIN
              unresolved[whr[jstart+j]] = 1
           ENDELSE
        ENDFOR
     ENDIF
     
     ;; Placing the unresolved clusters on the map
     ;;--------------------------------------------
     whu = WHERE(unresolved EQ 1, nwhu)
     ;; print, nwhr, nwhu
     IF KEYWORD_SET(verbose) THEN BEGIN
        PRINT, 'Unresolved cluster(s)' 
     ENDIF
     
     FOR i=0L, nwhu-1 DO BEGIN
        ;; if i/1000. eq i/1000 then print, i, nwhu
        clu_pix = catalog[whu[i]].pixel
        IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
           ;; IF KEYWORD_SET(nr500) THEN BEGIN -> bug if beta model and nr500 is set
           cluster_cy = catalog[whu[i]].cyn500
        ENDIF ELSE BEGIN
           cluster_cy = catalog[whu[i]].cy
        ENDELSE
        y_map[clu_pix] = y_map[clu_pix]+cluster_cy/(hpix*60)^2.
        taubeta_map[clu_pix] = taubeta_map[clu_pix]+cluster_cy/(hpix*60)^2.* $
                               electron_mass/catalog[whu[i]].Te*(catalog[whu[i]].v_radial)/c_light

        ;; relativistic SZ effect
        IF KEYWORD_SET(relativistic) THEN BEGIN
           IF relativistic GE 1 THEN ythetae_map[clu_pix] = ythetae_map[clu_pix] + cluster_cy/(hpix*60)^2.*catalog[whu[i]].te/electron_mass
           IF relativistic GE 2 THEN ythetae2_map[clu_pix] = ythetae2_map[clu_pix] + cluster_cy/(hpix*60)^2.*(catalog[whu[i]].te/electron_mass)^2
           IF relativistic GE 3 THEN ythetae3_map[clu_pix] = ythetae3_map[clu_pix] + cluster_cy/(hpix*60)^2.*(catalog[whu[i]].te/electron_mass)^3
           IF relativistic GE 4 THEN ythetae4_map[clu_pix] = ythetae4_map[clu_pix] + cluster_cy/(hpix*60)^2.*(catalog[whu[i]].te/electron_mass)^4
       ENDIF
        
        ;; polarised SZ effect
        IF KEYWORD_SET(polar) THEN BEGIN
           taubeta2_q_map[clu_pix] = taubeta2_q_map[clu_pix]+cluster_cy/(hpix*60)^2.* $
                                     electron_mass/catalog[whu[i]].Te* $
                                     ((catalog[whu[i]].v_transverse)/c_light)^2*COS(2*catalog[whu[i]].angle)
           taubeta2_u_map[clu_pix] = taubeta2_u_map[clu_pix]+cluster_cy/(hpix*60)^2.* $
                                     electron_mass/catalog[whu[i]].Te* $
                                     ((catalog[whu[i]].v_transverse)/c_light)^2*SIN(2*catalog[whu[i]].angle)
        ENDIF 
     ENDFOR
  ENDIF
  
  t1 = LONG(systime(1))
  
  ;; print, (t1-t0)/60.
  
closing:
  
END
