; -----------------------------------------------------------------------------
;
; Copyright (C) 2006, 2007, 2008, 2009, 2011, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SIMU_SZ
;
; PURPOSE:
;	Compute multiple SZ simulations
;
; CATEGORY:
;	SZ effect
;
; CALLING SEQUENCE:
;	SIMU_SZ, Nsim
;
; INPUTS:
;	Nsim:	The number of simulations you want.
;
; OPTIONAL INPUTS:
;	Parm2:	Describe optional inputs here. If you don't have any, just
;		delete this section.
;	
; KEYWORD PARAMETERS:
;	SIMDEB: 	First index of the simulations.
;	MAP_HD: 	The header of the map.
;	DIR:    	The directory where the simulations will be created.
;	SIM_NAME:    	The name of the simulation.
;	COSMO:    	The structure containing the cosmological parameters.
;	CLUSTER:    	The structure containing the structure formation parameters (cf. DEFINE_CLUSTER).
;	MASS_FUNCTION: 	The mass function ('press_schechter',
;	                'sheth_tormen', 'jenkins', 'evrard' or 'tinker').
;	POLAR:  	Set this keyword to compute polarised SZ maps.
;	IDL_SAVE:  	If set then an idl save file is created.
;
; OUTPUTS:
;	The code writes - fits files for thermal and kinetic SZ
;                       - fits files for polarised SZ if polar is set
;                       - fits file for the cluster catalog
;                       - idl save file for the cluster catalog and
;                         other parameters if idl_save is set 
;
; RESTRICTIONS:
;       Some files will be written
;
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	2002:   	Jean-Baptiste Melin, First version.
;	06/02/2006:	JBM, CLUSTER keyword added.
; 	February 2006:	JBM, POLAR keyword added.
; 	January 2007:	JBM, IDL_SAVE keyword added.
;	22/01/2007:	JBM, Use of HEADFITS and MODFITS to be faster.
;	21/03/2007:	JBM, Header of the cluster catalog improved.
;	17/07/2007:	JBM, rv (virial radius) and tv (theta virial) included.
;	05/01/2012:	Jacques Delabrouille, updated doc and PSM header.
;
;-
PRO SIMU_SZ, nsim, simdeb=simdeb, map_hd = map_hd, $
	     dir=dir, sim_name=sim_name, cosmo=cosmo, cluster=cluster, $
             mass_function=mass_function, polar=polar, idl_save=idl_save

IF NOT KEYWORD_SET(simdeb) THEN simdeb=0

IF KEYWORD_SET(map_hd) EQ 0 THEN BEGIN
	head=GENERATE_HEADER(nx=300, ny=300, lon_c=0, lat_c=0, pixdeg=2.5/60., angle=0., coord='G')
ENDIF ELSE BEGIN
	head=GENERATE_HEADER(nx=SXPAR(map_hd,'NAXIS1'), ny=SXPAR(map_hd,'NAXIS2'), lon_c=0, lat_c=0,$
		pixdeg=SXPAR(map_hd,'CDELT2'),angle=0., coord='G')
ENDELSE

IF NOT KEYWORD_SET(dir) THEN dir = dialog_pickfile(/dir)
dir = FIX_SEPARATOR(dir)

IF NOT KEYWORD_SET(sim_name) THEN BEGIN
  print, 'Give a sim_name !!!'
  goto, closing
ENDIF

seed = LONG(systime(1))
a = RANDOMU(seed,1)

FOR i=simdeb, nsim-1+simdeb DO BEGIN
 
 print, 'generating simulation ', i
 
 iseed = seed
 	
 CREATE_SZ_SIMULATION, head, catalog, y_map, taubeta_map, taubeta2_q_map, taubeta2_u_map, $
		         cosmo=cosmo, cluster=cluster, $
                         mass_function=mass_function, iseed=seed
	
 WRITEFITS, dir+sim_name+'_y_'+STRTRIM(string(i),1)+'.fits', y_map, head
 WRITEFITS, dir+sim_name+'_dT_kin_'+STRTRIM(string(i),1)+'.fits', -taubeta_map, head

 IF KEYWORD_SET(polar) THEN BEGIN
    WRITEFITS, dir+sim_name+'_q_pol_'+STRTRIM(string(i),1)+'.fits', taubeta2_q_map, head
    WRITEFITS, dir+sim_name+'_u_pol_'+STRTRIM(string(i),1)+'.fits', taubeta2_u_map, head
 ENDIF

 nclu = N_ELEMENTS(catalog)
 catalog.name=STRTRIM(STRING(LINDGEN(nclu)),1)
 MWRFITS, catalog, dir+sim_name+'_catalog_'+STRTRIM(string(i),1)+'.fits', /CREATE
 hd=HEADFITS(dir+sim_name+'_catalog_'+STRTRIM(string(i),1)+'.fits', exten=1)
 SXADDPAR, hd, 'TTYPE1', SXPAR(hd,'TTYPE1'), 'Name'
 SXADDPAR, hd, 'TTYPE2', SXPAR(hd,'TTYPE2'), 'Redshift'
 SXADDPAR, hd, 'TTYPE3', SXPAR(hd,'TTYPE3'), 'Mass [10^15 Msol unit]'
 SXADDPAR, hd, 'TTYPE4', SXPAR(hd,'TTYPE4'), 'Longitude [deg]'
 SXADDPAR, hd, 'TTYPE5', SXPAR(hd,'TTYPE5'), 'Latitude [deg]'
 SXADDPAR, hd, 'TTYPE6', SXPAR(hd,'TTYPE6'), 'Center pixel number'
 SXADDPAR, hd, 'TTYPE7', SXPAR(hd,'TTYPE7'), 'Radial velocity [km/s]'
 SXADDPAR, hd, 'TTYPE8', SXPAR(hd,'TTYPE8'), 'Transverse velocity [km/s]'
 SXADDPAR, hd, 'TTYPE9', SXPAR(hd,'TTYPE9'), 'Angle of transverse velocity [rad]'
 SXADDPAR, hd, 'TTYPE10', SXPAR(hd,'TTYPE10'), 'Temperature [keV]'
 SXADDPAR, hd, 'TTYPE11', SXPAR(hd,'TTYPE11'), 'Central electronic density [cm^-3]'
 SXADDPAR, hd, 'TTYPE12', SXPAR(hd,'TTYPE12'), 'Core radius [Mpc]'
 SXADDPAR, hd, 'TTYPE13', SXPAR(hd,'TTYPE13'), 'Virial radius [Mpc]'
 SXADDPAR, hd, 'TTYPE14', SXPAR(hd,'TTYPE14'), 'Central optical depth'
 SXADDPAR, hd, 'TTYPE15', SXPAR(hd,'TTYPE15'), 'Central Compton parameter'
 SXADDPAR, hd, 'TTYPE16', SXPAR(hd,'TTYPE16'), 'Core radius [arcmin]'
 SXADDPAR, hd, 'TTYPE17', SXPAR(hd,'TTYPE17'), 'Virial radius [arcmin]'
 SXADDPAR, hd, 'TTYPE18', SXPAR(hd,'TTYPE18'), 'Integrated Compton parameter Y [arcmin^2]'
 SXADDPAR, hd, 'h', cosmo.h, 'Hubble constant H0/(100 km/s/Mpc)'
 SXADDPAR, hd, 'omega_m', cosmo.omega_m, 'Omega matter'
 SXADDPAR, hd, 'omega_b', cosmo.omega_b, 'Omega baryon'
 SXADDPAR, hd, 'lambda', cosmo.omega_de, 'Omega lambda'
 SXADDPAR, hd, 'sigma 8', cluster.sigma_8, 'Sigma 8'
 SXADDPAR, hd, 'ns', cluster.ns, 'Spectral index'
 SXADDPAR, hd, 'gamma', cluster.gamma, 'Power spectrum shape'
 SXADDPAR, hd, 'mass function', mass_function, 'Cluster mass function'
 SXADDPAR, hd, 'gf', cluster.gf, 'Cluster gas fraction'
 SXADDPAR, hd, 'T*', cluster.t_star, 'Normalisation of the M-T relation'
 MODFITS, dir+sim_name+'_catalog_'+STRTRIM(string(i),1)+'.fits', 0, hd, exten=1

 IF KEYWORD_SET(idl_save) THEN BEGIN
   SAVE, cosmo, cluster, catalog, mass_function, iseed, $
       filename=dir+sim_name+'_variables_'+STRTRIM(string(i),1)+'.sav'
 ENDIF

ENDFOR

closing:
END
