; -----------------------------------------------------------------------------
;
; Copyright (C) 2011-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	COLORCOR
;
; PURPOSE:
;	Compute the color correction for a given spectral band and a given emission law
;
; CATEGORY:
;	Emission law, Instrument model
;
; CALLING SEQUENCE:
;	Result = COLORCOR(band, emlaw, nuref=, specind=, temp=, /double)
;
; INPUTS:
;	Band:	A spectral band created with GET_BAND_STRUCT, or
;		extracted from an instrument model created with LOAD_INSTRUMENT
;	Emlaw:	Name of an emission law (can be 'POWERLAW' or 'GREYBODY' so far).
;
; KEYWORD PARAMETERS:
;	NUREF:		The reference frequency for which the color correction is computed.
;	SPECIND:	The spectral index of the powerlaw or greybody emissivity.
;	TEMP:		The temperature of the greybody.
;
; OUTPUTS:
;	Result is the number by which one should MULTIPLY the brightness
;	averaged in the band (in W/m2/sr/Hz or in MJy/sr) to get the
;	brightness at the reference frequency.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	First define a band using the GET_BAND_STRUCT} procedure:
;	IDL> band = GET_BAND_STRUCT('INSTR', instr='HFI_RIMO', version='DX9-v1', channel=['857-1'])
;
;	Compute the color correction coefficient for a 10K greybody with nu^1.6 emissivity:
;	IDL> c = COLORCOR(band, 'GREYBODY', nuref=857e9, specind=1.6, temp=10.)
;
;	Print the result
;	IDL> PRINT, c
;	The printed result is 1.0140338.
;
; MODIFICATION HISTORY:
; 	16/11/2011:	Jacques Delabrouille, First version.
;-
FUNCTION COLORCOR, band, emlaw, nuref=nuref, specind=specind, temp=temp, double=double, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() LT 1)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'COLORCOR: function that provides the color correction for a band and an emission law'
     PSM_INFO_MESSAGE, 'SYNTAX: result = COLORCOR(band, emlaw, nuref=, specind=, temp=, /double)'
     PSM_INFO_MESSAGE, "'band' is the input frequency band (created with GET_BAND_STRUCT)"
     PSM_INFO_MESSAGE, "'emlaw' is the name of the emission law considered (POWERLAW, GREYBODY)"
     PSM_INFO_MESSAGE, "'nuref' is the reference frequency for which the color correction is computed"
     PSM_INFO_MESSAGE, "'specind' and 'temp' are a spectral index and a temperature for the emission law."
     PSM_INFO_MESSAGE, "Multiply the flux observed in the band by the output of COLORCOR to get the flux"
     PSM_INFO_MESSAGE, "at the reference frequency 'nuref'."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF


  CASE STRUPCASE(emlaw) OF
     'GREYBODY': ok=1B
     'POWERLAW': ok=1B
     ELSE: ok=0B
  ENDCASE

  IF ok THEN BEGIN
     RETURN, 1/BANDINTEGRATED_EMLAW(band, emission_name=emlaw, nuref=nuref, temp=temp, specind=specind, double=double)
  ENDIF ELSE BEGIN
     MESSAGE, 'Case not implemented'
  ENDELSE

END
