; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	EMLAW_PARAMLIST
;
; PURPOSE:
;	Function that returns the parameters of a given emission
;	law. It also gives (as an output keyword) the units for each
;	of the parameters, an information string, the type, and the
;	(minimal, normal) dimension of the parameter (which may be an
;	array). All units of flux are returned as 'unknown' (they
;	should be set elsewhere).  

;
; CATEGORY:
;	Emission law
;
; CALLING SEQUENCE:
;	result = EMLAW_PARAMLIST(Law)
;
; INPUTS:
;	Emission_name:	Name of the emission considered (Case
;	        	insensitive). Can be any among the following: 'CMB',
;	        	'BLACKBODY', 'GREYBODY', 'POWERLAW', 'CURVPOWERLAW',
;	        	'FREEFREE', 'SPINDUST', 'THERMALSZ', 'DIRAC', 
;	        	'RELATSZ1', 'RELATSZ2', 'RELATSZ3', 'RELATSZ4'
;	        	'SUBMMLAPI', 'SPIRALGAL', 'STARBURSTGAL'. 
;
; KEYWORD PARAMETERS:
;	INCLUDE_NURANGE:	If set, include NUMIN and NUMAX among
;	                        structure tags. Input keyword.
;
;	INCLUDE_AMPL:   	If set, include AMPL among
;	                        structure tags. Input keyword.
;
;	UNITS:           	Default units of the parameters of the
;	                        emission law. Output keyword.
;
;	INFO:           	Information about the parameters of the
;	                        emission law. Output keyword.
;
;	TYPE:           	Type of the parameters of the
;	                        emission law. Output keyword.
;
;	DIMENSION:           	Dimension of the parameters of the
;	                        emission law. Output keyword.
;
;	DOUBLE:           	Set for double precision type, simple
;	                        precision by default. Input keyword.
;
; OUTPUTS:
;	A list of tags for the given emission law.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	29/10/2009:	Jacques Delabrouille, First version.
;	11/06/2010:	JD, Implemented DIRAC emission law.
;	05/01/2012:	JD, general clean-up, replaced old 'AMPL' tag
;	                by 'VALUEATNUREF', updated doc header.
;-
FUNCTION EMLAW_PARAMLIST, emission_name, include_nurange=include_nurange, include_ampl=include_ampl, $
                          units=units, info=info, type=type, dimens=dimens, double=double, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'EMLAW_PARAMLIST: function to get the list of parameters used by an emission law'
     PSM_INFO_MESSAGE, 'SYNTAX: result = EMLAW_PARAMLIST(emission_name, /include_nurange, /include_ampl, $'
     PSM_INFO_MESSAGE, '                                 units=, info=, type=, dimens=, /double)'
     PSM_INFO_MESSAGE, "emission_name can be:"
     PSM_INFO_MESSAGE, "CMB, BLACKBODY, GREYBODY, POWERLAW, CURVPOWERLAW, FREEFREE, SPINDUST, THERMALSZ, DIRAC"
     PSM_INFO_MESSAGE, "RELATSZ1, RELATSZ2, RELATSZ3, RELATSZ4, SUBMMLAPI, SPIRALGAL, STARBURSTGAL"
     PSM_INFO_MESSAGE, "set /include_nurange to include 'NUMIN' and 'NUMAX' in parameter list"
     PSM_INFO_MESSAGE, "set /include_ampl to include amplitude 'VALUEATNUREF' in parameter list"
     PSM_INFO_MESSAGE, "Other keywords are output, except /double which sets the type returned in keyword 'type'"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF


CASE STRUPCASE(emission_name) OF 
'CMB': BEGIN 
   units = ['Hz']
   info = ['reference frequency']
;;   dimens = [1]
   IF NOT KEYWORD_SET(double) THEN type = ['E'] ELSE type=['D']
   res = ['NUREF']
END
'BLACKBODY': BEGIN
   units = ['Kelvin']
   info = ['blackbody temperature']
;;   dimens = [1]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E'] ELSE type=['D','D']
   res = ['NUREF','TEMP']
END
'GREYBODY': BEGIN
   units = ['Hz','unitless','Kelvin']
   info = ['reference frequency','spectral index','greybody temperature']
;;   dimens = [1,1,1,1]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E','E'] ELSE type=['D','D','D']
   res = ['NUREF','SPECIND','TEMP']
END
'POWERLAW': BEGIN
   units = ['Hz','unitless']
   info = ['reference frequency','spectral index']
;;   dimens = [1,1]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E'] ELSE type=['D','D']
   res = ['NUREF','SPECIND']
END
'CURVPOWERLAW': BEGIN
   units = ['Hz','unitless','Hz','unitless']
   info = ['reference frequency','spectral index','ref. freq. for curvature',$
           'curvature amplitude']
;;   dimens = [1,1,1,1]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E','E','E'] ELSE type=['D','D','D','D']
   res = ['NUREF','SPECIND','CURVNUREF','CURVAMPL']
END
'FREEFREE': BEGIN
   units = ['Hz','Kelvin']
   info = ['reference frequency','electron temperature']
;;   dimens = [1,1]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E'] ELSE type=['D','D']
   res = ['NUREF','TEMP']
END
'SPINDUST': BEGIN
   units = ['Hz','unitless']
   info = ['reference frequency','Draine-Lazarian composition']
   PSM_DEBUG_MESSAGE, 'Check that everything is OK with the possible EXTRA component in DL composition', routine='EMLAW_PARAMLIST', infolimit=3, stoplimit=3
;;   dimens = [1,7]
   IF NOT KEYWORD_SET(double) THEN type = ['E','7E'] ELSE type=['D','7D']
   res = ['NUREF','DLCOMP']
END
'THERMALSZ': BEGIN 
   units = ['Hz','keV']
   info = ['reference frequency','electron temperature']
;;   dimens = [2]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E'] ELSE type=['D','D']
   res = ['NUREF','TEMP']
END
'RELATSZ1': BEGIN
   units = ['Hz']
   info = ['reference frequency']
;;   dimens = [1]
   IF NOT KEYWORD_SET(double) THEN type = ['E'] ELSE type=['D']
   res = ['NUREF']
END
'RELATSZ2': BEGIN
   units = ['Hz']
   info = ['reference frequency']
;;   dimens = [1]
   IF NOT KEYWORD_SET(double) THEN type = ['E'] ELSE type=['D']
   res = ['NUREF']
END
'RELATSZ3': BEGIN
   units = ['Hz']
   info = ['reference frequency']
;;   dimens = [1]
   IF NOT KEYWORD_SET(double) THEN type = ['E'] ELSE type=['D']
   res = ['NUREF']
END
'RELATSZ4': BEGIN
   units = ['Hz']
   info = ['reference frequency']
;;   dimens = [1]
   IF NOT KEYWORD_SET(double) THEN type = ['E'] ELSE type=['D']
   res = ['NUREF']
END
'DIRAC': BEGIN
   units = ['Hz']
   info = ['reference frequency']
;;   dimens = [1]
   IF NOT KEYWORD_SET(double) THEN type = ['E'] ELSE type=['D']
   res = ['NUREF']
END
'SUBMMLAPI': BEGIN
   units = ['Hz','unitless']
   info = ['reference frequency','redshift']
;;   dimens = [2]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E'] ELSE type=['D','D']
   res = ['NUREF','REDSHIFT']
END
'SPIRALGAL': BEGIN
   units = ['Hz','unitless']
   info = ['reference frequency','redshift']
;;   dimens = [2]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E'] ELSE type=['D','D']
   res = ['NUREF','REDSHIFT']
END
'STARBURSTGAL': BEGIN
   units = ['Hz','unitless']
   info = ['reference frequency','redshift']
;;   dimens = [2]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E'] ELSE type=['D','D']
   res = ['NUREF','REDSHIFT']
END
'SUBMMCAI': BEGIN
   units = ['Hz','unitless']
   info = ['reference frequency','redshift']
;;   dimens = [2]
   IF NOT KEYWORD_SET(double) THEN type = ['E','E'] ELSE type=['D','D']
   res = ['NUREF','REDSHIFT']
END
   ELSE: MESSAGE, 'Emission law not implemented'
ENDCASE

IF KEYWORD_SET(include_ampl) NE 0 THEN BEGIN
   units = ['unknown',units]
   info = ['flux at ref. freq.',info]
;;   dimens = [1,dimens]
   res = ['AMPL',res]
ENDIF
IF KEYWORD_SET(include_nurange) NE 0 THEN BEGIN
   units = ['Hz','Hz',units]
   info = ['min. freq. range value','max. freq. range value',info]
;;   dimens = [1,1,dimens]
   res = ['NUMIN','NUMAX',res]
ENDIF

RETURN, res

END
