; -----------------------------------------------------------------------------
;
; Copyright (C) 2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	BORDER_OF_MASK
;
; PURPOSE:
;	...
;
;-
PRO BORDER_OF_MASK, mask, p0, p1, np0=np0, np1=np1, help=help, ordering=ordering

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'BORDER_OF_MASK: function to find border pixels in a mask'
     PSM_INFO_MESSAGE, 'SYNTAX: BORDER_OF_MASK, mask, p0, p1, np0=, np1=, ordering='
     PSM_INFO_MESSAGE, "mask is an input HEALPix map containing zeros and ones (can be file or memory array)"
     PSM_INFO_MESSAGE, "set ordering to either 'RING' (the default) or 'NESTED' for input in memory array format"
     PSM_INFO_MESSAGE, "p0 and p1 are border pix in (where mask=0) and outside (where mask=1) the mask respectively"
     PSM_INFO_MESSAGE, 'np0 and np1 are the number of valid pixels in p0 and p1'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT DEFINED(mask) THEN BEGIN
     PSM_INFO_MESSAGE, 'BORDER_OF_MASK: Input mask not defined !'
     GOTO, closing2
  ENDIF

  s = SIZE(mask, /type)

  ;; Find ordering and nside
  ;;-------------------------
  IF NOT KEYWORD_SET(ordering) THEN BEGIN
     IF s LE 5 THEN BEGIN
        ordering='RING'
     ENDIF
     IF s EQ 7 THEN BEGIN
        h = HEADFITS(mask,exten=1)
        ordering = STRTRIM(SXPAR(h,'ORDERING'),2)
     ENDIF
  ENDIF
  nside = GET_NSIDE(mask)
  npix = NSIDE2NPIX(nside)

  ;;-------------------------------------------------------
  ;; Border candidates 
  ;;-------------------------------------------------------
  PSM_DEBUG_MESSAGE, 'Fix the threshold between the two methods', routine='BORDER_OF_MASK'
  whmask = WHERE(mask EQ 1, nmasked)
  IF nmasked EQ 0 THEN BEGIN
     np0 = 0
     np1 = 0
     GOTO, closing
  ENDIF

  ;; Border candidates : Algorithm for low number of holes
  ;;-------------------------------------------------------
  IF TOTAL(1-mask)/N_ELEMENTS(mask) LE 5E-2 THEN BEGIN
     whmask = WHERE(mask EQ 0, nwhmask)
     p0 = LONARR(1)
     p1 = LONARR(1)
     FOR i=0L, nwhmask-1 DO BEGIN
        HEALPIX_NEIGHBOURS, whmask[i], ip, nside=nside, ordering=ordering
        whp1 = WHERE(mask[ip] EQ 1, nwhp1)
        IF (nwhp1 GE 1) THEN BEGIN
           p0 = [p0,whmask[i]]
           p1 = [p1,ip[whp1]]
        ENDIF
     ENDFOR
     p0 = p0[1:*]
     p1 = p1[1:*]
     np0 = N_ELEMENTS(p0)
     np1 = N_ELEMENTS(p1)
     GOTO, closing
  ENDIF

  ;; Border candidates : Algorithm for low number of valid pixels
  ;;--------------------------------------------------------------
  IF TOTAL(mask)/N_ELEMENTS(mask) LE 5e-2 THEN BEGIN
     whnomask = WHERE(mask EQ 1, nwhnomask)
     p0 = LONARR(1)
     p1 = LONARR(1)
     FOR i=0L, nwhnomask-1 DO BEGIN
        HEALPIX_NEIGHBOURS, whnomask[i], ip, nside=nside, ordering=ordering
        whp0 = WHERE(mask[ip] EQ 0, nwhp0)
        IF (nwhp0 GE 1) THEN BEGIN
           p1 = [p1,whnomask[i]]
           p0 = [p0,ip[whp0]]
        ENDIF
     ENDFOR
     p0 = p0[1:*]
     p1 = p1[1:*]
     np0 = N_ELEMENTS(p0)
     np1 = N_ELEMENTS(p1)
     GOTO, closing
  ENDIF

  ;; Border candidates : Algorithm for intermediate cases
  ;;------------------------------------------------------
     pixsize = NSIDE2PIXSIZE(nside, /arcmin)
     almfile = PSM_TMPFILENAME(extension='.fits')
     PSM_MAP2ALM, mask, almfile, ordering=ordering, /nohdr
     smoothalmfile = PSM_TMPFILENAME(extension='.fits')
     beam_in = GET_BEAM_STRUCT(0.)
     beam_out = GET_BEAM_STRUCT(pixsize*SQRT(2.))
     CHANGE_ALM_RESOLUTION, almfile, smoothalmfile, beam_in=beam_in, beam_out=beam_out
     PSM_ALM2MAP, smoothalmfile, smoothmask, hpx_nside=nside, ordering=ordering
     
     IF s LE 5 THEN BEGIN
        p0 = WHERE((mask EQ 0) AND (smoothmask GE 0.15), nwh0)
        p1 = WHERE((mask EQ 1) AND (TEMPORARY(smoothmask) LE 0.85), nwh1)
     ENDIF
     IF s EQ 7 THEN BEGIN
        READ_MAP, mask, ma, ordering=ordering
        p0 = WHERE((ma EQ 0) AND (smoothmask GE 0.15), nwh0)
        p1 = WHERE((ma EQ 1) AND (TEMPORARY(smoothmask) LE 0.85), nwh1)
     ENDIF

  ;; If input mask is a memory array
  ;;---------------------------------
  IF s LE 5 THEN BEGIN

     IF nwh0 NE 0 THEN BEGIN
        m = 0*mask
        m[p1] = 1
        FOR p=0L, nwh0-1 DO BEGIN
           HEALPIX_NEIGHBOURS, p0[p], ip, nside=nside, ordering=ordering
           m[ip] = 1
        ENDFOR
        m = m*mask
        p1 = WHERE((m EQ 1), np1)
     ENDIF
     
     IF np1 NE 0 THEN BEGIN
        m = 0*mask
        m[p0] = 1
        FOR p=0L, np1-1 DO BEGIN
           HEALPIX_NEIGHBOURS, p1[p], ip, nside=nside, ordering=ordering
           m[ip] = 1
        ENDFOR
        m = m*(1-mask)
        p0 = WHERE((m EQ 1), np0)
     ENDIF
     GOTO, closing

  ENDIF

  IF s EQ 7 THEN BEGIN

     IF nwh0 NE 0 THEN BEGIN
        m = 0*ma
        m[p1] = 1
        FOR p=0L, nwh0-1 DO BEGIN
           HEALPIX_NEIGHBOURS, p0[p], ip, nside=nside, ordering=ordering
           m[ip] = 1
        ENDFOR
        m = TEMPORARY(m)*ma
        p1 = WHERE((m EQ 1), np1)
     ENDIF
     
     IF np1 NE 0 THEN BEGIN
        m = 0*ma
        m[p0] = 1
        FOR p=0L, np1-1 DO BEGIN
           HEALPIX_NEIGHBOURS, p1[p], ip, nside=nside, ordering=ordering
           m[ip] = 1
        ENDFOR
        m = TEMPORARY(m)*(1-TEMPORARY(ma))
        p0 = WHERE((m EQ 1), np0)
     ENDIF
     GOTO, closing
     
  ENDIF


  MESSAGE, 'Format of input mask not allowed in call', /info

closing:

  p0 = p0[PSM_UNIQ(p0)]
  p1 = p1[PSM_UNIQ(p1)]

closing2:
END
