; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2011
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ERODE_BADPIX
;
; PURPOSE:
;	Procedure to perform inpainting into a region of bad pixels in
;	a healpix map, upt to some distance of the border of the bad region.
;
; CATEGORY:
;	Map tools
;
; CALLING SEQUENCE:
;	ERODE_BADPIX, Map, ORDERING=, BEAM=, ITER=, USE_NEST=
;
; INPUTS:
;	Map:		An input HEALPix map with some pixels set to the
;			healpix bad value (-1.63750e+30)
;
; KEYWORD PARAMETERS:
;	ORDERING:	'RING' or 'NESTED', default is RING
;
;	BEAM:		set this keyword to a neighborhood (in
;			arcminutes) which is used to erode the bad pixels.
;			If it is not defined, the software uses direct
;			neighbours, unless 'iter' is set, in which
;			case the software uses pixels in a
;			neighborhood of size proportional to 'iter'. 
;
;	USE_NEST:	If 'use_nest' is set, 'beam' is disregarded,
;			the code uses the nested structure of HEALPix
;			to erode the bad pixels. 
;
; OUTPUTS:
;	Map:		The input map is modified, border pixels in
;			the region flagged as bad being
;			interpolate/extrapolated from valid neighbours.
;
; SIDE EFFECTS:
;	The input map is modified.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	15/09/2010:	Jacques Delabrouille, First version.
; 	16/09/2010:	Guillaume Castex, implement the BEAM keyword.
; 	21/09/2010:	JD, implement the USE_NEST option.
; 	20/01/2011:	GC, implement the ordering keyword.
; 	22/09/2012:	JD, doc and license header.
;-
PRO ERODE_BADPIX, map, ordering=ordering, beam=beam, iter=iter, use_nest=use_nest, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'ERODE_BADPIX: procedure to perform inpainting in a healpix map'
     PSM_INFO_MESSAGE, 'SYNTAX: ERODE_BADPIX, map, ordering=, beam=, iter=, /use_nest'
     PSM_INFO_MESSAGE, "'map' is the input map (array), containing bad values (-1.63750e+30) which will be filled."
     PSM_INFO_MESSAGE, "Set 'ordering' to 'RING' [the default] or 'NESTED'."
     PSM_INFO_MESSAGE, "Set 'beam' to a FWHM (in arcmin) which defines the neighborhood used to erode the bad pixels."
     PSM_INFO_MESSAGE, "If 'beam' not set, the software uses direct neighbours, unless 'iter' is set, in which case"
     PSM_INFO_MESSAGE, "the software uses pixels in a neighborhood of size proportional to 'iter'"
     PSM_INFO_MESSAGE, "If 'use_nest' is set, 'beam' is disregarded, the code uses the nested structure of HEALPix"
     PSM_INFO_MESSAGE, "to erode the bad pixels."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF


  s = SIZE(map, /type)

  ;; Find ordering and nside
  ;;-------------------------
  IF NOT KEYWORD_SET(ordering) THEN BEGIN
     IF s LE 5 THEN BEGIN
        ordering='RING'
     ENDIF
     IF s EQ 7 THEN BEGIN
        h = HEADFITS(map,exten=1)
        ordering = STRTRIM(SXPAR(h,'ORDERING'),2)
     ENDIF
  ENDIF
  nside = GET_NSIDE(map)
  npix = NSIDE2NPIX(nside)
  masked_pix = WHERE(map EQ !healpix.bad_value, nwh)
  IF nwh EQ 0 THEN GOTO, closing

  mask = BYTARR(N_ELEMENTS(map)) + 1B
  mask[masked_pix] = 0B
  BORDER_OF_MASK, mask, p0, np0=np0,ordering=ordering

  If NOT KEYWORD_SET(beam) THEN BEGIN
     IF KEYWORD_SET(use_nest) THEN BEGIN
        FOR i=0L, np0-1 DO BEGIN
           HEALPIX_NEIGHBOURS, p0[i], neighbours, nside=nside, ordering=ordering 
           value = TOTAL(map[neighbours]*mask[neighbours])/TOTAL(mask[neighbours])
           map[p0[i]] = value
        ENDFOR
     ENDIF ELSE BEGIN
        omega=4*!pi/npix
        radius=4*iter*acos(1-omega/(2*!pi)) ; from solid angle of a circular cap: Omega = 2pi*(1-cos(alpha))
                                ; cos(alpha) = 1-Omega/2pi
                                ; Hence pixeldiameter = 2*acos(1-Omega/2pi), and we take the double, times the iteration order.
        FOR i=0L, np0-1 DO BEGIN
                     
           IF iter EQ 1 THEN BEGIN
              HEALPIX_NEIGHBOURS, p0[i], neighbours, nside=nside, ordering=ordering 
              value = TOTAL(map[neighbours]*mask[neighbours])/TOTAL(mask[neighbours])
           ENDIF ELSE BEGIN
              
              CASE STRUPCASE(ordering) OF
                 'RING': BEGIN
                            PIX2VEC_RING, nside, p0[i], vec
                            QUERY_DISC, nside, vec, radius, listpix, nlist
                         END
                 'NESTED': BEGIN
                              PIX2VEC_NEST, nside, p0[i], vec
                              QUERY_DISC, nside, vec, radius, listpix, nlist,/nested
                           END
              ENDCASE

              IF nlist NE 0 THEN BEGIN
                 value = TOTAL(map[listpix]*mask[listpix])/TOTAL(mask[listpix])
              ENDIF ELSE BEGIN
                 STOP
                 GOTO, closing
              ENDELSE
           ENDELSE
           map[p0[i]] = value
        ENDFOR
     ENDELSE
  ENDIF ELSE BEGIN
     beamradiusradian = beam/60.*!dtor ; radius of the beam image is taken to be the FWHM 
     FOR i=0L, np0-1 DO BEGIN
        CASE STRUPCASE(ordering) OF
           'RING': PIX2VEC_RING, nside, p0[i], vec
           'NESTED': PIX2VEC_NEST, nside, p0[i], vec
        ENDCASE
        QUERY_DIST, nside, vec, beamradiusradian, listpix, nlist, distpix, /inclusive
        value = TOTAL(map[listpix]*mask[listpix])/TOTAL(mask[listpix])
        map[p0[i]] = value
     ENDFOR
  ENDELSE

  closing:                      ; If there are no bad pixels, then do nothing
END
