; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Jacques Delabrouille
;   Mathieu Remazeilles
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HPX_LOCALCOVAR
;
; PURPOSE:
;	This function computes the local variance of a Healpix map or
;	the local covariance of two Healpix maps.	 
;
; CATEGORY:
;	Map tools
;
; CALLING SEQUENCE:
;	Result = HPX_LOCALCOVAR(Map1, Map2)
;
; INPUTS:
;	Map1: HEALPix map  
;
; OPTIONAL INPUTS:
;	Map2: HEALPix map
;	
; KEYWORD PARAMETERS:
;	ORDERING: HEALPix ordering of input maps Map1 and Map2, and of the
;	output map Result (default='RING')
;
;       DEGRADE: Factor to degrade the product of maps Map1*Map2 (must
;       be a power of 2, default is 8)
;
;       NITER: Sets the number of iterations of the HPX_PIXSMOOTHING
;       procedure (done 1+niter times)
;
;       NOCENTRALPIX: Set /nocentralpix for discarding the (degrade x degrade) central pixels in covariance calculation
;
; OUTPUTS:
;	Result: HEALPix map of the local variance (resp. local covariance) of the input HEALPix
;	map (resp. of the two input HEALPix maps)
;
; OPTIONAL OUTPUTS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Smooth the product of the maps map1*map2 by averaging over
;	all neighbouring pixels
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	16/09/2010:	Jacques Delabrouille, First version.
;       05/01/2012:	JD, PSM license header.
;       16/01/2012:     Mathieu Remazeilles, doc header.
;-

FUNCTION HPX_LOCALCOVAR, map1, map2, ordering=ordering, degrade=degrade, maxremote=maxremote, nocentralpix=nocentralpix, silent=silent, noupgrade=noupgrade, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'HPX_LOCALCOVAR: function to calculate local covariance of two maps'
     PSM_INFO_MESSAGE, 'SYNTAX: result = HPX_LOCALCOVAR(map1, map2, ordering=, degrade=, maxremote=, nocentralpix=, /silent)'
     PSM_INFO_MESSAGE, "'map1' and 'map2' are input healpix maps (omit 'map2' for computing local var of 'map1')"
     PSM_INFO_MESSAGE, "'ordering' is the ordering of map1 and map2, and of the output"
     PSM_INFO_MESSAGE, "The procedure first degrades map1.map2 by a factor degrade (must be a power of 2, "
     PSM_INFO_MESSAGE, "default is 8), and then uses HPX_PIXSMOOTHING to average over all neighbouring pixels"
     PSM_INFO_MESSAGE, "at degraded resolution"
     PSM_INFO_MESSAGE, "'maxremote' sets corresponding keyword in the HPX_PIXSMOOTHING procedure (defines size of kernel)"
     PSM_INFO_MESSAGE, "Set /nocentralpix for discarding the (degrade x degrade) central pixels in covariance calculation"
     PSM_INFO_MESSAGE, "Set /noupgrade to output covariance at degraded nside"
     PSM_INFO_MESSAGE, "Set /silent for silent processing"
     PSM_INFO_MESSAGE, "See also LOCALCOVAR for an alternate version that uses harmonic space smoothing"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ON_ERROR, 2

  npix = N_ELEMENTS(map1)
  IF DEFINED(map2) THEN IF N_ELEMENTS(map2) NE npix THEN STOP

  IF NOT DEFINED(ordering) THEN ordering='RING'
  IF DEFINED(map2) THEN BEGIN
     whok = WHERE((map2 NE !healpix.bad_value) AND (map1 NE !healpix.bad_value), nwhok)
     map = REPLICATE(!healpix.bad_value, npix)
     IF nwhok NE 0 THEN map[whok] = map2[whok]*map1[whok]
  ENDIF ELSE BEGIN
     whok = WHERE(map1 NE !healpix.bad_value, nwhok)
     map = map1
     IF nwhok NE 0 THEN map[whok] = map[whok]*map1[whok]
  ENDELSE
  npix = N_ELEMENTS(map)
  nside = NPIX2NSIDE(npix)

  ;; First degrade a bit to speed-up smoothing
  IF NOT KEYWORD_SET(degrade) THEN degrade=8
  nside_out = (nside/degrade) > 1
  UD_GRADE, TEMPORARY(map), stat, nside_out=nside_out, order_in=ordering, order_out='RING'

  ;; Then smooth in pixel space
  HPX_PIXSMOOTHING, TEMPORARY(stat), stat2, ordering='RING', maxremote=maxremote, nocentralpix=nocentralpix, silent=silent
  stat = TEMPORARY(stat2)

  ;; Then UD_GRADE back
  IF NOT KEYWORD_SET(noupgrade) THEN UD_GRADE, TEMPORARY(stat), stat, nside_out=nside, order_in='RING', order_out=ordering


  RETURN, stat
END
