; -----------------------------------------------------------------------------
;
; Copyright (C) 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HPX_PIX_NEIGHBOURS
;
; PURPOSE:
;	Finds iteratively the neighbours of a healpix pixel, up to a
;	maximum 'remoteness' (set by the keyword MAXREMOTE).
;
; CATEGORY:
;	Map tools
;
; CALLING SEQUENCE:
;	HPX_PIX_NEIGHBOURS, Ipix, Remote, List
;
; INPUTS:
;	Pixnum: 	Pixel number.
;
; KEYWORD PARAMETERS:
;	MAXREMOTE: 	Maximum 'remoteness'. 1 means direct
;	                neighbours, 2 neighbours of neighbours,
;	                etc. If maxremote is not set, then the
;	                function returns the input pixel itself.
;
;	NSIDE:  	HEALPix nside used for pixel numbering.
;
;	ORDERING:	HEALPix ordering used for pixel numbering
;	                (default='RING').
;
; OUTPUTS:
;
;	Remote: 	The 'remoteness' of the output pixel list,
;	                from 0 (the input pixel) to maxremote.
;	                Remote=1 corresponds to direct neighbours,
;	                Remote=2 to neighbours of neighbours, etc.
;
;	List:    	The output pixel list, in the vicinity of the
;	                input pixel.
;
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	02/02/2012:	Jacques Delabrouille, First version.
;
;-
PRO HPX_PIX_NEIGHBOURS, ipix, remote, list, maxremote=maxremote, nside=nside, ordering=ordering
  
  ON_ERROR, 2
  IF N_ELEMENTS(ipix) NE 1 THEN MESSAGE, 'Case not implemented'
  IF NOT KEYWORD_SET(maxremote) THEN maxremote=0
  IF NOT KEYWORD_SET(ordering) THEN use_ordering='RING' ELSE use_ordering = ordering

  remote = [0]
  list = [LONG(ipix)]

  IF maxremote NE 0 THEN BEGIN
     previouslist = list

     FOR r=1, maxremote DO BEGIN
        
        ;; Find list of new candidates
        ;;-----------------------------
        newlist=[-1L]
        FOR i=0, N_ELEMENTS(previouslist)-1 DO BEGIN
           HEALPIX_NEIGHBOURS, previouslist[i], neighbours, nside=nside, ordering=use_ordering
           newlist = [newlist,neighbours]
        ENDFOR
        newlist = newlist[1:*]
        newlist = newlist[SORT(newlist)]
        newlist = newlist[UNIQ(newlist)]

        keep = BYTARR(N_ELEMENTS(newlist))+1B

        ;; Eliminate those already in list
        ;;---------------------------------
        newp=0
        oldp=0
        
        WHILE (newp LT N_ELEMENTS(newlist) AND (oldp LT N_ELEMENTS(list))) DO BEGIN
           IF newlist[newp] LT list[oldp] THEN newp=newp+1L ELSE $
           IF newlist[newp] GT list[oldp] THEN oldp=oldp+1L ELSE $
           IF newlist[newp] EQ list[oldp] THEN BEGIN
              keep[newp] = 0
              newp = newp+1L
              oldp = oldp+1L
           ENDIF
        ENDWHILE

        newlist = newlist[WHERE(keep)]

        nel = N_ELEMENTS(list)
        newnel = N_ELEMENTS(newlist)
        remote = [remote, replicate(r,N_ELEMENTS(newlist))]
        list = [list, newlist]
        s = SORT(list)
        list = list[s]
        remote = remote[s]
        previouslist = newlist

     ENDFOR

  ENDIF
  
END
