; -----------------------------------------------------------------------------
;
; Copyright (C) 2005, 2009, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	REPIX_TANMAP
;
; PURPOSE:
;	This function repixelizes a map with a given header Map_hd to
;	an other header New_map_hd. Works for tangential maps
;	(gnomonic projection). 
;
; CATEGORY:
;	Map tools
;
; CALLING SEQUENCE:
;	result = REPIX_TANMAP(Map, Map_hd, New_map_hd)
;
; INPUTS:
;	Map:    	The map you want to repixelize.
;	Map_hd:  	The header of the input map.
;	Newap_hd:	The header you want for your output map.
;
; OUTPUTS:
;	A repixellised map
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	October 2005:	Jean-Baptiste Melin, First version.
;	December 2009:	Jacques Delabrouille, General revision.
;	05/01/2012:	JD, Added/modified header documentation.
;-
FUNCTION REPIX_TANMAP, map, map_hd, new_map_hd

  IF ( N_PARAMS() EQ 0 ) THEN BEGIN
     print,'CALL IS : new_map=REPIX_MAP(map, map_hd, new_map_hd)'
     GOTO, closing
  ENDIF
  
  ;; Map 
  nx = SXPAR(map_hd,'NAXIS1')
  ny = SXPAR(map_hd,'NAXIS2')
  d = 1d0/nx
  
  ;; New map
  new_nx = SXPAR(new_map_hd,'NAXIS1')
  new_ny = SXPAR(new_map_hd,'NAXIS2')
  new_d = 1d0/new_nx
  
  lon_val = d*(DINDGEN(nx)-(nx-1.)/2.)
  lat_val = d*(DINDGEN(ny)-(ny-1.)/2.)
  
  new_lon_val = new_d*(DINDGEN(new_nx)-(new_nx-1.)/2.)
  new_lat_val = new_d*(DINDGEN(new_ny)-(new_ny-1.)/2.)
  
  lon_pix = INTARR(nx)
  lon_frac = DBLARR(nx)
  
  lon_pix[0] = 0
  lon_frac[0] = (((new_lon_val[0]+new_d/2)-(lon_val[0]-d/2))/d)<1
  FOR i=1, nx-1 DO BEGIN
     whp = WHERE((lon_val[i]-d/2)-(new_lon_val-new_d/2) GE 0, nwhp)
     ind = MAX(whp)
     lon_pix[i] = ind
     lon_frac[i] = (((new_lon_val[ind]+new_d/2)-(lon_val[i]-d/2))/d)<1
  ENDFOR
  
  lat_pix = lon_pix
  lat_frac = lon_frac
  
  new_map = DBLARR(new_nx, new_ny)
  FOR i=0, nx-1 DO BEGIN
     FOR j=0, ny-1 DO BEGIN
        new_map[lon_pix[i],lat_pix[j]] = new_map[lon_pix[i],lat_pix[j]]+lon_frac[i]*lat_frac[j]*map[i,j]
        IF lon_pix[i] LT new_nx-1 THEN BEGIN
           new_map[lon_pix[i]+1,lat_pix[j]] = new_map[lon_pix[i]+1,lat_pix[j]]+(1-lon_frac[i])*lat_frac[j]*map[i,j]
        ENDIF
        IF lat_pix[j] LT new_ny-1 THEN BEGIN
           new_map[lon_pix[i],lat_pix[j]+1] = new_map[lon_pix[i],lat_pix[j]+1]+lon_frac[i]*(1-lat_frac[j])*map[i,j]
        ENDIF
        IF lon_pix[i] LT new_nx-1 AND lat_pix[j] LT new_ny-1 THEN BEGIN
           new_map[lon_pix[i]+1,lat_pix[j]+1] = new_map[lon_pix[i]+1,lat_pix[j]+1]+(1-lon_frac[i])*(1-lat_frac[j])*map[i,j]
        ENDIF
     ENDFOR
  ENDFOR
  
  n_map = new_map*(d/new_d)^2
  
  RETURN, n_map
  
closing:
END
