; -----------------------------------------------------------------------------
;
; Copyright (C) 2011, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	MATRIX_SQRT
;
; PURPOSE:
;	...
;
;-
FUNCTION MATRIX_SQRT, matrix, status, check=check, accept_error=accept_error, inverse=inverse, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() EQ 0)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'MATRIX_SQRT: function which computes the square root of a symmetric non-negative matrix'
     PSM_INFO_MESSAGE, 'SYNTAX: result = MATRIX_SQRT(matrix, status, /check, accept_error=)'
     PSM_INFO_MESSAGE, 'status is an optional output, equal to 1B if the computation went OK and 0B otherwise'
     PSM_INFO_MESSAGE, 'set /check to check that the square root is correct up to numerical errors'
     PSM_INFO_MESSAGE, "if /check is set, then set 'accept_error' to the maximum relative acceptable error (default = 1e-8)"
     PSM_INFO_MESSAGE, "if /inverse is set, then the inverse square root is computed instead"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ON_ERROR, 2

  mat = DOUBLE(matrix)
  status = 1B

  ;; First check that the matrix is symmetric
  ;;------------------------------------------
  matt = TRANSPOSE(mat)
  ok = 1B
  FOR i=0L, N_ELEMENTS(mat)-1 DO ok = ok AND mat[i] EQ matt[i]
  IF NOT ok THEN BEGIN
     MESSAGE, 'input matrix is not symmetric'
     status = 0B
  ENDIF

  ;; Do SVD
  ;;--------
  LA_SVD, mat, s, u, v, /double
  u = (u+v)/2
  
  ;; Check positivity
  ;;------------------
  wh = WHERE(s LT 0, nwh)
  IF nwh NE 0 THEN BEGIN
     MESSAGE, 'input matrix is not positive'
     status = 0B
  ENDIF

  IF NOT KEYWORD_SET(inverse) THEN d = SQRT(s) ELSE d = 1/SQRT(s)
  dim = N_ELEMENTS(d)
  t = DBLARR(dim,dim)
  FOR i=0, dim-1 DO t[i,i]=d[i]

  result = u##(t##TRANSPOSE(u))
  result = (result+transpose(result))/2

  ;; Check the result if requested
  ;;-------------------------------
  IF KEYWORD_SET(check) NE 0 THEN BEGIN
     matt = result ## result
     IF NOT KEYWORD_SET(inverse) THEN error = matt ## INVERT(mat) ELSE error = matt ## mat
     FOR i=0, dim-1 DO error[i,i] = error[i,i]-1D
     error = MAX(ABS(error))
     IF NOT KEYWORD_SET(accept_error) THEN accept_error = 1d-8 
     ok = error LT accept_error
     IF NOT ok THEN BEGIN
        status = 0B
        MESSAGE, 'Failure in computing the square root of the matrix', /info
     ENDIF
  ENDIF

  RETURN, result

END
