; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ALM_STRUCTURE
;
; PURPOSE:
;	Function to generate a structure used to store sets of
;	harmonic coefficients a_lm.
;
; CATEGORY:
;	Spherical transforms
;
; CALLING SEQUENCE:
;	Result = ALM_STRUCTURE(NFIELDS=, FIELDNAMES=, LMIN=, LMAX=, /DOUBLE)
;
; INPUTS:
;	None
;
; KEYWORD PARAMETERS:
;	NFIELDS:	Number of fields in the structure (e.g. 3 for T, E, B)
;
;	FIELDNAMES:	A string array conprising the field names, e.g. ['T','E','B'].
;
;	LMIN:		Minimum value of l for the harmonic coefficients
;	LMAX:		Maximum value of l for the harmonic coefficients
;
;	DOUBLE:		Set this keyword for double precision arrays
;			(default single precision).
;
; OUTPUTS:
;	Result is a structure that can be used to store sets of
;	harmonic coefficients
;
; OPTIONAL OUTPUTS:
;	None
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	19/09/2012:	JD, doc header.
;-
FUNCTION ALM_STRUCTURE, nfields=nfields, fieldnames=fieldnames, lmin=lmin, lmax=lmax, double=double, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, 'ALM_STRUCTURE: function to create a structure of spherical harmonics coefficients'
     PRINT, 'SYNTAX: result = ALM_STRUCTURE( nfields=, fieldnames=, lmin=, lmax=, /double)'
     RETURN, 0
  ENDIF

  IF KEYWORD_SET(fieldnames) NE 0 THEN BEGIN
     nfield = N_ELEMENTS(fieldnames)
     IF defined(nfields) THEN BEGIN
        IF nfield NE nfields THEN MESSAGE, 'Number of fields incompatible with dimension of fieldnames vector'
     ENDIF ELSE BEGIN
        nfields = nfield
     ENDELSE
     length = STRLEN(fieldnames)
     IF MAX(length) NE 1 THEN MESSAGE, 'Field names should be one letter or character only at present'
  ENDIF ELSE BEGIN
     IF NOT DEFINED(nfields) THEN nfields=1
     fieldnames = REPLICATE('',nfields)
  ENDELSE
  
  IF NOT KEYWORD_SET(lmin) THEN lmin=0L
  IF NOT KEYWORD_SET(lmax) THEN lmax=3000L

  index = ALM_INDEX(lmin,lmax)
  ni = N_ELEMENTS(index)

  creation_string = 'alm = {field:fieldnames,lmin:lmin,lmax:lmax'
  IF NOT KEYWORD_SET(double) THEN FOR i=0, nfields-1 DO creation_string = creation_string + ',a_'+ NUMBER2STRING(i+1) + ':FLTARR(ni,2)' $
  ELSE FOR i=0, nfields-1 DO creation_string = creation_string + ',a_'+ NUMBER2STRING(i+1) + ':DBLARR(ni,2)'
  creation_string = creation_string + '}'

  ok = EXECUTE(creation_string)
  RETURN, alm

END
