; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_ALM2CL
;
; PURPOSE:
;	...
;
;-
PRO PSM_ALM2CL, alm, cl, lmin=lmin, lmax=lmax, double=double, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_ALM2CL: procedure to calculate empirical cl from alm coefficients between lmin and lmax'
     PSM_INFO_MESSAGE, 'SYNTAX: PSM_ALM2CL, alm, cl, lmin=, lmax=, /double'
     PSM_INFO_MESSAGE, "Input 'alm' can be a file or an array of files, or a [n_alm,2,n_fields] memory array,"
     PSM_INFO_MESSAGE, "or an alm structure (alm.field, alm.lmin, alm.lmax, alm.a_1, alm.a_2, ...)"
     PSM_INFO_MESSAGE, 'set /double to force double precision for output cl'
     PSM_INFO_MESSAGE, 'alm can be either a structure created with alm_structure.pro, or an array [n_index,2,n_fields]'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  almtype = SIZE(alm, /type)

  ;; If the alm are provided as a file
  ;;-----------------------------------
  IF almtype EQ 7 THEN BEGIN

     nel = N_ELEMENTS(alm)
     nalm_in_file = INTARR(nel)

     FOR f=0, nel-1 DO BEGIN
        ok = ISFITSFILE(alm[0])
        IF NOT ok THEN BEGIN
           PSM_INFO_MESSAGE, "Input alm '"+alm[0]+"' does not seem to be a valid fits file, returning"
           RETURN
        ENDIF ELSE BEGIN
           FITS_INFO, alm[0], n_ext = n_ext 
           nalm_in_file[f] = n_ext
        ENDELSE 
     ENDFOR

     nnn = TOTAL(nalm_in_file)
     
     ;; Find lmax
     lmax=0L
     lmax = FITSFILE_LMAX(alm[0], exten=1) & IF nalm_in_file[0] GE 1 THEN FOR ex=2, nalm_in_file[0] DO lmax = (lmax > FITSFILE_LMAX(alm[0], exten=ex))

     FOR i=0, nel-1 DO BEGIN
        FOR ex=1, nalm_in_file[i] DO lmax  = (lmax > FITSFILE_LMAX(alm[i], exten=ex))
     ENDFOR

     ;; Make an array
     alm_array = REPLICATE(0.,LMAX2NALM(lmax),2,nnn)
     IF PRECISION_ISDOUBLE() THEN alm_array = DOUBLE(TEMPORARY(alm_array))

     ;; Read and store all alms
     current=0
     FOR i=0, nel-1 DO BEGIN
        inputalmfile = alm[i]
        FITS2ALM, index, a, inputalmfile, 'ALL'
        FOR n=0, nalm_in_file[i]-1 DO BEGIN
           alm_array[0:N_ELEMENTS(a[*,0])-1,0,current] = a[*,0,n]
           alm_array[0:N_ELEMENTS(a[*,1])-1,1,current] = a[*,1,n]  
           current = current+1
        ENDFOR     
     ENDFOR
     PSM_ALM2CL, alm_array, cl, lmin=lmin, lmax=lmax, double=double
  ENDIF

  ;; If the alm are provided as a structure
  ;;----------------------------------------
  IF almtype EQ 8 THEN BEGIN

     type = SIZE(alm.a_1,/type)
     IF type EQ 5 THEN BEGIN
        double=1 
     ENDIF ELSE BEGIN
        IF NOT KEYWORD_SET(double) THEN double=0
     ENDELSE
     IF NOT KEYWORD_SET(lmin) THEN lmin=alm.lmin ELSE lmin = lmin > alm.lmin
     IF NOT KEYWORD_SET(lmax) THEN lmax=alm.lmax ELSE lmax = lmax < alm.lmax

     index = ALM_INDEX(lmin,lmax)
     INDEX2LM, index, ll, mm

     ;; Create the cl
     ;;---------------
     cl = CL_STRUCTURE(fieldnames=alm.field, lmin=lmin, lmax=lmax, double=double)
     IF double THEN number=0d0 ELSE number=0.

     ;; Compute cl values
     ;;-------------------
     n = N_ELEMENTS(alm.field)
     FOR l=lmin,lmax DO BEGIN
        wh = WHERE(ll EQ l, nwh)
        IF nwh NE 0 THEN BEGIN
           imin = MIN(index[wh],p)
           FOR i=1, n DO FOR j=i, n DO BEGIN
              cxx = number

              ;; double precision
              ;;------------------
              IF DOUBLE THEN BEGIN
                 IF mm[wh[p]] EQ 0 THEN BEGIN
                    nmodes = 2L*nwh-1L
                    ok = EXECUTE('cxx = cxx - DOUBLE(alm.a_'+NUMBER2STRING(i)+'[wh[p],0])*DOUBLE(alm.a_'+NUMBER2STRING(j)+'[wh[p],0])') 
                    ;; this is done so that a_l0 is only once in the sum
                 ENDIF ELSE BEGIN
                    nmodes = 2L*nwh
                 ENDELSE
                 ok = EXECUTE('cxx = cxx + 2.*TOTAL(DOUBLE(alm.a_'+NUMBER2STRING(i)+'[wh,*])*DOUBLE(alm.a_'+NUMBER2STRING(j)+'[wh,*]))')

              ;; simple precision
              ;;---------------
              ENDIF ELSE BEGIN                 
                 IF mm[wh[p]] EQ 0 THEN BEGIN
                    nmodes = 2L*nwh-1L
                    ok = EXECUTE('cxx = cxx - alm.a_'+NUMBER2STRING(i)+'[wh[p],0]*alm.a_'+NUMBER2STRING(j)+'[wh[p],0]') 
                    ;; this is done so that a_l0 is only once in the sum
                 ENDIF ELSE BEGIN
                    nmodes = 2L*nwh
                 ENDELSE
                 ok = EXECUTE('cxx = cxx + 2.*TOTAL(alm.a_'+NUMBER2STRING(i)+'[wh,*]*alm.a_'+NUMBER2STRING(j)+'[wh,*])')
              ENDELSE


              cxx = cxx/nmodes
              ok = EXECUTE('cl.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[l-lmin] = cxx')
           ENDFOR
        ENDIF
     ENDFOR
  ENDIF


  ;; If the alm are provided as an array
  ;;-------------------------------------
  IF almtype LE 5 THEN BEGIN

     type = SIZE(alm,/type)
     ndim = SIZE(alm,/n_dimensions)
     dim = SIZE(alm,/dimensions)
     IF type EQ 5 THEN BEGIN
        double=1 
     ENDIF ELSE BEGIN
        IF NOT KEYWORD_SET(double) THEN double=0
     ENDELSE
     IF NOT KEYWORD_SET(lmin) THEN lmin=0
     almlmax = NALM2LMAX(dim[0])
     IF NOT KEYWORD_SET(lmax) THEN lmax=almlmax ELSE lmax = lmax < almlmax

     index = ALM_INDEX(lmin,lmax)
     INDEX2LM, index, ll, mm

     ;; Number of alm
     ;;---------------
     IF ndim EQ 3 THEN n = dim[2] ELSE BEGIN
        dim= [dim,1]
        alm = REFORM(TEMPORARY(alm), dim[0], dim[1], 1)
        n=1
     ENDELSE

     ;; Create the cl
     ;;---------------
     cl = CL_STRUCTURE(nfields=n, lmin=lmin, lmax=lmax, double=double)
     IF double THEN number=0d0 ELSE number=0.

     ;; Compute cl values
     ;;-------------------

     FOR l=lmin,lmax DO BEGIN
        wh = WHERE(ll EQ l, nwh)
        IF nwh NE 0 THEN BEGIN
           imin = MIN(index[wh],p)
           FOR i=1, n DO FOR j=i, n DO BEGIN
              cxx = number

              ;; double precision
              ;;------------------
              IF DOUBLE THEN BEGIN
                 IF mm[wh[p]] EQ 0 THEN BEGIN
                    nmodes = 2L*nwh-1L
                    cxx = cxx - DOUBLE(alm[wh[p],0,i-1])*DOUBLE(alm[wh[p],0,j-1])
                    ;; this is done so that a_l0 is only once in the sum
                 ENDIF ELSE BEGIN
                    nmodes = 2L*nwh
                 ENDELSE
                 cxx = cxx + 2.*TOTAL(DOUBLE(alm[wh,*,i-1])*DOUBLE(alm[wh,*,j-1]))

              ;; simple precision
              ;;---------------
              ENDIF ELSE BEGIN                 
                 IF mm[wh[p]] EQ 0 THEN BEGIN
                    nmodes = 2L*nwh-1L
                    cxx = cxx - alm[wh[p],0,i-1]*alm[wh[p],0,j-1]
                    ;; this is done so that a_l0 is only once in the sum
                 ENDIF ELSE BEGIN
                    nmodes = 2L*nwh
                 ENDELSE
                 cxx = cxx + 2.*TOTAL(alm[wh,*,i-1]*alm[wh,*,j-1])
              ENDELSE


              cxx = cxx/nmodes
              ok = EXECUTE('cl.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[l-lmin] = cxx')
           ENDFOR
        ENDIF
     ENDFOR
     
     ;; put alm back in original format if needed
     IF ndim EQ 2 THEN alm = REFORM(TEMPORARY(alm),dim[0],dim[1])
     
  ENDIF

  ;; fix units if the units are provided in the input alm fits files
  IF almtype EQ 7 THEN BEGIN
     nnn = N_ELEMENTS(alm)
     IF nnn NE 0 THEN BEGIN
        FOR i=0, nnn-1 DO BEGIN
           h = HEADFITS(alm[0],exten=1)
           units = SXPAR(h,'TUNIT2', count=count)
           IF count EQ 0 THEN BEGIN
              units='unknown'
           ENDIF ELSE BEGIN
              ok = CHECKUNIT(units)
              IF NOT ok THEN UNIT2PSMUNIT, units, units
           ENDELSE
           cl.units[i] = units
        ENDFOR
     ENDIF
  ENDIF
  

END
