; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2011, 2012
;   Soumen Basak
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_CL2ALM
;
; PURPOSE:
;	...
;
;-

PRO PSM_CL2ALM, cl_in, alm, lmin=lmin, lmax=lmax, seed=seed, almobs=almobs_in, nlobs=nlobs_in, double=double, tolerance=tolerance, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 2)) THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_CL2ALM: procedure to generate harmonic modes given a power spectrum'
     PSM_INFO_MESSAGE, 'SYNTAX: PSM_CL2ALM, cl_in, alm, lmin=, lmax=, seed=, almobs=, nlobs=, /double, tolerance='
     PSM_INFO_MESSAGE, 'cl_in is an input Cl structure (generated with CL_STRUCTURE)'
     PSM_INFO_MESSAGE, 'alm is the output table of alm harmonic modes, randomly generated'
     PSM_INFO_MESSAGE, 'Use almobs for an input set of observed alm that constrains the generated harmonic modes,'
     PSM_INFO_MESSAGE, 'with corresponding nlobs Cl structure for defining the errors in the constraining observed alm.'
     PSM_INFO_MESSAGE, 'Field names in nlobs that match field names in cl_in will be used for constraining the output.'
     PSM_INFO_MESSAGE, 'set lmin to the minimum value of l (default = MIN(cl.l))'
     PSM_INFO_MESSAGE, 'set lmax to the minimum value of l (default = MAX(cl.l))'
     PSM_INFO_MESSAGE, 'set /double to force double precision for output alm.'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF
  
  IF NOT KEYWORD_SET(tolerance) THEN tolerance=1e-5

  IF MAX(cl_in.l) NE (N_ELEMENTS(cl_in.l)-1) THEN MESSAGE, 'Unexpected ell values in c_ell, fix your code'

  nclfs = N_ELEMENTS(cl_in.field)
  cl = cl_in

  IF DEFINED(almobs_in) THEN BEGIN
     almobs = almobs_in
  ENDIF
  IF DEFINED(nlobs_in) THEN BEGIN
     nlobs = nlobs_in
  ENDIF

  IF NOT DEFINED(lmin) THEN lmin = MIN(cl.l)
  IF NOT DEFINED(lmax) THEN lmax = MAX(cl.l)

  IF DEFINED(nlobs) THEN BEGIN
     nnlobsfs = N_ELEMENTS(nlobs.field)
     IF DEFINED(almobs) THEN BEGIN
        nalmobs = N_ELEMENTS(almobs(*,0,0))
        lobsmax = NALM2LMAX(nalmobs)
        lobsmax = MIN([lobsmax,MAX(nlobs.l)])
        ;; lobsmax = lmax
        IF N_ELEMENTS(almobs(0,0,*)) NE nnlobsfs THEN BEGIN
           PRINT, "number of fields in almobs and nlobs does not match"
           PRINT, N_ELEMENTS(almobs(0,0,*)), nnlobsfs
           STOP
        ENDIF
     ENDIF ELSE BEGIN
        PRINT, "almobs in not defined"
        STOP
     ENDELSE
  ENDIF ELSE BEGIN
     nnlobsfs = 0
     lobsmax = lmax
  ENDELSE
  

  ;; Define a new set of Cl, alm and nlm, 'renormalised', i.e. such that diag
  ;; elements at each ell are either 0 or 1
  ;;------------------------------------------------------------
  nl = N_ELEMENTS(cl_in.l)
  IF KEYWORD_SET(double) THEN renorm_vector = DBLARR(nclfs, nl) ELSE renorm_vector = FLTARR(nclfs, nl)
  FOR l=0, nl-1 DO BEGIN
     FOR i=1, nclfs DO BEGIN
        ok = EXECUTE('cii = cl.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(i)+'[l]')
        IF NOT ok THEN STOP
        IF cii NE 0 THEN norm_cii=SQRT(cii) ELSE norm_cii=1
        renorm_vector[i-1,l] = 1/norm_cii
     ENDFOR
  ENDFOR

  FOR l=0, nl-1 DO BEGIN
     FOR i=1, nclfs DO BEGIN
        FOR j=i, nclfs DO BEGIN
           ok = EXECUTE('cl.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[l] = cl_in.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[l] * renorm_vector[i-1,l] * renorm_vector[j-1,l]')
           IF NOT ok THEN STOP
        ENDFOR
     ENDFOR
  ENDFOR

  IF DEFINED(almobs) THEN BEGIN
     PRINT, 'almobs defined'
     nfield_almobs = N_ELEMENTS(almobs_in[0,0,*])
     
     nlo = NALM2LMAX(N_ELEMENTS(almobs_in[*,0,0]))
     nlo = nlo<lmax
     
     FOR i=0, nfield_almobs-1 DO BEGIN
        FOR l=0, nlo DO BEGIN
           almobs[LMAX2NALM(l-1):LMAX2NALM(l-1)+l,*,i] = almobs_in[LMAX2NALM(l-1):LMAX2NALM(l-1)+l,*,i] * renorm_vector[i,l]
        ENDFOR
     ENDFOR
  ENDIF

  IF DEFINED(nlobs) THEN BEGIN
     PRINT, 'nlobs defined'
     nclfs_n = N_ELEMENTS(nlobs_in.field)
     nl_n = N_ELEMENTS(nlobs_in.l)
     nl_n = nl_n<(lmax+1)
     
     FOR l=0, nl_n-1 DO BEGIN
        FOR i=1, nclfs_n DO BEGIN
           FOR j=i, nclfs_n DO BEGIN
              ok = EXECUTE('nlobs.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[l] = nlobs_in.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+'[l] * renorm_vector[i-1,l] * renorm_vector[j-1,l]')
              IF NOT ok THEN STOP
           ENDFOR
        ENDFOR
     ENDFOR
  ENDIF

  index=ALM_INDEX(0,lmax)
  INDEX2LM, index, ll, mm

  IF NOT KEYWORD_SET(seed) THEN BEGIN
    PSM_SEEDSTATUS, verbosity=0, exists=exists
    IF exists THEN seed=GETSEED()
  ENDIF
  alm = RANDOMN(seed,N_ELEMENTS(index),2,nclfs, double=double)
  alm[WHERE(mm EQ 0),1,*] = 0
  IF lmin GT 0 THEN alm[WHERE(ll LT lmin),*,*] = 0

  FOR l = lmin, (lobsmax < MAX(cl.l)) DO BEGIN
     whl = WHERE(cl.l EQ l, nwhl)
     IF nwhl EQ 0 THEN STOP
     cyy = CL2COV(whl[0], cl, nclfs=nclfs, /double)

     whll = WHERE(ll EQ l, nwhll)

     IF DEFINED(nlobs) THEN BEGIN
        whlobs = WHERE(nlobs.l EQ l, nwhlobs)
     ENDIF ELSE BEGIN
        nwhlobs = 0
     ENDELSE

     ;;IF nnlobsfs GT 0 THEN BEGIN
     IF ((nnlobsfs GT 0) AND (nwhlobs NE 0)) THEN BEGIN
        ;;cnn = CL2COV(whl[0], nlobs, nclfs=nnlobsfs, double=double)
        cnn = CL2COV(whlobs[0], nlobs, nclfs=nnlobsfs, double=double)

        dxy =  INTARR(nclfs,nnlobsfs)
        fieldconstrained = BYTARR(nclfs)
        FOR f =0L, nnlobsfs-1 DO BEGIN & $
           ismatch = STRMATCH(cl.field,nlobs.field[f]) & $
           dxy(*,f) = ismatch & $
           fieldconstrained = fieldconstrained+ismatch & $
        ENDFOR
        scyy = dxy ## (cyy ## TRANSPOSE(dxy))
        sczz = cnn + scyy
        nzfs = nnlobsfs
        wzf = WHERE(DIAG_MATRIX(sczz) GT 0, nwzf)
        
        IF nwzf NE 0 THEN BEGIN
           nwzfs = N_ELEMENTS(wzf)
           dzwz = INTARR(nwzfs,nzfs)
           FOR f = 0L,nwzfs-1 DO dzwz(f,wzf(f))=1  
           scww = TRANSPOSE(dzwz) ## (sczz ## dzwz) 
           IF nwzfs GT 1 THEN BEGIN
              invscww = LA_INVERT(scww, /double, status=la_invert_status)   
              IF la_invert_status NE 0 THEN BEGIN
                 PRINT, "input array is not invertible"
                 PRINT, "la_invert_status=",la_invert_status
                 STOP
              ENDIF
           ENDIF ELSE invscww = 1 / scww
           invsczz = dzwz ## (invscww ## TRANSPOSE(dzwz))
           uyx = cyy ## (TRANSPOSE(dxy) ## invsczz)
           vyy = cyy - (uyx ## (dxy ## cyy))
           vyy = (vyy+TRANSPOSE(vyy))/2

           ;; put to 0 elements which should be 0 (tentative)
           ;;------------------------------------------------
           zeroing_matrix = DBLARR(nclfs,nclfs)
           diag_zeroing = REPLICATE(1d0,nclfs)
           diagnoise = DIAG_MATRIX(cnn)
           whnoise0 = WHERE(diagnoise EQ 0, nwh_noise0)
           IF nwh_noise0 NE 0 THEN BEGIN & $
              wh = WHERE(fieldconstrained EQ 1, nwh) & $
              diag_zeroing[wh[whnoise0]] = 0 & $
           ENDIF
           FOR i=0, nclfs-1 DO zeroing_matrix[i,i]=diag_zeroing[i]
           vyy = zeroing_matrix##vyy##zeroing_matrix

        ENDIF ELSE BEGIN
           vyy = cyy
        ENDELSE

     ENDIF ELSE vyy = cyy

     wyf = WHERE(DIAG_MATRIX(vyy) GT 0, nwyfs)
     
     IF nwyfs NE 0 THEN BEGIN
        dywy =  INTARR(nwyfs,nclfs)
        FOR f=0L,nwyfs-1 DO dywy(f,wyf(f))=1  
        svyy = TRANSPOSE(dywy) ## (vyy ## dywy)

        ;; diagpart = DIAG_MATRIX(svyy)
        ;; rescale = SQRT(diagpart)        
        ;; usedim = N_ELEMENTS(svyy[*,0])
        ;; FOR c1=0, usedim-1 DO svyy[c1,*] = svyy[c1,*] / rescale
        ;; FOR c1=0, usedim-1 DO svyy[*,c1] = svyy[*,c1] / rescale
        keepsvyy = svyy

        IF nwyfs GT 1 THEN BEGIN
           LA_CHOLDC, svyy, /double, status=la_choldc_status
           IF la_choldc_status NE 0 THEN BEGIN
              IF GET_CAREFULNESS() GE 2 THEN BEGIN
                 PRINT, 'l_error =',l
                 PRINT, "input array is not positive definite"
                 PRINT, "la_choldc_status=",la_choldc_status
                 STOP
              ENDIF ELSE BEGIN
                 ;; could use SVD...
                 STOP ;; To be tested thoroughly
                 svyy = keepsvyy
                 LA_SVD, svyy, W, U, V
                 u = (u+v)/2
                 u = u ## DIAG_MATRIX(SQRT(W>0))
                 newsvyy = u ## TRANSPOSE(u)
                 wh = WHERE(ABS(keepsvyy-newsvyy) GT tolerance, nwh)
                 IF nwh GT 0 THEN MESSAGE, 'Generation of alm has failed: a matrix is not definite positive up to allowed numerical errors' $
                 ELSE LA_CHOLDC, newsvyy, /double, status=la_choldc_status
              ENDELSE
           ENDIF
           FOR f=0L,nwyfs-2 DO svyy(f+1:*,f)=0 
        ENDIF ELSE svyy=SQRT(svyy)
        vyy = dywy ## (svyy ## TRANSPOSE(dywy)) 
        alm(whll[0],0,*) = vyy ## REFORM(alm(whll[0],0,*),1,nclfs)
        alm(whll[0],1,*) = 0
        IF KEYWORD_SET(double) THEN vyy = vyy/SQRT(2.d0) ELSE vyy = vyy/SQRT(2.0)
        IF nwhll GT 1 THEN BEGIN
           alm(whll[1:*],0,*) = vyy ## REFORM(alm(whll[1:*],0,*),nwhll-1,nclfs)
           alm(whll[1:*],1,*) = vyy ## REFORM(alm(whll[1:*],1,*),nwhll-1,nclfs)
        ENDIF
        IF ((nnlobsfs GT 0) AND (nwhlobs NE 0)) THEN BEGIN
           alm(whll[0],0,*) = alm(whll[0],0,*) + (uyx ## REFORM(almobs(whll[0],0,*),1,nnlobsfs))
           alm(whll[0],1,*) = 0
           IF nwhll GT 1 THEN BEGIN
              alm(whll[1:*],0,*) = alm(whll[1:*],0,*) + (uyx ## REFORM(almobs(whll[1:*],0,*),nwhll-1,nnlobsfs))
              alm(whll[1:*],1,*) = alm(whll[1:*],1,*) + (uyx ## REFORM(almobs(whll[1:*],1,*),nwhll-1,nnlobsfs))
           ENDIF
        ENDIF
     ENDIF ELSE BEGIN
        alm(whll,*,*) = 0
     ENDELSE
  ENDFOR
  
  IF lobsmax LT lmax THEN BEGIN
     FOR l = lobsmax+1, lmax DO BEGIN
        whl = WHERE(cl.l EQ l)
        cyy = CL2COV(whl[0], cl, nclfs=nclfs, double=double)
        whll = WHERE(ll EQ l, nwhll)
        vyy = cyy
        wyf = WHERE(DIAG_MATRIX(vyy) GT 0, nwyf)
        
        IF nwyf GT 0 THEN BEGIN
           nwyfs= N_ELEMENTS(wyf)
           dywy =  INTARR(nwyfs,nclfs)
           FOR f=0L,nwyfs-1 DO dywy(f,wyf(f))=1  
           svyy = TRANSPOSE(dywy) ## (vyy ## dywy) 
           IF nwyfs GT 1 THEN BEGIN
              LA_CHOLDC, svyy, double=double, status=la_choldc_status
              IF la_choldc_status NE 0 THEN BEGIN
                 PRINT, "input array is not positive definite"
                 PRINT, "la_choldc_status=",la_choldc_status
                 STOP
              ENDIF
              FOR f=0L,nwyfs-2 DO svyy(f+1:*,f)=0 
           ENDIF ELSE svyy=SQRT(svyy)
           vyy = dywy ## (svyy ## TRANSPOSE(dywy)) 
           alm(whll[0],0,*) = vyy ## REFORM(alm(whll[0],0,*),1,nclfs)
           alm(whll[0],1,*) = 0
           IF KEYWORD_SET(double) THEN vyy = vyy/SQRT(2.d0) ELSE vyy = vyy/SQRT(2.0)
           IF nwhll GT 1 THEN BEGIN
              alm(whll[1:*],0,*) = vyy ## REFORM(alm(whll[1:*],0,*),nwhll-1,nclfs)
              alm(whll[1:*],1,*) = vyy ## REFORM(alm(whll[1:*],1,*),nwhll-1,nclfs)
           ENDIF
        
        ENDIF ELSE BEGIN
           alm(whll,*,*) = 0
        ENDELSE
        
     ENDFOR
  ENDIF
  
  ;;alm_out = alm
  FOR i=0, nclfs-1 DO FOR l=0, lmax DO alm[LMAX2NALM(l-1):LMAX2NALM(l-1)+l,*,i] = alm[LMAX2NALM(l-1):LMAX2NALM(l-1)+l,*,i] / renorm_vector[i,l]
  


END

