; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_STR_REPLACE
;
; PURPOSE:
;	Replace a substring by another in an input string.
;
; CATEGORY:
;	String handling
;
; CALLING SEQUENCE:
;	Result = STR_REPLACE(Str, Outstring, Instring)
;
; INPUTS:
;	Str:    	Input string.
;	Outstring:	Substring to be searched for and replaced.
;	Instring:	Substring to be put in str in replacement of
;	                outstring.
;
; KEYWORDS:
;	None.
;
; OUTPUTS:
;	The result is the input string with outstring replaced by instring.
;
; PROCEDURE:
;	Simple string manipulation.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	Define a string:
;	> str = 'tralala tsoin tsoin'
;
;	Print the string obtained when 'tsoin' is replaced with 'ploum'
;	> PRINT, PSM_STR_REPLACE(str, 'tsoin', 'ploum')
;
;	(IDL prints 'tralala ploum ploum').
;
; REVISION HISTORY
;	June 2000:         Jacques Delabrouille, First version.
;	May 2001:          JD, Bug (infinite loop) when instring contains outstring fixed.
;	December 2008:     JD, Adapted to handle arrays of strings.
;	01/06/2010:        JD, Added help.
;	15/04/2011:        JD, Loop in long integer now, for handling
;	                   properly large arrays of strings.
;-

FUNCTION SINGLE_STR_REPLACE, str, outstring, instring

  search = 1
  begin_string = ''
  endstring = str
  WHILE search EQ 1 DO BEGIN
     whout = STRPOS(endstring,outstring)
     IF whout EQ -1 THEN BEGIN
        RETURN, begin_string+endstring
        GOTO, closing
     ENDIF ELSE BEGIN
        begin_string = begin_string + STRMID(endstring,0,whout) + instring
        endstring = STRMID(endstring,whout+STRLEN(outstring))
     ENDELSE
  ENDWHILE
closing:
END


FUNCTION PSM_STR_REPLACE, str, outstring, instring, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_STR_REPLACE: function which replaces a specified substring by another substring in a given string'
     PSM_INFO_MESSAGE, 'SYNTAX: PSM_STR_REPLACE, str, outstring, instring'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
   PROHELP, 'psm_str_replace'
ENDIF

string_is_an_array = ISARRAY(str)

IF string_is_an_array THEN BEGIN
   newstr = str
   nstr = N_ELEMENTS(str)
   FOR i=0L, nstr-1 DO newstr[i] = SINGLE_STR_REPLACE(str[i],outstring,instring)
ENDIF ELSE newstr = SINGLE_STR_REPLACE(str,outstring,instring)

RETURN, newstr
closing:
END
