; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	STRUCT2STRING
;
; PURPOSE:
;	Write a structure in the format of a single string that can be
;	used to define that structure in IDL via the EXECUTE command.
;	Written for simple structures (with tags equal to arrays of
;	numbers of less than 2 dimensions, for example). 
;
; CALLING SEQUENCE:
;	Result = STRUCT2STRING(Structure)
;
; INPUTS:
;	Structure: the structure to be written in the format of an
;	executable string.
;
; KEYWORDS:
;	COMPRESS:	Set this keyword to remove extra spaces added
;	                by the formatting of input numbers when
;	                converted to strings.
;
; OUTPUTS:
;	The result is a string that writes the structure definition in
;	proper IDL syntax.
;
; RESTRICTIONS:
;	Input arrays of numbers in the tags of the structure to be
;	written must have at most two dimensions. Beware that the type
;	of the IDL variables may not be properly taken care of by this
;	routine. Does not work properly (yet) for structures
;	comprising character strings.
;
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	Create a structure in IDL:
;	> struct = {a:LINDGEN(3), b:0.}
;
;	View the string array that defines that structure in IDL syntax:
;	> PRINT, STRUCT2STRING(struct, /compress)
;
;	(IDL prints '{A:[ 0, 1, 2], B: 0.00000}').
;
; REVISION HISTORY
;	28/11/2001:   Jean-Baptiste Melin, first version
;	10/01/2012:   Jacques Delabrouille, documentation update,
;	              COMPRESS keyword added, debugged case where the
;	              first tag is an array.
;-
FUNCTION STRUCT2STRING, structure, compress=compress

  PSM_DEBUG_MESSAGE, 'What to do with cases such as: struct = {a:LINDGEN(3), b:"mystring", c:0.}', routine='STRUCT2STRING', infolimit=2, stoplimit=3

  result = '{'
  tag = TAG_NAMES(structure)
  nt = N_ELEMENTS(tag)

  IF ISARRAY(structure.(0)) THEN result = result + tag[0] + ':' + ARRAY2STRING(structure.(0), compress=compress) $
  ELSE result = result + tag[0] + ':' + STRING(structure.(0))
  
  IF  nt GE 2 THEN BEGIN
     FOR i=1, nt-1 DO BEGIN
        IF ISARRAY(structure.(i)) THEN value = ARRAY2STRING(structure.(i), compress=compress) ELSE BEGIN
           IF NOT KEYWORD_SET(compress) THEN value = STRING(structure.(i)) ELSE value = STRCOMPRESS(STRING(structure.(i)))
        ENDELSE
        result = result + ', ' + tag[i] + ':' + value
     ENDFOR
  ENDIF
  
  result = result + '}'
  
  RETURN, result
  
END
