; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PRTSTRUCT
;
; PURPOSE:
;	Procedure for printing a structure, with help for all tags and
;	subtags recursively printed out 
;
; CATEGORY:
;	Structure handling
;
; CALLING SEQUENCE:
;	PRTSTRUCT, Structure
;
; INPUTS:
;	Structure: 	A structure to be printed
;
; KEYWORD PARAMETERS:
;	NAME:   	Name of the structure to be printed
;	NOBLANKS: 	The output, by default, is formatted so that tags
; 	                and subtags appear in formatted table style with blanks
; 	                added. Set this keyword for unformatted display of the info
;	SIZEMAX:   	maximum size of arrays printed out explicitely (default=6)
;
; MODIFICATION HISTORY:
;	27/08/2008: 	Jacques Delabrouille, first version.
;	16/03/2012: 	JD, expanded header and help, added file keyword.
;
;-
PRO PRTSTRUCT, struct, name=name, noblanks=noblanks, sizemax=sizemax, file=file, to_log_file=to_log_file, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PRTSTRUCT: procedure to print out a structure for visualisation'
     PSM_INFO_MESSAGE, 'SYNTAX: PRTSTRUCT, struct, name=, noblanks=, sizemax=, file='
     PSM_INFO_MESSAGE, "Input keyword 'name' is the name to appear first in each line [default = '<Expression>']"
     PSM_INFO_MESSAGE, "Set keyword /noblanksfor printing in compact form (no tabulations)"
     PSM_INFO_MESSAGE, "Set keyword 'sizemax' to define the maximum size of printed arrays (default=6)"
     PSM_INFO_MESSAGE, "Set keyword sizemax'file' to the name of a file in which to print the info"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT DEFINED(sizemax) THEN sizemax=6

  s = SIZE(struct,/type)
  IF s NE 8 THEN BEGIN
     MESSAGE, 'Input is not a structure', /info
     RETURN
  ENDIF

  IF NOT KEYWORD_SET(name) THEN rootname='<Expression>' ELSE rootname=name

  names = RECURSIVE_TAG_NAMES(struct)
  nnames = N_ELEMENTS(names)

  nlevels = INTARR(nnames)
  FOR i=0, nnames-1 DO BEGIN
     subtags = STRSPLIT(names(i),'.')
     nsubtags = N_ELEMENTS(subtags)
     nlevels[i] = nsubtags
  ENDFOR

  nl = MAX(nlevels)
  arraytags = STRARR(nnames,nl)
  newtag = BYTARR(nnames,nl)
  newtag[0,0:nl-1] = 1B
  mask = BYTARR(nnames,nl)
  mask[0,0:nlevels[0]-1] = 1B

  FOR i=0, nnames-1 DO BEGIN
     subtags = STRSPLIT(names(i),'.',/extract)
     nsubtags = N_ELEMENTS(subtags)
     arraytags[i,0:nsubtags-1] = subtags
     IF i NE 0 THEN BEGIN
        whchanged = WHERE(arraytags[i,*] NE arraytags[i-1,*])
        newtag[i,whchanged] = 1B
        mask[i,0:nlevels[i]-1] = 1B
     ENDIF
  ENDFOR

  IF nl GE 2 THEN FOR i=1, nl-1 DO newtag[*,i] = newtag[*,i] OR newtag[*,i-1]
  newtag = newtag*mask

  IF KEYWORD_SET(noblanks) EQ 0 THEN BEGIN
     wh = WHERE(newtag NE 0, nwh)
     IF nwh NE 0 THEN arraytags[wh] = ' | .' + arraytags[wh]
     wh = WHERE(newtag EQ 0, nwh)
     IF nwh NE 0 THEN arraytags[wh] = ' | '
  ENDIF ELSE BEGIN
     wh = WHERE(mask NE 0, nwh)
     IF nwh NE 0 THEN arraytags[wh] = '.' + arraytags[wh]
     wh = WHERE(mask EQ 0, nwh)
     IF nwh NE 0 THEN arraytags[wh] = ''
  ENDELSE

  printnames = STRARR(nnames)
  IF KEYWORD_SET(noblanks) EQ 0 THEN BEGIN
     tolength = STRARR(nl)
     FOR i=0, nl-1 DO tolength[i] = MAX(STRLEN(arraytags[*,i]))+1
     
     FOR i=0, nnames-1 DO BEGIN
        IF nl GE 2 THEN BEGIN
           FOR j=0, nl-2 DO BEGIN
              arraytags[i,j] = PSM_STRING_TO_LENGTH(arraytags[i,j],tolength[j])
           ENDFOR
        ENDIF
     ENDFOR
  ENDIF

  nblanks = STRLEN(rootname)
  blankroot = ''
  IF nblanks NE 0 THEN FOR i=0, nblanks-1 DO blankroot = blankroot+' '
  printnames[*] = rootname
  IF KEYWORD_SET(noblanks) EQ 0 THEN BEGIN
     wh = WHERE(newtag[*,0] EQ 0, nwh)
     IF nwh NE 0 THEN printnames[wh] = blankroot
  ENDIF

  FOR i=0, nnames-1 DO FOR j=0, nl-1 DO printnames[i] = printnames[i] + arraytags[i,j]
 
  maxlen = MAX(STRLEN(printnames))

  IF DEFINED(file) THEN BEGIN
     GET_LUN, unit
     OPENW, unit, file
  ENDIF

  FOR i=0, nnames-1 DO BEGIN
     printnames[i] = printnames[i] + ' '
     printnames[i] = PSM_STRING_TO_LENGTH(printnames[i],maxlen+4,character='-')
     printnames[i] = printnames[i]+'> '
     ok = EXECUTE('s = SIZE(struct.'+names[i]+',/type)')
     ;;IF s EQ 10 THEN BEGIN
     CASE s OF
        10: BEGIN
           ok = EXECUTE('valid = PTR_VALID(struct.'+names[i]+')')
           IF NOT ISARRAY(valid) THEN BEGIN
              IF valid THEN BEGIN
                 ok = EXECUTE('HELP, *(struct.'+names[i]+'), output=helpoutput') 
                 output = printnames[i] + '   ' + TEMPORARY(STRJOIN(helpoutput))
              ENDIF ELSE BEGIN
                 helpoutput = ' *** not a valid pointer ***'
                 output = printnames[i] + '   ' + TEMPORARY(helpoutput)
              ENDELSE
           ENDIF ELSE BEGIN
              ok = EXECUTE('HELP, (struct.'+names[i]+'), output=helpoutput') 
              output = printnames[i] + '   ' + ' ' + TEMPORARY(STRTRIM(PSM_STR_REPLACE(STRJOIN(helpoutput),"<Expression>",''),2))           
           ENDELSE
        END       
        ELSE: BEGIN
           doprint = 0B
           ok = EXECUTE('IF ISARRAY(struct.'+names[i]+') THEN IF N_ELEMENTS(struct.'+names[i]+') LE sizemax THEN doprint=1B')
           IF doprint THEN BEGIN
              ok = EXECUTE('a = struct.'+names[i])
              IF ISANUMBER(a) THEN BEGIN
                 str='['+NUMBER2STRING(a[0])
                 IF N_ELEMENTS(a) GE 2 THEN FOR kkk=1, N_ELEMENTS(a)-1 DO str = str+', '+NUMBER2STRING(a[kkk]) 
                 str=str+']'
                 ok = EXECUTE('HELP, struct.'+names[i]+', output=helpoutput')
                 output = PSM_STR_REPLACE(TEMPORARY(helpoutput),"<Expression>",printnames[i])
                 output = PSM_STR_REPLACE(TEMPORARY(output), "Array["+NUMBER2STRING(N_ELEMENTS(a))+"]", str)
                 GOTO, continuer
              ENDIF
              IF ISSTRING(a) THEN BEGIN
                 str="['"+a[0]+"'"
                 IF N_ELEMENTS(a) GE 2 THEN FOR kkk=1, N_ELEMENTS(a)-1 DO str = str+", '"+a[kkk]+"'" 
                 str=str+']'
                 ok = EXECUTE('HELP, struct.'+names[i]+', output=helpoutput')
                 output = PSM_STR_REPLACE(TEMPORARY(helpoutput),"<Expression>",printnames[i])
                 output = PSM_STR_REPLACE(TEMPORARY(output), "Array["+NUMBER2STRING(N_ELEMENTS(a))+"]", str)
                 GOTO, continuer
              ENDIF
              ok = EXECUTE('HELP, struct.'+names[i]+', output=helpoutput')
              output = PSM_STR_REPLACE(TEMPORARY(helpoutput),"<Expression>",printnames[i])
           ENDIF ELSE BEGIN
              ok = EXECUTE('HELP, struct.'+names[i]+', output=helpoutput')
              output = PSM_STR_REPLACE(TEMPORARY(helpoutput),"<Expression>",printnames[i])
           ENDELSE
           continuer:
        END
     ENDCASE
     DEFSYSV, '!psm_logfile', exists=exists
     IF ((exists AND NOT DEFINED(file)) AND KEYWORD_SET(to_log_file)) THEN BEGIN
        GET_LUN, unit2
        OPENU, unit2, !psm_logfile, /append
        PRINTF, unit2, output
        CLOSE, unit2
        FREE_LUN, unit2
     ENDIF

     IF DEFINED(file) THEN BEGIN
        PRINTF, unit, TEMPORARY(output)
     ENDIF ELSE BEGIN
        PRINT, TEMPORARY(output)
     ENDELSE

  ENDFOR
  IF DEFINED(file) THEN BEGIN
     CLOSE, unit
     FREE_LUN, unit
  ENDIF


END
  


