; -----------------------------------------------------------------------------
;
; Copyright (C) 2007, 2009, 2010, 2012
;   Marc Betoule
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CHECKUNIT
;
; PURPOSE:
;	Analyse a string to check if it corresponds to a recognized
;	PSM unit. Leading and trailing blanks are removed from the
;	input unit in the process, if any.
;
; CATEGORY:
;	Unit handling and conversion
;
; CALLING SEQUENCE:
;	Result = CHECKUNIT(Unit, Prefix, Core, Exponent, Type)
;
; INPUTS:
;	Unit:	A single string for the input unit among the list :
;               ['Jy/sr','K_CMB','K_RJ','K/KCMB','y_sz','W/m2/sr/Hz'] or
;               ['rad','deg','arcmin','arcsec'] or ['meter','parsec']
;               or ['gram','Msun'] or ['unknown']
;               optionally prefixed by ['n','u','m','k','M','G'] 
;               for nano, micro, milli, kilo,Mega, Giga, 
;               and optionally raised to an integer power, 
;               in which case the unit is in parentheses
;               and postfixed by ['**2','**3',...]
;
; OUTPUTS:
;	This function returns result=1B if the input unit is a valid
;	PSM unit, and result=0 if not.
;
; OPTIONAL OUTPUTS:
;	Prefix, Core, Exponent, Type
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple string manipulation and checking against list of
;	accepted values
;
; EXAMPLE:
;
; 	Check whether 'mK_thermodynamic' is a valid PSM unit:
; 	> PRINT, CHECKUNIT('mK_thermodynamic')
;
; 	IDL prints 0 because 'mK_thermodynamic' is not a valid PSM unit
;
; 	Check whether '(uK_CMB)**2' is a valid PSM unit, and get its
; 	prefix, core, exponent and type:
; 	> PRINT, CHECKUNIT('(uK_CMB)**2', Prefix, Core, Exponent, Type)
;
; 	IDL prints 0
;
; 	Check prefix, core, exponent and type:
; 	> HELP, prefix, core, exponent, type
;
; 	IDL PRINTS:
; 	  PREFIX          STRING    = 'u'
; 	  CORE            STRING    = 'K_CMB'
; 	  EXPONENT        STRING    = '2'
; 	  TYPE            STRING    = 'flux'

;
; MODIFICATION HISTORY:
; 	28/02/2007:	Marc Betoule, First version.
;	20/01/2009:	Jacques Delabrouille, Complete revision, more
;	                strict checking, added exponent and optional outputs.
;	28/09/2009:	JD, Added 'unknown' unit.
; 	08/01/2012:	JD, doc and PSM license header.
;-
FUNCTION CHECKUNIT, unit, prefix, core, exponent, type, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() EQ 0)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CHECKUNIT: function to check whether a string is a valid PSM unit, and decompose it'
     PSM_INFO_MESSAGE, 'SYNTAX: result = CHECKUNIT(unit, prefix, core, exponent, type)'
     PSM_INFO_MESSAGE, 'unit is the input string'
     PSM_INFO_MESSAGE, 'prefix, core, exponent, and type are optional outputs for unit decomposition'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF
  
  supported_units=['Jy/sr','K_CMB','K_RJ','K/KCMB','y_sz','W/m2/sr/Hz','unknown','rad','deg','arcmin','arcsec','meter','parsec','gram','Msun']
  type_units=['flux','flux','flux','flux','flux','flux','unknown','angle','angle','angle','angle','length','length','mass','mass']
  
  ;;nano, micro, milli, kilo, mega, giga
  supported_prefix=['n','u','m','k','M','G']
  ;;parse
  unit = STRTRIM(unit,2)
  
  ;; Check for exponent
  ;;--------------------
  exponent_ok = 0B
  unitexp = STR_SEP(unit,'**')
  IF N_ELEMENTS(unitexp) EQ 1 THEN BEGIN
     exponent_ok = 1B
     exponent = '1'
  ENDIF
  IF N_ELEMENTS(unitexp) EQ 2 THEN BEGIN
     exponent = unitexp[1]
     exponent = STRTRIM(exponent,2)
     exponent_ok = (FLOAT(exponent) EQ FIX(exponent))
     IF STRLEN(exponent) EQ 0 THEN exponent_ok = 0B
  ENDIF

  ;; Core of the unit
  ;;------------------
  core_ok = 0B
  FOR i = 0, n_elements(supported_units)-1 DO BEGIN
     pos = STRPOS(unit, supported_units[i])
     IF pos NE -1 THEN BEGIN 
        core = supported_units[i]
        core_ok = 1B
        IF pos EQ 0 THEN prefix = '' ELSE prefix = STRMID(unit,pos-1,1)
        IF prefix EQ '(' THEN prefix = '' 
     ENDIF
  ENDFOR

  IF NOT core_ok THEN BEGIN
     core=''
     prefix=''
     exponent=''
  ENDIF ELSE BEGIN
     which = WHERE(supported_units EQ core)
     type = (type_units(which))[0]
  ENDELSE
  
  ;; Check if prefix is valid
  ;;--------------------------
  prefix_ok = 0B
  IF core_ok THEN BEGIN
     IF STRLEN(prefix) EQ 0 THEN prefix_ok = 1B
     IF STRLEN(prefix) EQ 1 THEN BEGIN
        wh = WHERE(supported_prefix EQ prefix, nwh) 
        IF nwh EQ 1 THEN prefix_ok = 1B
     ENDIF
  ENDIF
  
  IF exponent NE 1 THEN new_unit = '('+prefix+core+')**'+exponent ELSE new_unit = prefix + core
  check_ok = (new_unit EQ unit)
  
  RETURN, exponent_ok * core_ok * prefix_ok * check_ok
END
