; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Marc Betoule
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CONVERSION_FACTOR
;
; PURPOSE:
;	Convert units used in the PSM.
;
; CATEGORY:
;	Unit handling and conversion
;
; CALLING SEQUENCE:
;	Result = CONVERSION_FACTOR(from_unit, to_unit, band, /double)
;
; INPUTS:
;	From_unit:	A valid PSM unit
;	To_unit:	A valid PSM unit
;	
;	Valid PSM units are character strings of format core, or prefix+core,
;	or (prefix+core)**exponent
;
;	core is the core unit name, among the following
;	Allowed angle units: ['rad', 'deg', 'arcmin', 'arcsec']
;	Allowed length units: ['meter', 'parsec']
;	Allowed flux units: ['Jy/sr','K_CMB','K_RJ','K/KCMB','y_sz','W/m2/sr/Hz']
;
;	Allowed prefixes : ['n','u','m','k','M','G'] for nano, micro, milli, kilo, Mega, Giga.
;	Exponents are taken into account with the conventional format, e.g. '(deg)**2'
;
; OPTIONAL INPUTS:
;	Band:	A spectral band created with GET_BAND_STRUCT, or
;		extracted from an instrument model created with LOAD_INSTRUMENT.
;		Can also be simpy a frequency in Hz, e.g. 3e10.
;		It is useful only to convert between frequency
;		dependent units, e.g. between MJy/sr and mK_CMB.	
;	
; KEYWORD PARAMETERS:
;	DOUBLE:	Set this keyword for double precision computation.
;
; OUTPUTS:
;	Result is the number by which one has to multiply a quantity
;	expressed in the first unit (From_unit) to get it expressed in
;	the second unit (To_unit).
;
; RESTRICTIONS:
;	The convention for MJy/sr is 1 MJy/sr = 1E-20 W/m2/sr/Hz, band-independent.
;	WARNING: IRAS uses a different convention.
;
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	To get the conversion factor to convert from mK_CMB to K_RJ at 100 GHz, type 
;	IDL> PRINT, CONVERSION_FACTOR('K_CMB','K_RJ',1e11)
;	The result is 0.000777164.
;
; MODIFICATION HISTORY:
; 	30/01/2007:	Marc Betoule, First version.
; 	01/02/2007:	Jacques Delabrouille, small bug fix in call to PLANCK_BNU
; 	22/02/2007:	MB, add error handling.
; 	12/08/2008:	JD, avoid computing when from_unit = to_unit, now allows arrays of freq.
; 	02/12/2008:	JD, Reorganisation with a subfunction for better handling band integration.
; 	22/01/2009:	JD, Convert prefix only when only the prefix is changed (band-independent).
; 	06/11/2009:	JD, Add angle conversion.
; 	26/11/2009:	JD, Conversion in tabulated bands, added tests
; 			for efficiency when conversion is trivial.
; 	08/01/2010:	JD, Corrected bug in conversion between K_CMB and K_KCMB.
; 	13/01/2010:	JD, Implement unit conversion for instrumental bands.
; 	08/09/2010:	JD, Implement mass and length conversion.
; 	30/03/2011:	JD, corrected a (vicious) bug in the special case where the input 
; 	 	 	band is a tophat and one unit is MJy...
; 	 	 	Put a stop to prevent a similar bug to happen without the user being aware.
; 	26/05/2011:	Guillaume Castex, handle case where a tabulated band contains the 0 frequency.
;-
; -----------------------------------------------------------------------------
FUNCTION MASS_CONVERSION, from_unit, to_unit

  inok = CHECKUNIT(from_unit, prefix_in, core_in, exponent_in, type_in)
  outok = CHECKUNIT(to_unit, prefix_out, core_out, exponent_out, type_out)

  IF from_unit EQ to_unit THEN RETURN, 1. ELSE BEGIN

     ;; nano, micro, milli, kilo, mega, giga
     ;;--------------------------------------
     supported_prefix=['n','u','m','k','M','G']
     pref_val=[1d-9, 1d-6, 1d-3, 1d3, 1d6, 1d9]
     
     ;; Argument parsing
     ;;------------------
     in = core_in
     pref_in = prefix_in
     out = core_out
     pref_out = prefix_out

     IF STRLEN(pref_in) EQ 0 THEN BEGIN
        pref_val_in = 1d 
     ENDIF ELSE BEGIN 
        ipi = WHERE(pref_in eq supported_prefix)
        if ipi EQ -1 then MESSAGE, pref_in + ' is not a recognised prefix, the list of recognised prefixes is : (' + string(supported_prefix,/print) +')'
        pref_val_in = pref_val[ipi]
        pref_val_in = pref_val_in[0]
     ENDELSE
     IF STRLEN(pref_out) EQ 0 THEN BEGIN
        pref_val_out = 1d 
     ENDIF ELSE BEGIN
        ipo = WHERE(pref_out EQ supported_prefix)
        if ipo eq -1 THEN MESSAGE, pref_in + ' is not a recognised prefix, the list of recognised prefixes is : (' + string(supported_prefix,/print) +')'
        pref_val_out = pref_val[ipo]
        pref_val_out = pref_val_out[0]
     ENDELSE

     ;; Factor computation for core unit
     ;;----------------------------------
     CASE core_in OF
        'gram': fac_in=1d0
        'Msun': BEGIN
           DEFSYSV, '!phyconst', exists=exists
           IF NOT exists THEN DEFINE_PHYSICAL_CONSTANTS
           fac_in=!phyconst.M_s*1d3
        END
        ELSE: MESSAGE, 'Unrecognised unit: '+core_in
     ENDCASE
     CASE core_out OF 
        'gram': fac_out=1d0
        'Msun': BEGIN
           DEFSYSV, '!phiconst', exists=exists
           IF NOT exists THEN DEFINE_PHYSICAL_CONSTANTS
           fac_out=1d-3/!phyconst.M_s
        END
     ENDCASE
     
     factor = [(fac_in * fac_out) * (pref_val_in/pref_val_out)]
     IF exponent_in NE 1 THEN factor = factor ^ exponent_in
     
     RETURN, factor[0]
  ENDELSE

END

;---------------------------------------------------------------------------------------------
FUNCTION LENGTH_CONVERSION, from_unit, to_unit

  inok = CHECKUNIT(from_unit, prefix_in, core_in, exponent_in, type_in)
  outok = CHECKUNIT(to_unit, prefix_out, core_out, exponent_out, type_out)

  IF from_unit EQ to_unit THEN RETURN, 1. ELSE BEGIN

     ;; nano, micro, milli, kilo, mega, giga
     ;;--------------------------------------
     supported_prefix=['n','u','m','k','M','G']
     pref_val=[1d-9, 1d-6, 1d-3, 1d3, 1d6, 1d9]
     
     ;; Argument parsing
     ;;------------------
     in = core_in
     pref_in = prefix_in
     out = core_out
     pref_out = prefix_out

     IF STRLEN(pref_in) EQ 0 THEN BEGIN
        pref_val_in = 1d 
     ENDIF ELSE BEGIN 
        ipi = WHERE(pref_in eq supported_prefix)
        if ipi EQ -1 then MESSAGE, pref_in + ' is not a recognised prefix, the list of recognised prefixes is : (' + string(supported_prefix,/print) +')'
        pref_val_in = pref_val[ipi]
        pref_val_in = pref_val_in[0]
     ENDELSE
     IF STRLEN(pref_out) EQ 0 THEN BEGIN
        pref_val_out = 1d 
     ENDIF ELSE BEGIN
        ipo = WHERE(pref_out EQ supported_prefix)
        if ipo eq -1 THEN MESSAGE, pref_in + ' is not a recognised prefix, the list of recognised prefixes is : (' + string(supported_prefix,/print) +')'
        pref_val_out = pref_val[ipo]
        pref_val_out = pref_val_out[0]
     ENDELSE

     ;; Factor computation for core unit
     ;;----------------------------------
     CASE core_in OF
        'meter': fac_in=1d0
        'parsec': fac_in=3.0856775807d+16
        ELSE: MESSAGE, 'Unrecognised unit: '+core_in
     ENDCASE
     CASE core_out OF 
        'meter': fac_out=1d0
        'parsec': fac_out=1d0/3.0856775807d+16
     ENDCASE
     
     factor = [(fac_in * fac_out) * (pref_val_in/pref_val_out)]
     IF exponent_in NE 1 THEN factor = factor ^ exponent_in
     
     RETURN, factor[0]
  ENDELSE

END

;---------------------------------------------------------------------------------------------

FUNCTION ANGLE_CONVERSION, from_unit, to_unit

  inok = CHECKUNIT(from_unit, prefix_in, core_in, exponent_in, type_in)
  outok = CHECKUNIT(to_unit, prefix_out, core_out, exponent_out, type_out)

  IF from_unit EQ to_unit THEN RETURN, 1. ELSE BEGIN

     ;; nano, micro, milli, kilo, mega, giga
     ;;--------------------------------------
     supported_prefix=['n','u','m','k','M','G']
     pref_val=[1d-9, 1d-6, 1d-3, 1d3, 1d6, 1d9]
     
     ;; Argument parsing
     ;;------------------
     in = core_in
     pref_in = prefix_in
     out = core_out
     pref_out = prefix_out

     IF STRLEN(pref_in) EQ 0 THEN BEGIN
        pref_val_in = 1d 
     ENDIF ELSE BEGIN 
        ipi = WHERE(pref_in eq supported_prefix)
        if ipi EQ -1 then MESSAGE, pref_in + ' is not a recognised prefix, the list of recognised prefixes is : (' + string(supported_prefix,/print) +')'
        pref_val_in = pref_val[ipi]
        pref_val_in = pref_val_in[0]
     ENDELSE
     IF STRLEN(pref_out) EQ 0 THEN BEGIN
        pref_val_out = 1d 
     ENDIF ELSE BEGIN
        ipo = WHERE(pref_out EQ supported_prefix)
        if ipo eq -1 THEN MESSAGE, pref_in + ' is not a recognised prefix, the list of recognised prefixes is : (' + string(supported_prefix,/print) +')'
        pref_val_out = pref_val[ipo]
        pref_val_out = pref_val_out[0]
     ENDELSE

     ;; Factor computation for core unit
     ;;----------------------------------
     CASE core_in OF
        'rad': fac_in=180D0/!dpi
        'deg': fac_in=1D0
        'arcmin': fac_in=1D0/60D0
        'arcsec': fac_in=1D0/3600D0
        ELSE: MESSAGE, 'Unrecognised unit: '+core_in
     ENDCASE
     CASE core_out OF 
        'rad': fac_out=!dpi/180D0
        'deg': fac_out=1D0
        'arcmin': fac_out=60D0
        'arcsec': fac_out=3600D0
     ENDCASE
     
     factor = [(fac_in * fac_out) * (pref_val_in/pref_val_out)]
     IF exponent_in NE 1 THEN factor = factor ^ exponent_in
     
     RETURN, factor[0]
  ENDELSE

END

;---------------------------------------------------------------------------------------------

FUNCTION CONVERSION_FACTOR_FROM_FREQ, from_unit, to_unit, freq_Hz
  
  inok = CHECKUNIT(from_unit, prefix_in, core_in, exponent_in, type_in)
  outok = CHECKUNIT(to_unit, prefix_out, core_out, exponent_out, type_out)

  IF DEFINED(freq_Hz) THEN BEGIN
     COMMON psm, param 
     IF DEFINED(param) THEN BEGIN
        IF param.run.carefulness GE 1 THEN BEGIN
           IF MIN(freq_Hz) LT 1e8 THEN MESSAGE, 'Input frequency to conversion_factor seems to be in GHz...'
        ENDIF
     ENDIF
     freq = freq_Hz/1e9
  ENDIF

  IF from_unit EQ to_unit THEN BEGIN
     IF ISARRAY(freq_Hz) THEN RETURN, REPLICATE(1., N_ELEMENTS(freq_Hz)) ELSE RETURN, 1. 
  ENDIF ELSE BEGIN

     supported_units=['Jy/sr','K_CMB','K_RJ','K/KCMB','y_sz','W/m2/sr/Hz','taubetat2']

     ;; nano, micro, milli, kilo, mega, giga
     ;;--------------------------------------
     supported_prefix=['n','u','m','k','M','G']
     pref_val=[1d-9, 1d-6, 1d-3, 1d3, 1d6, 1d9]
     
     ;; Argument parsing
     ;;------------------
     in = core_in
     pref_in = prefix_in
     out = core_out
     pref_out = prefix_out
  
     IF STRLEN(pref_in) EQ 0 THEN BEGIN
        pref_val_in = 1d 
     ENDIF ELSE BEGIN 
        ipi = WHERE(pref_in eq supported_prefix)
        if ipi EQ -1 then MESSAGE, pref_in + ' is not a recognised prefix, the list of recognised prefixes is : (' + string(supported_prefix,/print) +')'
        pref_val_in = pref_val[ipi]
        pref_val_in = pref_val_in[0]
     ENDELSE
     IF STRLEN(pref_out) EQ 0 THEN BEGIN
        pref_val_out = 1d 
     ENDIF ELSE BEGIN
        ipo = WHERE(pref_out EQ supported_prefix)
        if ipo eq -1 THEN MESSAGE, pref_out + ' is not a recognised prefix, the list of recognised prefixes is : (' + string(supported_prefix,/print) +')'
        pref_val_out = pref_val[ipo]
        pref_val_out = pref_val_out[0]
     ENDELSE
     
  
     ;; Factor computation for core unit
     ;;----------------------------------
     DEFSYSV, '!phyconst', exist=exist
     IF not exist THEN BEGIN
        IF PSM_CAREFULNESS() GE 1 THEN PSM_INFO_MESSAGE, 'Warning, physical constants not defined -- using default values in CONVERSION_FACTOR'
        h = 6.6260693d-34
        k = 1.3806505d-23
        c = 299792458d
     ENDIF ELSE BEGIN
        h = !phyconst.h
        k = !phyconst.k
        c = !phyconst.c
     ENDELSE
     T = CMB_TEMPERATURE()

     IF core_in EQ core_out THEN BEGIN
        IF ISARRAY(freq_Hz) THEN fac_in = REPLICATE(1,N_ELEMENTS(freq_Hz)) ELSE fac_in = 1
        fac_out=1
     ENDIF ELSE BEGIN

        skipconversion = 0B

        IF (core_in EQ 'K_CMB') AND (core_out EQ 'K/KCMB') THEN BEGIN
           skipconversion = 1B
           fac_in = 1
           fac_out = 1./T
        ENDIF
        
        IF (core_in EQ 'K/KCMB') AND (core_out EQ 'K_CMB') THEN BEGIN
           skipconversion = 1B
           fac_in = 1
           fac_out = T
        ENDIF
        
        IF NOT skipconversion THEN BEGIN
           nu = freq * 1d9
           x = h*nu/k/T
           ex = exp(x)
           CASE in OF
              'Jy/sr' : fac_in = 1d-26
              'K_CMB' : fac_in = planck_dBnu_dT(nu,temp=T)
              'K_RJ' : fac_in = 2d*nu^2*k/c^2
              'K/KCMB' : fac_in = T * planck_dBnu_dT(nu,temp=T)
              'y_sz' : fac_in = x * ex / (ex-1d) * (x*(ex+1d)/(ex-1d) - 4d) * planck_bnu(nu,temp=T)
              'W/m2/sr/Hz': fac_in = REPLICATE(1d, N_ELEMENTS(nu))
              'taubetat2': fac_in = 0.1d * ex * (ex + 1d) / (2d * (ex - 1d)^2) * x^2 * planck_bnu(nu,temp=T)
              ELSE : MESSAGE, from_unit + ' is not a recognised unit, the list of recognised units is :'+string(10b)+$
                              ' (' + string(supported_units,/print) + ')' +string(10b)+$
                              'with optionally a prefix in : (' + string(supported_prefix,/print) +')'
           ENDCASE
           CASE out OF
              'Jy/sr' : fac_out = 1d26
              'K_CMB' : fac_out = 1d / planck_dBnu_dT(nu,temp=T)
              'K_RJ' : fac_out = 1d/(2d*nu^2*k/c^2)
              'K/KCMB' : fac_out = 1d/(planck_dBnu_dT(nu,temp=T) * T)
              'y_sz' : fac_out = 1d / (x * ex / (ex-1d) * (x*(ex+1d)/(ex-1d) - 4d) * planck_bnu(nu,temp=T))
              'W/m2/sr/Hz': fac_out = REPLICATE(1d, N_ELEMENTS(nu))
              'taubetat2': fac_out = 1d/(0.1d * ex * (ex + 1d) / (2d * (ex - 1d)^2) * x^2 * planck_bnu(nu,temp=T))
              ELSE : MESSAGE, to_unit + ' is not a recognised unit, the list of recognised units is :'+string(10b)+$
                              ' (' + string(supported_units,/print) + ')' +string(10b)+$
                              'with optionally a prefix in : (' + string(supported_prefix,/print) +')'
           ENDCASE
        ENDIF
     ENDELSE

     factor = [(fac_in * fac_out) * (pref_val_in/pref_val_out)]
     IF N_PARAMS() GE 3 THEN BEGIN
        IF N_ELEMENTS(factor) NE N_ELEMENTS(freq_Hz) THEN STOP ;; potential problem
     ENDIF
     IF exponent_in NE 1 THEN factor = factor ^ exponent_in
     IF N_PARAMS() EQ 2 THEN RETURN, factor[0] ELSE $
     IF (N_ELEMENTS(factor) EQ 1) THEN RETURN, factor[0] ELSE RETURN, factor
     
  ENDELSE

END

;---------------------------------------------------------------------------------------------
; MAIN FUNCTION
;---------------------------------------------------------------------------------------------

FUNCTION CONVERSION_FACTOR, from_unit, to_unit, band, double=double, help=help

  IF (KEYWORD_SET(help) NE 0 OR (N_PARAMS() LT 2)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CONVERSION_FACTOR: function which gives the conversion between an input unit and an output unit'
     PSM_INFO_MESSAGE, 'SYNTAX: result = CONVERSION_FACTOR(from_unit, to_unit, band, double=)'
     PSM_INFO_MESSAGE, "'from_unit' is the input unit"
     PSM_INFO_MESSAGE, "'to_unit' is the output unit"
     PSM_INFO_MESSAGE, "'band' optionnally specifies the frequency band in which the conversion is to be made."
     PSM_INFO_MESSAGE, "allowed angle units: ['rad', 'deg', 'arcmin', 'arcsec']"
     PSM_INFO_MESSAGE, "allowed length units: ['meter', 'parsec']"
     PSM_INFO_MESSAGE, "allowed flux units: ['Jy/sr','K_CMB','K_RJ','K/KCMB','y_sz','W/m2/sr/Hz']"
     PSM_INFO_MESSAGE, "allowed prefixes : ['n','u','m','k','M','G']"
     PSM_INFO_MESSAGE, "exponents are taken into account with the conventional format, e.g. '(deg)**2'"
     PSM_INFO_MESSAGE, "'band' is used for frequency-dependent conversion, e.g. mK_RJ --> MJy/sr"
     PSM_INFO_MESSAGE, "set /double for double precision output"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ON_ERROR, 2

  inok = CHECKUNIT(from_unit, prefix_in, core_in, exponent_in, type_in)
  outok = CHECKUNIT(to_unit, prefix_out, core_out, exponent_out, type_out)
  IF NOT inok THEN MESSAGE, "Input units not recognised: '"+from_unit+"'"
  IF NOT outok THEN MESSAGE, "Output units not recognised: '"+to_unit+"'"

  IF type_in NE type_out THEN MESSAGE, 'Cannot convert '+type_in+' into '+type_out
  IF exponent_in NE exponent_out THEN MESSAGE, 'Cannot convert '+from_unit+' into '+to_unit



  ;; Process differently different types of units
  ;;----------------------------------------------
  CASE type_in OF 

     ;; Flux conversion
     ;;-----------------
     'flux':BEGIN

        ;; Create output factor (may be an array)
        IF DEFINED(band) THEN nbands = N_ELEMENTS(band) ELSE nbands=1
        IF KEYWORD_SET(double) NE 0 THEN output_factor = DBLARR(nbands) ELSE output_factor = FLTARR(nbands) 
        
        ;; If band is a number or array of numbers, then assumed to be DIRAC everywhere, otherwise read the type of band
        IF DEFINED(band) THEN s = SIZE(band, /type) ELSE s = 0
        IF s LE 5 THEN BEGIN
           IF DEFINED(band) THEN nu_c = band ELSE nu_c = FLOAT('Inf')
           bandtype = REPLICATE('DIRAC',N_ELEMENTS(nu_c)) 
        ENDIF
        IF s EQ 8 THEN BEGIN
           nbands = N_ELEMENTS(band)
           IF nbands NE 1 THEN MESSAGE, 'Case not implemented'
           IF band.shape EQ 'DIRAC' THEN nu_c = band.nu_c
           bandtype = STRARR(nbands)
           FOR i=0, nbands-1 DO BEGIN
              bandtype[i] = STRSPLIT(band[i].shape, '-', /extract)
           ENDFOR
        ENDIF
        
        ;; Compute conversion factor for each band
        ;;-----------------------------------------
        FOR i=0, nbands-1 DO BEGIN
           CASE bandtype[i] OF 
              'DIRAC': BEGIN
                 IF nu_c[i] LE 1e8 THEN MESSAGE, 'Now conversion_factor accepts input in Hz, not GHz. Sorry, but please fix your code !'
                 output_factor[i] = CONVERSION_FACTOR_FROM_FREQ(from_unit, to_unit, nu_c[i])
              END
              'TOPHAT': BEGIN         
                 ;; generate an array of frequencies
                 freq = ((DINDGEN(1000)+0.5)/1000d)*(band[i].nu_max-band[i].nu_min) + band[i].nu_min

                 ;; compute the emission law of the input units
                 numerator = CONVERSION_FACTOR_FROM_FREQ(from_unit, 'MJy/sr', freq)
                 ;; compute the emission law of the output units
                 denominator = CONVERSION_FACTOR_FROM_FREQ(to_unit, 'MJy/sr', freq)

                 ;; STOP ;; for testing purposes only ----------------------------------
                 ;; ;; compute the emission law of the input units
                 ;; numerator = CONVERSION_FACTOR_FROM_FREQ(from_unit, 'K_RJ', freq)
                 ;; ;; compute the emission law of the output units
                 ;; denominator = CONVERSION_FACTOR_FROM_FREQ(to_unit, 'K_RJ', freq)
                 ;; ;;------------------------------------------------------------------

                 ;; simple integral (no normalisation of the band
                 ;; because it cancels out)
                 output_factor[i] = TOTAL(numerator)/TOTAL(denominator)
              END
              'TABLE': BEGIN
                 good_nu=where(band.nu ne 0)

                 ;; compute the emission law of the input units at all band frequencies
                 numerator = band.value[good_nu] * CONVERSION_FACTOR_FROM_FREQ(from_unit, 'MJy/sr', band.nu[good_nu])
                 ;; compute the emission law of the output units
                 denominator = band.value[good_nu] * CONVERSION_FACTOR_FROM_FREQ(to_unit, 'MJy/sr', band.nu[good_nu])

                 ;; STOP ;; for testing purposes only ----------------------------------
                 ;; ;; compute the emission law of the input units at all band frequencies
                 ;; numerator = band.value[good_nu] * CONVERSION_FACTOR_FROM_FREQ(from_unit, 'K_RJ', band.nu[good_nu])
                 ;; ;; compute the emission law of the output units
                 ;; denominator = band.value[good_nu] * CONVERSION_FACTOR_FROM_FREQ(to_unit, 'K_RJ', band.nu[good_nu])
                 ;; ;;------------------------------------------------------------------


                 ;; integrate as in bandintegrated_emlaw (no normalisation of the band because it cancels out)
                 output_factor[i] = INT_TABULATED(band.nu[good_nu], numerator) / INT_TABULATED(band.nu[good_nu], denominator)

              END
              'INSTR': BEGIN
                 ;; load band data if necessary
                 IF NOT PTR_VALID(band.data) THEN FILL_BAND_DATA, band
                 output_factor[i] = CONVERSION_FACTOR(from_unit, to_unit, *band.data, double=double)
              END
              ELSE: BEGIN
                 MESSAGE, 'Band shape not implemented yet in conversion_factor', /info
                 output_factor[i] = 'NaN'
              END
           ENDCASE
        ENDFOR
        
        IF N_ELEMENTS(output_factor) GT 1 THEN BEGIN
           IF KEYWORD_SET(double) NE 0 THEN RETURN, DOUBLE(output_factor) ELSE RETURN, FLOAT(output_factor)
        ENDIF ELSE BEGIN
           IF KEYWORD_SET(double) NE 0 THEN RETURN, DOUBLE(output_factor[0]) ELSE RETURN, FLOAT(output_factor[0])
        ENDELSE
        
     END

     ;; Angle conversion
     ;;------------------
     'angle': BEGIN
        result = ANGLE_CONVERSION(from_unit, to_unit)
        IF KEYWORD_SET(double) NE 0 THEN RETURN, DOUBLE(result) ELSE RETURN, FLOAT(result)
     END

     ;; Length conversion
     ;;------------------
     'length': BEGIN
        result = LENGTH_CONVERSION(from_unit, to_unit)
        IF KEYWORD_SET(double) NE 0 THEN RETURN, DOUBLE(result) ELSE RETURN, FLOAT(result)
     END

     ;; Length conversion
     ;;------------------
     'mass': BEGIN
        result = MASS_CONVERSION(from_unit, to_unit)
        IF KEYWORD_SET(double) NE 0 THEN RETURN, DOUBLE(result) ELSE RETURN, FLOAT(result)
     END

  ENDCASE

END
