; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Jacques Delabrouille
;   Mathieu Remazeilles
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ALM2WAVELETS
;
; PURPOSE:
;	Compute wavelets coefficients from spherical harmonics coefficients
;
; CATEGORY:
;	Wavelets (needlets) processing
;
; CALLING SEQUENCE:
;	ALM2WAVELETS, Alm, Wavelets
;
; INPUTS:
;	Alm:    	Fits file name for input harmonic coefficients.
;	Wavelets:	Fits file name for output wavelets.
;
; KEYWORD PARAMETERS:
;	BANDS:  	The wavelets bands to be used.
;	DOUBLE: 	Set this keyword for double precision.
;	OVERWRITE: 	Normally, this code does not overwrite any
;                	existing wavelets file if it already.
;                	exists. Set the OVERWRITE keyword to erase and
;                	replace an existing file.
;	SIGNAL: 	Signal type (can be 'T' for temperature only,
;                  	or 'TP' for temperature and polarisation).  
;                  	Only 'T' case [the default] is implemented for now.
;	WAVELETNSIDE: 	Set this keyword to a usual Healpix NSIDE
;	                (e.g. 512) to force storing the needlet
;	                coefficients at that NSIDE for all scales. 
;	NODEGRADE: 	Set this keyword to store needlet coefficients
;	                at the same NSIDE at all scales.
;	NSIDE_MAX_WIN: 	Set this keyword to define an upper limit to
;                	the HEALPix NSIDE used for storing needlet coefficients.
;	NSIDE_MIN_WIN: 	Set this keyword to define a lower limit to
;                	the HEALPix NSIDE used for storing needlet coefficients.
;
; OUTPUTS:
;	A file is written, which contains the output needlet
;	coefficients and the bands used to compute them.
;
; SIDE EFFECTS:
;	A file is written to disk.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	09/09/2010:	Jacques Delabrouille, First version (adapted
; 	                from my previous implementation of the needlet
; 	                transform).
; 	07/07/2011:	JD, Implemented the WAVELETNSIDE and NODEGRADE
;                	keywords.
; 	07/07/2011:	Mathieu Remazeilles and JD, lower memory used.
; 	25/07/2011:	MR, limit NSIDE to 2048 for lower memory consumption.
; 	06/10/2011:	JD, Implemented the NSIDE_MAX_WIN and
;                   	NSIDE_MIN_WIN keywords.
; 	09/01/2012:	JD, doc and PSM license header.
; 	18/01/2012:	JD, changed call to NALM2LMAX (which is now a function).
;-
PRO ALM2WAVELETS, alm, wavelets, bands=bands, double=double, overwrite=overwrite, signal=signal, waveletnside=waveletnside, $
                  nodegrade=nodegrade, nside_max_w=nside_max_w, nside_min_w=nside_min_w, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 2)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'ALM2WAVELETS: procedure to compute wavelet (needlet) coefficients of an input map'
     PSM_INFO_MESSAGE, 'SYNTAX: ALM2WAVELETS, alm, wavelets, bands=, /double, /overwrite, signal=, waveletnside=, /nodegrade=, nside_max_w=, nside_min_w='
     PSM_INFO_MESSAGE, "'alm' is the input alm array, wavelets the name of an output fits file in which wavelet coefficients are written."
     PSM_INFO_MESSAGE, "'bands' is a 2D input array of dimension [lmax+1, nbands] that define the bands and can be created with functions suche as"
     PSM_INFO_MESSAGE, "COSINE_ALL_BANDS, GAUSSBEAM_ELL_BANDS, QUASIGAUSS_ELL_BANDS."
     PSM_INFO_MESSAGE, "Set /double for double precision, /overwrite to overwrite any existing file."
     PSM_INFO_MESSAGE, "Set 'waveletnside' to the value of nside for needlet coefficients (default is automatic and depends on scale)."
     PSM_INFO_MESSAGE, "Set /nodegrade to use the same maximum nside for all scales."
     PSM_INFO_MESSAGE, "Set 'nside_max_w' and 'nside_min_w' to limit the values of nside at all scales."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT KEYWORD_SET(signal) THEN signal='T'

  ;; Type of the output (file?), cleanup if necessary
  ;;--------------------------------------------------
  waveletstype = SIZE(wavelets, /type)
  IF waveletstype EQ 7 THEN waveletsfile = wavelets ELSE STOP
  exists = FILE_TEST(waveletsfile)
  IF exists THEN BEGIN
     IF NOT KEYWORD_SET(overwrite) THEN BEGIN
        PSM_INFO_MESSAGE, 'File exists, aborting. Set /overwrite to overwrite file: '+waveletsfile
        GOTO, Closing
     ENDIF ELSE BEGIN
        FILE_DELETE, waveletsfile
     ENDELSE
  ENDIF

  IF NOT KEYWORD_SET(bands) THEN BEGIN
     IF NOT KEYWORD_SET(waveletnside) THEN BEGIN
        IF ISFITSFILE(alm) THEN lmax = FITSFILE_LMAX(alm) ELSE lmax=NALM2LMAX(N_ELEMENTS(alm[*,0]))
        nside = LOWER_POWEROF2(lmax) 
        IF KEYWORD_SET(nside_min_w) NE 0 THEN nside = (nside > nside_min_w)
        IF KEYWORD_SET(nside_max_w) NE 0 THEN nside = (nside < nside_max_w)
     ENDIF ELSE BEGIN
        nside = waveletnside 
        IF KEYWORD_SET(nside_min_w) NE 0 THEN nside = (nside > nside_min_w)
        IF KEYWORD_SET(nside_max_w) NE 0 THEN nside = (nside < nside_max_w)
     ENDELSE
     bandcenters = [0,16,32,64,128,256,512,1024,2048,4096,8192]
     wh = WHERE(bandcenters LT (2*nside))
     bandcenters = [bandcenters[wh],2*nside]
     bands = COSINE_ELL_BANDS(bandcenters=bandcenters)
     IF NOT KEYWORD_SET(double) THEN bands=FLOAT(bands)
  ENDIF ELSE BEGIN
     uselmax = MAX(WHERE(bands[*,0] NE 0))
     nbands = N_ELEMENTS(bands[0,*])
     IF nbands GT 1 THEN FOR i=1, nbands-1 DO BEGIN
        uselmax = MAX(WHERE(bands[*,i] NE 0)) > uselmax
     ENDFOR
     IF NOT KEYWORD_SET(waveletnside) THEN BEGIN
        nside = LOWER_POWEROF2(uselmax) 
        IF KEYWORD_SET(nside_min_w) NE 0 THEN nside = (nside > nside_min_w)
        IF KEYWORD_SET(nside_max_w) NE 0 THEN nside = (nside < nside_max_w)
     ENDIF ELSE BEGIN
        nside = waveletnside 
        IF KEYWORD_SET(nside_min_w) NE 0 THEN nside = (nside > nside_min_w)
        IF KEYWORD_SET(nside_max_w) NE 0 THEN nside = (nside < nside_max_w)
     ENDELSE
  ENDELSE

  ;; Get the alm
  ;;--------------
  almtype = SIZE(alm,/type)
  isnum=[1,2,3,4,5,12,13,14,15]
  wh = WHERE(isnum EQ almtype, nwh)
  IF nwh NE 1 THEN STOP ;; Case to be implemented

  lmin = 0
  lmax = N_ELEMENTS(bands[*,0])-1
  almlmax = N_ELEMENTS(alm[*,0])-1
  IF lmax GE almlmax THEN BEGIN
     MESSAGE, 'Bands extend beyond the maximum ell for alm', /info
     STOP ;; Case to be implemented
  ENDIF

  nbands = N_ELEMENTS(bands[0,*])
  index = ALM_INDEX(lmin, lmax)
  INDEX2L_ONLY, TEMPORARY(index), ell

  ;; Write the bands in fits file
  ;;------------------------------
  MWRFITS, bands, waveletsfile

  CASE signal OF
     'T': BEGIN
        
        ;; Start the band loop
        ;;---------------------
        FOR i=0, nbands-1 DO BEGIN
           ;; index_need = index

           ;; Now filter, restricting the alm and index to the needed ones
           ;;--------------------------------------------------------------
           uselmax = MAX(WHERE(bands[*,i] NE 0))
           IF NOT KEYWORD_SET(nodegrade) THEN BEGIN
              possiblenside = [1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192]
              whereok = WHERE(possiblenside GE uselmax) ; used to be uselmax/2, but this creates artefacts
              usenside = MIN(possiblenside[whereok])
              IF KEYWORD_SET(nside_min_w) NE 0 THEN usenside = (usenside > nside_min_w)
              IF KEYWORD_SET(nside_max_w) NE 0 THEN usenside = (usenside < nside_max_w)
           ENDIF ELSE BEGIN
              usenside = nside 
              IF KEYWORD_SET(nside_min_w) NE 0 THEN usenside = (usenside > nside_min_w)
              IF KEYWORD_SET(nside_max_w) NE 0 THEN usenside = (usenside < nside_max_w)
           ENDELSE
           whkeep = WHERE(ell LE uselmax, nwhkeep)
           ;; index_need = index_need[whkeep]
           IF NOT KEYWORD_SET(double) THEN alm_write = FLTARR(nwhkeep,2) ELSE alm_write = DBLARR(nwhkeep,2)
           FOR j=0, 1 DO alm_write[*,j] = TEMPORARY(alm[whkeep,j])*bands[ell[whkeep],i]

           ;; Transform back to Healpix format
           ;;----------------------------------
           PSM_ALM2MAP, TEMPORARY(alm_write), wavelets_write, ordering='RING', hpx_nside=usenside, double=double, /overwrite

           ;; Write in wavelets fits file
           ;;-----------------------------
           MWRFITS, TEMPORARY(wavelets_write), waveletsfile

        ENDFOR
     END
  ENDCASE

Closing:
END
