; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	WAVELETS2ALM
;
; PURPOSE:
;	Combine wavelets coefficients into spherical harmonics
;	coefficients (procedure reverse of ALM2WAVELETS).
;
; CATEGORY:
;	Wavelets (needlets) processing
;
; CALLING SEQUENCE:
;	WAVELETS2ALM, Wavelets, Alm
;
; INPUTS:
;	Wavelets:	Fits file name for input wavelets.
;	Alm:    	Fits file name for output harmonic coefficients.
;
; KEYWORD PARAMETERS:
;	BL_EFF: 	Output keyword, returning the effective window
;	                function of the total wavelet (needlet) bands
;	NOLASTWINDOW: 	Set this keyword to omit the last window in
;	                the recombination of the Alm. This in effect
;	                creates an attenuation of the harmonic modes
;	                at high ell. The impact is seen in BL_EFF.
;
; OUTPUTS:
;	A file is written, which contains the output harmonic
;	coefficients.
;
; SIDE EFFECTS:
;	A file is written to disk.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple access to information in the input file, harmonic
;	transform, and weighted summation.
;
; MODIFICATION HISTORY:
; 	09/09/2010:	Jacques Delabrouille, First version (adapted
; 	                from my previous implementation of the needlet
; 	                transform).
; 	04/01/2012:	JD, PSM license header.
; 	09/01/2012:	JD, doc header.
;-
PRO WAVELETS2ALM, wavelets, alm, bl_eff=bl_eff, nolastwindow=nolastwindow, double=double

  waveletstype = SIZE(wavelets, /type)
  
  IF waveletstype EQ 7 THEN BEGIN
     waveletsfile = wavelets

     ;; Get synthesis windows
     ;;-----------------------
     windows = MRDFITS(waveletsfile, 0)
     nwindows = N_ELEMENTS(windows[0,*])
     IF NOT KEYWORD_SET(nolastwindow) THEN nw=nwindows ELSE nw=nwindows-1
     windows = windows[*,0:nw-1]
     synwindows = windows
     bl_eff = TOTAL((windows*synwindows),2)
     ;FOR b=0, nwindows-1 DO synwindows[*,b] = synwindows[*,b]/bl_eff
     PSM_DEBUG_MESSAGE, 'Limit lmax to non-vanishing band upper limit', routine='WAVELETS2ALM'

     ;; Create output alm, alm index and ell values
     ;;---------------------------------------------
     lmin = 0
     lmax = N_ELEMENTS(windows[*,0]) - 1
     index = ALM_INDEX(lmin, lmax)
     INDEX2L_ONLY, index, ell
     nalm = LMAX2NALM(lmax)
     IF NOT KEYWORD_SET(double) THEN alm = FLTARR(nalm,2) ELSE alm = DBLARR(nalm,2)


     FOR i=1, nw DO BEGIN
        map = MRDFITS(waveletsfile, i)
        wh = WHERE(windows(*,i-1) NE 0)
        win_lmax = MAX(wh)<lmax
        PSM_MAP2ALM, TEMPORARY(map), almtemp, lmax=win_lmax, /overwrite, double=double      
        nnn = N_ELEMENTS(almtemp[*,0])
        alm[0:nnn-1,0] = alm[0:nnn-1,0] + almtemp[*,0] * synwindows[ell[0:nnn-1],i-1]       
        alm[0:nnn-1,1] = alm[0:nnn-1,1] + almtemp[*,1] * synwindows[ell[0:nnn-1],i-1]       
     ENDFOR

  ENDIF ELSE MESSAGE, 'Case not implemented'




END
